@@ -1,5 +1,5 @@
 /*
- * $Id: common.h,v 1.363 2006/02/22 16:08:59 rmanfredi Exp $
+ * $Id: common.h 12293 2006-11-09 22:55:17Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -44,10 +44,10 @@
 
 #define GTA_VERSION 0				/**< major version */
 #define GTA_SUBVERSION 96			/**< minor version */
-#define GTA_PATCHLEVEL 1			/**< patch level or teeny version */
-#define GTA_REVISION "stable"			/**< unstable, beta, stable */
-#define GTA_REVCHAR ""			/**< u - unstable, b - beta, none - stable */
-#define GTA_RELEASE "2006-02-22"	/**< ISO 8601 format YYYY-MM-DD */
+#define GTA_PATCHLEVEL 3			/**< patch level or teeny version */
+#define GTA_REVISION "stable"		/**< unstable, beta, stable */
+#define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
+#define GTA_RELEASE "2006-11-09"	/**< ISO 8601 format YYYY-MM-DD */
 #define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
 
 #if defined(USE_GTK1)
@@ -57,7 +57,7 @@
 #elif defined(USE_TOPLESS)
 #define GTA_INTERFACE "Topless"
 #else
-#define GTA_INTERFACE "X11"
+#error "Expected Gtk+ 1.2, Gtk+ 2.x or headless as user-interface."
 #endif
 
 #ifndef HAS_LIBXML2
@@ -69,22 +69,32 @@
  */
 
 #include <errno.h>
+
+#ifdef I_STDLIB
 #include <stdlib.h>
+#endif
+
 #include <stdio.h>
 #include <signal.h>
+
+#ifdef I_UNISTD
 #include <unistd.h>
+#endif
+
+#ifdef I_SYS_TYPES
 #include <sys/types.h>
+#endif
 
 #ifdef I_SYS_TIME
 #include <sys/time.h>
 #endif
+
 #ifdef I_SYS_TIME_KERNEL
 #define KERNEL
 #include <sys/time.h>
 #undef KERNEL
 #endif
 
-
 #ifdef MINGW32
 #include <ws2tcpip.h>
 #include <winsock.h>
@@ -115,13 +125,23 @@
 #define S_IRWXG _S_IREAD
 #define S_IRWXO _S_IREAD
 
-#else /* !MINGW32 */
+#endif /* !MINGW32 */
 
-#include <sys/resource.h>
+#ifdef I_SYS_SOCKET
 #include <sys/socket.h>
 #endif
 
+#ifdef I_SYS_RESOURCE
+#include <sys/resource.h>
+#endif
+
+#ifdef I_SYS_UN
+#include <sys/un.h>
+#endif
+
+#ifdef I_SYS_STAT
 #include <sys/stat.h>
+#endif
 
 #ifdef MINGW32
 struct iovec 
@@ -146,16 +166,42 @@ struct passwd
 #else /* !MINGW32 */
 
 #include <sys/uio.h>		/* For writev(), readv(), struct iovec */
+
+#ifdef I_NETINET_IN
 #include <netinet/in.h>
+#endif
+
 #include <netinet/tcp.h>
-#include <arpa/inet.h>		/* For ntohl(), htonl() */
 
 #endif /* MINGW32 */
 
+#ifdef I_ARPA_INET
+#include <arpa/inet.h>		/* For ntohl(), htonl() */
+#endif
+
 #include <ctype.h>
+
+#ifdef I_FCNTL
 #include <fcntl.h>
+#endif
+#ifdef I_SYS_FILE
+#include <sys/file.h>
+#endif
+
+#if !defined(I_FCNTL) && !defined(I_SYS_FILE)
+#include <sys/fcntl.h>		/* Fallback */
+#endif
+
+#ifdef I_STRING
 #include <string.h>
+#else
+#include <strings.h>
+#endif
+
+#ifdef I_DIRENT
 #include <dirent.h>
+#endif
+
 #include <setjmp.h>
 
 #ifdef I_TIME
@@ -176,7 +222,9 @@ struct passwd
 #include <inttypes.h>
 #endif /* I_INTTYPES */
 
+#ifdef I_SYS_MMAN
 #include <sys/mman.h>
+#endif
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *) -1)
@@ -192,7 +240,7 @@ struct passwd
  * Proper mmap() support for memory-mapped files requires ISO C functions like
  * sigsetjmp().
  */
-#if defined(__STDC_VERSION__)
+#if defined(HAS_MMAP) && defined(HAS_SIGSETJMP)
 #define USE_MMAP 1
 #endif	/* ISO C */
 
@@ -204,6 +252,18 @@ struct passwd
 #include <netinet/ip.h>
 #endif
 
+#if defined(HAS_POLL)
+#include <poll.h>
+#endif	/* HAS_POLL */
+
+#if !defined(AF_LOCAL) && defined(AF_UNIX)
+#define AF_LOCAL AF_UNIX
+#endif	/* !AF_LOCAL && AF_UNIX */
+
+#if !defined(PF_LOCAL) && defined(PF_UNIX)
+#define PF_LOCAL PF_UNIX
+#endif	/* !PF_LOCAL && PF_UNIX */
+
 /* For pedantic lint checks, define USE_LINT. We override some definitions
  * and hide ``inline'' to prevent certain useless warnings. */
 #ifdef USE_LINT
@@ -232,6 +292,14 @@ struct passwd
 
 #include <glib.h>
 
+#if defined(USE_GLIB1) && !defined(GLIB_MAJOR_VERSION)
+#error "Install GLib 1.2 to compile Gtk-Gnutella against GLib 1.2."
+#endif
+
+#if defined(USE_GLIB2) && !defined(GLIB_MAJOR_VERSION)
+#error "Install GLib 2.x to compile Gtk-Gnutella against GLib 2.x."
+#endif
+
 #ifdef USE_LINT
 #undef G_GNUC_INTERNAL
 #define G_GNUC_INTERNAL
@@ -242,8 +310,13 @@ struct passwd
 
 typedef guint64 filesize_t; /**< Use filesize_t to hold filesizes */
 
+#ifdef I_STDARG
 #include <stdarg.h>
+#endif
+
+#ifdef I_REGEX
 #include <regex.h>
+#endif
 
 #include <zlib.h>
 
@@ -255,6 +328,13 @@ typedef void (*GCallback) (void);
 #include <glib-object.h>
 #endif
 
+/*
+ * Array size determination
+ */
+#ifndef G_N_ELEMENTS
+#define G_N_ELEMENTS(arr) (sizeof (arr) / sizeof ((arr)[0]))
+#endif
+
 /*
  * Portability macros.
  */
@@ -288,6 +368,21 @@ typedef void (*GCallback) (void);
 #define STDERR_FILENO 2
 #endif /* STDERR_FILENO */
 
+/* Determines the maximum value of the given integer type "t". This
+ * works for signed as well as unsigned types. However, it's assumed
+ * the type consists of exactly sizeof (type) * CHAR_BIT bits. */
+#define MAX_INT_VAL(t) \
+	(((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 0))) \
+   	- 1 + ((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 0))))
+
+#ifndef TIME_T_MAX
+/* This assumes time_t is an integer, not a float */
+#define TIME_T_MAX MAX_INT_VAL(time_t)
+#endif /* TIME_T_MAX */
+
+#ifndef OFF_T_MAX
+#define OFF_T_MAX MAX_INT_VAL(off_t)
+#endif /* OFF_T_MAX */
 
 /*
  * Other common macros.
@@ -330,26 +425,6 @@ G_STMT_START {			\
 	}					\
 } G_STMT_END
 
-/**
- * Stores a RCS ID tag inside the object file. Every .c source file should
- * use this macro once as `RCSID("<dollar>Id$")' on top. The ID tag is
- * automagically updated each time the file is committed to the CVS repository.
- * The RCS IDs can be looked up from the compiled binary with e.g. `what',
- * `ident' or `strings'. See also rcs(1) and ident(1).
- */
-#define RCSID(x) \
-static inline const char *	\
-get_rcsid(void)		\
-{	\
-	static const char rcsid[] = "@(#) " x;	\
-	const char *s = rcsid;	\
-	while (*s != '\0') {	\
-		if (*s++ == '$')	\
-			break;	\
-	}	\
-	return s;	\
-}
-
 #if defined(__GNUC__) && defined(__GNUC_MINOR__)
 #define HAVE_GCC(major, minor) \
 	((__GNUC__ > (major)) || \
@@ -390,13 +465,30 @@ get_rcsid(void)		\
 #define WARN_UNUSED_RESULT
 #endif
 
+/* Instructs the compiler to emit code for this function even if it is
+ * or seems to be unused. */
+#if HAVE_GCC(3, 1)
+#define KEEP_FUNCTION __attribute__((used))
+#else /* GCC < 3.1 || !GCC */
+#define KEEP_FUNCTION
+#endif
+
+/* The antidote for WARN_UNUSED_RESULT. This attribute is sometimes
+ * misused for functions that return a result which SHOULD NOT be
+ * ignored in contrast to MUST NOT. Unfortunately, a simple "(void)"
+ * does not suppress this warning.
+ */
+#define IGNORE_RESULT(x) G_STMT_START { (void) (0 != (x)); }  G_STMT_END
+
 /* Functions using this attribute cause a warning if the variable
  * argument list does not contain a NULL pointer. */
+#ifndef G_GNUC_NULL_TERMINATED
 #if HAVE_GCC(4, 0)
-#define WARN_NEED_SENTINEL __attribute__((sentinel))
-#else /* GCC < 4 */
-#define WARN_NEED_SENTINEL
-#endif /* GCC >= 4 */
+#define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
+#else	/* GCC < 4 */
+#define G_GNUC_NULL_TERMINATED
+#endif	/* GCC >= 4 */
+#endif	/* G_GNUC_NULL_TERMINATED */
 
 /* Define G_LIKELY() and G_UNLIKELY() so that they are available when
  * using GLib 1.2 as well. These allow optimization by static branch
@@ -411,11 +503,37 @@ get_rcsid(void)		\
 #endif /* GCC >= 3.4 */
 #endif /* !G_LIKELY */
 
+#ifndef G_GNUC_MALLOC
+#if HAVE_GCC(3, 0)
+#define G_GNUC_MALLOC __attribute__((__malloc__))
+#else
+#define G_GNUC_MALLOC
+#endif	/* GCC >= 3.0 */
+#endif	/* G_GNUC_MALLOC */
+
+#if HAVE_GCC(3, 0)
+#define REGPARM(n)	__attribute__((__regparm__((n))))
+#else
+#define REGPARM(n)
+#endif	/* GCC >= 3.0 */
+
+#if HAVE_GCC(3, 0)
+#define ALWAYS_INLINE __attribute__((__always_inline__))
+#else
+#define ALWAYS_INLINE
+#endif	/* GCC >= 3.0 */
+
 /**
  * CMP() returns the sign of a-b, that means -1, 0, or 1.
  */
 #define CMP(a, b) ((a) == (b) ? 0 : (a) > (b) ? 1 : (-1))
 
+/**
+ * SIGN() returns the sign of an integer value.
+ */
+#define SIGN(x) ((x) == 0 ? 0 : (x) > 0 ? 1 : (-1))
+
+
 /**
  * STATIC_ASSERT() can be used to verify conditions at compile-time. For
  * example, it can be used to ensure that an array has a minimum or exact
@@ -444,16 +562,19 @@ get_rcsid(void)		\
 
 #define GTA_PORT			6346	/**< Default "standard" port */
 #define MAX_HOSTLEN			256		/**< Max length for FQDN host */
+#define GTA_BUILD					"$Revision: 12293 $"
 
 /* The next two defines came from huge.h --- Emile */
 #define SHA1_BASE32_SIZE 	32		/**< 160 bits in base32 representation */
 #define SHA1_RAW_SIZE		20		/**< 160 bits in binary representation */
 
+#define TTH_BASE32_SIZE 	39		/**< 160 bits in base32 representation */
+#define TTH_RAW_SIZE		24		/**< 160 bits in binary representation */
+
 #define BITPRINT_BASE32_SIZE 	72	/**< 352 bits in base32 representation
 									 **  includes dot between sha1 and tiger */
 #define BITPRINT_RAW_SIZE		44	/**< 352 bits in binary representation */
 
-
 /*
  * Forbidden glib calls.
  */
@@ -510,6 +631,28 @@ ngettext_(const gchar *msg1, const gchar *msg2, gulong n)
  */
 #define NG_(Single, Plural, Number) ngettext_((Single), (Plural), (Number))
 
+/**
+ * Stores a RCS ID tag inside the object file. Every .c source file should
+ * use this macro once as `RCSID("<dollar>Id$")' on top. The ID tag is
+ * automagically updated each time the file is committed to the CVS repository.
+ * The RCS IDs can be looked up from the compiled binary with e.g. `what',
+ * `ident' or `strings'. See also rcs(1) and ident(1).
+ */
+#define RCSID(x) \
+static KEEP_FUNCTION inline const char *	\
+get_rcsid(void)	\
+{	\
+	static const char rcsid[] = "@(#) " x;	\
+	const char *s = rcsid;	\
+	while (*s != '\0') {	\
+		if (*s++ == '$')	\
+			break;	\
+	}	\
+	return s;	\
+}
+
+#include "casts.h"
+
 #endif /* _common_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
