@@ -1,5 +1,5 @@
 /*
- * $Id: alive.c,v 1.17 2005/10/21 00:55:14 cbiere Exp $
+ * $Id: alive.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: alive.c,v 1.17 2005/10/21 00:55:14 cbiere Exp $");
+RCSID("$Id: alive.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "alive.h"
 #include "nodes.h"
@@ -176,7 +176,7 @@ alive_ping_can_send(pmsg_t *mb, const mqueue_t *q)
 	struct alive *a = n->alive_pings;
 	gchar *muid = pmsg_start(mb);
 
-	g_assert(((const struct gnutella_header *) muid)->function == GTA_MSG_INIT);
+	g_assert(gnutella_header_get_function(muid) == GTA_MSG_INIT);
 	g_assert(a->count == 0 || a->pings != NULL);
 
 	/*
@@ -242,11 +242,10 @@ alive_send_ping(gpointer obj)
 	struct alive *a = (struct alive *) obj;
 	gchar muid[GUID_RAW_SIZE];
 	struct alive_ping *ap;
-	struct gnutella_msg_init *m;
+	gnutella_msg_init_t *m;
 	guint32 size;
 	pmsg_t *mb;
 
-	STATIC_ASSERT(23 == sizeof *m);
 	g_assert(a->count == 0 || a->pings != NULL);
 
 	if (a->count >= a->maxcount)
@@ -343,7 +342,7 @@ alive_trim_upto(struct alive *a, GSList *item)
  * @return TRUE if it was indeed an ACK for a ping we sent.
  */
 gboolean
-alive_ack_ping(gpointer obj, gchar *muid)
+alive_ack_ping(gpointer obj, const gchar *muid)
 {
 	struct alive *a = obj;
 	GSList *sl;
@@ -372,7 +371,7 @@ alive_ack_ping(gpointer obj, gchar *muid)
  * replying to an alive ping.
  */
 void
-alive_ack_first(gpointer obj, gchar *muid)
+alive_ack_first(gpointer obj, const gchar *muid)
 {
 	struct alive *a = obj;
 	struct alive_ping *ap;
