@@ -1,5 +1,5 @@
 /*
- * $Id: bh_download.c,v 1.6 2005/11/21 21:11:23 rmanfredi Exp $
+ * $Id: bh_download.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2005, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_download.c,v 1.6 2005/11/21 21:11:23 rmanfredi Exp $");
+RCSID("$Id: bh_download.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "bh_download.h"
 #include "downloads.h"
@@ -51,6 +51,21 @@ RCSID("$Id: bh_download.c,v 1.6 2005/11/21 21:11:23 rmanfredi Exp $");
 #define BH_DL_DEFAULT_SIZE	4096	/* Default data buffer size */
 #define BH_DL_MAX_SIZE		65536	/* Maximum payload size we allow */
 
+struct browse_ctx {
+	gpointer owner;					/**< Download owning us */
+	rxdrv_t *rx;					/**< RX stack top */
+	gnet_host_t host;				/**< Host we're browsing, for logging */
+	gnet_search_t sh;				/**< Search ID to which hits are given */
+	gchar *vendor;					/**< Vendor version string */
+	gnutella_header_t header;		/**< Received header */
+	gchar *data;					/**< Where payload data is stored */
+	guint data_size;				/**< Size of data buffer */
+	guint pos;						/**< Reading position */
+	guint32 size;					/**< Payload size */
+	gboolean has_header;			/**< True when header has been read */
+	gboolean closed;				/**< Set when search is closed */
+};
+
 /**
  * Initialize the browse host context.
  */
@@ -89,23 +104,21 @@ browse_host_dl_for_search(struct browse_ctx *bc, gnet_search_t sh)
 static gboolean
 browse_data_read(struct browse_ctx *bc, pmsg_t *mb)
 {
-	gint r;
-
 	/*
 	 * Read header if it has not been fully fetched yet.
 	 */
 
 	if (!bc->has_header) {
-		gchar *w = (gchar *) &bc->header;
+		gchar *w = cast_to_gpointer(&bc->header);
 
-		r = pmsg_read(mb, w + bc->pos, sizeof(bc->header) - bc->pos);
-		bc->pos += r;
-
-		if (bc->pos < sizeof(struct gnutella_header))
+		g_assert(sizeof bc->header >= bc->pos);
+		bc->pos += pmsg_read(mb, &w[bc->pos], sizeof bc->header - bc->pos);
+		if (bc->pos < sizeof bc->header)
 			return FALSE;
 
 		bc->has_header = TRUE;		/* We have read the full header */
-		bc->size = peek_le32(bc->header.size);
+
+		bc->size = gnutella_header_get_size(&bc->header);
 
 		/*
 		 * Protect against too large data.
@@ -135,17 +148,17 @@ browse_data_read(struct browse_ctx *bc, pmsg_t *mb)
 	 */
 
 	if (bc->size) {
-		r = pmsg_read(mb, bc->data + bc->pos, bc->size - bc->pos);
-		bc->pos += r;
+		g_assert(bc->size >= bc->pos);
+		bc->pos += pmsg_read(mb, &bc->data[bc->pos], bc->size - bc->pos);
 	}
 
-	if (bc->pos < bc->size)
+	if (bc->pos >= bc->size) {
+		bc->has_header = FALSE;		/* For next message */
+		bc->pos = 0;
+		return TRUE; /* Must process message and continue */
+	} else {
 		return FALSE;
-
-	bc->has_header = FALSE;		/* For next message */
-	bc->pos = 0;
-
-	return TRUE;				/* Must process message and continue */
+	}
 }
 
 /**
@@ -162,7 +175,7 @@ browse_data_process(struct browse_ctx *bc)
 	 * We accept only query hits.
 	 */
 
-	if (bc->header.function != GTA_MSG_SEARCH_RESULTS) {
+	if (gnutella_header_get_function(&bc->header) != GTA_MSG_SEARCH_RESULTS) {
 		download_stop(bc->owner, GTA_DL_ERROR, "Non query-hit received");
 		return FALSE;
 	}
@@ -182,11 +195,14 @@ browse_data_process(struct browse_ctx *bc)
 /**
  * RX data indication callback used to give us some new Gnet traffic in a
  * low-level message structure (which can contain several Gnet messages).
+ *
+ * @return FALSE if an error occurred.
  */
-static void
+static gboolean
 browse_data_ind(rxdrv_t *rx, pmsg_t *mb)
 {
 	struct browse_ctx *bc = rx_owner(rx);
+	struct download *d;
 	gboolean error = FALSE;
 
 	while (browse_data_read(bc, mb)) {
@@ -207,10 +223,16 @@ browse_data_ind(rxdrv_t *rx, pmsg_t *mb)
 	 * layer, but in that case it is harmless to make the call anyway.
 	 */
 
-	if (!error)
-		download_browse_maybe_finished(bc->owner);
+	d = bc->owner;
+	download_check(d);
+
+	if (!error) {
+		download_browse_maybe_finished(d);
+		download_check(d);
+	}
 
 	pmsg_free(mb);
+	return !error && DOWNLOAD_IS_RUNNING(d);
 }
 
 /***
@@ -389,13 +411,12 @@ browse_host_dl_free(struct browse_ctx *bc)
 {
 	g_assert(bc != NULL);
 
-	if (bc->vendor)
-		atom_str_free(bc->vendor);
+	atom_str_free_null(&bc->vendor);
 	if (bc->rx)
 		rx_free(bc->rx);
 	if (!bc->closed)
 		search_dissociate_browse(bc->sh, bc->owner);
-	g_free(bc->data);
+	G_FREE_NULL(bc->data);
 	wfree(bc, sizeof *bc);
 }
 
