@@ -1,5 +1,5 @@
 /*
- * $Id: bh_upload.c,v 1.21 2006/01/01 19:09:02 rmanfredi Exp $
+ * $Id: bh_upload.c 12058 2006-09-27 19:33:55Z cbiere $
  *
  * Copyright (c) 2005, Christian Biere & Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_upload.c,v 1.21 2006/01/01 19:09:02 rmanfredi Exp $");
+RCSID("$Id: bh_upload.c 12058 2006-09-27 19:33:55Z cbiere $")
 
 #include "bh_upload.h"
 #include "share.h"
@@ -171,9 +171,13 @@ static ssize_t
 browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 {
 	static const gchar header[] =
-		"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">"
-		"<html><head><title>Browse Host</title></head><body>\r\n";
-	static const gchar trailer[] = "</ul></body></html>";
+		"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\r\n"
+		"<html>\r\n"
+		"<head>\r\n"
+		"<title>Browse Host</title>\r\n"
+		"</head>\r\n"
+		"<body>\r\n";
+	static const gchar trailer[] = "</ul>\r\n</body>\r\n</html>\r\n";
 	struct browse_host_ctx *bh = ctx;
 	gchar *p = dest;
 
@@ -207,7 +211,7 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 					" file",
 					shared_files_scanned() == 1 ? "" : "s",
 					" ",
-					short_kb_size(shared_kbytes_scanned()),
+					short_kb_size(shared_kbytes_scanned(), FALSE),
 					" total</h3>\r\n"
 					"<ul>\r\n", (void *) 0);
 				bh->b_data = bh->w_buf;
@@ -239,7 +243,7 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 			}
 
 			if (!bh->b_data) {
-				shared_file_t *sf;
+				const shared_file_t *sf;
 
 				bh->file_index++;
 				sf = shared_file(bh->file_index);
@@ -250,34 +254,40 @@ browse_host_read_html(gpointer ctx, gpointer const dest, size_t size)
 				} else if (SHARE_REBUILDING == sf) {
 					browse_host_next_state(bh, BH_STATE_REBUILDING);
 				} else {
+					const gchar * const name_nfc = shared_file_name_nfc(sf);
+					const filesize_t file_size = shared_file_size(sf);
 					size_t html_size;
 					gchar *html_name;
 
-					html_size = 1 + html_escape(sf->name_nfc, NULL, 0);
+					html_size = 1 + html_escape(name_nfc, NULL, 0);
 					html_name = walloc(html_size);
-					html_escape(sf->name_nfc, html_name, html_size);
+					html_escape(name_nfc, html_name, html_size);
 
 					if (sha1_hash_available(sf)) {
+						const gchar * const sha1 = shared_file_sha1(sf);
+
 						bh->w_buf_size = w_concat_strings(&bh->w_buf,
 							"<li><a href=\"/uri-res/N2R?urn:sha1:",
-							sha1_base32(sf->sha1_digest),
+							sha1_base32(sha1),
 							"\">", html_name, "</a>&nbsp;[",
-							short_html_size(sf->file_size),
+							short_html_size(file_size, display_metric_units),
 							"]</li>\r\n",
 							(void *) 0);
 					} else {
 						gchar *escaped;
 
-						escaped = url_escape(sf->name_nfc);
+						escaped = url_escape(name_nfc);
 						bh->w_buf_size = w_concat_strings(&bh->w_buf,
 							"<li><a href=\"/get/",
-							uint64_to_string(sf->file_index),
+							uint64_to_string(shared_file_index(sf)),
 							"/", escaped, "\">", html_name, "</a>"
-							"&nbsp;[", short_html_size(sf->file_size),
+							"&nbsp;[",
+							short_html_size(file_size, display_metric_units),
 							"]</li>\r\n", (void *) 0);
 
-						if (escaped != sf->name_nfc)
+						if (escaped != name_nfc) {
 							G_FREE_NULL(escaped);
+						}
 					}
 
 					wfree(html_name, html_size);
@@ -364,7 +374,7 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
 		gint i;
 
 		for (i = 0; i < BH_SCAN_AHEAD; i++) {
-			shared_file_t *sf;
+			const shared_file_t *sf;
 
 		skip:
 			bh->file_index++;
@@ -377,7 +387,7 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
 			} else if (SHARE_REBUILDING == sf)
 				break;
 			else
-				files = g_slist_prepend(files, sf);
+				files = g_slist_prepend(files, deconstify_gpointer(sf));
 		}
 
 		if (NULL == files)		/* Did not find any more file to include */
@@ -423,7 +433,7 @@ browse_host_read_qhits(gpointer ctx, gpointer const dest, size_t size)
  * Write data to the TX stack.
  */
 ssize_t
-browse_host_write(gpointer ctx, gpointer data, size_t size)
+browse_host_write(gpointer ctx, gconstpointer data, size_t size)
 {
 	struct browse_host_ctx *bh = ctx;
 
@@ -500,12 +510,13 @@ browse_host_close(gpointer ctx, gboolean fully_served)
 	 */
 
 	if (fully_served) {
-		if (bh->flags & BH_HTML)
+		if (bh->flags & BH_HTML) {
 			gnet_prop_set_guint32_val(PROP_HTML_BROWSE_SERVED,
 				html_browse_served + 1);
-		else if (bh->flags & BH_QHITS)
+		} else if (bh->flags & BH_QHITS) {
 			gnet_prop_set_guint32_val(PROP_QHITS_BROWSE_SERVED,
 				qhits_browse_served + 1);
+		}
 	}
 
 	wfree(bh, sizeof *bh);
@@ -604,14 +615,14 @@ browse_host_open(
 	 * Update statistics.
 	 */
 
-	if (flags & BH_HTML)
+	if (flags & BH_HTML) {
 		gnet_prop_set_guint32_val(PROP_HTML_BROWSE_COUNT,
 			html_browse_count + 1);
-
-	if (flags & BH_QHITS)
+	}
+	if (flags & BH_QHITS) {
 		gnet_prop_set_guint32_val(PROP_QHITS_BROWSE_COUNT,
 			qhits_browse_count + 1);
-
+	}
 	return &bh->special;
 }
 
