@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.h,v 1.2 2004/10/04 11:00:07 rmanfredi Exp $
+ * $Id: bitzi.h 11703 2006-08-23 17:40:39Z cbiere $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -23,11 +23,22 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Bitzi Core search code.
+ *
+ * @author Alex Bennee <alex@bennee.com>
+ * @date 2004
+ */
+
 #ifndef _core_bitzi_h_
 #define _core_bitzi_h_
 
 #include "if/core/bitzi.h"
 
 void bitzi_init(void);
+void bitzi_close(void);
 
 #endif /* _core_bitzi_h_ */
