@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.c,v 1.13 2004/10/25 23:06:26 cbiere Exp $
+ * $Id: bsched.c 12282 2006-11-09 18:26:21Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -24,24 +24,29 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Bandwidth scheduling.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: bsched.c,v 1.13 2004/10/25 23:06:26 cbiere Exp $");
+RCSID("$Id: bsched.c 12282 2006-11-09 18:26:21Z cbiere $")
 
 #include "bsched.h"
 #include "inet.h"
 #include "uploads.h"
 
-#include "if/core/wrap.h"		/* wrapped_io_t */
+#include "if/core/wrap.h"		/* For wrapped_io_t */
 #include "if/gnet_property_priv.h"
 
 #include "lib/glib-missing.h"
 #include "lib/walloc.h"
+#include "lib/misc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 
@@ -57,91 +62,36 @@ static GSList *bws_in_list = NULL;
 static gint bws_out_ema = 0;
 static gint bws_in_ema = 0;
 
-#define BW_SLOT_MIN		64		/* Minimum bandwidth/slot for realloc */
+#define BW_SLOT_MIN		64	 /**< Minimum bandwidth/slot for realloc */
 
-#define BW_OUT_UP_MIN	8192	/* Minimum out bandwidth for becoming ultra */
-#define BW_OUT_GNET_MIN	128		/* Minimum out bandwidth per Gnet connection */
-#define BW_OUT_LEAF_MIN	32		/* Minimum out bandwidth per leaf connection */
+#define BW_OUT_UP_MIN	8192 /**< Minimum out bandwidth for becoming ultra */
+#define BW_OUT_GNET_MIN	128	 /**< Minimum out bandwidth per Gnet connection */
+#define BW_OUT_LEAF_MIN	32	 /**< Minimum out bandwidth per leaf connection */
 
-#define BW_TCP_MSG		40		/* Smallest size of a TCP message */
-#define BW_UDP_MSG		20		/* Minimal IP overhead for a UDP message */
+#define BW_TCP_MSG		40	 /**< Smallest size of a TCP message */
+#define BW_UDP_MSG		28	 /**< Minimal IP+UDP overhead for a UDP message */
 
-#define BW_UDP_OVERSIZE	512		/* Allow that many bytes over available b/w */
+#define BW_UDP_OVERSIZE	512	 /**< Allow that many bytes over available b/w */
 
-/*
- * Determine how large an I/O vector the kernel can accept.
- */
-
-#if defined(MAXIOV)
-#define MAX_IOV_COUNT	MAXIOV			/* Regular */
-#elif defined(UIO_MAXIOV)
-#define MAX_IOV_COUNT	UIO_MAXIOV		/* Linux */
-#elif defined(IOV_MAX)
-#define MAX_IOV_COUNT	IOV_MAX			/* Solaris */
-#else
-#define MAX_IOV_COUNT	16				/* Unknown, use required minimum */
-#endif
-
-/**
- * Wrapper over writev() ensuring that we don't request more than
- * MAX_IOV_COUNT entries at a time.
- */
-static ssize_t
-safe_writev(wrap_io_t *wio, struct iovec *iov, gint iovcnt)
+static inline void
+bio_check(const bio_source_t *bio)
 {
-	size_t sent = 0;
-	struct iovec *end = iov + iovcnt;
-	struct iovec *siov;
-	gint siovcnt = MAX_IOV_COUNT;
-	gint iovsent = 0;
-
-	for (siov = iov; siov < end; siov += siovcnt) {
-		ssize_t r;
-		size_t size;
-		struct iovec *xiv;
-		struct iovec *xend;
-
-		siovcnt = iovcnt - iovsent;
-		if (siovcnt > MAX_IOV_COUNT)
-			siovcnt = MAX_IOV_COUNT;
-		g_assert(siovcnt > 0);
-		
-		r = wio->writev(wio, siov, siovcnt);
-
-		if ((ssize_t) -1 == r || 0 == r) {
-			if (r == 0 || sent)
-				break;				/* Don't flag error if bytes sent */
-			return -1;				/* Propagate error */
-		}
-
-		sent += r;
-		iovsent += siovcnt;		/* We'll break out if we did not send it all */
-
-		/*
-		 * How much did we sent?  If not the whole vector, we're blocking,
-		 * so stop writing and return amount we sent.
-		 */
-
-		for (size = 0, xiv = siov, xend = siov + siovcnt; xiv < xend; xiv++)
-			size += xiv->iov_len;
-
-		if ((size_t) r < size)
-			break;
-	}
-
-	return sent;
+	g_assert(bio);
+	g_assert(BIO_SOURCE_MAGIC == bio->magic);
 }
 
 /**
  * Create a new bandwidth scheduler.
  *
- * `type' refers to the scheduling model.  Only BS_T_STREAM for now.
- * `mode' refers to the nature of the sources: either reading or writing.
- * `bandwidth' is the expected bandwidth in bytes per second.
- * `period' is the scheduling period in ms.
+ * @param `name' no brief description.
+ * @param `type' refers to the scheduling model.  Only BS_T_STREAM for now.
+ * @param `mode' refers to the nature of the sources: either reading or writing.
+ * @param `bandwidth' is the expected bandwidth in bytes per second.
+ * @param `period' is the scheduling period in ms.
  */
 bsched_t *
-bsched_make(gchar *name, gint type, guint32 mode, gint bandwidth, gint period)
+bsched_make(const gchar *name, gint type, guint32 mode,
+	gint bandwidth, gint period)
 {
 	bsched_t *bs;
 
@@ -155,7 +105,7 @@ bsched_make(gchar *name, gint type, guint32 mode, gint bandwidth, gint period)
 	g_assert(type == BS_T_STREAM);		/* XXX only mode supported for now */
 	g_assert(bandwidth <= BS_BW_MAX);	/* Signed, and multiplied by 1000 */
 
-	bs = (bsched_t *) g_malloc0(sizeof(*bs));
+	bs = g_malloc0(sizeof(*bs));
 
 	bs->name = g_strdup(name);
 	bs->flags = mode;
@@ -179,11 +129,12 @@ bsched_make(gchar *name, gint type, guint32 mode, gint bandwidth, gint period)
 static void
 bsched_free(bsched_t *bs)
 {
-	GList *l;
+	GList *iter;
 
-	for (l = bs->sources; l; l = g_list_next(l)) {
-		bio_source_t *bio = (bio_source_t *) l->data;
-		
+	for (iter = bs->sources; iter; iter = g_list_next(iter)) {
+		bio_source_t *bio = iter->data;
+
+		bio_check(bio);
 		g_assert(bio->bs == bs);
 		bio->bs = NULL;				/* Mark orphan source */
 	}
@@ -224,10 +175,10 @@ bsched_reset_stealers(bsched_t *bs)
 void
 bsched_config_steal_http_gnet(void)
 {
-	GSList *l;
+	GSList *iter;
 
-	for (l = bws_list; l; l = g_slist_next(l)) {
-		bsched_t *bs = (bsched_t *) l->data;
+	for (iter = bws_list; iter; iter = g_slist_next(iter)) {
+		bsched_t *bs = iter->data;
 		bsched_reset_stealers(bs);
 	}
 
@@ -270,10 +221,10 @@ bsched_config_steal_http_gnet(void)
 void
 bsched_config_steal_gnet(void)
 {
-	GSList *l;
+	GSList *iter;
 
-	for (l = bws_list; l; l = g_slist_next(l)) {
-		bsched_t *bs = (bsched_t *) l->data;
+	for (iter = bws_list; iter; iter = g_slist_next(iter)) {
+		bsched_t *bs = iter->data;
 		bsched_reset_stealers(bs);
 	}
 
@@ -357,10 +308,10 @@ bsched_init(void)
 void
 bsched_close(void)
 {
-	GSList *l;
+	GSList *iter;
 
-	for (l = bws_list; l; l = g_slist_next(l))
-		bsched_free(l->data);
+	for (iter = bws_list; iter; iter = g_slist_next(iter))
+		bsched_free(iter->data);
 
 	g_slist_free(bws_list);
 	g_slist_free(bws_out_list);
@@ -435,7 +386,7 @@ bsched_enable(bsched_t *bs)
 	g_assert(bs);
 
 	bs->flags |= BS_F_ENABLED;
-	g_get_current_time(&bs->last_period);
+	tm_now(&bs->last_period);
 }
 
 /**
@@ -486,10 +437,10 @@ bsched_enable_all(void)
 void
 bsched_shutdown(void)
 {
-	GSList *l;
+	GSList *sl;
 
-	for (l = bws_list; l; l = g_slist_next(l))
-		bsched_disable(l->data);
+	for (sl = bws_list; sl; sl = g_slist_next(sl))
+		bsched_disable(sl->data);
 }
 
 /**
@@ -498,13 +449,13 @@ bsched_shutdown(void)
 static void
 bio_enable(bio_source_t *bio)
 {
-	g_assert(bio->io_tag == 0);
+	bio_check(bio);
+	g_assert(0 == bio->io_tag);
 	g_assert(bio->io_callback);		/* "passive" sources not concerned */
 
 	bio->io_tag = inputevt_add(bio->wio->fd(bio->wio),
-		(inputevt_cond_t) INPUT_EVENT_EXCEPTION |
-			((bio->flags & BIO_F_READ) ? INPUT_EVENT_READ : INPUT_EVENT_WRITE),
-		bio->io_callback, bio->io_arg);
+			(bio->flags & BIO_F_READ) ? INPUT_EVENT_RX : INPUT_EVENT_WX,
+			bio->io_callback, bio->io_arg);
 
 	g_assert(bio->io_tag);
 }
@@ -519,10 +470,11 @@ bio_enable(bio_source_t *bio)
 static void
 bio_disable(bio_source_t *bio)
 {
+	bio_check(bio);
 	g_assert(bio->io_tag);
 	g_assert(bio->io_callback);		/* "passive" sources not concerned */
 
-	g_source_remove(bio->io_tag);
+	inputevt_remove(bio->io_tag);
 	bio->io_tag = 0;
 }
 
@@ -532,7 +484,7 @@ bio_disable(bio_source_t *bio)
 void
 bio_add_callback(bio_source_t *bio, inputevt_handler_t callback, gpointer arg)
 {
-	g_assert(bio);
+	bio_check(bio);
 	g_assert(bio->io_callback == NULL);	/* "passive" source */
 	g_assert(callback);
 
@@ -549,7 +501,7 @@ bio_add_callback(bio_source_t *bio, inputevt_handler_t callback, gpointer arg)
 void
 bio_remove_callback(bio_source_t *bio)
 {
-	g_assert(bio);
+	bio_check(bio);
 	g_assert(bio->io_callback);		/* Not a "passive" source */
 
 	if (bio->io_tag)
@@ -566,12 +518,12 @@ bio_remove_callback(bio_source_t *bio)
 static void
 bsched_no_more_bandwidth(bsched_t *bs)
 {
-	GList *l;
+	GList *iter;
 
-	for (l = bs->sources; l; l = g_list_next(l)) {
-		bio_source_t *bio = (bio_source_t *) l->data;
+	for (iter = bs->sources; iter; iter = g_list_next(iter)) {
+		bio_source_t *bio = iter->data;
 
-		g_assert(bio != NULL);
+		bio_check(bio);
 
 		if (bio->io_tag)
 			bio_disable(bio);
@@ -586,13 +538,12 @@ bsched_no_more_bandwidth(bsched_t *bs)
 static void
 bsched_clear_active(bsched_t *bs)
 {
-	GList *l;
+	GList *iter;
 
-	for (l = bs->sources; l; l = g_list_next(l)) {
-		bio_source_t *bio = (bio_source_t *) l->data;
-
-		g_assert(bio != NULL);
+	for (iter = bs->sources; iter; iter = g_list_next(iter)) {
+		bio_source_t *bio = iter->data;
 
+		bio_check(bio);
 		bio->flags &= ~BIO_F_ACTIVE;
 	}
 }
@@ -607,18 +558,18 @@ bsched_clear_active(bsched_t *bs)
 static void
 bsched_begin_timeslice(bsched_t *bs)
 {
-	GList *l;
+	GList *iter;
 	GList *last = NULL;
 	gdouble norm_factor = 1000.0 / bs->period;
 	gint count;
 
-	for (count = 0, l = bs->sources; l; l = g_list_next(l)) {
-		bio_source_t *bio = (bio_source_t *) l->data;
+	for (count = 0, iter = bs->sources; iter; iter = g_list_next(iter)) {
+		bio_source_t *bio = iter->data;
 		guint32 actual;
 
-		g_assert(bio != NULL);
+		bio_check(bio);
 
-		last = l;			/* Remember last seen source  for rotation */
+		last = iter;		/* Remember last seen source  for rotation */
 		count++;			/* Count them for assertion */
 
 		bio->flags &= ~(BIO_F_ACTIVE | BIO_F_USED);
@@ -661,9 +612,10 @@ bsched_begin_timeslice(bsched_t *bs)
 
 	if (last != NULL && last != bs->sources) {
 		bio_source_t *bio;
+
 		g_assert(bs->sources != NULL);
-		bio = (bio_source_t *) bs->sources->data;
-		g_assert(bio != NULL);
+		bio = bs->sources->data;
+		bio_check(bio);
 		bs->sources = g_list_remove(bs->sources, bio);
 		bs->sources = gm_list_insert_after(bs->sources, last, bio);
 	}
@@ -726,7 +678,7 @@ bsched_begin_timeslice(bsched_t *bs)
 static void
 bsched_bio_add(bsched_t *bs, bio_source_t *bio)
 {
-	g_assert(bio != NULL);
+	bio_check(bio);
 
 	bs->sources = g_list_append(bs->sources, bio);
 	bs->count++;
@@ -748,7 +700,7 @@ bsched_bio_add(bsched_t *bs, bio_source_t *bio)
 static void
 bsched_bio_remove(bsched_t *bs, bio_source_t *bio)
 {
-	g_assert(bio != NULL);
+	bio_check(bio);
 
 	bs->sources = g_list_remove(bs->sources, bio);
 	bs->count--;
@@ -767,7 +719,7 @@ bsched_bio_remove(bsched_t *bs, bio_source_t *bio)
  * whether the source can accept those calls without blocking will have to
  * be determined explicitly.
  *
- * Returns new bio_source object.
+ * @returns new bio_source object.
  */
 bio_source_t *
 bsched_source_add(
@@ -786,8 +738,9 @@ bsched_source_add(
 	g_assert((flags & BIO_F_RW) != BIO_F_RW);	/* Either reading or writing */
 	g_assert(!(flags & ~BIO_F_RW));				/* Can only specify r/w flags */
 
-	bio = (bio_source_t *) walloc0(sizeof(*bio));
+	bio = walloc0(sizeof *bio);
 
+	bio->magic = BIO_SOURCE_MAGIC;
 	bio->bs = bs;
 	bio->wio = wio;
 	bio->flags = flags;
@@ -816,14 +769,20 @@ bsched_source_add(
 void
 bsched_source_remove(bio_source_t *bio)
 {
-	bsched_t *bs = bio->bs;
+	bsched_t *bs;
+	
+	bio_check(bio);
 
+	bs = bio->bs;
 	if (bs)
 		bsched_bio_remove(bs, bio);
-	if (bio->io_tag)
-		g_source_remove(bio->io_tag);
+	if (bio->io_tag) {
+		inputevt_remove(bio->io_tag);
+		bio->io_tag = 0;
+	}
 
-	wfree(bio, sizeof(*bio));
+	bio->magic = 0;
+	wfree(bio, sizeof *bio);
 }
 
 /**
@@ -864,19 +823,24 @@ bsched_set_bandwidth(bsched_t *bs, gint bandwidth)
 
 
 /**
- * Returns the bandwidth available for a given source.
- * `len' is the amount of bytes requested by the application.
+ * @param `bio' no brief description.
+ * @param `len' is the amount of bytes requested by the application.
+ *
+ * @returns the bandwidth available for a given source.
  */
 static gint
 bw_available(bio_source_t *bio, gint len)
 {
-	bsched_t *bs = bio->bs;
+	bsched_t *bs;
 	gint available;
 	gint result;
 	gboolean capped = FALSE;
 	gboolean used;
 	gboolean active;
 
+	bio_check(bio);
+
+	bs = bio->bs;
 	if (!(bs->flags & BS_F_ENABLED))		/* Scheduler disabled */
 		return len;							/* Use amount requested */
 
@@ -1084,8 +1048,9 @@ bw_available(bio_source_t *bio, gint len)
  * Update bandwidth used, and scheduler statistics.
  * If no more bandwidth is available, disable all sources.
  *
- * `used' is the amount of bytes used by the I/O.
- * `requested' is the amount of bytes requested for the I/O.
+ * @param `bs' no brief description.
+ * @param `used' is the amount of bytes used by the I/O.
+ * @param `requested' is the amount of bytes requested for the I/O.
  */
 static void
 bsched_bw_update(bsched_t *bs, gint used, gint requested)
@@ -1135,18 +1100,18 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
 	size_t amount;
 	ssize_t r;
 
-	g_assert(bio);
+	bio_check(bio);
 	g_assert(bio->flags & BIO_F_WRITE);
 
-	/* 
-	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN 
+	/*
+	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN
 	 * to signal that we cannot perform any I/O right now.
 	 */
 
 	available = bw_available(bio, len);
 
 	if (available == 0) {
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 		return -1;
 	}
 
@@ -1166,10 +1131,10 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
 	 *		--RAM, 05/10/2003
 	 */
 
-	if ((ssize_t) -1  == r && errno == 0) {
+	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("wio->write(fd=%d, len=%d) returned -1 with errno = 0, "
 			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 	}
 
 	if (r > 0) {
@@ -1181,7 +1146,8 @@ bio_write(bio_source_t *bio, gconstpointer data, size_t len)
 }
 
 /**
- * Write at most `len' bytes from `iov' to source's fd, as bandwidth permits.
+ * Write at most `len' bytes from `iov' to source's fd, as bandwidth permits,
+ * `len' being determined by the size of the supplied I/O vector.
  * If we cannot write anything due to bandwidth constraints, return -1 with
  * errno set to EAGAIN.
  */
@@ -1194,7 +1160,7 @@ bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 	struct iovec *siov;
 	gint slen = -1;			/* Avoid "may be used uninitialized" warning */
 
-	g_assert(bio);
+	bio_check(bio);
 	g_assert(bio->flags & BIO_F_WRITE);
 
 	/*
@@ -1204,15 +1170,15 @@ bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 	for (r = 0, siov = iov, len = 0; r < iovcnt; r++, siov++)
 		len += siov->iov_len;
 
-	/* 
-	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN 
+	/*
+	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN
 	 * to signal that we cannot perform any I/O right now.
 	 */
 
 	available = bw_available(bio, len);
 
 	if (available == 0) {
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 		return -1;
 	}
 
@@ -1280,10 +1246,10 @@ bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 	 *		--RAM, 05/10/2003
 	 */
 
-	if ((ssize_t) -1 == r && errno == 0) {
+	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("writev(fd=%d, len=%d) returned -1 with errno = 0, "
 			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 	}
 
 	if (r > 0) {
@@ -1306,20 +1272,22 @@ bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt)
 
 /**
  * Send UDP datagram to specified destination `to'.
- * If we cannot write anything due to bandwidth constraints, return -1 with
- * errno set to EAGAIN.
+ *
+ * @return -1 with errno set to EAGAIN, if we cannot write anything due
+ * to bandwidth constraints.
  */
 ssize_t
-bio_sendto(bio_source_t *bio, gnet_host_t *to, gconstpointer data, size_t len)
+bio_sendto(bio_source_t *bio, const gnet_host_t *to,
+	gconstpointer data, size_t len)
 {
 	size_t available;
 	ssize_t r;
 
-	g_assert(bio);
+	bio_check(bio);
 	g_assert(bio->flags & BIO_F_WRITE);
 
-	/* 
-	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN 
+	/*
+	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN
 	 * to signal that we cannot perform any I/O right now.
 	 *
 	 * Note that datagrams are necessarily atomic operations, therefore
@@ -1333,7 +1301,7 @@ bio_sendto(bio_source_t *bio, gnet_host_t *to, gconstpointer data, size_t len)
 	available = bw_available(bio, len);
 
 	if (available == 0 || available + BW_UDP_OVERSIZE < len) {
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 		return -1;
 	}
 
@@ -1341,6 +1309,8 @@ bio_sendto(bio_source_t *bio, gnet_host_t *to, gconstpointer data, size_t len)
 		printf("bio_sendto(wio=%d, len=%d) available=%d\n",
 			bio->wio->fd(bio->wio), (gint) len, (gint) available);
 
+	g_assert(bio->wio != NULL);
+	g_assert(bio->wio->sendto != NULL);
 	r = bio->wio->sendto(bio->wio, to, data, len);
 
 	/*
@@ -1351,10 +1321,10 @@ bio_sendto(bio_source_t *bio, gnet_host_t *to, gconstpointer data, size_t len)
 	 *		--RAM, 05/10/2003
 	 */
 
-	if ((ssize_t) -1 == r && errno == 0) {
+	if ((ssize_t) -1 == r && 0 == errno) {
 		g_warning("wio->sendto(fd=%d, len=%d) returned -1 with errno = 0, "
 			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 	}
 
 	if (r > 0) {
@@ -1365,43 +1335,107 @@ bio_sendto(bio_source_t *bio, gnet_host_t *to, gconstpointer data, size_t len)
 	return r;
 }
 
+#ifdef USE_MMAP
+static sigjmp_buf mmap_env;
+static volatile sig_atomic_t mmap_access;
+static volatile signal_handler_t old_sigbus_handler;
+static volatile signal_handler_t old_sigsegv_handler;
+
+/**
+ * Handles SIGBUS or SIGSEGV when accessing mmap()ed areas. This may
+ * happen when the underlying file gets truncated or the device fails
+ * with an I/O error. When using read() one would see "errno == EIO"
+ * or similar. With mmap() we have to catch this ourselves.
+ */
+static void
+signal_handler(int signo)
+{
+	if (mmap_access) {
+		mmap_access = 0;
+		siglongjmp(mmap_env, signo);
+	}
+	
+	/*
+	 * If this signal did not occur whilst accessing a mmap()ed area,
+	 * there is something wrong and we delegate the signal to the old
+	 * handler.
+	 */
+
+	switch (signo) {
+	case SIGBUS:
+		set_signal(signo, old_sigbus_handler);
+		break;
+	case SIGSEGV:
+		set_signal(signo, old_sigsegv_handler);
+		break;
+	}
+	raise(signo);
+}
+#endif /* !USE_MMAP */
+
 /**
  * Write at most `len' bytes to source's fd, as bandwidth permits.
+ *
  * Bytes are read from `offset' in the in_fd file descriptor, and the value
  * is updated in place by the kernel.
  *
- * If we cannot write anything due to bandwidth constraints, return -1 with
- * errno set to EAGAIN.
+ * @return -1 with errno set to EAGAIN, if we cannot write anything due to
+ * bandwidth constraints.
  */
 ssize_t
-bio_sendfile(bio_source_t *bio, gint in_fd, off_t *offset, size_t len)
+bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd, off_t *offset,
+	size_t len)
 {
-#ifndef HAS_SENDFILE
+#if !defined(USE_MMAP) && !defined(HAS_SENDFILE)
+
+	(void) ctx;
 	(void) bio;
 	(void) in_fd;
 	(void) offset;
 	(void) len;
-	g_error("missing sendfile(2), should not have been called");
-	return EOPNOTSUPP;		/* g_error() is fatal, just shut warnings */
-#else
-	size_t available;
-	size_t amount;
-	ssize_t r;
-	off_t start = *offset;
 
-	g_assert(bio);
+	g_assert_not_reached();
+	/* NOTREACHED */
+
+	errno = ENOSYS;
+
+	return (ssize_t) -1;
+
+#else /* USE_MMAP || HAS_SENDFILE */
+
+	/*
+	 * amount is only declared volatile to shut up the bogus warning by GCC
+	 * 2.95.x that ``amount'' *might* be clobbered by sigsetjmp - which is
+	 * actually not the case.
+	 */
+	volatile size_t amount;
+	size_t available;
+	ssize_t r;
+	gint out_fd;
+	off_t start;
+
+	g_assert(ctx);
+	bio_check(bio);
+	g_assert(bio->wio);
 	g_assert(bio->flags & BIO_F_WRITE);
+	g_assert(offset);
 	g_assert(len > 0);
 
-	/* 
-	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN 
+	start = *offset;
+	g_assert(start >= 0);
+	g_assert(start + (off_t) len > start);
+
+	out_fd = bio->wio->fd(bio->wio);
+
+	/*
+	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN
 	 * to signal that we cannot perform any I/O right now.
 	 */
 
 	available = bw_available(bio, len);
 
 	if (available == 0) {
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 		return -1;
 	}
 
@@ -1411,14 +1445,146 @@ bio_sendfile(bio_source_t *bio, gint in_fd, off_t *offset, size_t len)
 		printf("bsched_write(fd=%d, len=%d) available=%d\n",
 			bio->wio->fd(bio->wio), (gint) len, (gint) available);
 
+#ifdef USE_MMAP
+	{
+		static gboolean first_call = TRUE;
+		const gchar *data;
+		gint n;
+
+		if (first_call) {
+			first_call = FALSE;
+			
+			/*
+			 * It would be a waste to change the signal handler each
+			 * time. Therefore, the handler is set up only once and
+			 * signals that do not occur whilst mmap_access is set
+			 * are delegated to the original handler.
+			 */
+			old_sigbus_handler = set_signal(SIGBUS, signal_handler);
+			old_sigsegv_handler = set_signal(SIGSEGV, signal_handler);
+		}
+
+		if (
+			ctx->map == NULL ||
+			start < ctx->map_start ||
+			start + amount > ctx->map_end
+		) {
+			static const size_t min_map_size = 64 * 1024;
+			size_t map_len, old_len;
+			off_t map_start;
+			int flags = MAP_PRIVATE;
+
+			/*
+			 * Make sure ``off'' is page-aligned for mmap(); some
+			 * implementations require this.
+			 */
+
+			map_start = start - (start % compat_pagesize());
+			map_len = amount + (start - map_start);
+
+			/*
+			 * Map at least 64 KiB so that mmap() isn't called too frequently.
+			 */
+
+			if (
+				map_len < min_map_size &&
+				map_start + min_map_size > map_start
+			) {
+				map_len = min_map_size;
+			}
+
+			old_len = ctx->map_end - ctx->map_start;
+			if (ctx->map) {
+			   	if (old_len != map_len) {
+					munmap(ctx->map, old_len);
+					ctx->map = NULL;
+				} else {
+					flags |= MAP_FIXED;
+				}
+			}
+
+			ctx->map_start = map_start;
+			ctx->map_end = ctx->map_start + map_len;
+			g_assert(ctx->map_start < ctx->map_end);
+
+			for (;;) {
+				void *addr;
+
+				addr = mmap(ctx->map, map_len, PROT_READ, flags, in_fd,
+						ctx->map_start);
+				if (addr == MAP_FAILED && ctx->map) {
+					munmap(ctx->map, old_len);
+					ctx->map = NULL;
+					old_len = 0;
+					flags &= ~MAP_FIXED;
+				} else {
+					ctx->map = addr;
+					break;
+				}
+			}
+
+			if (MAP_FAILED == ctx->map) {
+				ctx->map = NULL;
+				ctx->map_start = 0;
+				ctx->map_end = 0;
+				return (ssize_t) -1;
+			}
+			
+#ifdef HAS_MADVISE
+			madvise(ctx->map, map_len, MADV_SEQUENTIAL);
+#endif
+		}
+
+		g_assert(ctx->map != NULL);
+
+		if (0 != (n = sigsetjmp(mmap_env, 1))) {
+			switch (n) {
+			case SIGBUS:
+				g_warning("bio_sendfile(): Caught SIGBUS");
+				break;
+			case SIGSEGV:
+				g_warning("bio_sendfile(): Caught SIGSEGV");
+				break;
+			default:
+				g_assert_not_reached();
+			}
+			errno = EPIPE;	/* Don't consider this fatal, keep mmap() enabled */
+			return -1;
+		}
+
+		data = ctx->map;
+
+		g_assert(start >= ctx->map_start);
+		data = &data[start - ctx->map_start];
+		
+		g_assert(ctx->map_end > start);
+		amount = MIN(ctx->map_end - start, amount);
+
+		g_assert(mmap_access == 0);
+		mmap_access = 1;
+		r = write(out_fd, data, amount);
+		mmap_access = 0;
+
+		switch (r) {
+		case (ssize_t) -1:
+			break;
+		case 0:
+			munmap(ctx->map, ctx->map_end - ctx->map_start);
+			ctx->map = NULL;
+			break;
+		default:
+			*offset = start + r;
+		}
+	}
+#else /* !USE_MMAP */
 #ifdef USE_BSD_SENDFILE
 	/*
-	 * The BSD semantics for sendfile() differ from the Linux one:
+	 * The FreeBSD semantics for sendfile() differ from the Linux one:
 	 *
-	 * . BSD sendfile() returns 0 on succes, -1 on failure.
-	 * . BSD sendfile() returns the amount of written bytes via a parameter
+	 * . FreeBSD sendfile() returns 0 on success, -1 on failure.
+	 * . FreeBSD sendfile() returns the amount of written bytes via a parameter
 	 *   when EAGAIN.
-	 * . BSD sendfile() does not update the offset inplace.
+	 * . FreeBSD sendfile() does not update the offset inplace.
 	 *
 	 * Emulate the Linux semantics: set `r' to the amount of bytes written,
 	 * and update the `offset' variable.
@@ -1426,34 +1592,39 @@ bio_sendfile(bio_source_t *bio, gint in_fd, off_t *offset, size_t len)
 
 	{
 		off_t written;
+		int flags = 0;
 
-		r = sendfile(in_fd, bio->wio->fd(bio->wio), start, amount, NULL,
-				&written, 0);
-
+#if defined(SF_NODISKIO)
+		flags |= SF_NODISKIO;
+#endif	/* SF_NODISKIO */
+		
+		r = sendfile(in_fd, out_fd, start, amount, NULL, &written, flags);
 		if ((ssize_t) -1 == r) {
-			if (errno == EAGAIN)
-				r = (ssize_t) written;
-		} else
+			if (is_temporary_error(errno) || EBUSY == errno)
+				r = written > 0 ? (ssize_t) written : (ssize_t) -1;
+		} else {
 			r = amount;			/* Everything written, but returns 0 if OK */
-
+		}
 		if (r > 0)
 			*offset = start + r;
 	}
 
 #else	/* !USE_BSD_SENDFILE */
 
-	r = sendfile(bio->wio->fd(bio->wio), in_fd, offset, amount);
+	r = sendfile(out_fd, in_fd, offset, amount);
 
 	if (r >= 0 && *offset != start + r) {		/* Paranoid, as usual */
 		g_warning("FIXED SENDFILE returned offset: "
-			"was set to %" PRIu64 " instead of %" PRIu64 " (%d byte%s written)",
-			(guint64) *offset, (guint64) (start + r),
+			"was set to %s instead of %s (%d byte%s written)",
+			uint64_to_string(*offset), uint64_to_string2(start + r),
 			(gint) r, r == 1 ? "" : "s");
 		*offset = start + r;
-	} else if ((ssize_t) -1 == r)
+	} else if ((ssize_t) -1 == r) {
 		*offset = start;	/* Paranoid: in case sendfile() touched it */
+	}
 
 #endif	/* USE_BSD_SENDFILE */
+#endif	/* USE_MMAP */
 
 	if (r > 0) {
 		bsched_bw_update(bio->bs, r, amount);
@@ -1461,13 +1632,15 @@ bio_sendfile(bio_source_t *bio, gint in_fd, off_t *offset, size_t len)
 	}
 
 	return r;
-#endif	/* HAVE_SENDFILE */
+#endif /* !USE_MMAP && !HAS_SENDFILE */
 }
 
 /**
- * Read at most `len' bytes from `buf' from source's fd, as bandwidth permits.
- * If we cannot read anything due to bandwidth constraints, return -1 with
- * errno set to EAGAIN.
+ * Read at most `len' bytes from `buf' from source's fd, as bandwidth
+ * permits.
+ *
+ * @return -1 with errno set to EAGAIN, if we cannot read anything due to
+ * bandwidth constraints.
  */
 ssize_t
 bio_read(bio_source_t *bio, gpointer data, size_t len)
@@ -1476,17 +1649,17 @@ bio_read(bio_source_t *bio, gpointer data, size_t len)
 	size_t amount;
 	ssize_t r;
 
-	g_assert(bio);
+	bio_check(bio);
 	g_assert(bio->flags & BIO_F_READ);
 
-	/* 
-	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN 
+	/*
+	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN
 	 * to signal that we cannot perform any I/O right now.
 	 */
 
 	available = bw_available(bio, len);
 	if (available == 0) {
-		errno = EAGAIN;
+		errno = VAL_EAGAIN;
 		return -1;
 	}
 
@@ -1499,6 +1672,135 @@ bio_read(bio_source_t *bio, gpointer data, size_t len)
 	if (r > 0) {
 		bsched_bw_update(bio->bs, r, amount);
 		bio->bw_actual += r;
+		bio->bs->flags |= BS_F_DATA_READ;
+	}
+
+	return r;
+}
+
+/**
+ * XXX: This is copy-paste bullshit:
+ *
+ * Read at most `len' bytes from `iov' to source's fd, as bandwidth permits,
+ * `len' being determined by the size of the supplied I/O vector.
+ * If we cannot write anything due to bandwidth constraints, return -1 with
+ * errno set to EAGAIN.
+ */
+ssize_t
+bio_readv(bio_source_t *bio, struct iovec *iov, gint iovcnt)
+{
+	size_t available;
+	ssize_t r;
+	size_t len;
+	struct iovec *siov;
+	gint slen = -1;			/* Avoid "may be used uninitialized" warning */
+
+	bio_check(bio);
+	g_assert(bio->flags & BIO_F_READ);
+
+	/*
+	 * Compute I/O vector's length.
+	 */
+
+	for (r = 0, siov = iov, len = 0; r < iovcnt; r++, siov++)
+		len += siov->iov_len;
+
+	/*
+	 * If we don't have any bandwidth, return -1 with errno set to EAGAIN
+	 * to signal that we cannot perform any I/O right now.
+	 */
+
+	available = bw_available(bio, len);
+
+	if (available == 0) {
+		errno = VAL_EAGAIN;
+		return -1;
+	}
+
+	/*
+	 * If we cannot read the whole vector, we need to trim it.
+	 * Because we promise to not corrupt the original I/O vector, we
+	 * save the original length of the last I/O entry, should we modify it.
+	 */
+
+	if (len > available) {
+		size_t curlen;
+
+		for (r = 0, siov = iov, curlen = 0; r < iovcnt; r++, siov++) {
+			curlen += siov->iov_len;
+
+			/*
+			 * Exact size reached, we just need to adjust down the iov count.
+			 * Force siov to NULL before leaving the loop to indicate we did
+			 * not have to alter it.
+			 */
+
+			if (curlen == available) {
+				siov = NULL;
+				iovcnt = r + 1;
+				break;
+			}
+
+			/*
+			 * Maximum size reached...  Need to adjust both the iov count
+			 * and the length of the current siov entry.
+			 */
+
+			if (curlen > available) {
+				slen = siov->iov_len;		/* Save for later restore */
+				siov->iov_len -= (curlen - available);
+				iovcnt = r + 1;
+				g_assert(siov->iov_len > 0);
+				break;
+			}
+		}
+	}
+
+	/*
+	 * Read into I/O vector, updating used bandwidth.
+	 *
+	 * When `iovcnt' is greater than MAX_IOV_COUNT, use our custom readv()
+	 * wrapper to avoid failure with EINVAL.
+	 *		--RAM, 17/03/2002
+	 */
+
+	if (dbg > 7)
+		printf("bio_readv(fd=%d, len=%d) available=%d\n",
+			bio->wio->fd(bio->wio), (gint) len, (gint) available);
+
+	if (iovcnt > MAX_IOV_COUNT)
+		r = safe_readv(bio->wio, iov, iovcnt);
+	else
+		r = bio->wio->readv(bio->wio, iov, iovcnt);
+
+	/*
+	 * XXX hack for broken libc, which can return -1 with errno = 0!
+	 *
+	 * Apparently, when compiling with gcc-3.3.x, one can have a system
+	 * call return -1 with errno set to EOK.
+	 *		--RAM, 05/10/2003
+	 */
+
+	if ((ssize_t) -1 == r && 0 == errno) {
+		g_warning("readv(fd=%d, len=%d) returned -1 with errno = 0, "
+			"assuming EAGAIN", bio->wio->fd(bio->wio), (gint) len);
+		errno = VAL_EAGAIN;
+	}
+
+	if (r > 0) {
+		g_assert((size_t) r <= available);
+		bsched_bw_update(bio->bs, r, MIN(len, available));
+		bio->bw_actual += r;
+		bio->bs->flags |= BS_F_DATA_READ;
+	}
+
+	/*
+	 * Restore original I/O vector if we trimmed it.
+	 */
+
+	if (len > available && siov) {
+		g_assert(slen >= 0);			/* Ensure it was initialized */
+		siov->iov_len = slen;
 	}
 
 	return r;
@@ -1518,8 +1820,10 @@ bws_write(bsched_t *bs, wrap_io_t *wio, gconstpointer data, size_t len)
 
 	g_assert(bs);
 	g_assert(bs->flags & BS_F_WRITE);
-
+	g_assert(wio);
+	g_assert(wio->write);
 	g_assert(len <= INT_MAX);
+
 	r = wio->write(wio, data, len);
 	if (r > 0)
 		bsched_bw_update(bs, r, len);
@@ -1539,11 +1843,15 @@ bws_read(bsched_t *bs, wrap_io_t *wio, gpointer data, size_t len)
 
 	g_assert(bs);
 	g_assert(bs->flags & BS_F_READ);
-
+	g_assert(wio);
+	g_assert(wio->read);
 	g_assert(len <= INT_MAX);
+
 	r = wio->read(wio, data, len);
-	if (r > 0)
+	if (r > 0) {
 		bsched_bw_update(bs, r, len);
+		bs->flags |= BS_F_DATA_READ;
+	}
 
 	return r;
 }
@@ -1557,6 +1865,7 @@ bws_udp_count_read(gint len)
 	gint count = BW_UDP_MSG + len;
 
 	bsched_bw_update(bws.gin_udp, count, count);
+	bws.gin_udp->flags |= BS_F_DATA_READ;
 }
 
 /**
@@ -1572,7 +1881,8 @@ bws_udp_count_written(gint len)
 
 /**
  * Returns adequate b/w shaper depending on the socket type.
- * Returns NULL if there is no b/w shaper to consider.
+ *
+ * @returns NULL if there is no b/w shaper to consider.
  */
 static bsched_t *
 bs_socket(enum socket_direction dir, enum socket_type type)
@@ -1615,7 +1925,7 @@ bws_sock_connect(enum socket_type type)
 	 */
 
 	if (bs != NULL)
-		bsched_bw_update(bs, 1.5 * BW_TCP_MSG, 1.5 * BW_TCP_MSG);
+		bsched_bw_update(bs, (3 * BW_TCP_MSG) / 2, (3 * BW_TCP_MSG) / 2);
 }
 
 /**
@@ -1759,9 +2069,9 @@ bws_can_connect(enum socket_type type)
  * Periodic heartbeat.
  */
 static void
-bsched_heartbeat(bsched_t *bs, GTimeVal *tv)
+bsched_heartbeat(bsched_t *bs, tm_t *tv)
 {
-	GList *l;
+	GList *iter;
 	gint delay;
 	gint overused;
 	gint theoric;
@@ -1907,10 +2217,10 @@ bsched_heartbeat(bsched_t *bs, GTimeVal *tv)
 
 	last_used = 0;
 
-	for (l = bs->sources; l; l = g_list_next(l)) {
-		bio_source_t *bio = (bio_source_t *) l->data;
+	for (iter = bs->sources; iter; iter = g_list_next(iter)) {
+		bio_source_t *bio = iter->data;
 
-		g_assert(bio != NULL);
+		bio_check(bio);
 
 		if (bio->flags & BIO_F_USED)
 			last_used++;
@@ -1969,7 +2279,7 @@ bsched_stealbeat(bsched_t *bs)
 	if (!(bs->flags & BS_F_ENABLED))	/* Scheduler disabled */
 		return;
 
-	/*
+	/**
 	 * Note that we do not use the theoric bandwidth, but bs->bw_max to
 	 * estimate the amount of underused bandwidth.  The reason is that
 	 * bs->bw_max can be corrected due to traffic spikes.
@@ -2025,7 +2335,7 @@ bsched_stealbeat(bsched_t *bs)
 	 */
 
 	for (l = bs->stealers; l; l = g_slist_next(l)) {
-		bsched_t *xbs = (bsched_t *) l->data;
+		bsched_t *xbs = l->data;
 
 		steal_count++;
 
@@ -2046,7 +2356,7 @@ bsched_stealbeat(bsched_t *bs)
 
 	if (all_used_count == 0) {
 		for (l = bs->stealers; l; l = g_slist_next(l)) {
-			bsched_t *xbs = (bsched_t *) l->data;
+			bsched_t *xbs = l->data;
 			xbs->bw_stolen += underused / steal_count;
 
 			if (dbg > 4)
@@ -2055,7 +2365,7 @@ bsched_stealbeat(bsched_t *bs)
 		}
 	} else {
 		for (l = all_used; l; l = g_slist_next(l)) {
-			bsched_t *xbs = (bsched_t *) l->data;
+			bsched_t *xbs = l->data;
 			gdouble amount;
 
 			if (xbs->bw_max == 0)
@@ -2082,12 +2392,13 @@ bsched_stealbeat(bsched_t *bs)
 void
 bsched_timer(void)
 {
-	GTimeVal tv;
+	tm_t tv;
 	GSList *l;
 	gint out_used = 0;
 	gint in_used = 0;
+	gboolean read_data = FALSE;
 
-	g_get_current_time(&tv);
+	tm_now(&tv);
 
 	/*
 	 * First pass: compute bandwidth used.
@@ -2116,7 +2427,7 @@ bsched_timer(void)
 	 */
 
 	for (l = bws_out_list; l; l = g_slist_next(l)) {
-		bsched_t *bs = (bsched_t *) l->data;
+		bsched_t *bs = l->data;
 		out_used += (gint) (bs->bw_last_period * 1000.0 / bs->period_ema);
 	}
 
@@ -2126,8 +2437,14 @@ bsched_timer(void)
 		printf("Outgoing b/w EMA = %d bytes/s\n", bws_out_ema);
 
 	for (l = bws_in_list; l; l = g_slist_next(l)) {
-		bsched_t *bs = (bsched_t *) l->data;
+		bsched_t *bs = l->data;
+
 		in_used += (gint) (bs->bw_last_period * 1000.0 / bs->period_ema);
+
+		if (bs->flags & BS_F_DATA_READ) {
+			read_data = TRUE;
+			bs->flags &= ~BS_F_DATA_READ;
+		}
 	}
 
 	bws_in_ema += (in_used >> 6) - (bws_in_ema >> 6);		/* Slow EMA */
@@ -2135,39 +2452,74 @@ bsched_timer(void)
 	if (dbg > 4)
 		printf("Incoming b/w EMA = %d bytes/s\n", bws_in_ema);
 
-	if (in_used)
+	/*
+	 * Don't simply rely on in_used > 0 since we fake input data when
+	 * sockets are closed or timeout on connection.  We really want to know
+	 * whether there has been actual data read.
+	 *		--RAM, 2005-06-29
+	 */
+
+	if (read_data)
 		inet_read_activity();
 }
 
+static gboolean
+true_expr(const gchar *expr)
+{
+	if (dbg > 0) {
+		g_message("%s", expr);
+	}
+	return TRUE;
+}
+
+#define noisy_check(expr) ((expr) ? true_expr(G_STRLOC ": " #expr) : 0)
+
 /**
+ * Needs very short description so that doxygen can parse the following
+ * list properly.
+ *
  * Determine whether we have enough bandwidth to possibly become an
  * ultra node:
  *
- * 1. There must be more than BW_OUT_UP_MIN outgoing bandwidth available.
- * 2. If bandwidth schedulers are enabled, leaf nodes must not be configured
- *    to steal all the HTTP outgoing bandwidth, unless they disabled uploads.
- * 3. If Gnet out scheduler is enabled, there must be at least BW_OUT_GNET_MIN
- *    bytes per gnet connection.
- * 4. Overall, there must be BW_OUT_LEAF_MIN bytes per configured leaf plus
- *    BW_OUT_GNET_MIN bytes per gnet connection available.
+ *  -# There must be more than BW_OUT_UP_MIN outgoing bandwidth available.
+ *  -# If bandwidth schedulers are enabled, leaf nodes must not be configured
+ *     to steal all the HTTP outgoing bandwidth, unless they disabled uploads.
+ *  -# If Gnet out scheduler is enabled, there must be at least BW_OUT_GNET_MIN
+ *     bytes per gnet connection.
+ *  -# Overall, there must be BW_OUT_LEAF_MIN bytes per configured leaf plus
+ *     BW_OUT_GNET_MIN bytes per gnet connection available.
  */
 gboolean
 bsched_enough_up_bandwidth(void)
 {
 	guint32 total = 0;
-
-	if (ul_running && bws_out_ema < BW_OUT_UP_MIN)
+	
+	/**
+	 * FIXME: If all upload slots are used by clients which download
+	 *		  very slowly, the below check would cause a demotion
+	 *		  to leaf mode. It is in general not possible to know
+	 *		  whether we are out of bandwidth or if the remote clients
+	 *		  have insufficient available bandwidth.
+	 */
+#if 0
+	if (noisy_check(ul_running && bws_out_ema < BW_OUT_UP_MIN))
 		return FALSE;		/* 1. */
+#endif
 
 	if (
-		bws_glout_enabled && bws_out_enabled &&
-		bw_gnet_lout >= bw_http_out && upload_is_enabled()
+		noisy_check(
+			bws_glout_enabled &&
+			bws_out_enabled &&
+			bw_gnet_lout >= bw_http_out &&
+			upload_is_enabled())
 	)
 		return FALSE;		/* 2. */
 
 	if (
-		bws_gout_enabled &&
-		bw_gnet_out < BW_OUT_GNET_MIN * (up_connections + max_connections) / 2
+		noisy_check(
+			bws_gout_enabled &&
+			bw_gnet_out < BW_OUT_GNET_MIN *
+				(up_connections + max_connections) / 2)
 	)
 		return FALSE;		/* 3. */
 
@@ -2180,10 +2532,11 @@ bsched_enough_up_bandwidth(void)
 		total += bw_gnet_lout;
 
 	if (
-		total <
-			(BW_OUT_GNET_MIN * max_connections + BW_OUT_LEAF_MIN * max_leaves)
-	)
+		noisy_check(total <
+			(BW_OUT_GNET_MIN * max_connections + BW_OUT_LEAF_MIN * max_leaves))
+	) {
 		return FALSE;		/* 4. */
+	}
 
 	return TRUE;
 }
