@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.h,v 1.10 2005/09/26 20:21:48 rmanfredi Exp $
+ * $Id: bsched.h 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -161,7 +161,7 @@ extern struct bws_set bws;
  * Public interface.
  */
 
-bsched_t *bsched_make(gchar *name,
+bsched_t *bsched_make(const gchar *name,
 	gint type, guint32 mode, gint bandwidth, gint period);
 void bsched_init(void);
 void bsched_shutdown(void);
@@ -181,7 +181,7 @@ void bio_add_callback(bio_source_t *bio,
 void bio_remove_callback(bio_source_t *bio);
 ssize_t bio_write(bio_source_t *bio, gconstpointer data, size_t len);
 ssize_t bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt);
-ssize_t bio_sendto(bio_source_t *bio, gnet_host_t *to,
+ssize_t bio_sendto(bio_source_t *bio, const gnet_host_t *to,
 	gconstpointer data, size_t len);
 ssize_t bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd,
 	off_t *offset, size_t len);
