@@ -1,5 +1,5 @@
 /*
- * $Id: dh.c,v 1.9 2004/11/13 14:06:06 rmanfredi Exp $
+ * $Id: dh.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -24,14 +24,18 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Dynamic query hits.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
  */
 
 #include "common.h"
 
-RCSID("$Id: dh.c,v 1.9 2004/11/13 14:06:06 rmanfredi Exp $");
+RCSID("$Id: dh.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "dh.h"
 #include "nodes.h"
@@ -41,43 +45,44 @@ RCSID("$Id: dh.c,v 1.9 2004/11/13 14:06:06 rmanfredi Exp $");
 
 #include "lib/atoms.h"
 #include "lib/misc.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define DH_HALF_LIFE	300		/* 5 minutes */
-#define DH_MIN_HITS		250		/* Minimum amount of hits we try to relay */
-#define DH_POPULAR_HITS	500		/* Query deemed popular after that many hits */
-#define DH_MAX_HITS		1000	/* Maximum hits after which we heavily drop */
-#define DH_THRESH_HITS	10		/* Consider we have no hits if less than that */
+#define DH_HALF_LIFE	300		/**< 5 minutes */
+#define DH_MIN_HITS		250		/**< Minimum amount of hits we try to relay */
+#define DH_POPULAR_HITS	500		/**< Query deemed popular after that many hits */
+#define DH_MAX_HITS		1000	/**< Maximum hits after which we heavily drop */
+#define DH_THRESH_HITS	10		/**< We have no hits if less than that */
 
-/*
+/**
  * Information about query hits received.
  */
 typedef struct dqhit {
-	guint32 msg_recv;		/* Amount of individual messages we got */
-	guint32 msg_queued;		/* # of messages queued */
-	guint32 hits_recv;		/* Total amount of results we saw */
-	guint32 hits_sent;		/* Total amount of results we sent back */
-	guint32 hits_queued;	/* Amount of hits queued */
+	guint32 msg_recv;		/**< Amount of individual messages we got */
+	guint32 msg_queued;		/**< # of messages queued */
+	guint32 hits_recv;		/**< Total amount of results we saw */
+	guint32 hits_sent;		/**< Total amount of results we sent back */
+	guint32 hits_queued;	/**< Amount of hits queued */
 } dqhit_t;
 
 /*
  * Meta-information about the query hit message.
  */
 struct pmsg_info {
-	guint32 hits;			/* Amount of query hits held in message */
+	guint32 hits;			/**< Amount of query hits held in message */
 };
 
-/*
+/**
  * Drop reasons.
  */
 enum dh_drop {
-	DH_FORWARD = 0,			/* Don't drop */
-	DH_DROP_FC,				/* Drop because of flow-control */
-	DH_DROP_THROTTLE,		/* Drop because of hit throttling */
+	DH_FORWARD = 0,			/**< Don't drop */
+	DH_DROP_FC,				/**< Drop because of flow-control */
+	DH_DROP_THROTTLE		/**< Drop because of hit throttling */
 };
 
 /*
@@ -229,12 +234,12 @@ static void
 dh_pmsg_free(pmsg_t *mb, gpointer arg)
 {
 	struct pmsg_info *pmi = (struct pmsg_info *) arg;
-	gchar *muid;
+	const gchar *muid;
 	dqhit_t *dh;
 
 	g_assert(pmsg_is_extended(mb));
 
-	muid = ((struct gnutella_header *) pmsg_start(mb))->muid;
+	muid = gnutella_header_get_muid(pmsg_start(mb));
 	dh = dh_locate(muid);
 
 	if (dh == NULL)
@@ -396,17 +401,18 @@ dh_route(gnutella_node_t *src, gnutella_node_t *dest, gint count)
 {
 	pmsg_t *mb;
 	struct pmsg_info *pmi;
-	gchar *muid;
+	const gchar *muid;
 	dqhit_t *dh;
 	mqueue_t *mq;
 
-	g_assert(src->header.function == GTA_MSG_SEARCH_RESULTS);
+	g_assert(
+		gnutella_header_get_function(&src->header) == GTA_MSG_SEARCH_RESULTS);
 	g_assert(count >= 0);
 
 	if (!NODE_IS_WRITABLE(dest))
 		goto drop_shutdown;
 
-	muid = src->header.muid;
+	muid = gnutella_header_get_muid(&src->header);
 	dh = dh_locate(muid);
 
 	g_assert(dh != NULL);		/* Must have called dh_got_results() first! */
@@ -455,10 +461,8 @@ dh_route(gnutella_node_t *src, gnutella_node_t *dest, gint count)
 	 * This enables us to track how much results we already queued/sent.
 	 */
 
-	mb = gmsg_split_to_pmsg_extend(
-		(guchar *) &src->header, src->data,
-		src->size + sizeof(struct gnutella_header),
-		dh_pmsg_free, pmi);
+	mb = gmsg_split_to_pmsg_extend(&src->header, src->data,
+			src->size + GTA_HEADER_SIZE, dh_pmsg_free, pmi);
 
 	mq_putq(mq, mb);
 
@@ -476,7 +480,7 @@ drop_flow_control:
 	gnet_stats_count_dropped(src, MSG_DROP_FLOW_CONTROL);
 	gnet_stats_count_flowc(&src->header);
 	return;
-	
+
 drop_throttle:
 	gnet_stats_count_dropped(src, MSG_DROP_THROTTLE);
 	return;
@@ -509,7 +513,7 @@ dh_init(void)
 {
 	by_muid = g_hash_table_new(guid_hash, guid_eq);
 	by_muid_old = g_hash_table_new(guid_hash, guid_eq);
-	last_rotation = time(NULL);
+	last_rotation = tm_time();
 }
 
 /**
