@@ -1,5 +1,5 @@
 /*
- * $Id: dmesh.c,v 1.35 2005/11/10 23:54:15 cbiere Exp $
+ * $Id: dmesh.c 12058 2006-09-27 19:33:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: dmesh.c,v 1.35 2005/11/10 23:54:15 cbiere Exp $");
+RCSID("$Id: dmesh.c 12058 2006-09-27 19:33:55Z cbiere $")
 
 #include "gnutella.h"
 #include "downloads.h"
@@ -149,7 +149,7 @@ urlinfo_hash(gconstpointer key)
 	guint hash;
 
 	hash = host_addr_hash(info->addr);
-	hash ^= (info->port << 16) | info->port;
+	hash ^= ((guint32) info->port << 16) | info->port;
 	hash ^= info->idx;
 	hash ^= g_str_hash(info->name);
 
@@ -164,9 +164,9 @@ urlinfo_eq(gconstpointer a, gconstpointer b)
 {
 	const dmesh_urlinfo_t *ia = a, *ib = b;
 
-	return host_addr_equal(ia->addr, ib->addr) &&
-		ia->port == ib->port	&&
-		ia->idx == ib->idx		&&
+	return ia->port == ib->port &&
+		ia->idx == ib->idx &&
+		host_addr_equal(ia->addr, ib->addr) &&
 		(ia->name == ib->name || 0 == strcmp(ia->name, ib->name));
 }
 
@@ -274,7 +274,7 @@ dmesh_ban_add(const gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 {
 	time_t now = tm_time();
 	struct dmesh_banned *dmb;
-	gint lifetime = BAN_LIFETIME;
+	time_delta_t lifetime = BAN_LIFETIME;
 
 	if (stamp == 0)
 		stamp = now;
@@ -284,6 +284,7 @@ dmesh_ban_add(const gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	 */
 
 	lifetime -= delta_time(now, stamp);
+	lifetime = MIN(lifetime, INT_MAX / 1000);
 
 	if (lifetime <= 0)
 		return;
@@ -349,7 +350,7 @@ dmesh_ban_add(const gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
  * Check whether URL is banned from the mesh.
  */
 static gboolean
-dmesh_is_banned(dmesh_urlinfo_t *info)
+dmesh_is_banned(const dmesh_urlinfo_t *info)
 {
 	return NULL != g_hash_table_lookup(ban_mesh, info);
 }
@@ -365,6 +366,7 @@ static const gchar * const parse_errstr[] = {
 	"Index in /get/index is reserved",		/**< DMESH_URL_RESERVED_INDEX */
 	"No filename after /get/index",			/**< DMESH_URL_NO_FILENAME */
 	"Bad URL encoding",						/**< DMESH_URL_BAD_ENCODING */
+	"Malformed /uri-res/N2R?",				/**< DMESH_URL_BAD_URI_RES */
 };
 
 /**
@@ -407,7 +409,10 @@ dmesh_url_parse(const gchar *url, dmesh_urlinfo_t *info)
 		return FALSE;
 	}
 
-	addr = name_to_host_addr(host);	/* FIXME: This can block */
+	/* FIXME:	This can block; we should never keep resolved hostnames as IP
+	 *			addresses around but always resolve hostnames just in time.
+	 */
+	addr = name_to_single_host_addr(host, settings_dns_net());
 	if (!is_host_addr(addr))
 		return FALSE;
 
@@ -470,10 +475,19 @@ dmesh_url_parse(const gchar *url, dmesh_urlinfo_t *info)
 			return FALSE;
 		}
 		info->name = atom_str_get(unescaped);
-		if (unescaped != file)
+		if (unescaped != file) {
 			G_FREE_NULL(unescaped);
-	} else
+		}
+	} else {
+		gchar digest[SHA1_RAW_SIZE];
+		
+		if (!urn_get_sha1(file, digest)) {
+			dmesh_url_errno = DMESH_URL_BAD_URI_RES;
+			return FALSE;
+		}
 		info->name = atom_str_get(file);
+	}
+
 
 	dmesh_url_errno = DMESH_URL_OK;
 
@@ -485,14 +499,14 @@ dmesh_url_parse(const gchar *url, dmesh_urlinfo_t *info)
  * `sha1' is only passed in case we want to log the removal.
  */
 static void
-dm_expire(struct dmesh *dm, guint32 agemax, const gchar *sha1)
+dm_expire(struct dmesh *dm, glong agemax, const gchar *sha1)
 {
 	GSList *l;
 	GSList *prev;
 	time_t now = tm_time();
 
 	for (prev = NULL, l = dm->entries; l; /* empty */) {
-		struct dmesh_entry *dme = (struct dmesh_entry *) l->data;
+		struct dmesh_entry *dme = l->data;
 		GSList *next;
 
 		if (delta_time(now, dme->stamp) <= agemax) {
@@ -512,7 +526,7 @@ dm_expire(struct dmesh *dm, guint32 agemax, const gchar *sha1)
 
 		if (dmesh_debug > 4)
 			g_message("MESH %s: EXPIRED \"%s\", age=%d",
-				sha1_base32(sha1),
+				sha1 ? sha1_base32(sha1) : "<no SHA-1 known>",
 				dmesh_urlinfo_to_string(&dme->url),
 				(gint) delta_time(now, dme->stamp));
 
@@ -602,13 +616,12 @@ dmesh_dispose(const gchar *sha1)
 
 /**
  * Fill URL info from externally supplied sha1, addr, port, idx and name.
- * When `idx' is URN_INDEX, then `name' is ignored, and we use the
- * stringified SHA1.
+ * If sha1 is NULL, we use the name, otherwise the urn:sha1.
  */
 static void
 dmesh_fill_info(dmesh_urlinfo_t *info,
 	const gchar *sha1, const host_addr_t addr,
-	guint16 port, guint idx, gchar *name)
+	guint16 port, guint idx, const gchar *name)
 {
 	static const gchar urnsha1[] = "urn:sha1:";
 	static gchar urn[SHA1_BASE32_SIZE + sizeof urnsha1];
@@ -617,11 +630,12 @@ dmesh_fill_info(dmesh_urlinfo_t *info,
 	info->port = port;
 	info->idx = idx;
 
-	if (idx == URN_INDEX) {
+	if (sha1) {
 		concat_strings(urn, sizeof urn, urnsha1, sha1_base32(sha1), (void *) 0);
 		info->name = urn;
-	} else
+	} else {
 		info->name = name;
+	}
 }
 
 /**
@@ -629,7 +643,7 @@ dmesh_fill_info(dmesh_urlinfo_t *info,
  */
 gboolean
 dmesh_remove(const gchar *sha1, const host_addr_t addr, guint16 port,
-	guint idx, gchar *name)
+	guint idx, const gchar *name)
 {
 	struct dmesh *dm;
 	dmesh_urlinfo_t info;
@@ -696,7 +710,7 @@ dmesh_count(const gchar *sha1)
  * it was the oldest record and we have enough already.
  */
 static gboolean
-dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
+dmesh_raw_add(const gchar *sha1, const dmesh_urlinfo_t *info, time_t stamp)
 {
 	struct dmesh_entry *dme;
 	struct dmesh *dm;
@@ -704,7 +718,9 @@ dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	host_addr_t addr = info->addr;
 	guint16 port = info->port;
 	guint idx = info->idx;
-	gchar *name = info->name;
+	const gchar *name = info->name;
+
+	g_return_val_if_fail(sha1, FALSE);
 
 	if (stamp == 0 || delta_time(stamp, now) > 0)
 		stamp = now;
@@ -716,7 +732,7 @@ dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	 * Reject if this is for our host, or if the host is a private/hostile IP.
 	 */
 
-	if (host_addr_equal(addr, listen_addr()) && port == listen_port)
+	if (is_my_address(addr, port))
 		return FALSE;
 
 	if (!host_is_valid(addr, port))
@@ -742,8 +758,7 @@ dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	 * than the one we're trying to add).
 	 */
 
-	dm = g_hash_table_lookup(mesh, sha1);
-
+	dm =  g_hash_table_lookup(mesh, sha1);
 	if (dm == NULL) {
 		dm = walloc(sizeof *dm);
 
@@ -793,8 +808,7 @@ dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	dm->last_update = now;
 
 	if (dm->count == MAX_ENTRIES) {
-		struct dmesh_entry *last =
-			(struct dmesh_entry *) g_slist_last(dm->entries)->data;
+		struct dmesh_entry *last = g_slist_last(dm->entries)->data;
 
 		dm->entries = g_slist_remove(dm->entries, last);
 
@@ -810,8 +824,14 @@ dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
 	 * downloading that file.
 	 */
 
-	if (dme != NULL)
-		file_info_try_to_swarm_with(name, idx, addr, port, sha1);
+	if (dme != NULL) {
+		/*
+		 * If this is from a uri-res URI, don't use the SHA1 as
+		 * filename, so that the existing name is used instead.
+		 */
+		file_info_try_to_swarm_with(URN_INDEX == idx && sha1 ? NULL : name,
+			idx, addr, port, sha1);
+	}
 
 	return dme != NULL;			/* TRUE means we added the entry */
 }
@@ -821,7 +841,7 @@ dmesh_raw_add(gchar *sha1, dmesh_urlinfo_t *info, time_t stamp)
  */
 gboolean
 dmesh_add(gchar *sha1, const host_addr_t addr, guint16 port, guint idx,
-	gchar *name, time_t stamp)
+	const gchar *name, time_t stamp)
 {
 	dmesh_urlinfo_t info;
 
@@ -1250,7 +1270,13 @@ dmesh_alternate_location(const gchar *sha1,
 	 */
 
 	if (
-		fi != NULL && fi->done != 0 && pfsp_server && !is_firewalled &&
+		fi != NULL &&
+		fi->size != 0 &&
+		fi->file_size_known &&
+		fi->done != 0 &&
+		pfsp_server &&
+		!is_firewalled &&
+		is_host_addr(listen_addr()) &&
 		(
 			fi->done >= MIN_PFSP_SIZE ||
 			fi->done * 100 / fi->size > MIN_PFSP_PCT
@@ -1260,6 +1286,8 @@ dmesh_alternate_location(const gchar *sha1,
 		struct dmesh_entry ourselves;
 		time_t now = tm_time();
 
+		file_info_check(fi);
+
 		ourselves.inserted = now;
 		ourselves.stamp = now;
 		ourselves.url.addr = listen_addr();
@@ -1458,8 +1486,7 @@ dmesh_defer_nonurn_altloc(GSList *list, dmesh_urlinfo_t *url, time_t stamp)
 	 * Copy the structure, beware of atoms.
 	 */
 
-    defer->dmesh_url = walloc(sizeof *defer->dmesh_url);
-	*defer->dmesh_url = *url;		/* struct copy */
+    defer->dmesh_url = wcopy(url, sizeof *url);
     defer->dmesh_url->name = atom_str_get(url->name);
     defer->stamp = stamp;
 
@@ -1477,11 +1504,9 @@ dmesh_defer_nonurn_altloc(GSList *list, dmesh_urlinfo_t *url, time_t stamp)
  * list itself needs to be freed afterwards
  */
 static inline void
-dmesh_free_deferred_altloc(dmesh_deferred_url_t *info, gpointer unused_udata)
+dmesh_free_deferred_altloc(dmesh_deferred_url_t *info)
 {
-	(void) unused_udata;
-	atom_str_free(info->dmesh_url->name);
-	wfree(info->dmesh_url, sizeof *info->dmesh_url);
+	dmesh_urlinfo_free(info->dmesh_url);
 	wfree(info, sizeof *info);
 }
 
@@ -1500,7 +1525,7 @@ dmesh_free_deferred_altloc(dmesh_deferred_url_t *info, gpointer unused_udata)
  * and `n' the amount of existing entries.
  */
 static void
-dmesh_check_deferred_against_existing(gchar *sha1,
+dmesh_check_deferred_against_existing(const gchar *sha1,
 	GSList *existing_urls, GSList *deferred_urls)
 {
     GSList *ex, *def;
@@ -1578,7 +1603,8 @@ dmesh_check_deferred_against_existing(gchar *sha1,
  * after itself yet.
  */
 static void
-dmesh_check_deferred_against_themselves(gchar *sha1, GSList *deferred_urls)
+dmesh_check_deferred_against_themselves(const gchar *sha1,
+	GSList *deferred_urls)
 {
     dmesh_deferred_url_t *first;
     GSList *sl = deferred_urls;
@@ -1649,7 +1675,7 @@ dmesh_check_deferred_against_themselves(gchar *sha1, GSList *deferred_urls)
  *
  */
 static void
-dmesh_check_deferred_altlocs(gchar *sha1, GSList *deferred_urls)
+dmesh_check_deferred_altlocs(const gchar *sha1, GSList *deferred_urls)
 {
     GSList *existing_urls;
 
@@ -1665,7 +1691,7 @@ dmesh_check_deferred_altlocs(gchar *sha1, GSList *deferred_urls)
 	} else
 		dmesh_check_deferred_against_themselves(sha1, deferred_urls);
 
-	G_SLIST_FOREACH(deferred_urls, dmesh_free_deferred_altloc, NULL);
+	G_SLIST_FOREACH(deferred_urls, dmesh_free_deferred_altloc);
     g_slist_free(deferred_urls);
 }
 
@@ -1708,76 +1734,59 @@ dmesh_collect_sha1(const gchar *value, gchar *digest)
  * for a given SHA1 key given in the new compact form.
  */
 void
-dmesh_collect_compact_locations(gchar *sha1, gchar *value)
+dmesh_collect_compact_locations(const gchar *sha1, const gchar *value)
 {
 	time_t now = tm_time();
-	guchar c;
-	gchar *p = value;
+	const gchar *p = value;
 
-	for (;;) {
-		gboolean has_port = FALSE;
-		gboolean got_full = FALSE;
-		gchar *start = p = skip_ascii_blanks(p);
+	do {
+		const gchar *start, *endptr;
+		host_addr_t addr;
+		guint16 port;
+		gboolean ok;
 
-		for (;;) {
-			c = *p;
-			switch (c) {
-			case ':':
-				has_port = TRUE;	/* Remember there is a port */
-				break;
-			case ',':
-			case '\0':
-				got_full = TRUE;	/* Reached end of header or separator */
-				break;
-			default:
-				break;
-			}
+		start = skip_ascii_blanks(p);
+		if ('\0' == *start)
+			break;
 
-			/*
-			 * If we got the full address, parse it.
-			 */
-
-			if (got_full) {
-				host_addr_t addr;
-				guint16 port = GTA_PORT;	/* Could be missing in address */
-				gboolean ok;
-
-				*p = '\0';
-				if (has_port)
-					ok = string_to_host_addr_port(start, NULL, &addr, &port);
-				else {
-					addr = string_to_host_addr(start, NULL);
-					ok = is_host_addr(addr);
-				}
-
-				if (ok) {
-					dmesh_urlinfo_t info;
-
-					dmesh_fill_info(&info, sha1, addr, port, URN_INDEX, NULL);
-					ok = dmesh_raw_add(sha1, &info, now);
-
-					if (dmesh_debug > 4)
-						g_message("MESH %s: %s compact \"%s\", stamp=%u",
-							sha1_base32(sha1),
-							ok ? "added" : "rejected",
-							dmesh_urlinfo_to_string(&info), (guint) now);
-				} else if (dmesh_debug)
-					g_warning("ignoring invalid compact alt-loc \"%s\"", start);
-
-				*p = c;
-				p++;
-
-				break;
-			}
-
-			p++;
+		endptr = strchr(start, ',');
+		if (!endptr) {
+			endptr = strchr(start, ';');
+			if (!endptr)
+				endptr = strchr(start, '\0');
 		}
 
-		if (c == '\0')
-			return;
-	}
+		/*
+		 * There could be a GUID here if the host is not directly connectible
+		 * but we ignore this apparently.
+		 */	
+		ok = string_to_host_addr(start, &endptr, &addr);
+		if (ok && ':' == *endptr) {
+			gint error;
+				
+			port = parse_uint16(&endptr[1], &endptr, 10, &error);
+			ok = !error && port > 0; 
+		} else {
+			port = GTA_PORT;
+		}
+		
+		if (ok) {
+			dmesh_urlinfo_t info;
 
-	/* NOTREACHED */
+			dmesh_fill_info(&info, sha1, addr, port, URN_INDEX, NULL);
+			ok = dmesh_raw_add(sha1, &info, now);
+
+			if (dmesh_debug > 4)
+				g_message("MESH %s: %s compact \"%s\", stamp=%u",
+						sha1_base32(sha1),
+						ok ? "added" : "rejected",
+						dmesh_urlinfo_to_string(&info), (guint) now);
+		} else if (dmesh_debug) {
+			g_warning("ignoring invalid compact alt-loc \"%s\"", start);
+		}
+
+		p = '\0' != *endptr ? &endptr[1] : NULL;
+	} while (p);
 }
 
 /**
@@ -1785,20 +1794,20 @@ dmesh_collect_compact_locations(gchar *sha1, gchar *value)
  * sources for a given SHA1 key.
  */
 void
-dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
+dmesh_collect_locations(const gchar *sha1, gchar *value, gboolean defer)
 {
 	gchar *p = value;
 	guchar c;
 	time_t now = tm_time();
     GSList *nonurn_altlocs = NULL;
+	gboolean finished = FALSE;
 
-	for (;;) {
+	do {
 		gchar *url;
 		gchar *date;
 		time_t stamp;
 		gboolean ok;
 		dmesh_urlinfo_t info;
-		gboolean non_space_seen;
 		gboolean skip_date;
 		gboolean in_quote;
 
@@ -1808,20 +1817,17 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 		 * All leading spaces are skipped.
 		 */
 
-		url = NULL;
-		non_space_seen = FALSE;
 		in_quote = FALSE;
 		info.name = NULL;
 		info.addr = zero_host_addr;
 
-		while ((c = *p)) {
-			if (!non_space_seen) {
-				if (is_ascii_space(c))
-					goto next;
-				non_space_seen = TRUE;
-				url = p;
-			}
+		p = skip_ascii_spaces(p);
+		if ('\0' == *p) {				/* Only seen spaces */
+			finished = TRUE;
+			goto free_urlinfo;
+		}
 
+		for (url = p; '\0' != (c = *p); p++) {
 			/*
 			 * Quoted identifiers are one big token.
 			 */
@@ -1837,7 +1843,7 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 			}
 
 			if (in_quote)
-				goto next;
+				continue;
 
 			/*
 			 * The "," may appear un-escaped in the URL.
@@ -1854,26 +1860,14 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 			if (c == ',') {
 				if (is_strcaseprefix(&p[1], "http://"))
 					break;
-				if (p[1] != ' ')
-					goto next;
+				if (!is_ascii_space(p[1]))
+					continue;
 			}
 
-			if (c == ' ' || c == ',')
+			if (is_ascii_space(c) || c == ',')
 				break;
-
-		next:
-			p++;
 		}
 
-		if (url == NULL) {				/* Only seen spaces */
-			if (c == '\0')
-				goto finish;
-			continue;
-		}
-
-		if (c == '\0' && p == url)		/* Empty string remained */
-			goto finish;
-
 		/*
 		 * Parse URL.
 		 */
@@ -1913,8 +1907,6 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 		 * Maybe there is no date following the URL?
 		 */
 
-		if (c == '\0')				/* Reached end of string */
-			goto finish;
 		if (c == ',') {				/* There's no following date then */
 			p++;					/* Skip separator */
 			goto free_urlinfo;		/* continue */
@@ -1926,7 +1918,10 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 		 * Advance to next ',', expecting a date.
 		 */
 
-		date = ++p;
+		if (c != '\0')
+			p++;
+
+		date = p;
 
 	more_date:
 		for (/* NOTHING */; '\0' != (c = *p); p++) {
@@ -1951,8 +1946,8 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 
 		if (skip_date) {				/* URL was not parsed, just skipping */
 			if (c == '\0')				/* Reached end of string */
-				goto finish;
-            if (c == ',')
+				finished = TRUE;
+			else if (c == ',')
                 p++;					/* Skip the "," separator */
 			goto free_urlinfo;			/* continue */
 		}
@@ -1987,16 +1982,13 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 
 		if (info.idx == URN_INDEX) {
 			gchar digest[SHA1_RAW_SIZE];
-
+		
 			ok = urn_get_sha1(info.name, digest);
-			if (!ok) {
-				g_warning("malformed /uri-res/N2R? Alternate-Location: %s",
-					info.name);
-				goto skip_add;
-			}
+			g_assert(ok);
 
 			ok = sha1_eq(sha1, digest);
 			if (!ok) {
+				g_assert(sha1);
 				g_warning("mismatch in /uri-res/N2R? Alternate-Location "
 					"for SHA1=%s: got %s", sha1_base32(sha1), info.name);
 				goto skip_add;
@@ -2018,7 +2010,7 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 				 * all in one block.
 				 */
 				nonurn_altlocs = dmesh_defer_nonurn_altloc(nonurn_altlocs,
-						&info, stamp);
+									&info, stamp);
 				goto nolog;
 			} else
 				ok = dmesh_raw_add(sha1, &info, stamp);
@@ -2034,23 +2026,15 @@ dmesh_collect_locations(gchar *sha1, gchar *value, gboolean defer)
 
 	nolog:
 		if (c == '\0')				/* Reached end of string */
-			goto finish;
-
-        if (c == ',')
+			finished = TRUE;
+		else if (c == ',')
             p++;					/* Skip separator */
 
 	free_urlinfo:
 		if (info.name)
 			atom_str_free(info.name);
 
-		continue;
-
-	finish:
-		if (info.name)
-			atom_str_free(info.name);
-
-		break;
-	}
+	} while (!finished);
 
 	/*
 	 * Once everyone is done we can sort out deferred urls, if any.
@@ -2089,17 +2073,12 @@ dmesh_alt_loc_fill(const gchar *sha1, dmesh_urlinfo_t *buf, gint count)
 
 	for (i = 0, sl = dm->entries; sl && i < count; sl = g_slist_next(sl)) {
 		struct dmesh_entry *dme = sl->data;
-		dmesh_urlinfo_t *from, *to;
+		dmesh_urlinfo_t *from;
 
 		g_assert(i < MAX_ENTRIES);
 
 		from = &dme->url;
-		to = &buf[i++];
-
-		to->addr = from->addr;
-		to->port = from->port;
-		to->idx = from->idx;
-		to->name = from->name;
+		buf[i++] = *from;
 	}
 
 	return i;
@@ -2134,9 +2113,10 @@ dmesh_check_results_set(gnet_results_set_t *rs)
 		has = NULL != g_hash_table_lookup(mesh, rc->sha1);
 
 		if (!has) {
-			struct shared_file *sf = shared_file_by_sha1(rc->sha1);
-			if (sf != NULL && sf != SHARE_REBUILDING && sf->fi == NULL)
-				has = TRUE;
+			const struct shared_file *sf = shared_file_by_sha1(rc->sha1);
+			has =	sf != NULL &&
+					sf != SHARE_REBUILDING &&
+					!shared_file_is_partial(sf);
 		}
 
 		if (has) {
@@ -2181,7 +2161,7 @@ dmesh_check_results_set(gnet_results_set_t *rs)
  * @param `fi' no brief description.
  */
 void
-dmesh_multiple_downloads(gchar *sha1, filesize_t size, fileinfo_t *fi)
+dmesh_multiple_downloads(const gchar *sha1, filesize_t size, fileinfo_t *fi)
 {
 	dmesh_urlinfo_t buffer[DMESH_MAX], *p;
 	gint n;
@@ -2194,12 +2174,17 @@ dmesh_multiple_downloads(gchar *sha1, filesize_t size, fileinfo_t *fi)
 	now = tm_time();
 
 	for (p = buffer; n > 0; n--, p++) {
+		const gchar *filename;
+
 		if (dmesh_debug > 2)
 			g_message("ALT-LOC queuing from MESH: %s",
 				dmesh_urlinfo_to_string(p));
 
+		filename = URN_INDEX == p->idx && fi && fi->file_name
+			? fi->file_name
+			: p->name;
 		download_auto_new(p->name, size, p->idx, p->addr, p->port,
-			blank_guid, NULL, sha1, now, FALSE,
+			blank_guid, NULL, sha1, now,
 			fi->file_size_known, fi, NULL, /* XXX: TLS? */ 0);
 	}
 }
@@ -2214,6 +2199,7 @@ dmesh_store_kv(gpointer key, gpointer value, gpointer udata)
 	FILE *out = udata;
 	GSList *sl;
 
+	g_assert(key);
 	fprintf(out, "%s\n", sha1_base32(key));
 
 	for (sl = dm->entries; sl; sl = g_slist_next(sl)) {
@@ -2421,7 +2407,7 @@ dmesh_ban_retrieve(void)
 	 * Lines starting with a # are skipped.
 	 */
 
-	while (fgets(tmp, sizeof(tmp), in)) {
+	while (fgets(tmp, sizeof tmp, in)) {
 		line++;
 
 		if (tmp[0] == '#')
@@ -2461,14 +2447,14 @@ dmesh_ban_retrieve(void)
 static gboolean
 dmesh_free_kv(gpointer key, gpointer value, gpointer unused_udata)
 {
-	struct dmesh *dm = (struct dmesh *) value;
+	struct dmesh *dm = value;
 	GSList *sl;
 
 	(void) unused_udata;
 	atom_sha1_free(key);
 
 	for (sl = dm->entries; sl; sl = g_slist_next(sl))
-		dmesh_entry_free((struct dmesh_entry *) sl->data);
+		dmesh_entry_free(sl->data);
 
 	g_slist_free(dm->entries);
 	wfree(dm, sizeof *dm);
@@ -2514,7 +2500,7 @@ dmesh_close(void)
 	g_hash_table_foreach(ban_mesh, dmesh_ban_prepend_list, &banned);
 
 	for (sl = banned; sl; sl = g_slist_next(sl)) {
-		struct dmesh_banned *dmb = (struct dmesh_banned *) sl->data;
+		struct dmesh_banned *dmb = sl->data;
 		cq_cancel(callout_queue, dmb->cq_ev);
 		dmesh_ban_expire(callout_queue, dmb);
 	}
