@@ -1,5 +1,5 @@
 /*
- * $Id: dq.h,v 1.7 2005/06/25 01:37:39 daichik Exp $
+ * $Id: dq.h 11786 2006-08-27 17:36:37Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -54,13 +54,13 @@ void dq_close(void);
 
 void dq_launch_net(struct gnutella_node *n, struct query_hashvec *qhv);
 void dq_node_removed(guint32 node_id);
-gboolean dq_got_results(gchar *muid, guint count);
-gboolean dq_oob_results_ind(gchar *muid, gint count);
+gboolean dq_got_results(const gchar *muid, guint count, guint32 status);
+gboolean dq_oob_results_ind(const gchar *muid, gint count);
 void dq_oob_results_got(const gchar *muid, guint count);
-void dq_got_query_status(gchar *muid, guint32 node_id, guint16 kept);
+void dq_got_query_status(const gchar *muid, guint32 node_id, guint16 kept);
 void dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv);
 void dq_search_closed(gnet_search_t handle);
-gboolean dq_get_results_wanted(gchar *muid, guint32 *wanted);
+gboolean dq_get_results_wanted(const gchar *muid, guint32 *wanted);
 
 #endif	/* _core_dq_h_ */
 
