@@ -1,10 +1,8 @@
 /*
- * $Id: extensions.c,v 1.10 2004/11/14 14:25:37 rmanfredi Exp $
+ * $Id: extensions.c 12020 2006-09-21 17:17:35Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Gnutella message extension handling.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Gnutella message extension handling.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: extensions.c,v 1.10 2004/11/14 14:25:37 rmanfredi Exp $");
+RCSID("$Id: extensions.c 12020 2006-09-21 17:17:35Z cbiere $")
 
 #include "extensions.h"
 #include "ggep.h"
@@ -39,25 +47,25 @@ RCSID("$Id: extensions.c,v 1.10 2004/11/14 14:25:37 rmanfredi Exp $");
 
 #include "if/gnet_property_priv.h"
 
-#define HUGE_FS		'\x1c'		/* Field separator (HUGE) */
+#define HUGE_FS		'\x1c'		/**< Field separator (HUGE) */
 
-#define GGEP_MAXLEN	65535		/* Maximum decompressed length */
-#define GGEP_GROW	512			/* Minimum chunk growth when resizing */
+#define GGEP_MAXLEN	65535		/**< Maximum decompressed length */
+#define GGEP_GROW	512			/**< Minimum chunk growth when resizing */
 
-/*
+/**
  * An extension descriptor.
  *
  * The extension block is structured thustly:
  *
- *    <.................len.......................>
- *    <..headlen.><..........paylen...............>
- *    +-----------+-------------------------------+
- *    |   header  |      extension payload        |
- *    +-----------+-------------------------------+
- *    ^           ^
- *    base        payload
+ *    - <.................len.......................>
+ *    - <..headlen.><..........paylen...............>
+ *    - +-----------+-------------------------------+
+ *    - |   header  |      extension payload        |
+ *    - +-----------+-------------------------------+
+ *    - ^           ^
+ *    - base        payload
  *
- * The <headlen> part is simply <len> - <paylen> so it is not stored.
+ * The "<headlen>" part is simply "<len>" - "<paylen>" so it is not stored.
  * Likewise, we store only the beginning of the payload, the base can be
  * computed if needed.
  *
@@ -84,18 +92,18 @@ RCSID("$Id: extensions.c,v 1.10 2004/11/14 14:25:37 rmanfredi Exp $");
  * each time a new extension is found.
  */
 typedef struct extdesc {
-	gchar *ext_phys_payload;	/* Start of payload buffer */
-	gchar *ext_payload;			/* "virtual" payload */
-	guint16 ext_phys_len;		/* Extension length (header + payload) */
-	guint16 ext_phys_paylen;	/* Extension payload length */
-	guint16 ext_paylen;			/* "virtual" payload length */
-	guint16 ext_rpaylen;		/* Length of buffer for "virtual" payload */
+	gchar *ext_phys_payload;	/**< Start of payload buffer */
+	gchar *ext_payload;			/**< "virtual" payload */
+	guint16 ext_phys_len;		/**< Extension length (header + payload) */
+	guint16 ext_phys_paylen;	/**< Extension payload length */
+	guint16 ext_paylen;			/**< "virtual" payload length */
+	guint16 ext_rpaylen;		/**< Length of buffer for "virtual" payload */
 
 	union {
 		struct {
-			gboolean extu_cobs;			/* Payload is COBS-encoded */
-			gboolean extu_deflate;		/* Payload is deflated */
-			const gchar *extu_id;		/* Extension ID */
+			gboolean extu_cobs;			/**< Payload is COBS-encoded */
+			gboolean extu_deflate;		/**< Payload is deflated */
+			const gchar *extu_id;		/**< Extension ID */
 		} extu_ggep;
 	} ext_u;
 
@@ -104,7 +112,7 @@ typedef struct extdesc {
 #define ext_phys_headlen(d)	((d)->ext_phys_len - (d)->ext_phys_paylen)
 #define ext_phys_base(d)	((d)->ext_phys_payload - ext_phys_headlen(d))
 
-/* 
+/*
  * Union access shortcuts.
  */
 
@@ -113,59 +121,70 @@ typedef struct extdesc {
 #define ext_ggep_id			ext_u.extu_ggep.extu_id
 
 static const gchar * const extype[] = {
-	"UNKNOWN",					/* EXT_UNKNOWN */
-	"XML",						/* EXT_XML */
-	"HUGE",						/* EXT_HUGE */
-	"GGEP",						/* EXT_GGEP */
-	"NONE",						/* EXT_NONE */
+	"UNKNOWN",					/**< EXT_UNKNOWN */
+	"XML",						/**< EXT_XML */
+	"HUGE",						/**< EXT_HUGE */
+	"GGEP",						/**< EXT_GGEP */
+	"NONE",						/**< EXT_NONE */
 };
 
 /***
  *** Extension name screener.
  ***/
 
-struct rwtable {			/* Reserved word description */
-	const gchar *rw_name;	/* Representation */
-	ext_token_t rw_token;	/* Token value */
+/**
+ * Reserved word description.
+ */
+struct rwtable {
+	const gchar *rw_name;	/**< Representation */
+	ext_token_t rw_token;	/**< Token value */
 };
 
-static const struct rwtable urntable[] =	/* URN name table (sorted) */
+/** URN name table (sorted) */
+static const struct rwtable urntable[] =
 {
 	{ "bitprint",		EXT_T_URN_BITPRINT },
 	{ "sha1",			EXT_T_URN_SHA1 },
 };
 
-static const struct rwtable ggeptable[] =	/* GGEP extension table (sorted) */
+/** GGEP extension table (sorted) */
+static const struct rwtable ggeptable[] =
 {
-#define GGEP_ID(x) { #x, EXT_T_GGEP_ ## x }
-   	
-	{ "<", EXT_T_GGEP_LIME_XML },	/* '<' is less that 'A' */
-	GGEP_ID(ALT),					/* Alt-locs in qhits */
-	GGEP_ID(BH),					/* Browseable host indication */
-	GGEP_ID(CT),					/* Resource creation time */
-	GGEP_ID(DU),					/* Average servent uptime */
-	GGEP_ID(GTKGV1),				/* GTKG complete version number (binary) */
-	GGEP_ID(GUE),					/* GUESS support */
-	GGEP_ID(H),						/* Hashes in binary form */
-	GGEP_ID(HNAME),					/* Hostname */
-	GGEP_ID(IPP),					/* IP:port in pongs (UHC) */
-	GGEP_ID(LF),					/* Large file size in qhits */
-	GGEP_ID(LOC),					/* Locale preferences, for clustering  */
-	GGEP_ID(PATH),					/* Shared file path, in query hits */
-	GGEP_ID(PHC),					/* Packed host caches (UHC) in pongs */
-	GGEP_ID(PUSH),					/* Push proxy info, in qhits */
-	GGEP_ID(SCP),					/* Supports cached pongs, in pings (UHC) */
-	GGEP_ID(T),						/* Textual information in qhits */
-	GGEP_ID(UDPHC),					/* Is an UDP hostcache (UHC) , in pongs */
-	GGEP_ID(UP),					/* Ultrapeer information about free slots */
-	GGEP_ID(VC),					/* Vendor code, in pongs */
-	GGEP_ID(u),						/* HUGE URN in ASCII */
+#define GGEP_ID(x) { STRINGIFY(x), CAT2(EXT_T_GGEP_,x) }
+#define GGEP_GTKG_ID(x) { "GTKG." STRINGIFY(x), CAT2(EXT_T_GGEP_GTKG_,x) }
+
+	{ "<", EXT_T_GGEP_LIME_XML }, /**< '<' is less that 'A' */
+	GGEP_ID(ALT),		/**< Alt-locs in qhits */
+	GGEP_ID(BH),		/**< Browseable host indication */
+	GGEP_ID(CT),		/**< Resource creation time */
+	GGEP_ID(DU),		/**< Average servent uptime */
+	GGEP_ID(FW),		/**< Firewalled-to-Firewalled protocol version */
+	GGEP_ID(GGEP),		/**< GGEP extension names known, NUL-separated */
+	GGEP_GTKG_ID(IPV6),	/**< GTKG IPv6 address */
+	GGEP_GTKG_ID(TLS),	/**< GTKG TLS support indication */
+	GGEP_ID(GTKGV1),	/**< GTKG complete version number (binary) */
+	GGEP_ID(GUE),		/**< GUESS support */
+	GGEP_ID(H),			/**< Hashes in binary form */
+	GGEP_ID(HNAME),		/**< Hostname */
+	GGEP_ID(IP),		/**< Ip:Port in ping and pongs (F2F) */
+	GGEP_ID(IPP),		/**< IP:Port in pongs (UHC) */
+	GGEP_ID(LF),		/**< Large file size in qhits */
+	GGEP_ID(LOC),		/**< Locale preferences, for clustering  */
+	GGEP_ID(PATH),		/**< Shared file path, in query hits */
+	GGEP_ID(PHC),		/**< Packed host caches (UHC) in pongs */
+	GGEP_ID(PUSH),		/**< Push proxy info, in qhits */
+	GGEP_ID(SCP),		/**< Supports cached pongs, in pings (UHC) */
+	GGEP_ID(T),			/**< Textual information in qhits */
+	GGEP_ID(UA),		/**< User-Agent string */
+	GGEP_ID(UDPHC),		/**< Is an UDP hostcache (UHC) , in pongs */
+	GGEP_ID(UP),		/**< Ultrapeer information about free slots */
+	GGEP_ID(VC),		/**< Vendor code, in pongs */
+	GGEP_ID(VMSG),		/**< Array of supported vendor message codes */
+	GGEP_ID(u),			/**< HUGE URN in ASCII */
 
 #undef GGEP_ID
 };
 
-#define END(v)		(v - 1 + G_N_ELEMENTS(v))
-
 /**
  * Perform a dichotomic search for keywords in the reserved-word table.
  * The `case_sensitive' parameter governs whether lookup is done with or
@@ -176,29 +195,30 @@ static const struct rwtable ggeptable[] =	/* GGEP extension table (sorted) */
  */
 static ext_token_t
 rw_screen(gboolean case_sensitive,
-	const struct rwtable *low, const struct rwtable *high,
+	const struct rwtable *table, size_t size,
 	const gchar *word, const gchar **retkw)
 {
-	const struct rwtable *mid;
-	gint c;
-
 	g_assert(retkw);
 
-	while (low <= high) {
-		mid = low + (high-low)/2;
-		c = case_sensitive ?
-			strcmp(mid->rw_name, word) : strcasecmp(mid->rw_name, word);
-		if (c == 0) {
-			*retkw = mid->rw_name;
-			return mid->rw_token;
-		} else if (c < 0)
-			low = mid + 1;
-		else
-			high = mid - 1;
-	}
+#define GET_KEY(i) (table[(i)].rw_name)
+#define FOUND(i) \
+	G_STMT_START { \
+		*retkw = table[(i)].rw_name; \
+	   	return table[(i)].rw_token; \
+		/* NOTREACHED */ \
+	} G_STMT_END
+
+	if (case_sensitive)
+		BINARY_SEARCH(const gchar *, word, size,
+				strcmp, GET_KEY, FOUND);
+	else
+		BINARY_SEARCH(const gchar *, word, size,
+				ascii_strcasecmp, GET_KEY, FOUND);
+
+#undef FOUND
+#undef GET_KEY
 
 	*retkw = NULL;
-
 	return EXT_T_UNKNOWN;
 }
 
@@ -207,15 +227,31 @@ rw_screen(gboolean case_sensitive,
  */
 static void
 rw_is_sorted(const gchar *name,
-	const struct rwtable *low, const struct rwtable *high)
+	const struct rwtable *table, size_t size)
 {
-	const struct rwtable *e;
+	size_t i;
 
-	for (e = low + 1; e <= high; e++) {
-		const struct rwtable *prev = e - 1;
-		if (strcmp(prev->rw_name, e->rw_name) >= 0)
+	/* Skip the first to have a previous element, tables with a single
+	 * element are sorted anyway. */
+	for (i = 1; i < size; i++) {
+		const struct rwtable *prev = &table[i - 1], *e = &table[i];
+
+		if (
+			prev->rw_token >= e->rw_token ||
+			strcmp(prev->rw_name, e->rw_name) >= 0
+		)
 			g_error("reserved word table \"%s\" unsorted (near item \"%s\")",
 				name, e->rw_name);
+
+		if (ggeptable == table) {
+			const gchar *s;
+
+		   	s = ext_ggep_name(e->rw_token);
+			if (0 != strcmp(s, e->rw_name)) {
+				g_error("table \"%s\" has wrong GGEP ID (near item \"%s\")",
+					name, e->rw_name);
+			}
+		}
 	}
 }
 
@@ -228,7 +264,7 @@ rw_ggep_screen(gchar *word, const gchar **retkw)
 {
 	ext_token_t t;
 
-	t = rw_screen(TRUE, ggeptable, END(ggeptable), word, retkw);
+	t = rw_screen(TRUE, ggeptable, G_N_ELEMENTS(ggeptable), word, retkw);
 
 	return (t == EXT_T_UNKNOWN) ? EXT_T_UNKNOWN_GGEP : t;
 }
@@ -240,7 +276,7 @@ rw_ggep_screen(gchar *word, const gchar **retkw)
 static ext_token_t
 rw_urn_screen(gchar *word, const gchar **retkw)
 {
-	return rw_screen(FALSE, urntable, END(urntable), word, retkw);
+	return rw_screen(FALSE, urntable, G_N_ELEMENTS(urntable), word, retkw);
 }
 
 /***
@@ -250,8 +286,9 @@ rw_urn_screen(gchar *word, const gchar **retkw)
 static GHashTable *ext_names = NULL;
 
 /**
- * Transform the name into a printable form, and return an atom string
- * of that printable form.
+ * Transform the name into a printable form.
+ *
+ * @return an atom string of that printable form.
  */
 static gchar *
 ext_name_atom(const gchar *name)
@@ -276,7 +313,7 @@ ext_name_atom(const gchar *name)
 	 * no escaping, then the name is also the key (same object).
 	 */
 
-	key = g_strdup(name);
+	key = wcopy(name, 1 + strlen(name));
 	atom = hex_escape(key, TRUE); /* strict escaping */
 
 	g_hash_table_insert(ext_names, key, atom);
@@ -292,10 +329,10 @@ ext_names_kv_free(gpointer key, gpointer value, gpointer unused_udata)
 {
 	(void) unused_udata;
 
-	if (0 != strcmp((gchar *) key, (gchar *) value))
+	if (0 != strcmp(key, value)) {
 		G_FREE_NULL(value);
-
-	G_FREE_NULL(key);
+	}
+    wfree(key, 1 + strlen(key));
 
 	return TRUE;
 }
@@ -311,16 +348,14 @@ ext_names_kv_free(gpointer key, gpointer value, gpointer unused_udata)
  *** is structured and can therefore grab more than one extension in one call).
  ***
  *** Upon entry, `*retp' points to the start of the extension, and there are
- ** `len' bytes to parse.  There are `exvcnt' slots available in the extension
+ *** `len' bytes to parse.  There are `exvcnt' slots available in the extension
  *** vector, starting at `exv'.
  ***
  *** On exit, `p' is updated to the first byte following the last successfully
  *** parsed byte.  If the returned value is 0, then `p' is not updated.
  ***/
 
-/*
- * ext_ggep_parse
- *
+/**
  * Parses a GGEP block (can hold several extensions).
  */
 static gint
@@ -333,11 +368,8 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 
 	for (count = 0; count < exvcnt && p < end; /* empty */) {
 		guchar flags;
-		gchar id[16];
-		gint id_len;
-		gint data_length;
-		gint i;
-		gchar *ip = id;
+		gchar id[GGEP_F_IDLEN + 1];
+		guint id_len, data_length, i;
 		gboolean length_ended = FALSE;
 		const gchar *name;
 		extdesc_t *d;
@@ -354,11 +386,12 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 			goto abort;
 
 		id_len = flags & GGEP_F_IDLEN;
+		g_assert(id_len < sizeof id);
 
 		if (id_len == 0)
 			goto abort;
 
-		if (end - p < id_len)		/* Not enough bytes to store the ID! */
+		if ((size_t) (end - p) < id_len) /* Not enough bytes to store the ID! */
 			goto abort;
 
 		/*
@@ -373,12 +406,11 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 
 		for (i = 0; i < id_len; i++) {
 			gint c = *p++;
-			if (c == '\0' || !isascii(c) || iscntrl(c))
+			if (c == '\0' || !isascii(c) || is_ascii_cntrl(c))
 				goto abort;
-			*ip++ = c;
+			id[i] = c; 
 		}
-
-		*ip++ = '\0';
+		id[i] = '\0';
 
 		/*
 		 * Read the payload length (maximum of 3 bytes).
@@ -407,12 +439,13 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 		if (!length_ended)
 			goto abort;
 
-		/* 
+		/*
 		 * Ensure we have enough bytes left for the payload.  If not, it
 		 * means the length is garbage.
 		 */
 
-		if (end - p < data_length)		/* Not enough bytes for the payload */
+		/* Check whether there are enough bytes for the payload */
+		if ((size_t) (end - p) < data_length)
 			goto abort;
 
 		/*
@@ -423,18 +456,18 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 		 */
 
 		if (flags & (GGEP_F_COBS|GGEP_F_DEFLATE)) {
-			gint len = data_length;
+			guint d_len = data_length;
 
 			if (flags & GGEP_F_COBS) {
-				if (len == 0 || !cobs_is_valid(p, len))
+				if (d_len == 0 || !cobs_is_valid(p, d_len))
 					goto abort;
-				len--;					/* One byte of overhead */
+				d_len--;					/* One byte of overhead */
 			}
 
 			if (flags & GGEP_F_DEFLATE) {
-				gint offset = 0;
+				guint offset = 0;
 
-				if (len < 6)
+				if (d_len < 6)
 					goto abort;
 
 				/*
@@ -448,12 +481,12 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 				 */
 
 				if (flags & GGEP_F_COBS) {
-					if ((guint) *p < 3)
+					if ((guchar) *p < 3)
 						goto abort;
 					offset = 1;			/* Skip leading byte */
 				}
 
-				if (!zlib_is_valid_header(p + offset, len))
+				if (!zlib_is_valid_header(p + offset, d_len))
 					goto abort;
 			}
 		}
@@ -462,7 +495,7 @@ ext_ggep_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 		 * OK, at this point we have validated the GGEP header.
 		 */
 
-		d = walloc(sizeof(*d));
+		d = walloc(sizeof *d);
 
 		d->ext_phys_payload = p;
 		d->ext_phys_paylen = data_length;
@@ -561,11 +594,10 @@ ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 	 * Recognize "urn:".
 	 */
 
-	if (0 != strncasecmp(p, "urn:", 4))
+	p = is_strcaseprefix(p, "urn:");
+	if (!p)
 		return 0;
 
-	p += 4;
-
 	/*
 	 * Maybe it's simply a "urn:" empty specification?
 	 */
@@ -612,7 +644,7 @@ ext_huge_parse(gchar **retp, gint len, extvec_t *exv, gint exvcnt)
 
 	while (p < end) {
 		guchar c = *p++;
-		if (!isalnum(c) || c == (guchar) GGEP_MAGIC) {
+		if (!is_ascii_alnum(c) || c == (guchar) GGEP_MAGIC) {
 			p--;
 			break;
 		}
@@ -701,7 +733,6 @@ ext_unknown_parse(gchar **retp, gint len, extvec_t *exv,
 	gint exvcnt, gboolean skip)
 {
 	gchar *p = *retp;
-	gchar *end = p + len;
 	gchar *lastp = p;				/* Last parsed point */
 	extdesc_t *d;
 
@@ -713,23 +744,32 @@ ext_unknown_parse(gchar **retp, gint len, extvec_t *exv,
 	 * could appear to be the start of a GGEP block or XML.
 	 */
 
-	while (p < end) {
-		guchar c = *p++;
-		if (
-			(c == '\0' || c == (guchar) HUGE_FS || c == (guchar) GGEP_MAGIC) ||
-			(
-				(c == 'u' || c == 'U') &&
-				(end - p) >= 3 &&
-				0 == strncasecmp(p, "rn:", 3)
-			) ||
-			(c == '<' && (p < end) && isalpha((guchar) *p))
-		) {
+	for (/* NOTHING*/; len > 0; p++, len--) {
+		gboolean found;
+
+		switch ((guchar) *p) {
+		case '\0':
+		case (guchar) HUGE_FS:
+		case (guchar) GGEP_MAGIC:
+			found = TRUE;
+			break;
+		case 'u':
+		case 'U':
+			found = len >= 4 && is_strcaseprefix(p, "urn:");
+			break;
+		case '<':
+			found = len >= 2 && is_ascii_alpha((guchar) p[1]);
+			break;
+		default:
+			found = FALSE;
+		}
+		
+		if (found) {
 			if (skip) {
 				skip = FALSE;
-				continue;
+			} else {
+				break;
 			}
-			p--;
-			break;
 		}
 	}
 
@@ -898,7 +938,7 @@ ext_parse(gchar *buf, gint len, extvec_t *exv, gint exvcnt)
 
 		g_assert(len > 0);
 
-		/* 
+		/*
 		 * From now on, all new Gnutella extensions will be done via GGEP.
 		 * However, we have to be backward compatible with legacy extensions
 		 * that predate GGEP (HUGE and XML) and were not properly encapsulated.
@@ -993,8 +1033,8 @@ out:
  * Inflate `len' bytes starting at `buf', up to GGEP_MAXLEN bytes.
  * The payload `name' is given only in case there is an error to report.
  *
- * Returns the allocated inflated buffer, and its inflated length in `retlen'.
- * Returns NULL on error.
+ * @returns the allocated inflated buffer, and its inflated length in `retlen'.
+ * @returns NULL on error.
  */
 static gchar *
 ext_ggep_inflate(gchar *buf, gint len, guint16 *retlen, const gchar *name)
@@ -1016,8 +1056,8 @@ ext_ggep_inflate(gchar *buf, gint len, guint16 *retlen, const gchar *name)
 
 	inz = walloc(sizeof(*inz));
 
-	inz->zalloc = NULL;
-	inz->zfree = NULL;
+	inz->zalloc = zlib_alloc_func;
+	inz->zfree = zlib_free_func;
 	inz->opaque = NULL;
 
 	ret = inflateInit(inz);
@@ -1099,7 +1139,7 @@ ext_ggep_inflate(gchar *buf, gint len, guint16 *retlen, const gchar *name)
 	wfree(inz, sizeof(*inz));
 
 	/*
-	 * Return NULL on error, fill `retlen' if OK.
+	 * @return NULL on error, fill `retlen' if OK.
 	 */
 
 	if (failed) {
@@ -1111,7 +1151,7 @@ ext_ggep_inflate(gchar *buf, gint len, guint16 *retlen, const gchar *name)
 
 	g_assert(*retlen == inflated);	/* Make sure it was not truncated */
 
-	return result;				/* OK, successfully inflated */
+	return result;					/* OK, successfully inflated */
 }
 
 /**
@@ -1125,10 +1165,10 @@ static void
 ext_ggep_decode(const extvec_t *e)
 {
 	gchar *pbase;					/* Current payload base */
-	gint plen;						/* Curernt payload length */
+	size_t plen;					/* Curernt payload length */
 	gchar *uncobs = NULL;			/* COBS-decoded buffer */
-	gint uncobs_len = 0;			/* Length of walloc()'ed buffer */
-	gint result;					/* Decoded length */
+	size_t uncobs_len = 0;			/* Length of walloc()'ed buffer */
+	size_t result;					/* Decoded length */
 	extdesc_t *d;
 
 	g_assert(e);
@@ -1224,7 +1264,7 @@ out:
 }
 
 /**
- * Returns a pointer to the extension's payload.
+ * @returns a pointer to the extension's payload.
  */
 const gchar *
 ext_payload(const extvec_t *e)
@@ -1246,7 +1286,7 @@ ext_payload(const extvec_t *e)
 }
 
 /**
- * Returns a pointer to the extension's payload length.
+ * @returns a pointer to the extension's payload length.
  */
 guint16
 ext_paylen(const extvec_t *e)
@@ -1268,9 +1308,9 @@ ext_paylen(const extvec_t *e)
 }
 
 /**
- * Returns a pointer to the extension's header.
+ * @returns a pointer to the extension's header.
  *
- * WARNING: the actual "virtual" payload may not be contiguous to the end
+ * @warning the actual "virtual" payload may not be contiguous to the end
  * of the header: don't read past the ext_headlen() first bytes of the
  * header.
  */
@@ -1285,7 +1325,7 @@ ext_base(const extvec_t *e)
 }
 
 /**
- * Returns the length of the extensions's header.
+ * @returns the length of the extensions's header.
  */
 guint16
 ext_headlen(const extvec_t *e)
@@ -1298,7 +1338,7 @@ ext_headlen(const extvec_t *e)
 }
 
 /**
- * Returns the total length of the extension (payload + extension header).
+ * @returns the total length of the extension (payload + extension header).
  */
 guint16
 ext_len(const extvec_t *e)
@@ -1317,7 +1357,7 @@ ext_len(const extvec_t *e)
 }
 
 /**
- * Returns extension's GGEP ID, or "" if not a GGEP one.
+ * @returns extension's GGEP ID, or "" if not a GGEP one.
  */
 const gchar *
 ext_ggep_id_str(const extvec_t *e)
@@ -1385,7 +1425,7 @@ ext_has_ascii_word(const extvec_t *e)
 		guchar c = *p++;
 		if (!isascii(c))
 			return FALSE;
-		if (!has_alnum && isalnum(c))
+		if (!has_alnum && is_ascii_alnum(c))
 			has_alnum = TRUE;
 	}
 
@@ -1409,7 +1449,7 @@ ext_dump_one(FILE *f, const extvec_t *e, const gchar *prefix,
 
 	fputs(extype[e->ext_type], f);
 	fprintf(f, " (token=%d) ", e->ext_token);
-	
+
 	if (e->ext_name)
 		fprintf(f, "\"%s\" ", e->ext_name);
 
@@ -1501,6 +1541,21 @@ ext_reset(extvec_t *exv, gint exvcnt)
 	}
 }
 
+const gchar *
+ext_ggep_name(ext_token_t id)
+{
+	size_t i;
+
+	g_assert(id < EXT_T_TOKEN_COUNT);
+	g_assert(id >= ggeptable[0].rw_token);
+
+	i = id - ggeptable[0].rw_token;
+	g_assert(i < G_N_ELEMENTS(ggeptable));
+	g_assert(id == ggeptable[i].rw_token);
+
+	return ggeptable[i].rw_name;
+}
+
 /***
  *** Init & Shutdown
  ***/
@@ -1513,8 +1568,8 @@ ext_init(void)
 {
 	ext_names = g_hash_table_new(g_str_hash, g_str_equal);
 
-	rw_is_sorted("ggeptable", ggeptable, END(ggeptable));
-	rw_is_sorted("urntable", urntable, END(urntable));
+	rw_is_sorted("ggeptable", ggeptable, G_N_ELEMENTS(ggeptable));
+	rw_is_sorted("urntable", urntable, G_N_ELEMENTS(urntable));
 }
 
 /**
