@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.c,v 1.5 2004/10/17 21:11:00 cbiere Exp $
+ * $Id: geo_ip.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -24,14 +24,18 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Support for geographic (country-level) IP mapping.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
  */
 
 #include "common.h"
 
-RCSID("$Id: geo_ip.c,v 1.5 2004/10/17 21:11:00 cbiere Exp $");
+RCSID("$Id: geo_ip.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "geo_ip.h"
 #include "settings.h"
@@ -52,18 +56,18 @@ RCSID("$Id: geo_ip.c,v 1.5 2004/10/17 21:11:00 cbiere Exp $");
 static const gchar gip_file[] = "geo-ip.txt";
 static const gchar gip_what[] = "Geographic IP mappings";
 
-static gpointer geo_db;		/* The database of bogus CIDR ranges */
+static gpointer geo_db;			/**< The database of bogus CIDR ranges */
 
-/*
+/**
  * Context used during ip_range_split() calls.
  */
 struct range_context {
-	guint32 ip1;				/* Original lower IP in global range */
-	guint32 ip2;				/* Original upper IP in global range */
-	gint country;				/* Country code (numerical encoded) */
-	gint count;					/* Amount of ranges we added, for stats */
-	gchar *line;				/* The line from the input file */
-	gint linenum;				/* Line number in input file, for errors */
+	guint32 ip1;				/**< Original lower IP in global range */
+	guint32 ip2;				/**< Original upper IP in global range */
+	gint country;				/**< Country code (numerical encoded) */
+	gint count;					/**< Amount of ranges we added, for stats */
+	gchar *line;				/**< The line from the input file */
+	gint linenum;				/**< Line number in input file, for errors */
 };
 
 /**
@@ -72,7 +76,7 @@ struct range_context {
  * Insert IP range in database, linking it to the proper country code.
  */
 static void
-gip_add_cidr(guint32 ip, guint8 bits, gpointer udata)
+gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 {
 	struct range_context *ctx = (struct range_context *) udata;
 	iprange_err_t error;
@@ -80,7 +84,7 @@ gip_add_cidr(guint32 ip, guint8 bits, gpointer udata)
 
 	if (dbg > 4)
 		printf("GEO adding %s/%d for \"%s\"\n",
-			ip_to_gchar(ip), bits, ctx->line);
+			ip_to_string(ip), bits, ctx->line);
 
 	ccode = GUINT_TO_POINTER(ctx->country);
 	error = iprange_add_cidr(geo_db, ip, bits, ccode);
@@ -93,13 +97,13 @@ gip_add_cidr(guint32 ip, guint8 bits, gpointer udata)
 		if (error == IPR_ERR_OK) {
 			g_warning("%s, line %d: "
 				"entry \"%s\" (%s/%d) superseded earlier smaller range",
-				gip_file, ctx->linenum, ctx->line, ip_to_gchar(ip), bits);
+				gip_file, ctx->linenum, ctx->line, ip_to_string(ip), bits);
 			break;
 		}
 		/* FALL THROUGH */
 	default:
 		g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
-			gip_file, ctx->linenum, ctx->line, ip_to_gchar(ip), bits,
+			gip_file, ctx->linenum, ctx->line, ip_to_string(ip), bits,
 			iprange_strerror(error));
 		return;
 	}
@@ -109,7 +113,8 @@ gip_add_cidr(guint32 ip, guint8 bits, gpointer udata)
 
 /**
  * Load geographic IP data from the supplied FILE.
- * Returns the amount of entries loaded.
+ *
+ * @return The amount of entries loaded.
  */
 static gint
 gip_load(FILE *f)
@@ -117,7 +122,7 @@ gip_load(FILE *f)
 	gchar line[1024];
 	gchar *p;
 	gint linenum = 0;
-	gchar *end;
+	const gchar *end;
 	gint c, code;
 	struct range_context ctx;
 
@@ -133,12 +138,9 @@ gip_load(FILE *f)
 		 * Remove all trailing spaces in string.
 		 * Otherwise, lines which contain only spaces would cause a warning.
 		 */
-	
-		p = strchr(line, '\0');	
-		while (--p >= line) {
-			guchar c = (guchar) *p;
-			if (!is_ascii_space(c))
-				break;
+
+		p = strchr(line, '\0');
+		while (p-- != line && is_ascii_space(*p)) {
 			*p = '\0';
 		}
 		if ('\0' == *line)
@@ -161,7 +163,7 @@ gip_load(FILE *f)
 			continue;
 		}
 
-		if (!gchar_to_ip_strict(line, &ctx.ip1, NULL)) {
+		if (!string_to_ip_strict(line, &ctx.ip1, NULL)) {
 			g_warning("%s, line %d: invalid first IP in \"%s\"",
 				gip_file, linenum, line);
 			continue;
@@ -179,7 +181,7 @@ gip_load(FILE *f)
 			end++;
 		}
 
-		if (!gchar_to_ip_strict(end, &ctx.ip2, (gchar const **) &end)) {
+		if (!string_to_ip_strict(end, &ctx.ip2, &end)) {
 			g_warning("%s, line %d: invalid second IP in \"%s\"",
 				gip_file, linenum, line);
 			continue;
@@ -265,18 +267,21 @@ gip_changed(const gchar *filename, gpointer unused)
 
 	gip_close();
 	count = gip_load(f);
+	fclose(f);
 
 	gm_snprintf(buf, sizeof(buf), "Reloaded %d geographic IP ranges.", count);
 	gcu_statusbar_message(buf);
 }
 
 /**
- * Loads the geo-ip.txt into memory, choosing the first file we find
- * among the several places we look at, typically:
+ * Loads the geo-ip.txt into memory.
  *
- *    ~/.gtk-gnutella/geo-ip.txt
- *    /usr/share/gtk-gnutella/geo-ip.txt
- *    /home/src/gtk-gnutella/geo-ip.txt
+ * Choosing the first file we find among the several places we look at,
+ * typically:
+ *
+ *		-# ~/.gtk-gnutella/geo-ip.txt
+ *		-# /usr/share/gtk-gnutella/geo-ip.txt
+ *		-# /home/src/gtk-gnutella/geo-ip.txt
  *
  * The selected file will then be monitored and a reloading will occur
  * shortly after a modification.
@@ -287,16 +292,16 @@ gip_retrieve(void)
 	FILE *f;
 	gint idx;
 	gchar *filename;
-#ifndef OFFICIAL_BUILD 
-	file_path_t fp[3];
+#ifndef OFFICIAL_BUILD
+	static file_path_t fp[3];
 #else
-	file_path_t fp[2];
+	static file_path_t fp[2];
 #endif
 
 	file_path_set(&fp[0], settings_config_dir(), gip_file);
 	file_path_set(&fp[1], PRIVLIB_EXP, gip_file);
-#ifndef OFFICIAL_BUILD 
-	file_path_set(&fp[2], PACKAGE_SOURCE_DIR, gip_file);
+#ifndef OFFICIAL_BUILD
+	file_path_set(&fp[2], PACKAGE_EXTRA_SOURCE_DIR, gip_file);
 #endif
 
 	f = file_config_open_read_norename_chosen(
@@ -310,6 +315,7 @@ gip_retrieve(void)
 	G_FREE_NULL(filename);
 
 	gip_load(f);
+	fclose(f);
 }
 
 /**
@@ -327,21 +333,36 @@ gip_init(void)
 void
 gip_close(void)
 {
-	iprange_free_each(geo_db, NULL);
-	geo_db = NULL;
+	if (geo_db) {
+		iprange_free_each(geo_db, NULL);
+		geo_db = NULL;
+	}
 }
 
 /**
- * Returns the country mapped to this IP address, as an numerical encoded country code,
- * or -1 when unknown.
+ * Retrieves the country an address is assigned to.
+ *
+ * @param ha the host address to look up.
+ * @return the country mapped to this IP address as an numerical encoded
+ * country code, * or -1 when unknown.
  */
 gint
-gip_country(guint32 ip)
+gip_country(const host_addr_t ha)
 {
-	gpointer code;
+	host_addr_t to;
 
-	code = iprange_get(geo_db, ip);
-	return NULL != code ? (GPOINTER_TO_INT(code) >> 1) - 1 : -1;
+	if (
+		host_addr_convert(ha, &to, NET_TYPE_IPV4) ||
+		host_addr_6to4_to_ipv4(ha, &to)
+	) {
+		gpointer code;
+		guint32 ip;
+
+		ip = host_addr_ipv4(to);
+		if (geo_db && NULL != (code = iprange_get(geo_db, ip)))
+			return (GPOINTER_TO_INT(code) >> 1) - 1;
+	}
+	return -1;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
