@@ -1,10 +1,8 @@
 /*
- * $Id: geo_ip.h,v 1.2 2004/10/17 21:11:00 cbiere Exp $
+ * $Id: geo_ip.h 9450 2005-09-10 08:17:29Z daichik $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
- * Support for geographic IP mapping.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,12 +23,23 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Support for geographic IP mapping.
+ *
+ * @author Raphael Manfredi
+ * @date 2004
+ */
+
 #ifndef _core_geo_ip_h_
 #define _core_geo_ip_h_
 
 #include <glib.h>
+#include "lib/host_addr.h"
 
-gint gip_country(guint32 ip);
+gint gip_country(const host_addr_t addr);
 void gip_init(void);
 void gip_close(void);
 
