@@ -1,5 +1,5 @@
 /*
- * $Id: ggep_type.c,v 1.19 2005/09/10 08:17:28 daichik Exp $
+ * $Id: ggep_type.c 11743 2006-08-26 14:11:32Z rmanfredi $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ggep_type.c,v 1.19 2005/09/10 08:17:28 daichik Exp $");
+RCSID("$Id: ggep_type.c 11743 2006-08-26 14:11:32Z rmanfredi $")
 
 #include "ggep.h"
 #include "ggep_type.h"
@@ -123,7 +123,7 @@ ggept_gtkgv1_extract(extvec_t *exv, struct ggep_gtkgv1 *info)
 
 	READ_GUINT32_BE(p, info->release);
 	p += 4;
-	READ_GUINT32_BE(p, info->start);
+	READ_GUINT32_BE(p, info->build);
 	p += 4;
 
 	g_assert(p - payload == 12);
@@ -157,7 +157,7 @@ ggept_ip_vec_extract(extvec_t *exv, struct gnutella_host **hvec, gint *hvcnt)
 	vec = walloc(cnt * sizeof *vec);
 
 	for (i = 0; i < cnt; i++) {
-		vec[i].addr = host_addr_set_ipv4(peek_be32(p));
+		vec[i].addr = host_addr_get_ipv4(peek_be32(p));
 		p += 4;
 		vec[i].port = peek_le16(p);
 		p += 2;
@@ -219,6 +219,7 @@ ggept_hname_extract(extvec_t *exv, gchar *buf, gint len)
 	gint slen;
 	const gchar *payload;
 
+	g_assert(len >= 0);
 	g_assert(exv->ext_type == EXT_GGEP);
 	g_assert(exv->ext_token == EXT_T_GGEP_HNAME);
 
@@ -228,17 +229,32 @@ ggept_hname_extract(extvec_t *exv, gchar *buf, gint len)
 	 */
 
 	tlen = ext_paylen(exv);
-
-	if (tlen <= 0)
+	if (tlen <= 0 || tlen >= len)
 		return GGEP_INVALID;
 
 	payload = ext_payload(exv);
 	slen = MIN(tlen, len - 1);
 
 	memcpy(buf, payload, slen);
-
 	buf[slen] = '\0';
 
+	/*
+	 * Make sure the full string qualifies as hostname and is not an
+	 * IP address.
+	 */
+	{
+		const gchar *endptr;
+		host_addr_t addr;
+
+		if (
+			!string_to_host_or_addr(buf, &endptr, &addr) ||
+			&buf[slen] != endptr ||
+			is_host_addr(addr)
+		) {
+			return GGEP_INVALID;
+		}
+	}
+	
 	return GGEP_OK;
 }
 
@@ -296,19 +312,12 @@ ggept_gtkg_ipv6_extract(extvec_t *exv, host_addr_t *addr)
 		return GGEP_INVALID;
 
 	if (addr) {
-#ifdef USE_IPV6
 		if (0 == len) {
 			*addr = zero_host_addr;
 		} else {
-			const gchar *ipv6;
-
 			g_assert(len >= 16);
-			ipv6 = ext_payload(exv);
-			host_addr_set_ipv6(addr, cast_to_gconstpointer(ipv6));
+			*addr = host_addr_get_ipv6(cast_to_gconstpointer(ext_payload(exv)));
 		}
-#else /* !USE_IPV6 */
-		*addr = zero_host_addr;
-#endif /* USE_IPV6 */
 	}
 
 	return GGEP_OK;
