@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c,v 1.15 2005/11/12 11:13:42 cbiere Exp $
+ * $Id: gnet_stats.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gnet_stats.c,v 1.15 2005/11/12 11:13:42 cbiere Exp $");
+RCSID("$Id: gnet_stats.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "gnet_stats.h"
 #include "gmsg.h"
@@ -43,7 +43,6 @@ RCSID("$Id: gnet_stats.c,v 1.15 2005/11/12 11:13:42 cbiere Exp $");
 #include "if/gnet_property_priv.h"
 
 #include "lib/event.h"
-#include "lib/endian.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 static guint8 stats_lut[256];
@@ -80,6 +79,8 @@ static const gchar * const msg_drop_reason[MSG_DROP_REASON_COUNT] = {
 	N_("Malformed Query Hit"),				/**< MSG_DROP_BAD_RESULT */
 	N_("Bad return address"),				/**< MSG_DROP_BAD_RETURN_ADDRESS */
 	N_("Hostile IP address"),				/**< MSG_DROP_HOSTILE_IP */
+	N_("Spam"),								/**< MSG_DROP_SPAM */
+	N_("Evil filename"),					/**< MSG_DROP_EVIL */
 };
 
 /***
@@ -89,18 +90,27 @@ static const gchar * const msg_drop_reason[MSG_DROP_REASON_COUNT] = {
 void
 gnet_stats_init(void)
 {
-    memset(stats_lut, MSG_UNKNOWN, sizeof(stats_lut));
-    stats_lut[GTA_MSG_INIT] = MSG_INIT;
-    stats_lut[GTA_MSG_INIT_RESPONSE]= MSG_INIT_RESPONSE;
-    stats_lut[GTA_MSG_BYE] = MSG_BYE;
-    stats_lut[GTA_MSG_QRP] = MSG_QRP;
-    stats_lut[GTA_MSG_VENDOR] = MSG_VENDOR;
-    stats_lut[GTA_MSG_STANDARD] = MSG_STANDARD;
-    stats_lut[GTA_MSG_PUSH_REQUEST] = MSG_PUSH_REQUEST;
-    stats_lut[GTA_MSG_SEARCH] = MSG_SEARCH;
-    stats_lut[GTA_MSG_SEARCH_RESULTS] = MSG_SEARCH_RESULTS;
-    stats_lut[GTA_MSG_HSEP_DATA] = MSG_HSEP;
+	guint i;
 
+	for (i = 0; i < 256; i++) {
+		guchar m = MSG_UNKNOWN;
+		
+    	switch ((enum gta_msg) i) {
+		case GTA_MSG_INIT:           m = MSG_INIT; break;
+		case GTA_MSG_INIT_RESPONSE:  m = MSG_INIT_RESPONSE; break;
+		case GTA_MSG_SEARCH:         m = MSG_SEARCH; break;
+		case GTA_MSG_SEARCH_RESULTS: m = MSG_SEARCH_RESULTS; break;
+		case GTA_MSG_PUSH_REQUEST:   m = MSG_PUSH_REQUEST; break;
+		case GTA_MSG_RUDP:			 m = MSG_RUDP; break;
+		case GTA_MSG_VENDOR:		 m = MSG_VENDOR; break;
+		case GTA_MSG_STANDARD:		 m = MSG_STANDARD; break;
+		case GTA_MSG_QRP:            m = MSG_QRP; break;
+		case GTA_MSG_HSEP_DATA:		 m = MSG_HSEP; break;
+		case GTA_MSG_BYE:      		 m = MSG_BYE; break;
+		}
+		stats_lut[i] = m;
+	}
+		
     memset(&gnet_stats, 0, sizeof(gnet_stats));
     memset(&gnet_udp_stats, 0, sizeof(gnet_udp_stats));
 }
@@ -111,7 +121,7 @@ gnet_stats_init(void)
 void
 gnet_stats_count_received_header(gnutella_node_t *n)
 {
-	guint t = stats_lut[n->header.function];
+	guint t = stats_lut[gnutella_header_get_function(&n->header)];
 	guint i;
 	gnet_stats_t *stats;
 
@@ -129,11 +139,11 @@ gnet_stats_count_received_header(gnutella_node_t *n)
     stats->byte.received[MSG_TOTAL] += GTA_HEADER_SIZE;
     stats->byte.received[t] += GTA_HEADER_SIZE;
 
-	i = MIN(n->header.ttl, STATS_RECV_COLUMNS-1);
+	i = MIN(gnutella_header_get_ttl(&n->header), STATS_RECV_COLUMNS - 1);
     stats->pkg.received_ttl[i][MSG_TOTAL]++;
     stats->pkg.received_ttl[i][t]++;
 
-	i = MIN(n->header.hops, STATS_RECV_COLUMNS-1);
+	i = MIN(gnutella_header_get_hops(&n->header), STATS_RECV_COLUMNS - 1);
     stats->pkg.received_hops[i][MSG_TOTAL]++;
     stats->pkg.received_hops[i][t]++;
 }
@@ -144,8 +154,8 @@ gnet_stats_count_received_header(gnutella_node_t *n)
 void
 gnet_stats_count_received_payload(const gnutella_node_t *n)
 {
-    guint32 size = n->size;
-	guint t = stats_lut[n->header.function];
+    guint32 size = gnutella_header_get_size(&n->header);
+	guint t = stats_lut[gnutella_header_get_function(&n->header)];
 	guint i;
 	gnet_stats_t *stats;
 
@@ -157,11 +167,11 @@ gnet_stats_count_received_payload(const gnutella_node_t *n)
     stats->byte.received[MSG_TOTAL] += size;
     stats->byte.received[t] += size;
 
-	i = MIN(n->header.ttl, STATS_RECV_COLUMNS-1);
+	i = MIN(gnutella_header_get_ttl(&n->header), STATS_RECV_COLUMNS - 1);
     stats->byte.received_ttl[i][MSG_TOTAL] += size;
     stats->byte.received_ttl[i][t] += size;
 
-	i = MIN(n->header.hops, STATS_RECV_COLUMNS-1);
+	i = MIN(gnutella_header_get_hops(&n->header), STATS_RECV_COLUMNS - 1);
     stats->byte.received_hops[i][MSG_TOTAL] += size;
     stats->byte.received_hops[i][t] += size;
 }
@@ -230,7 +240,7 @@ void
 gnet_stats_count_expired(const gnutella_node_t *n)
 {
     guint32 size = n->size + sizeof(n->header);
-	guint t = stats_lut[n->header.function];
+	guint t = stats_lut[gnutella_header_get_function(&n->header)];
 	gnet_stats_t *stats;
 
 	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
@@ -276,14 +286,18 @@ gnet_stats_count_dropped(gnutella_node_t *n, msg_drop_reason_t reason)
 	g_assert((gint) reason >= 0 && reason < MSG_DROP_REASON_COUNT);
 
     size = n->size + sizeof(n->header);
-	type = stats_lut[n->header.function];
+	type = stats_lut[gnutella_header_get_function(&n->header)];
 	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
 	DROP_STATS(stats, type, size);
 	node_inc_rxdrop(n);
 
-	if (reason == MSG_DROP_HOSTILE_IP)
-		n->n_hostile++;
+	switch (reason) {
+	case MSG_DROP_HOSTILE_IP: n->n_hostile++; break;
+	case MSG_DROP_SPAM: n->n_spam++; break;
+	case MSG_DROP_EVIL: n->n_evil++; break;
+	default: ;
+	}
 
 	if (dbg > 4)
 		gmsg_log_dropped(&n->header, "from %s <%s>: %s",
@@ -306,7 +320,7 @@ gnet_stats_count_dropped_nosize(
 	gnet_stats_t *stats;
 	g_assert((gint) reason >= 0 && reason < MSG_DROP_REASON_COUNT);
 
-	type = stats_lut[n->header.function];
+	type = stats_lut[gnutella_header_get_function(&n->header)];
 	stats = NODE_IS_UDP(n) ? &gnet_udp_stats : &gnet_tcp_stats;
 
 	/* Data part of message not read */
@@ -320,25 +334,23 @@ gnet_stats_count_dropped_nosize(
 void
 gnet_stats_count_flowc(gconstpointer head)
 {
-    const struct gnutella_header *h = head;
 	guint t;
 	guint i;
-	guint32 size;
+	guint16 size = gmsg_size(head);
 
-	READ_GUINT32_LE(h->size, size);
 #if 0
 	g_message("FLOWC function=%d ttl=%d hops=%d", h->function, h->ttl, h->hops);
 #endif
 
-	t = stats_lut[h->function];
+	t = stats_lut[gnutella_header_get_function(head)];
 
-	i = MIN(h->hops, STATS_FLOWC_COLUMNS-1);
+	i = MIN(gnutella_header_get_hops(head), STATS_FLOWC_COLUMNS - 1);
 	gnet_stats.pkg.flowc_hops[i][t]++;
 	gnet_stats.pkg.flowc_hops[i][MSG_TOTAL]++;
 	gnet_stats.byte.flowc_hops[i][t] += size;
 	gnet_stats.byte.flowc_hops[i][MSG_TOTAL] += size;
 
-	i = MIN(h->ttl, STATS_FLOWC_COLUMNS-1);
+	i = MIN(gnutella_header_get_ttl(head), STATS_FLOWC_COLUMNS - 1);
 
 	g_assert(i != 0);			/* Cannot send a message with TTL=0 */
 
