@@ -1,10 +1,8 @@
 /*
- * $Id: gwcache.c,v 1.10 2004/11/14 21:59:51 rmanfredi Exp $
+ * $Id: gwcache.c 12203 2006-11-03 05:36:01Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
- * Gnutella Web Cache.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Gnutella Web Cache.
+ *
+ * @author Raphael Manfredi
+ * @date 2001-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: gwcache.c,v 1.10 2004/11/14 21:59:51 rmanfredi Exp $");
+RCSID("$Id: gwcache.c 12203 2006-11-03 05:36:01Z cbiere $")
 
 #include "gwcache.h"
 #include "http.h"
@@ -35,6 +43,7 @@ RCSID("$Id: gwcache.c,v 1.10 2004/11/14 21:59:51 rmanfredi Exp $");
 #include "hcache.h"
 #include "version.h"
 #include "settings.h"
+#include "sockets.h"		/* For socket_listen_addr() */
 
 #include "lib/atoms.h"
 #include "lib/cq.h"
@@ -42,6 +51,7 @@ RCSID("$Id: gwcache.c,v 1.10 2004/11/14 21:59:51 rmanfredi Exp $");
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/misc.h"
+#include "lib/tm.h"
 #include "lib/url.h"
 #include "lib/walloc.h"
 
@@ -65,12 +75,17 @@ static gchar gwc_tmp[1024];
  * to prevent insertion of duplicates in our cache.
  */
 
-#define MAX_GWC_URLS	200					/* Max URLs we store */
-#define MAX_GWC_REUSE	1					/* Max amount of uses for one URL */
+#define MAX_GWC_URLS	200		/**< Max URLs we store */
+#define MAX_GWC_REUSE	1		/**< Max amount of uses for one URL */
+
+struct gwc {
+	gchar *url;					/**< atom */
+	time_t stamp;				/**< time of last access */
+} gwc_url[MAX_GWC_URLS];		/**< Holds string atoms */
 
-static gchar *gwc_url[MAX_GWC_URLS];		/* Holds string atoms */
 static gint gwc_url_slot = -1;
 static GHashTable *gwc_known_url = NULL;
+static GHashTable *gwc_failed_url = NULL;
 
 /*
  * Web cache update policy:
@@ -99,18 +114,18 @@ static GHashTable *gwc_known_url = NULL;
  * . All requests include "client=GTKG" and "version" information.
  */
 
-static gchar *current_url = NULL;			/* Cache we're currently using */
-static gint current_reused = 0;				/* Amount of times we reused it */
+static gchar *current_url = NULL;			/**< Cache we're currently using */
+static gint current_reused = 0;				/**< Amount of times we reused it */
 
-#define MAX_URL_LINES	50					/* Max lines on a urlfile req */
-#define MAX_IP_LINES	150					/* Max lines on a hostfile req */
-#define MAX_OK_LINES	3					/* Max lines when expecting OK */
-#define MIN_IP_LINES	5					/* Min lines expected */
-#define MIN_URL_LINES	5					/* Min lines expected */
-#define HOUR_MS			(3600 * 1000)		/* Callout queue time in ms */
-#define URL_RETRY_MS	(20 * 1000)			/* Retry timer for urlfile, in ms */
-#define REFRESH_MS		(8 * HOUR_MS)		/* Refresh every 8 hours */
-#define REUSE_PERIOD	3600				/* Period between GET hostfile */
+#define MAX_URL_LINES	50					/**< Max lines on a urlfile req */
+#define MAX_IP_LINES	150					/**< Max lines on a hostfile req */
+#define MAX_OK_LINES	3					/**< Max lines when expecting OK */
+#define MIN_IP_LINES	5					/**< Min lines expected */
+#define MIN_URL_LINES	5					/**< Min lines expected */
+#define HOUR_MS			(3600 * 1000)		/**< Callout queue time in ms */
+#define URL_RETRY_MS	(20 * 1000)			/**< Retry timer for urlfile, in ms */
+#define REFRESH_MS		(8 * HOUR_MS)		/**< Refresh every 8 hours */
+#define REUSE_PERIOD	3600				/**< Period between GET hostfile */
 
 #define CLIENT_INFO "client=GTKG&version=" GTA_VERSION_NUMBER
 
@@ -127,17 +142,12 @@ static void gwc_get_urls(void);
 static void gwc_update_ip_url(void);
 static void gwc_seed_cache(gchar *cache_url);
 
-/*
+/**
  * The following URLs are there for bootstrapping purposes only.
  */
 
 static const gchar * const boot_url[] = {
-	"http://cache.kicks-ass.net:8000/",
-	"http://galvatron.dyndns.org:59009/gwcache/",
-	"http://gwebcache.bearshare.net/gcache.php",
-	"http://gwebcache2.limewire.com:9000/gwc",
-	"http://kisama.ath.cx:8080/",
-	"http://krill.shacknet.nu:20095/gwc",
+	NULL,
 };
 
 /**
@@ -165,7 +175,7 @@ gwc_add(const gchar *new_url)
 			url = ret;
 		}
 	} else {
-		/* This is superfluous with GLib but this way the above ``ret'' is
+		/* This is superfluous with GLib but this way the above ``ret''
 		 * is local and quitting isn't really appropriate anyway. */
 		g_warning("Out of memory");
 		return;
@@ -175,7 +185,10 @@ gwc_add(const gchar *new_url)
 	 * Don't add duplicates to the cache.
   	 */
 
-	if (g_hash_table_lookup(gwc_known_url, url)) {
+	if (
+		g_hash_table_lookup(gwc_known_url, url) ||
+		g_hash_table_lookup(gwc_failed_url, url)
+	) {
 		G_FREE_NULL(url);
 		return;
 	}
@@ -195,51 +208,70 @@ gwc_add(const gchar *new_url)
 	 * Expire any entry present at the slot we're about to write into.
 	 */
 
-	old_url = gwc_url[gwc_url_slot];
+	old_url = gwc_url[gwc_url_slot].url;
 
 	if (old_url != NULL) {
 		g_assert(g_hash_table_lookup(gwc_known_url, old_url));
 		g_hash_table_remove(gwc_known_url, old_url);
 		atom_str_free(old_url);
+		gwc_url[gwc_url_slot].url = NULL;
 	}
 
 	g_hash_table_insert(gwc_known_url, url_atom, GUINT_TO_POINTER(1));
 
-	gwc_url[gwc_url_slot] = url_atom;
+	gwc_url[gwc_url_slot].url = url_atom;
+	gwc_url[gwc_url_slot].stamp = 0;
 	gwc_file_dirty = TRUE;
 }
 
 /**
- * Pickup a cache randomly from the known set.
+ * Pickup a cache randomly from the known set. If there's no URL used that
+ * has not been used recently, NULL will be returned. The timestamp for
+ * the picked URL is updated automatically.
  *
  * Try to avoid using default bootstrapping URLs if we have more than the
  * minimum set of caches in stock...
+ *
+ * @return a GWebCache URL or NULL on failure.
  */
 static gchar *
 gwc_pick(void)
 {
 	gint count = g_hash_table_size(gwc_known_url);
-	gint idx;
+	gint idx, i;
+	gchar *url = NULL;
+	time_t now = tm_time();
+
+	if (0 == count)
+		return NULL;
 
 	g_assert(count > 0);
 	g_assert(count <= MAX_GWC_URLS);
 	g_assert(count == MAX_GWC_URLS || gwc_url_slot < count);
 
 	idx = random_value(count - 1);
+	for (i = 0; i < count; i++) {
+		time_t stamp = gwc_url[idx].stamp;
 
-	if (gwc_debug)
-		g_warning("picked webcache %s", gwc_url[idx]);
+		if (0 == stamp || delta_time(now, stamp) > 3900) {
+			url = gwc_url[idx].url;
+			gwc_url[idx].stamp = now;
+			break;
+		}
+	}
 
-	return gwc_url[idx];
+	if (gwc_debug && url)
+		g_message("picked webcache \"%s\"", url);
+
+	return url;
 }
 
-/*
- * gwc_store
- *
+/**
  * Store known GWC URLs.
  * They are normally saved in ~/.gtk-gnutella/gwcache.
  */
-static void gwc_store(void)
+static void
+gwc_store(void)
 {
 	FILE *out;
 	gint i;
@@ -264,8 +296,9 @@ static void gwc_store(void)
 		i = 0;
 
 	for (j = 0; j < MAX_GWC_URLS; j++) {
-		gchar *url = gwc_url[i];
-		i = (i == MAX_GWC_URLS - 1) ? 0 : (i + 1);
+		gchar *url = gwc_url[i].url;
+
+		i = (i + 1) % MAX_GWC_URLS;
 		if (url == NULL)
 			continue;
 		fprintf(out, "%s\n", url);
@@ -275,12 +308,11 @@ static void gwc_store(void)
 		gwc_file_dirty = FALSE;
 }
 
-/*
- * gwc_store_if_dirty
- *
+/**
  * Store known GWC URLs if dirty.
  */
-void gwc_store_if_dirty(void)
+void
+gwc_store_if_dirty(void)
 {
 	if (gwc_file_dirty)
 		gwc_store();
@@ -293,18 +325,22 @@ void gwc_store_if_dirty(void)
 static void
 gwc_retrieve(void)
 {
-	const file_path_t fpvec[] = {
-		{ settings_config_dir(), gwc_file },
-		{ PRIVLIB_EXP, gwc_bootfile },
 #ifndef OFFICIAL_BUILD
-		{ PACKAGE_SOURCE_DIR, gwc_bootfile }
+	static file_path_t fp[3];
+#else
+	static file_path_t fp[2];
 #endif
-	};
 	gint line;
 	FILE *in;
 	gchar tmp[1024];
 
-	in = file_config_open_read(gwc_what, fpvec, G_N_ELEMENTS(fpvec));
+	file_path_set(&fp[0], settings_config_dir(), gwc_file);
+	file_path_set(&fp[1], PRIVLIB_EXP, gwc_bootfile);
+#ifndef OFFICIAL_BUILD
+	file_path_set(&fp[2], PACKAGE_SOURCE_DIR, gwc_bootfile);
+#endif
+
+	in = file_config_open_read(gwc_what, fp, G_N_ELEMENTS(fp));
 	if (!in)
 		return;
 
@@ -398,15 +434,18 @@ gwc_init(void)
 	guint i;
 
 	gwc_known_url = g_hash_table_new(g_str_hash, g_str_equal);
-
-	for (i = 0; i < G_N_ELEMENTS(boot_url); i++)
-		gwc_add(boot_url[i]);
+	gwc_failed_url = g_hash_table_new(g_str_hash, g_str_equal);
 
 	gwc_retrieve();
+	if (0 == g_hash_table_size(gwc_known_url)) {
+		for (i = 0; i < G_N_ELEMENTS(boot_url) && boot_url[i]; i++)
+			gwc_add(boot_url[i]);
+	}
 
 	if (ancient_version)
 		return;				/* Older versions must have a harder time */
 
+#if 0
 	/*
 	 * Schedule hourly updates, starting our first in 10 minutes:
 	 * It is hoped that by then, we'll have a stable IP and will know
@@ -418,6 +457,7 @@ gwc_init(void)
 
 	periodic_refresh_ev = cq_insert(callout_queue,
 		REFRESH_MS, gwc_periodic_refresh, NULL);
+#endif
 }
 
 /**
@@ -449,12 +489,13 @@ check_current_url(void)
 }
 
 /**
- * Remove all knowledge about given URL, but do not free its memory.
+ * Removes the URL from the set of known URL, but do not free its memory
+ * and keeps it in the set of failed URLs for the session.
  */
 static void
 forget_url(gchar *url)
 {
-	gchar *url_tmp[MAX_GWC_URLS];			/* Temporary copy */
+	struct gwc url_tmp[MAX_GWC_URLS];			/* Temporary copy */
 	gint count = g_hash_table_size(gwc_known_url);
 	gint i;
 	gint j = 0;
@@ -463,7 +504,7 @@ forget_url(gchar *url)
 	g_assert(count <= MAX_GWC_URLS);
 	g_assert(count == MAX_GWC_URLS || gwc_url_slot < count);
 	g_assert(gwc_url_slot >= 0);
-	g_assert(sizeof(url_tmp) == sizeof(gwc_url));
+	STATIC_ASSERT(sizeof(url_tmp) == sizeof(gwc_url));
 
 	if (gwc_debug)
 		g_warning("forgetting GWC URL \"%s\"", url);
@@ -482,6 +523,7 @@ forget_url(gchar *url)
 			g_warning("URL was already gone from GWC");
 		return;
 	}
+	g_hash_table_insert(gwc_failed_url, url, GUINT_TO_POINTER(1));
 
 	/*
 	 * Because we have a round-robin buffer, removing something in the
@@ -499,10 +541,8 @@ forget_url(gchar *url)
 
 	if (count == MAX_GWC_URLS) {		/* Buffer was full */
 		for (i = gwc_url_slot;;) {
-			if (gwc_url[i] != url)		/* Atoms: we can compare addresses */
+			if (gwc_url[i].url != url)	/* Atoms: we can compare addresses */
 				url_tmp[j++] = gwc_url[i];
-			else
-				atom_str_free(url);		/* Or gwc_url[i], same pointer */
 			i++;
 			if (i == MAX_GWC_URLS)
 				i = 0;
@@ -511,35 +551,18 @@ forget_url(gchar *url)
 		}
 	} else {							/* Buffer was partially filled */
 		for (i = 0; i <= gwc_url_slot; i++) {
-			if (gwc_url[i] != url)		/* Atoms: we can compare addresses */
+			if (gwc_url[i].url != url)	/* Atoms: we can compare addresses */
 				url_tmp[j++] = gwc_url[i];
-			else
-				atom_str_free(url);		/* Or gwc_url[i], same pointer */
 		}
 	}
 
 	count--;							/* New amount of data in cache */
 	gwc_url_slot = j - 1;				/* Last position we filled */
-	memcpy(gwc_url, url_tmp, sizeof(gwc_url));
-
-	g_assert(gwc_url_slot >= 0 && gwc_url_slot < MAX_GWC_URLS);
-	g_assert(gwc_url_slot == count - 1);
-
-	/*
-	 * If we have less that the amount of bootstrapping URLs, fill
-	 * the cache with those.
-	 */
-
-	j = G_N_ELEMENTS(boot_url);
-
-	for (i = 0; i < j && count < j; i++) {
-		if (0 != strcmp(url, boot_url[i])) {
-			gwc_add(boot_url[i]);		/* Only if not the removed URL */
-			count++;
-		}
-	}
-
 	gwc_url_slot = MAX(0, gwc_url_slot);	/* If we removed ALL entries */
+	g_assert(gwc_url_slot == MAX(0, count - 1));
+	memcpy(gwc_url, url_tmp, sizeof(gwc_url));
+	g_assert(gwc_url_slot >= 0 && gwc_url_slot < MAX_GWC_URLS);
+
 	gwc_file_dirty = TRUE;
 }
 
@@ -560,6 +583,17 @@ clear_current_url(gboolean discard)
 	current_url = NULL;
 }
 
+/**
+ * Frees the atom used as hash table key 
+ */
+static void
+free_failed_url(gpointer key, gpointer unused_value, gpointer unused_udata)
+{
+	(void) unused_value;
+	(void) unused_udata;
+	atom_str_free(key);
+}
+
 /**
  * Called when servent shuts down.
  */
@@ -577,9 +611,11 @@ gwc_close(void)
 
 	gwc_store();
 	g_hash_table_destroy(gwc_known_url);
+	g_hash_table_foreach(gwc_failed_url, free_failed_url, NULL);
+	g_hash_table_destroy(gwc_failed_url);
 
 	for (i = 0; i < MAX_GWC_URLS; i++) {
-		gchar *url = gwc_url[i];
+		gchar *url = gwc_url[i].url;
 		if (url == NULL)
 			continue;
 		atom_str_free(url);
@@ -593,15 +629,15 @@ gwc_close(void)
  ***/
 
 struct parse_context {
-	getline_t *getline;			/* Used to hold partially read line */
-	gpointer handle;			/* Request handle */
-	gint maxlines;				/* Maximum number of lines we want to process */
-	gint lines;					/* Amount of lines so far */
-	gint processed;				/* User callback can count retained lines */
+	getline_t *getline;			/**< Used to hold partially read line */
+	gpointer handle;			/**< Request handle */
+	gint maxlines;				/**< Maximum number of lines we want to process */
+	gint lines;					/**< Amount of lines so far */
+	gint processed;				/**< User callback can count retained lines */
 };
 
 typedef gboolean (parse_dispatch_t)
-	(struct parse_context *c, gchar *buf, gint len);
+	(struct parse_context *c, const gchar *buf, size_t len);
 typedef void (parse_eof_t)(struct parse_context *c);
 
 /**
@@ -619,8 +655,8 @@ parse_context_free(gpointer obj)
 /**
  * Allocate new parsing context for handle and record it.
  *
- * `handle' is the asynchronous HTTP request handle.
- * `maxlines' is the max number of lines we want to parse.
+ * @param `handle'		the asynchronous HTTP request handle.
+ * @param `maxlines'	the max number of lines we want to parse.
  */
 static void
 parse_context_set(gpointer handle, gint maxlines)
@@ -644,23 +680,19 @@ parse_context_set(gpointer handle, gint maxlines)
  * to finalize parsing.
  */
 static void
-parse_dispatch_lines(gpointer handle, gchar *buf, gint len,
+parse_dispatch_lines(gpointer handle, const gchar *buf, size_t len,
 		parse_dispatch_t cb, parse_eof_t eof)
 {
 	struct parse_context *ctx;
-	getline_t *getline;
-	gchar *p = buf;
-	gint remain = len;
-	gint parsed;
-	gint linelen;
-	gchar *linep;
+	const gchar *p = buf;
+	size_t remain = len;
 
 	/*
 	 * Retrieve parsing context, stored as an opaque attribute in the
 	 * asynchronous HTTP request handle.
 	 */
-	
-	ctx = (struct parse_context *) http_async_get_opaque(handle);
+
+	ctx = http_async_get_opaque(handle);
 
 	g_assert(ctx->handle == handle);	/* Make sure it's the right context */
 
@@ -674,10 +706,13 @@ parse_dispatch_lines(gpointer handle, gchar *buf, gint len,
 	 * Read a line at a time.
 	 */
 
-	getline = ctx->getline;
-
 	for (;;) {
-		switch (getline_read(getline, p, remain, &parsed)) {
+		gchar *line;
+		gboolean error;
+		size_t line_len;
+		size_t parsed;
+
+		switch (getline_read(ctx->getline, p, remain, &parsed)) {
 		case READ_OVERFLOW:
 			http_async_cancel(handle);
 			return;
@@ -686,7 +721,6 @@ parse_dispatch_lines(gpointer handle, gchar *buf, gint len,
 			remain -= parsed;
 			break;
 		case READ_MORE:			/* ok, but needs more data */
-		default:
 			g_assert(parsed == remain);
 			return;
 		}
@@ -695,11 +729,15 @@ parse_dispatch_lines(gpointer handle, gchar *buf, gint len,
 		 * We come here everytime we get a full line.
 		 */
 
-		linep = getline_str(getline);
-		linelen = str_chomp(linep, getline_length(getline));
+		line = g_strdup(getline_str(ctx->getline));
+		line_len = getline_length(ctx->getline);
+		line_len = str_chomp(line, line_len);
 
-		if (!(*cb)(ctx, linep, linelen)) {
-			clear_current_url(FALSE);	/* An ERROR was reported */
+		error = !(*cb)(ctx, line, line_len); /* An ERROR was reported */
+		G_FREE_NULL(line);
+
+		if (error) {
+	   		clear_current_url(FALSE);
 			return;
 		}
 
@@ -717,7 +755,7 @@ parse_dispatch_lines(gpointer handle, gchar *buf, gint len,
 			return;
 		}
 
-		getline_reset(getline);
+		getline_reset(ctx->getline);
 	}
 }
 
@@ -731,12 +769,12 @@ parse_dispatch_lines(gpointer handle, gchar *buf, gint len,
  * @return FALSE to stop processing of any remaining data.
  */
 static gboolean
-gwc_url_line(struct parse_context *ctx, gchar *buf, gint len)
+gwc_url_line(struct parse_context *ctx, const gchar *buf, size_t len)
 {
 	if (gwc_debug > 3)
-		printf("GWC URL line (%d bytes): %s\n", len, buf);
+		g_message("GWC URL line (%lu bytes): %s", (gulong) len, buf);
 
-	if (0 == strncmp(buf, "ERROR", 5)) {
+	if (is_strprefix(buf, "ERROR")) {
 		g_warning("GWC cache \"%s\" returned %s",
 			http_async_info(ctx->handle, NULL, NULL, NULL, NULL), buf);
 		http_async_cancel(ctx->handle);
@@ -762,7 +800,7 @@ static void
 gwc_url_eof(struct parse_context *ctx)
 {
 	if (gwc_debug > 2)
-		printf("GWC URL all done (%d/%d lines processed)\n",
+		g_message("GWC URL all done (%d/%d lines processed)",
 			ctx->processed, ctx->lines);
 
 	if (ctx->processed < MIN_URL_LINES) {
@@ -795,7 +833,7 @@ gwc_url_data_ind(gpointer handle, gchar *data, gint len)
 static void
 gwc_url_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
-	http_async_log_error(handle, type, v);
+	http_async_log_error_dbg(handle, type, v, gwc_debug);
 
 	clear_current_url(TRUE);		/* This webcache is not good */
 
@@ -829,7 +867,7 @@ gwc_get_urls(void)
 		"%s?urlfile=1&%s", current_url, CLIENT_INFO);
 
 	if (gwc_debug > 2)
-		printf("GWC URL request: %s\n", gwc_tmp);
+		g_message("GWC URL request: %s", gwc_tmp);
 
 	/*
 	 * Launch the asynchronous request and attach parsing information.
@@ -869,12 +907,12 @@ gwc_is_waiting(void)
  * @return FALSE to stop processing of any remaining data.
  */
 static gboolean
-gwc_host_line(struct parse_context *ctx, gchar *buf, gint len)
+gwc_host_line(struct parse_context *ctx, const gchar *buf, size_t len)
 {
-	if (gwc_debug > 3)
-		printf("GWC host line (%d bytes): %s\n", len, buf);
+	if (gwc_debug > 3 || bootstrap_debug > 2)
+		g_message("BOOT GWC host line (%lu bytes): %s", (gulong) len, buf);
 
-	if (0 == strncmp(buf, "ERROR", 5)) {
+	if (is_strprefix(buf, "ERROR")) {
 		g_warning("GWC cache \"%s\" returned %s",
 			http_async_info(ctx->handle, NULL, NULL, NULL, NULL), buf);
 		http_async_cancel(ctx->handle);
@@ -882,12 +920,17 @@ gwc_host_line(struct parse_context *ctx, gchar *buf, gint len)
 	}
 
 	if (len) {
-		guint32 ip;
+		host_addr_t addr;
 		guint16 port;
 
-		if (gchar_to_ip_port(buf, &ip, &port)) {
+		if (string_to_host_addr_port(buf, NULL, &addr, &port)) {
 			ctx->processed++;
-			hcache_add_caught(HOST_ULTRA, ip, port, "GWC");
+			hcache_add_caught(HOST_ULTRA, addr, port, "GWC");
+
+			if (bootstrap_debug > 1)
+				g_message("BOOT collected %s from GWC %s",
+					host_addr_to_string(addr),
+					http_async_info(ctx->handle, NULL, NULL, NULL, NULL));
 		}
 	}
 
@@ -901,22 +944,23 @@ static void
 gwc_host_eof(struct parse_context *ctx)
 {
 	if (gwc_debug > 2)
-		printf("GWC host all done (%d/%d lines processed)\n",
+		g_message("GWC host all done (%d/%d lines processed)",
 			ctx->processed, ctx->lines);
 
 	/*
 	 * Provide GUI feedback.
 	 */
 
-	if (ctx->processed == 1)
-		gm_snprintf(gwc_tmp, sizeof(gwc_tmp), _("Got %d host from %s"),
-			ctx->processed, current_url);
-	else
-		gm_snprintf(gwc_tmp, sizeof(gwc_tmp), _("Got %d hosts from %s"),
-			ctx->processed, current_url);
+	gm_snprintf(gwc_tmp, sizeof(gwc_tmp),
+		NG_("Got %d host from %s", "Got %d hosts from %s", ctx->processed),
+		ctx->processed, current_url);
 
 	gcu_statusbar_message(gwc_tmp);
 
+	if (bootstrap_debug)
+		g_message("BOOT got %d host%s from GWC %s",
+			ctx->processed, ctx->processed == 1 ? "" : "s", current_url);
+
 	/*
 	 * If we did not get enough addresses, try to feed the cache with ours.
 	 */
@@ -944,7 +988,7 @@ gwc_host_data_ind(gpointer handle, gchar *data, gint len)
 static void
 gwc_host_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
-	http_async_log_error(handle, type, v);
+	http_async_log_error_dbg(handle, type, v, MAX(gwc_debug, bootstrap_debug));
 	hostfile_running = FALSE;
 
 	clear_current_url(TRUE);			/* This webcache is not good */
@@ -958,7 +1002,7 @@ gwc_get_hosts(void)
 {
 	gpointer handle;
 	static time_t last_called = 0;
-	time_t now = time(NULL);
+	time_t now = tm_time();
 
 	/*
 	 * Make sure we don't probe more than one webcache at a time.
@@ -995,10 +1039,13 @@ gwc_get_hosts(void)
 	 */
 
 	gm_snprintf(gwc_tmp, sizeof(gwc_tmp),
-		"_(Connecting to web cache %s)", current_url);
+		_("Connecting to web cache %s"), current_url);
 
 	gcu_statusbar_message(gwc_tmp);
 
+	if (bootstrap_debug)
+		g_message("BOOT connecting to web cache %s", current_url);
+
 	/*
 	 * Launch the asynchronous request and attach parsing information.
 	 */
@@ -1007,7 +1054,7 @@ gwc_get_hosts(void)
 		"%s?hostfile=1&%s", current_url, CLIENT_INFO);
 
 	if (gwc_debug > 2)
-		printf("GWC host request: %s\n", gwc_tmp);
+		g_message("GWC host request: %s", gwc_tmp);
 
 	handle = http_async_get(gwc_tmp,
 		NULL, gwc_host_data_ind, gwc_host_error_ind);
@@ -1024,7 +1071,7 @@ gwc_get_hosts(void)
 }
 
 /***
- *** GET ...?ip=....&url=....
+ *** GET ...?ip=....
  ***/
 
 /**
@@ -1033,19 +1080,18 @@ gwc_get_hosts(void)
  * @return FALSE to stop processing of any remaining data.
  */
 static gboolean
-gwc_update_line(struct parse_context *ctx, gchar *buf, gint len)
+gwc_update_line(struct parse_context *ctx, const gchar *buf, size_t len)
 {
 	if (gwc_debug > 3)
-		printf("GWC update line (%d bytes): %s\n", len, buf);
+		g_message("GWC update line (%lu bytes): %s", (gulong) len, buf);
 
-	if (0 == strncmp(buf, "OK", 2)) {
+	if (is_strprefix(buf, "OK")) {
 		if (gwc_debug > 2)
-			printf("GWC update OK for \"%s\"\n",
+			g_message("GWC update OK for \"%s\"",
 				http_async_info(ctx->handle, NULL, NULL, NULL, NULL));
 		http_async_close(ctx->handle);		/* OK, don't read more */
 		return FALSE;
-	}
-	else if (0 == strncmp(buf, "ERROR", 5)) {
+	} else if (is_strprefix(buf, "ERROR")) {
 		g_warning("GWC cache \"%s\" returned %s",
 			http_async_info(ctx->handle, NULL, NULL, NULL, NULL), buf);
 		http_async_cancel(ctx->handle);
@@ -1070,7 +1116,7 @@ gwc_update_data_ind(gpointer handle, gchar *data, gint len)
 static void
 gwc_update_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
-	http_async_log_error(handle, type, v);
+	http_async_log_error_dbg(handle, type, v, gwc_debug);
 	clear_current_url(TRUE);			/* This webcache is not good */
 }
 
@@ -1081,44 +1127,12 @@ static void
 gwc_update_this(gchar *cache_url)
 {
 	gpointer handle;
-	gchar *url = NULL;
-	gboolean found_alternate = FALSE;
-	gboolean has_data = FALSE;
-	gint rw;
-	gint i;
+	size_t rw;
 
 	g_assert(cache_url != NULL);
 
-	rw = gm_snprintf(gwc_tmp, sizeof(gwc_tmp), "%s?", cache_url);
-
-	/*
-	 * Choose another URL randomly.
-	 */
-
-	for (i = 0; i < MAX_GWC_URLS; i++) {
-		url = gwc_pick();
-		if (0 != strcmp(url, cache_url)) {
-			found_alternate = TRUE;
-			break;
-		}
-	}
-
-	/*
-	 * If we found an URL different from the cache we're going to update,
-	 * publish the escaped URL in the request.
-	 */
-
-	if (found_alternate) {
-		gchar *escaped_url = url_escape_query(url);		/* For query string */
-
-		rw += gm_snprintf(&gwc_tmp[rw], sizeof(gwc_tmp)-rw,
-			"url=%s&", escaped_url);
-
-		if (escaped_url != url)
-			G_FREE_NULL(escaped_url);
-
-		has_data = TRUE;		/* We have something to submit */
-	}
+	rw = concat_strings(gwc_tmp, sizeof gwc_tmp, cache_url, "?", (void *) 0);
+	g_return_if_fail(rw < sizeof gwc_tmp);
 
 	/*
 	 * Send our IP:port information if we're connectible, we are
@@ -1128,20 +1142,18 @@ gwc_update_this(gchar *cache_url)
 	if (
 		!is_firewalled &&
 		current_peermode == NODE_P_ULTRA &&
-		host_is_valid(listen_ip(), listen_port)
+		host_is_valid(listen_addr(), socket_listen_port())
 	) {
-		rw += gm_snprintf(&gwc_tmp[rw], sizeof(gwc_tmp)-rw,
-			"ip=%s&", ip_port_to_gchar(listen_ip(), listen_port));
-		has_data = TRUE;
-	}
+		rw += gm_snprintf(&gwc_tmp[rw], sizeof(gwc_tmp)-rw, "ip=%s&",
+			host_addr_port_to_string(listen_addr(), socket_listen_port()));
+	} else {
 
-	/*
-	 * If we don't have anything to submit, we're done.
-	 */
+		/*
+		 * If we don't have anything to submit, we're done.
+		 */
 
-	if (!has_data) {
 		if (gwc_debug > 2)
-			printf("GWC update has nothing to send\n");
+			g_message("GWC update has nothing to send");
 		return;
 	}
 
@@ -1152,7 +1164,7 @@ gwc_update_this(gchar *cache_url)
 	g_strlcpy(&gwc_tmp[rw], CLIENT_INFO, sizeof(gwc_tmp)-rw);
 
 	if (gwc_debug > 2)
-		printf("GWC update request: %s\n", gwc_tmp);
+		g_message("GWC update request: %s", gwc_tmp);
 
 	/*
 	 * Launch the asynchronous request and attach parsing information.
@@ -1173,7 +1185,8 @@ gwc_update_this(gchar *cache_url)
 	 * Provide GUI feedback in the statusbar.
 	 */
 
-	gm_snprintf(gwc_tmp, sizeof(gwc_tmp), _("Updated web cache %s"), cache_url);
+	concat_strings(gwc_tmp, sizeof(gwc_tmp),
+		_("Updated web cache "), cache_url, (void *) 0);
 	gcu_statusbar_message(gwc_tmp);
 
 	parse_context_set(handle, MAX_OK_LINES);
@@ -1192,7 +1205,7 @@ gwc_seed_cache(gchar *cache_url)
 		return;
 
 	if (gwc_debug > 2)
-		printf("GWC seeding cache \"%s\"\n", cache_url);
+		g_message("GWC seeding cache \"%s\"", cache_url);
 
 	gwc_update_this(cache_url);
 }
@@ -1203,7 +1216,7 @@ gwc_seed_cache(gchar *cache_url)
 static void
 gwc_update_ip_url(void)
 {
-	if (!check_current_url())
+	if (current_peermode == NODE_P_LEAF || !check_current_url())
 		return;
 
 	gwc_update_this(current_url);
