@@ -1,5 +1,5 @@
 /*
- * $Id: http.c,v 1.46 2005/12/17 10:33:09 cbiere Exp $
+ * $Id: http.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: http.c,v 1.46 2005/12/17 10:33:09 cbiere Exp $");
+RCSID("$Id: http.c 12242 2006-11-06 20:41:55Z cbiere $")
 
 #include "nodes.h"
 #include "http.h"
@@ -151,14 +151,14 @@ http_send_status(
 	if (saturated && code >= 300) {
 		xlive[0] = '\0';
 		version = version_short_string;
-		token = tok_short_version();
+		token = socket_omit_token(s) ? NULL : tok_short_version();
 		header_size = 512;
 		cb_flags |= HTTP_CBF_SMALL_REPLY;
 	} else {
 		gm_snprintf(xlive, sizeof(xlive)-1,
 			"X-Live-Since: %s\r\n", start_rfc822_date);
 		version = version_string;
-		token = tok_version();
+		token = socket_omit_token(s) ? NULL : tok_version();
 	}
 
 	for (i = 0; i < hevcnt; i++) {
@@ -182,10 +182,14 @@ http_send_status(
 		"Server: %s\r\n"
 		"Date: %s\r\n"
 		"%s"			/* Connection */
-		"X-Token: %s\r\n"
+		"%s%s%s"		/* X-Token (optional) */
 		"%s"			/* X-Live-Since */
 		"%s",			/* Content length */
-		code, status_msg, version, date, conn_close, token, xlive, no_content);
+		code, status_msg, version, date, conn_close,
+		token ? "X-Token: " : "",
+		token ? token : "",
+		token ? "\r\n" : "",
+		xlive, no_content);
 
 	mrw = rw;		/* Minimal header length */
 
@@ -458,10 +462,10 @@ http_status_parse(const gchar *line,
  */
 gboolean
 http_extract_version(
-	const gchar *request, gint len, guint *major, guint *minor)
+	const gchar *request, size_t len, guint *major, guint *minor)
 {
 	const gchar *p;
-	gint limit, i;
+	size_t limit, i;
 
 	/*
 	 * The smallest request would be "X / HTTP/1.0".
@@ -470,7 +474,7 @@ http_extract_version(
 	limit = sizeof("X / HTTP/1.0") - 1;
 
 	if (http_debug > 4)
-		printf("HTTP req (%d bytes): %s\n", len, request);
+		printf("HTTP req (%lu bytes): %s\n", (gulong) len, request);
 
 	if (len < limit)
 		return FALSE;
@@ -486,7 +490,7 @@ http_extract_version(
 	}
 
 	if (http_debug > 4)
-		printf("HTTP i = %d, limit = %d\n", i, limit);
+		g_message("HTTP i = %lu, limit = %lu", (gulong) i, (gulong) limit);
 
 	if (i == limit)
 		return FALSE;		/* Reached our limit without finding a space */
@@ -503,7 +507,8 @@ http_extract_version(
 		0 != parse_major_minor(p, NULL, major, minor)
 	) {
 		if (http_debug > 1)
-			printf("HTTP req (%d bytes): no protocol tag: %s\n", len, request);
+			printf("HTTP req (%lu bytes): no protocol tag: %s\n",
+				(gulong) len, request);
 		return FALSE;
 	}
 
@@ -522,7 +527,7 @@ http_extract_version(
  *** HTTP URL parsing.
  ***/
 
-static gchar *parse_errstr[] = {
+static const gchar * const parse_errstr[] = {
 	"OK",								/**< HTTP_URL_OK */
 	"Not an http URI",					/**< HTTP_URL_NOT_HTTP */
 	"More than one <user>:<password>",	/**< HTTP_URL_MULTIPLE_CREDENTIALS */
@@ -1570,8 +1575,7 @@ http_async_free_recursive(struct http_async *ha)
 		io_free(ha->io_opaque);
 	if (ha->bio)
 		bsched_source_remove(ha->bio);
-	if (ha->socket)
-		socket_free(ha->socket);
+	socket_free_null(&ha->socket);
 	if (ha->user_free)
 		(*ha->user_free)(ha->user_opaque);
 	if (ha->delayed)
@@ -2085,8 +2089,7 @@ http_redirect(struct http_async *ha, gchar *url)
 
 	g_assert(ha->socket);
 
-	socket_free(ha->socket);
-	ha->socket = NULL;
+	socket_free_null(&ha->socket);
 	http_async_newstate(ha, HTTP_AS_REDIRECTED);
 
 	/*
@@ -2128,7 +2131,7 @@ http_got_data(struct http_async *ha, gboolean eof)
 
 	if (s->pos > 0) {
 		ha->last_update = tm_time();
-		(*ha->data_ind)(ha, s->buffer, s->pos);
+		(*ha->data_ind)(ha, s->buf, s->pos);
 		if (ha->flags & HA_F_FREED)		/* Callback decided to cancel/close */
 			return;
 		s->pos = 0;
@@ -2151,7 +2154,7 @@ http_data_read(gpointer data, gint unused_source, inputevt_cond_t cond)
 {
 	struct http_async *ha = (struct http_async *) data;
 	struct gnutella_socket *s = ha->socket;
-	gint r;
+	ssize_t r;
 
 	(void) unused_source;
 	g_assert(ha->magic == HTTP_ASYNC_MAGIC);
@@ -2162,23 +2165,23 @@ http_data_read(gpointer data, gint unused_source, inputevt_cond_t cond)
 		return;
 	}
 
-	g_assert((gint) s->pos >= 0 && s->pos <= sizeof(s->buffer));
+	g_assert((gint) s->pos >= 0 && s->pos <= s->buf_size);
 
-	if (s->pos == sizeof(s->buffer)) {
+	if (s->pos == s->buf_size) {
 		http_async_error(ha, HTTP_ASYNC_IO_ERROR);
 		return;
 	}
 
-	r = bio_read(ha->bio, s->buffer + s->pos, sizeof(s->buffer) - s->pos);
+	r = bio_read(ha->bio, &s->buf[s->pos], s->buf_size - s->pos);
 	if (r == 0) {
 		socket_eof(s);
 		http_got_data(ha, TRUE);			/* Signals EOF */
 		return;
-	} else if (r < 0 && errno == VAL_EAGAIN)
-		return;
-	else if (r < 0) {
-		socket_eof(s);
-		http_async_syserr(ha, errno);
+	} else if ((ssize_t) -1 == r) {
+		if (!is_temporary_error(errno)) {
+			socket_eof(s);
+			http_async_syserr(ha, errno);
+		}
 		return;
 	}
 
@@ -2194,7 +2197,7 @@ static void
 http_got_header(struct http_async *ha, header_t *header)
 {
 	struct gnutella_socket *s = ha->socket;
-	gchar *status = getline_str(s->getline);
+	const gchar *status = getline_str(s->getline);
 	gint ack_code;
 	const gchar *ack_message = "";
 	gchar *buf;
