@@ -1,5 +1,5 @@
 /*
- * $Id: ioheader.c,v 1.12 2005/09/17 22:30:28 rmanfredi Exp $
+ * $Id: ioheader.c 12203 2006-11-03 05:36:01Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ioheader.c,v 1.12 2005/09/17 22:30:28 rmanfredi Exp $");
+RCSID("$Id: ioheader.c 12203 2006-11-03 05:36:01Z cbiere $")
 
 #include "ioheader.h"
 #include "sockets.h"
@@ -58,16 +58,16 @@ RCSID("$Id: ioheader.c,v 1.12 2005/09/17 22:30:28 rmanfredi Exp $");
  * by the header parsing I/O callbacks.
  */
 struct io_header {
-	gpointer resource;					/**< Resource for which we're parsing */
-	gpointer *io_opaque;				/**< Where we're referenced in resource */
-	struct gnutella_socket *socket;		/**< Socket on which we're reading */
-	bsched_t *bs;						/**< Bandwidth scheduler to use */
+	gpointer resource;				/**< Resource for which we're parsing */
+	gpointer *io_opaque;			/**< Where we're referenced in resource */
+	struct gnutella_socket *socket;	/**< Socket on which we're reading */
+	bsched_t *bs;					/**< Bandwidth scheduler to use */
 	header_t *header;
 	getline_t *getline;
-	const struct io_error *error;		/**< Error callbacks */
-	io_done_cb_t process_header;		/**< Called when all headers are read */
+	const struct io_error *error;	/**< Error callbacks */
+	io_done_cb_t process_header;	/**< Called when all headers are read */
 	io_start_cb_t header_read_start;	/**< Called when reading first byte */
-	GString *text;						/**< Full header text */
+	GString *text;					/**< Full header text */
 	gint flags;
 };
 
@@ -161,9 +161,8 @@ static void
 io_header_parse(struct io_header *ih)
 {
 	struct gnutella_socket *s = ih->socket;
-	getline_t *getline = ih->getline;
 	header_t *header = ih->header;
-	gint parsed;
+	size_t parsed;
 	gint error;
 
 	/*
@@ -172,11 +171,11 @@ io_header_parse(struct io_header *ih)
 	 */
 
 nextline:
-	switch (getline_read(getline, s->buffer, s->pos, &parsed)) {
+	switch (getline_read(ih->getline, s->buf, s->pos, &parsed)) {
 	case READ_OVERFLOW:
 		g_warning("io_header_parse: line too long, disconnecting from %s",
 			host_addr_to_string(s->addr));
-		dump_hex(stderr, "Leading Data", s->buffer, MIN(s->pos, 256));
+		dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
 		fprintf(stderr, "------ Header Dump:\n");
 		header_dump(header, stderr);
 		fprintf(stderr, "------\n");
@@ -184,13 +183,12 @@ nextline:
 		return;
 		/* NOTREACHED */
 	case READ_DONE:
-		if (s->pos != (size_t) parsed)
-			memmove(s->buffer, s->buffer + parsed, s->pos - parsed);
+		if (s->pos != parsed)
+			memmove(s->buf, &s->buf[parsed], s->pos - parsed);
 		s->pos -= parsed;
 		break;
 	case READ_MORE:		/* ok, but needs more data */
-	default:
-		g_assert((size_t) parsed == s->pos);
+		g_assert(parsed == s->pos);
 		s->pos = 0;
 		return;
 	}
@@ -210,7 +208,7 @@ nextline:
 
 		g_assert(ih->text != NULL);
 		/* No g_string_append_len() in glib 1.2, use g_string_append() --RAM */
-		g_string_append(ih->text, getline_str(getline));
+		g_string_append(ih->text, getline_str(ih->getline));
 		g_string_append_c(ih->text, '\n');
 	}
 
@@ -221,11 +219,10 @@ nextline:
 		 * the following header lines.
 		 */
 
-		g_assert(s->getline == 0);
-		s->getline = getline_make(getline_length(getline) + 1);
-
-		getline_copy(getline, s->getline);
-		getline_reset(getline);
+		g_assert(NULL == s->getline);
+		s->getline = getline_make(getline_length(ih->getline) + 1);
+		getline_copy(ih->getline, s->getline);
+		getline_reset(ih->getline);
 		ih->flags &= ~IO_SAVE_FIRST;
 		goto nextline;
 	}
@@ -245,11 +242,11 @@ nextline:
 	}
 
 	error = header_append(header,
-		getline_str(getline), getline_length(getline));
-
+		getline_str(ih->getline), getline_length(ih->getline));
+	
 	switch (error) {
 	case HEAD_OK:
-		getline_reset(getline);
+		getline_reset(ih->getline);
 		goto nextline;			/* Go process other lines we may have read */
 		/* NOTREACHED */
 	case HEAD_EOH:				/* We reached the end of the header */
@@ -265,8 +262,8 @@ nextline:
 		fprintf(stderr, "------ Header Dump:\n");
 		header_dump(header, stderr);
 		fprintf(stderr, "------\n");
-		dump_hex(stderr, "Header Line", getline_str(getline),
-			MIN(getline_length(getline), 128));
+		dump_hex(stderr, "Header Line", getline_str(ih->getline),
+			MIN(getline_length(ih->getline), 128));
 		(*ih->error->header_error)(ih->resource, error);
 		return;
 		/* NOTREACHED */
@@ -275,12 +272,12 @@ nextline:
 			g_warning("io_header_parse: %s, from %s",
 				header_strerror(error), host_addr_to_string(s->addr));
 			dump_hex(stderr, "Header Line",
-				getline_str(getline), getline_length(getline));
+				getline_str(ih->getline), getline_length(ih->getline));
 			fprintf(stderr, "------ Header Dump (so far):\n");
 			header_dump(header, stderr);
 			fprintf(stderr, "------\n");
 		}
-		getline_reset(getline);
+		getline_reset(ih->getline);
 		goto nextline;			/* Go process other lines we may have read */
 	}
 
@@ -292,7 +289,7 @@ nextline:
         if (dbg) {
             g_message("remote %s sent extra bytes after headers",
                 host_addr_to_string(s->addr));
-            dump_hex(stderr, "Extra Data", s->buffer, MIN(s->pos, 512));
+            dump_hex(stderr, "Extra Data", s->buf, MIN(s->pos, 512));
         }
 		(*ih->error->header_extra_data)(ih->resource);
 		return;
@@ -372,12 +369,12 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	 * never happen.
 	 */
 
-	g_assert(sizeof(s->buffer) >= s->pos);
-	count = sizeof(s->buffer) - s->pos;
+	g_assert(s->buf_size >= s->pos);
+	count = s->buf_size - s->pos;
 	if (count < 1) {
 		g_warning("ih_header_read: incoming buffer full, "
 			"disconnecting from %s", host_addr_to_string(s->addr));
-		dump_hex(stderr, "Leading Data", s->buffer, MIN(s->pos, 256));
+		dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
 		(*ih->error->input_buffer_full)(ih->resource);
 		return;
 	}
@@ -388,13 +385,13 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	 * errors to our client.
 	 */
 
-	r = bws_read(ih->bs, &s->wio, &s->buffer[s->pos], count);
+	r = bws_read(ih->bs, &s->wio, &s->buf[s->pos], count);
 	if (r == 0) {
 		socket_eof(s);
 		(*ih->error->header_read_eof)(ih->resource);
 		return;
 	} else if ((ssize_t) -1 == r) {
-	  	if (errno != VAL_EAGAIN) {
+	  	if (!is_temporary_error(errno)) {
 			socket_eof(s);
 			(*ih->error->header_read_error)(ih->resource, errno);
 		}
