@@ -1,5 +1,5 @@
 /*
- * $Id: mq.c,v 1.12 2005/08/07 22:32:09 cbiere Exp $
+ * $Id: mq.c 12265 2006-11-08 06:43:13Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: mq.c,v 1.12 2005/08/07 22:32:09 cbiere Exp $");
+RCSID("$Id: mq.c 12265 2006-11-08 06:43:13Z cbiere $")
 
 #include "nodes.h"
 #include "mq.h"
@@ -44,6 +44,7 @@ RCSID("$Id: mq.c,v 1.12 2005/08/07 22:32:09 cbiere Exp $");
 #include "gnet_stats.h"
 
 #include "lib/walloc.h"
+#include "lib/glib-missing.h"	/* For gm_snprintf() */
 #include "lib/cq.h"
 
 #include "if/gnet_property_priv.h"
@@ -56,6 +57,158 @@ static gboolean make_room_header(
 	mqueue_t *q, gchar *header, guint prio, gint needed, gint *offset);
 static void mq_swift_timer(cqueue_t *cq, gpointer obj);
 
+/**
+ * Compute readable queue information.
+ */
+static const gchar *
+mq_info(const mqueue_t *q)
+{
+	static gchar buf[160];
+
+	if (q->magic != MQ_MAGIC) {
+		gm_snprintf(buf, sizeof(buf),
+			"queue 0x%lx INVALID (bad magic)", (gulong) q);
+	} else {
+		gboolean udp = NODE_IS_UDP(q->node);
+
+		gm_snprintf(buf, sizeof(buf),
+			"queue 0x%lx [%s %s node %s%s%s%s%s] (%d item%s, %d byte%s)",
+			(gulong) q, udp ? "UDP" : "TCP",
+			NODE_IS_ULTRA(q->node) ? "ultra" :
+			udp ? "remote" : "leaf", node_addr(q->node),
+			(q->flags & MQ_FLOWC) ? " FLOWC" : "",
+			(q->flags & MQ_DISCARD) ? " DISCARD" : "",
+			(q->flags & MQ_SWIFT) ? " SWIFT" : "",
+			(q->flags & MQ_WARNZONE) ? " WARNZONE" : "",
+			q->count, q->count == 1 ? "" : "s",
+			q->size, q->size == 1 ? "" : "s"
+		);
+	}
+
+	return buf;
+}
+
+#ifdef MQ_DEBUG
+/*
+ * This hashtable tracks the queue owning a given glist linkable.
+ */
+static GHashTable *qown = NULL;
+
+/**
+ * Add linkable into queue
+ */
+static void mq_add_linkable(mqueue_t *q, GList *l)
+{
+	mqueue_t *owner;
+
+	g_assert(q->magic == MQ_MAGIC);
+	g_assert(l != NULL);
+	g_assert(l->data != NULL);
+	
+	if (qown == NULL)
+		qown = g_hash_table_new(NULL, NULL);
+
+	owner = g_hash_table_lookup(qown, l);
+	if (owner) {
+		g_warning("BUG: added linkable 0x%lx already owned by %s%s",
+			(gulong) l, owner == q ? "ourselves" : "other", mq_info(owner));
+		if (owner != q)
+			g_warning("BUG: will make linkable 0x%lx belong to %s",
+				(gulong) l, mq_info(q));
+		g_assert_not_reached();
+	}
+
+	g_hash_table_insert(qown, l, q);
+}
+
+/**
+ * Remove linkable from queue
+ */
+static void mq_remove_linkable(mqueue_t *q, GList *l)
+{
+	mqueue_t *owner;
+
+	g_assert(q->magic == MQ_MAGIC);
+	g_assert(l != NULL);
+	g_assert(l->data != NULL);
+	g_assert(qown != NULL);		/* Must have added something before */
+
+	owner = g_hash_table_lookup(qown, l);
+
+	if (owner == NULL)
+		g_error("BUG: removed linkable 0x%lx from %s belongs to no queue!",
+			(gulong) l, mq_info(q));
+	else if (owner != q)
+		g_error("BUG: removed linkable 0x%lx from %s is from another queue!",
+			(gulong) l, mq_info(q));
+	else
+		g_hash_table_remove(qown, l);
+}
+
+/**
+ * Check queue's sanity.
+ */
+void
+mq_check_track(mqueue_t *q, gint offset, const gchar *where, gint line)
+{
+	gint qcount;
+	gint qlink_alive = 0;
+	gint n;
+
+	g_assert(q);
+
+	if (qown == NULL)
+		qown = g_hash_table_new(NULL, NULL);
+
+	if (q->magic != MQ_MAGIC)
+		g_error("BUG: %s at %s:%d", mq_info(q), where, line);
+
+	qcount = g_list_length(q->qhead);
+	if (qcount != q->count)
+		g_error("BUG: "
+			"%s has wrong q->count of %d (counted %d in list) at %s:%d",
+			mq_info(q), q->count, qcount, where, line);
+
+	if (q->qlink == NULL)
+		return;
+
+	for (n = 0; n < q->qlink_count; n++) {
+		GList *item = q->qlink[n];
+		mqueue_t *owner;
+
+		if (item == NULL)
+			continue;
+
+		qlink_alive++;
+		if (item->data == NULL)
+			g_error("BUG: linkable #%d/%d from %s is NULL at %s:%d",
+				n, q->qlink_count, mq_info(q), where, line);
+
+		g_assert(qown);		/* If we have a qlink, we have added items */
+
+		owner = g_hash_table_lookup(qown, item);
+		if (owner != q)
+			g_error("BUG: linkable #%d/%d from %s "
+				"%s at %s:%d",
+				n, q->qlink_count, mq_info(q),
+				owner == NULL ?
+					"does not belong to any queue" :
+					"belongs to foreign queue",
+				where, line);
+	}
+
+	if (qlink_alive != qcount + offset)
+		g_error("BUG: qlink discrepancy for %s "
+		"(counted %d alive linkable, expected %d, queue has %d items) at %s:%d",
+		mq_info(q), qlink_alive, qcount + offset, qcount, where, line);
+}
+#else	/* !MQ_DEBUG */
+
+#define	mq_add_linkable(x,y)
+#define	mq_remove_linkable(x,y)
+
+#endif	/* MQ_DEBUG */
+
 /*
  * Polymorphic operations.
  */
@@ -78,7 +231,8 @@ mq_free(mqueue_t *q)
 
 	for (n = 0, l = q->qhead; l; l = g_list_next(l)) {
 		n++;
-		pmsg_free((pmsg_t *) l->data);
+		pmsg_free(l->data);
+		mq_remove_linkable(q, l);
 	}
 
 	g_assert(n == q->count);
@@ -90,6 +244,7 @@ mq_free(mqueue_t *q)
 		cq_cancel(callout_queue, q->swift_ev);
 
 	g_list_free(q->qhead);
+	q->magic = 0;
 	wfree(q, sizeof(*q));
 }
 
@@ -105,13 +260,14 @@ mq_rmlink_prev(mqueue_t *q, GList *l, gint size)
 {
 	GList *prev = g_list_previous(l);
 
+	mq_remove_linkable(q, l);
 	q->qhead = g_list_remove_link(q->qhead, l);
 	if (q->qtail == l)
 		q->qtail = prev;
 	q->size -= size;
 	q->count--;
 
-	pmsg_free((pmsg_t *) l->data);
+	pmsg_free(l->data);
 	g_list_free_1(l);
 
 	return prev;
@@ -142,6 +298,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	 */
 
 	q->swift_elapsed = node_flowc_swift_period(q->node) * 1000;
+	q->swift_elapsed = MAX(q->swift_elapsed, 1);
 
 	/*
 	 * Compute target to reach the low watermark, and then the amount we
@@ -193,9 +350,9 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 * Leave our queries in for now (they have hops=0).
 		 */
 
-		q->header.function = GTA_MSG_SEARCH;
-		q->header.hops = 1;
-		q->header.ttl = max_ttl;
+		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH);
+		gnutella_header_set_hops(&q->header, 1);
+		gnutella_header_set_ttl(&q->header, max_ttl);
 
 		if (needed > 0)
 			make_room_header(q, (gchar*) &q->header, PMSG_P_DATA, needed, NULL);
@@ -223,7 +380,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 * than our comparison point.
 		 */
 
-		q->header.function = GTA_MSG_SEARCH_RESULTS;
+		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH_RESULTS);
 
 		/*
 		 * Loop until we reach hops=hard_ttl_limit or we have finished
@@ -233,8 +390,8 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		for (ttl = hard_ttl_limit; needed > 0 && ttl >= 0; ttl--) {
 			gint old_size = q->size;
 
-			q->header.hops = hard_ttl_limit - ttl;
-			q->header.ttl = ttl;
+			gnutella_header_set_hops(&q->header, hard_ttl_limit - ttl);
+			gnutella_header_set_ttl(&q->header, ttl);
 
 			if (
 				make_room_header(q, (gchar*) &q->header, PMSG_P_DATA,
@@ -269,7 +426,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 static void
 mq_swift_timer(cqueue_t *unused_cq, gpointer obj)
 {
-	mqueue_t *q = (mqueue_t *) obj;
+	mqueue_t *q = obj;
 
 	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == (MQ_FLOWC|MQ_SWIFT));
@@ -283,7 +440,7 @@ mq_swift_timer(cqueue_t *unused_cq, gpointer obj)
 static void
 mq_enter_swift(cqueue_t *unused_cq, gpointer obj)
 {
-	mqueue_t *q = (mqueue_t *) obj;
+	mqueue_t *q = obj;
 
 	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == MQ_FLOWC);
@@ -315,6 +472,8 @@ mq_enter_flowc(mqueue_t *q)
 	q->flowc_written = 0;
 	q->last_size = q->size;
 	q->swift_elapsed = node_flowc_swift_grace(q->node) * 1000;
+	q->swift_elapsed = MAX(q->swift_elapsed, 1);
+
 	q->swift_ev =
 		cq_insert(callout_queue, q->swift_elapsed, mq_enter_swift, q);
 
@@ -384,15 +543,14 @@ mq_update_flowc(mqueue_t *q)
 void
 mq_clear(mqueue_t *q)
 {
-	GList *l;
-
 	g_assert(q);
 
 	if (q->count == 0)
 		return;					/* Queue is empty */
 
-	while ((l = q->qhead)) {
-		pmsg_t *mb = (pmsg_t *) l->data;
+	while (q->qhead) {
+		GList *l = q->qhead;
+		pmsg_t *mb = l->data;
 
 		/*
 		 * Break if we started to write this message, i.e. if we read
@@ -451,15 +609,15 @@ mq_shutdown(mqueue_t *q)
  * -- qsort() callback
  */
 static gint
-qlink_cmp(const void *lp1, const void *lp2)
+qlink_cmp(const void *a, const void *b)
 {
-	const pmsg_t *m1 = (const pmsg_t *) (*(const GList * const *) lp1)->data;
-	const pmsg_t *m2 = (const pmsg_t *) (*(const GList * const *) lp2)->data;
+	const GList * const *l1 = a, * const *l2 = b;
+	const pmsg_t *m1 = (*l1)->data, *m2 = (*l2)->data;
 
 	if (pmsg_prio(m1) == pmsg_prio(m2))
 		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2));
-
-	return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
+	else
+		return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
 }
 
 /**
@@ -468,13 +626,12 @@ qlink_cmp(const void *lp1, const void *lp2)
 static void
 qlink_create(mqueue_t *q)
 {
-	GList **qlink;
 	GList *l;
 	gint n;
 
 	g_assert(q->qlink == NULL);
 
-	qlink = q->qlink = (GList **) g_malloc(q->count * sizeof(GList *));
+	q->qlink = g_malloc(q->count * sizeof q->qlink[0]);
 
 	/*
 	 * Prepare sorting of queued messages.
@@ -483,12 +640,14 @@ qlink_create(mqueue_t *q)
 	 * gmsg_cmp() routine to compare the Gnutella messages.
 	 */
 
-	for (l = q->qhead, n = 0; l && n < q->count; l = g_list_next(l), n++)
-		qlink[n] = l;
+	for (l = q->qhead, n = 0; l && n < q->count; l = g_list_next(l), n++) {
+		g_assert(l->data != NULL);
+		q->qlink[n] = l;
+	}
 
 	if (l || n != q->count)
-		g_warning("BUG: queue count of %d for 0x%lx is wrong (found %d)",
-			q->count, (gulong) q, n);
+		g_error("BUG: queue count of %d for 0x%lx is wrong (has %d)",
+			q->count, (gulong) q, g_list_length(q->qhead));
 
 	/*
 	 * We use `n' and not `q->count' in case the warning above is emitted,
@@ -496,7 +655,9 @@ qlink_create(mqueue_t *q)
 	 */
 
 	q->qlink_count = n;
-	qsort(qlink, n, sizeof(GList *), qlink_cmp);
+	qsort(q->qlink, n, sizeof q->qlink[0], qlink_cmp);
+
+	mq_check(q, 0);
 }
 
 /**
@@ -520,13 +681,16 @@ qlink_insert_before(mqueue_t *q, gint hint, GList *l)
 {
 	g_assert(hint >= 0 && hint < q->qlink_count);
 	g_assert(qlink_cmp(&q->qlink[hint], &l) >= 0);	/* `hint' >= `l' */
+	g_assert(l->data != NULL);
+
+	mq_check(q, -1);
 
 	/*
 	 * Lookup before the message for a NULL entry that we could fill.
 	 */
 
-	if (hint > 0 && q->qlink[hint-1] == NULL) {
-		q->qlink[hint-1] = l;
+	if (hint > 0 && q->qlink[hint - 1] == NULL) {
+		q->qlink[hint - 1] = l;
 		return;
 	}
 
@@ -534,11 +698,17 @@ qlink_insert_before(mqueue_t *q, gint hint, GList *l)
 	 * Extend the array then, and insert right at `hint' in the new array.
 	 */
 
-	q->qlink = g_realloc(q->qlink, ++q->qlink_count * sizeof(GList *));
+	q->qlink_count++;
+	q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 
-	memmove(&q->qlink[hint+1], &q->qlink[hint],
-		(q->qlink_count - hint - 1) * sizeof(GList *));	/* Shift right */
+	{
+		gint i;
 
+		/* Shift right */
+		for (i = q->qlink_count - 1; i > hint; i--) {
+			q->qlink[i] = q->qlink[i - 1];
+		} 
+	}
 	q->qlink[hint] = l;
 }
 
@@ -552,13 +722,18 @@ qlink_insert(mqueue_t *q, GList *l)
 	gint high = q->qlink_count - 1;
 	GList **qlink = q->qlink;
 
+	g_assert(l->data != NULL);
+
+	mq_check(q, -1);
+
 	/*
 	 * If `qlink' is empty, create a slot for the new entry.
 	 */
 
 	if (high < 0) {
 		g_assert(q->count == 1);		/* `l' is already part of the queue */
-		q->qlink = g_realloc(q->qlink, ++q->qlink_count * sizeof(GList *));
+		q->qlink_count++;
+		q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 		q->qlink[0] = l;
 		return;
 	}
@@ -577,7 +752,8 @@ qlink_insert(mqueue_t *q, GList *l)
 	 */
 
 	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high]) >= 0) {
-		q->qlink = g_realloc(q->qlink, ++q->qlink_count * sizeof(GList *));
+		q->qlink_count++;
+		q->qlink = g_realloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 		q->qlink[q->qlink_count - 1] = l;
 		return;
 	}
@@ -695,7 +871,11 @@ qlink_insert(mqueue_t *q, GList *l)
 }
 
 /**
- * Remove the entry in the `qlink' linkable array.
+ * Remove the entry in the `qlink' linkable array, allowing compaction
+ * when there are too many holes.
+ *
+ * @param q			the message queue
+ * @param l			the linkable to remove from the qlink indexer
  */
 static void
 qlink_remove(mqueue_t *q, GList *l)
@@ -705,22 +885,26 @@ qlink_remove(mqueue_t *q, GList *l)
 
 	g_assert(qlink);
 	g_assert(n > 0);
+	g_assert(l->data != NULL);
+
+	mq_check(q, 0);
 
 	/*
 	 * If more entries in `qlink' than 3 times the amount of queued messages,
 	 * we have too many NULL in the array.
 	 */
 
-	if  (n > q->count * 3) {
+	if (n > q->count * 3) {
 		GList **dest = qlink;
 		gint copied = 0;
 		gboolean found = FALSE;
 
 		while (n-- > 0) {
-			GList *entry = *qlink++;;
-			if (l == entry || entry == NULL) {
-				if (entry == l)
-					found = TRUE;
+			GList *entry = *qlink++;
+			if (entry == NULL)
+				continue;
+			else if (l == entry) {
+				found = TRUE;
 				continue;
 			}
 			*dest++ = entry;
@@ -729,18 +913,29 @@ qlink_remove(mqueue_t *q, GList *l)
 
 		q->qlink_count = copied;
 
-		g_assert(found);
-
+		if (found)
+			return;
 	} else {
 		while (n-- > 0) {
-			if (l == *qlink++) {
-				*(--qlink) = NULL;
+			if (l == *qlink) {
+				*qlink = NULL;
 				return;
 			}
+			qlink++;
 		}
 
-		g_assert(0);		/* Must have been found */
+		/* Should have been found -- FALL THROUGH */
 	}
+
+	/*
+	 * Used to be an assertion, but it is non-fatal.  Warn copiously though.
+	 */
+
+	g_error("BUG: linkable 0x%lx for %s not found "
+		"(qlink has %d slots, queue has %d counted items, really %d) at %s:%d",
+		(gulong) l, mq_info(q),
+		q->qlink_count, q->count, g_list_length(q->qhead),
+		_WHERE_, __LINE__);
 }
 
 /**
@@ -773,8 +968,10 @@ make_room_header(
 {
 	gint n;
 	gint dropped = 0;				/* Amount of messages dropped */
+	gboolean qlink_corrupted = FALSE;	/* BUG catcher */
 
 	g_assert(needed > 0);
+	mq_check(q, 0);
 
 	if (dbg > 5)
 		printf("%s try to make room for %d bytes in queue 0x%lx (node %s)\n",
@@ -809,6 +1006,7 @@ make_room_header(
 	 * network, so we can NULLify the corresponding slot in the qlink array.
 	 */
 
+restart:
 	for (n = 0; needed >= 0 && n < q->qlink_count; n++) {
 		GList *item = q->qlink[n];
 		pmsg_t *cmb;
@@ -822,7 +1020,33 @@ make_room_header(
 		if (item == NULL)
 			continue;
 
-		cmb = (pmsg_t *) item->data;
+		/*
+		 * BUG catcher -- I've seen situations where item->data is NULL
+		 * at this point, which means something is deeply corrupted...
+		 *		--RAM, 2006-07-14
+		 */
+
+		if (item->data == NULL) {
+			g_error("BUG: NULL data for qlink item #%d/%d in %s at %s:%d",
+				n, q->qlink_count, mq_info(q), _WHERE_, __LINE__);
+
+			if (qlink_corrupted) {
+				g_error(
+					"BUG: trying to ignore still invalid qlink entry at %s:%d",
+						_WHERE_, __LINE__);
+				continue;
+			}
+
+			qlink_corrupted = TRUE;		/* Try to mend it */
+			qlink_free(q);
+			qlink_create(q);
+
+			g_error("BUG: recreated qlink and restarting at %s:%d",
+				_WHERE_, __LINE__);
+			goto restart;
+		}
+
+		cmb = item->data;
 		cmb_start = pmsg_start(cmb);
 
 		/*
@@ -873,10 +1097,15 @@ make_room_header(
 		gnet_stats_count_flowc(pmsg_start(cmb));
 		cmb_size = pmsg_size(cmb);
 
+		g_assert(q->qlink[n] == item);
+
 		needed -= cmb_size;
-		(void) mq_rmlink_prev(q, q->qlink[n], cmb_size);
+		(void) mq_rmlink_prev(q, item, cmb_size);
 		q->qlink[n] = NULL;
+
 		dropped++;
+
+		mq_check(q, 0);
 	}
 
 	if (dropped)
@@ -926,6 +1155,8 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 	gboolean make_room_called = FALSE;
 	gboolean has_normal_prio = (pmsg_prio(mb) == PMSG_P_DATA);
 
+	mq_check(q, 0);
+
 	/*
 	 * If we're flow-controlled and the message can be dropped, acccept it
 	 * if we can manage to make room for at least the size of the message,
@@ -1034,7 +1265,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		 */
 
 		for (l = q->qtail; l; l = l->prev) {
-			pmsg_t *m = (pmsg_t *) l->data;
+			pmsg_t *m = l->data;
 
 			if (
 				pmsg_is_unread(m) &&			/* Not partially written */
@@ -1078,6 +1309,8 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		}
 	}
 
+	mq_add_linkable(q, new);
+
 	q->size += msize;
 	q->count++;
 
