@@ -1,5 +1,5 @@
 /*
- * $Id: mq.h,v 1.8 2005/08/02 10:29:50 rmanfredi Exp $
+ * $Id: mq.h 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -59,6 +59,10 @@ struct mq_cops {
 	void (*update_flowc)(mqueue_t *q);
 };
 
+enum mq_magic {
+	MQ_MAGIC = 0x33990ee
+};
+
 /**
  * A message queue.
  *
@@ -78,7 +82,8 @@ struct mq_cops {
  * to be used as a comparison point when speeding up dropping in flow-control.
  */
 struct mqueue {
-	struct gnutella_header header;	/**< Comparison point during flow control */
+	enum mq_magic magic;	/**< Magic number */
+	gnutella_header_t header;	/**< Comparison point during flow control */
 	struct gnutella_node *node;		/**< Node to which this queue belongs */
 	const struct mq_ops *ops;		/**< Polymorphic operations */
 	const struct mq_cops *cops;		/**< Common operations */
@@ -133,6 +138,19 @@ void mq_fill_ops(struct mq_ops *ops);
 
 const struct mq_cops *mq_get_cops(void);
 
+/*
+ * Message queue assertions.
+ */
+
+#define MQ_DEBUG
+#ifdef MQ_DEBUG
+void mq_check_track(mqueue_t *q, gint offset, const gchar *where, gint line);
+
+#define mq_check(x,y)	mq_check_track((x), (y), _WHERE_, __LINE__)
+#else
+#define mq_check(x,y)
+#endif
+
 #endif	/* _core_mq_h_ */
 
 /* vi: set ts=4: */
