@@ -1,5 +1,5 @@
 /*
- * $Id: mq_udp.c,v 1.18 2005/09/10 08:17:28 daichik Exp $
+ * $Id: mq_udp.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: mq_udp.c,v 1.18 2005/09/10 08:17:28 daichik Exp $");
+RCSID("$Id: mq_udp.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "nodes.h"
 #include "mq.h"
@@ -108,7 +108,7 @@ mq_udp_pmsg_free_extended(pmsg_t *mb, gpointer arg)
  * new message block to use.
  */
 static pmsg_t *
-mq_udp_attach_metadata(pmsg_t *mb, gnet_host_t *to)
+mq_udp_attach_metadata(pmsg_t *mb, const gnet_host_t *to)
 {
 	pmsg_t *result;
 
@@ -153,6 +153,7 @@ mq_udp_make(gint maxsize, struct gnutella_node *n, struct txdriver *nd)
 
 	q = walloc0(sizeof(*q));
 
+	q->magic = MQ_MAGIC;
 	q->node = n;
 	q->tx_drv = nd;
 	q->maxsize = maxsize;
@@ -178,6 +179,7 @@ mq_udp_service(gpointer data)
 	gint sent;
 	gint dropped;
 
+	mq_check(q, 0);
 	g_assert(q->count);		/* Queue is serviced, we must have something */
 
 	sent = 0;
@@ -210,9 +212,8 @@ mq_udp_service(gpointer data)
 		if (r != mb_size) {
 			g_warning("partial UDP write (%d bytes) to %s for %d-byte datagram",
 				r, host_addr_port_to_string(mi->to.addr, mi->to.port), mb_size);
-			l = q->cops->rmlink_prev(q, l, mb_size);
 			dropped++;
-			continue;
+			goto skip;
 		}
 
 		node_add_tx_given(q->node, r);
@@ -241,6 +242,7 @@ mq_udp_service(gpointer data)
 		l = q->cops->rmlink_prev(q, l, mb_size);
 	}
 
+	mq_check(q, 0);
 	g_assert(q->size >= 0 && q->count >= 0);
 
 	if (sent)
@@ -264,6 +266,8 @@ mq_udp_service(gpointer data)
 		tx_srv_disable((txdrv_t *) q->tx_drv);
 		node_tx_service(q->node, FALSE);
 	}
+
+	mq_check(q, 0);
 }
 
 /**
@@ -273,7 +277,7 @@ mq_udp_service(gpointer data)
  * immediately by the caller.
  */
 void
-mq_udp_putq(mqueue_t *q, pmsg_t *mb, gnet_host_t *to)
+mq_udp_putq(mqueue_t *q, pmsg_t *mb, const gnet_host_t *to)
 {
 	size_t size = pmsg_size(mb);
 	gchar *mbs = pmsg_start(mb);
@@ -285,6 +289,7 @@ mq_udp_putq(mqueue_t *q, pmsg_t *mb, gnet_host_t *to)
 	g_assert(!pmsg_was_sent(mb));
 	g_assert(pmsg_is_unread(mb));
 	g_assert(q->ops == &mq_udp_ops);	/* Is an UDP queue */
+	mq_check(q, 0);
 
 	if (size == 0) {
 		g_warning("mq_putq: called with empty message");
@@ -363,28 +368,12 @@ mq_udp_putq(mqueue_t *q, pmsg_t *mb, gnet_host_t *to)
 	mbe = mq_udp_attach_metadata(mb, to);
 
 	q->cops->puthere(q, mbe, size);
+	mq_check(q, 0);
 	return;
 
 cleanup:
-
-#if 0
-	/* XXX: This is a temporary additional check for debugging purposes */
-	{
-		gint i;
-
-		for (i = 0; i < q->qlink_count; i++) {
-			GList *l;
-			gpointer found;
-
-			l = q->qlink[i];
-			g_assert(l != NULL);
-			found = g_list_find(l, mb);
-			g_assert(!found);
-		}
-	}
-#endif
-
 	pmsg_free(mb);
+	mq_check(q, 0);
 	return;
 }
 
@@ -392,7 +381,7 @@ cleanup:
  * Enqueue message to be sent to the ip:port held in the supplied node.
  */
 void
-mq_udp_node_putq(mqueue_t *q, pmsg_t *mb, gnutella_node_t *n)
+mq_udp_node_putq(mqueue_t *q, pmsg_t *mb, const gnutella_node_t *n)
 {
 	gnet_host_t to;
 
