@@ -1,5 +1,5 @@
 /*
- * $Id: oob.c,v 1.22 2005/10/01 11:45:48 nonamer Exp $
+ * $Id: oob.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: oob.c,v 1.22 2005/10/01 11:45:48 nonamer Exp $");
+RCSID("$Id: oob.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "oob.h"
 #include "hosts.h"
@@ -60,12 +60,14 @@ RCSID("$Id: oob.c,v 1.22 2005/10/01 11:45:48 nonamer Exp $");
 
 #define OOB_EXPIRE_MS		(2*60*1000)		/**< 2 minutes at most */
 #define OOB_TIMEOUT_MS		(45*1000)		/**< 45 secs for them to reply */
-#define OOB_DELIVER_MS		(5*1000)		/**< 1 message queued every 5 secs */
+#define OOB_DELIVER_BASE_MS	2500			/**< 1 msg queued every 2.5 secs */
+#define OOB_DELIVER_RAND_MS	5000			/**< ... + up to 5 random secs */
 
 #define OOB_MAX_QUEUED		50				/**< Max # of messages per host */
 #define OOB_MAX_RETRY		3				/**< Retry # if LIME/12v2 dropped */
 
 #define OOB_MAX_QHIT_SIZE	645				/**< Flush hits larger than this */
+#define OOB_MAX_DQHIT_SIZE	1075			/**< Flush limit for deflated hits */
 
 /**
  * A set of hits awaiting delivery.
@@ -104,6 +106,7 @@ struct gservent {
 	gpointer ev_service;	/**< Callout event for servicing FIFO */
 	gnet_host_t *host;		/**< The servent host (also used as key for table) */
 	fifo_t *fifo;			/**< The servent's FIFO, holding pmsg_t items */
+	gboolean can_deflate;	/**< Whether servent supports UDP compression */
 };
 
 /*
@@ -132,8 +135,8 @@ static gint num_oob_records;	/**< Leak and duplicate free detector */
  * results delivery via the sent LIME/12v2 and the expected LIME/11v2 reply.
  */
 static struct oob_results *
-results_make(
-	gchar *muid, GSList *files, gint count, gnet_host_t *to, gboolean ggep_h)
+results_make(const gchar *muid, GSList *files,
+	gint count, gnet_host_t *to, gboolean ggep_h)
 {
 	static const struct oob_results zero_results;
 	struct oob_results *r;
@@ -221,7 +224,7 @@ results_destroy(cqueue_t *unused_cq, gpointer obj)
 	(void) unused_cq;
 
 	if (query_debug)
-		printf("OOB query %s from %s expired with unclaimed %d hit%s\n",
+		g_message("OOB query %s from %s expired with unclaimed %d hit%s",
 			guid_hex_str(r->muid),
 			host_addr_port_to_string(r->dest.addr, r->dest.port),
 			r->count, r->count == 1 ? "" : "s");
@@ -243,7 +246,7 @@ results_timeout(cqueue_t *unused_cq, gpointer obj)
 	(void) unused_cq;
 
 	if (query_debug)
-		printf("OOB query %s, no ACK from %s to claim %d hit%s\n",
+		g_message("OOB query %s, no ACK from %s to claim %d hit%s",
 			guid_hex_str(r->muid),
 			host_addr_port_to_string(r->dest.addr, r->dest.port),
 			r->count, r->count == 1 ? "" : "s");
@@ -264,6 +267,18 @@ servent_free_remove(struct gservent *s)
 	servent_free(s);
 }
 
+/**
+ * Computes the amount of milliseconds before the next OOB hit delivery,
+ *
+ * Per a suggestion of Daniel Stutzbach, we wait BASE + RAND*random secs,
+ * where "random" is a real random number between 0 and 1.
+ */
+static gint
+deliver_delay(void)
+{
+	return OOB_DELIVER_BASE_MS + random_value(OOB_DELIVER_RAND_MS);
+}
+
 /**
  * Service servent's FIFO: send next packet, and re-arm servicing callback
  * if there are more data to send.
@@ -281,22 +296,32 @@ servent_service(cqueue_t *cq, gpointer obj)
 	if (mb == NULL)
 		goto remove;
 
-	q = node_udp_get_outq();
+	q = node_udp_get_outq(host_addr_net(s->host->addr));
 	if (q == NULL)
 		goto udp_disabled;
 
 	if (udp_debug > 19)
-		printf("UDP queuing OOB %s to %s for %s\n",
+		g_message("UDP queuing OOB %s to %s for %s",
 			gmsg_infostr_full(pmsg_start(mb)),
 			host_addr_port_to_string(s->host->addr, s->host->port),
 			guid_hex_str(pmsg_start(mb)));
 
+	/*
+	 * Count enqueued deflated payloads, only when server was marked as
+	 * supporting compression anyway...
+	 */
+
+	if (s->can_deflate) {
+		if (gnutella_header_get_ttl(pmsg_start(mb)) & GTA_UDP_DEFLATED)
+			gnet_stats_count_general(GNR_UDP_TX_COMPRESSED, 1);
+	}
+
 	mq_udp_putq(q, mb, s->host);
 
 	if (0 == fifo_count(s->fifo))
 		goto remove;
 
-	s->ev_service = cq_insert(cq, OOB_DELIVER_MS, servent_service, s);
+	s->ev_service = cq_insert(cq, deliver_delay(), servent_service, s);
 
 	return;
 
@@ -314,7 +339,7 @@ remove:
  * @param host the servent's IP:port.  Caller may free it upon return.
  */
 static struct gservent *
-servent_make(gnet_host_t *host)
+servent_make(gnet_host_t *host, gboolean can_deflate)
 {
 	struct gservent *s;
 
@@ -323,6 +348,7 @@ servent_make(gnet_host_t *host)
 	*s->host = *host;		/* Struct copy */
 	s->fifo = fifo_make();
 	s->ev_service = NULL;
+	s->can_deflate = can_deflate;
 
 	return s;
 }
@@ -363,7 +389,9 @@ oob_record_hit(gpointer data, size_t len, gpointer udata)
 	struct gservent *s = (struct gservent *) udata;
 
 	g_assert(len <= INT_MAX);
-	fifo_put(s->fifo, gmsg_to_pmsg(data, len));
+	fifo_put(s->fifo, s->can_deflate ?
+		gmsg_to_deflated_pmsg(data, len) :
+		gmsg_to_pmsg(data, len));
 }
 
 /**
@@ -375,7 +403,7 @@ oob_record_hit(gpointer data, size_t len, gpointer udata)
  * @param wanted	the amount of results they want delivered
  */
 void
-oob_deliver_hits(struct gnutella_node *n, gchar *muid, guint8 wanted)
+oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted)
 {
 	struct oob_results *r;
 	struct gservent *s;
@@ -387,9 +415,10 @@ oob_deliver_hits(struct gnutella_node *n, gchar *muid, guint8 wanted)
 	r = g_hash_table_lookup(results_by_muid, muid);
 
 	if (r == NULL) {
+		gnet_stats_count_general(GNR_SPURIOUS_OOB_HIT_CLAIM, 1);
 		if (query_debug)
-			printf("OOB got spurious LIME/11 from %s for %s, "
-				"asking for %d hit%s\n",
+			g_warning("OOB got spurious LIME/11 from %s for %s, "
+				"asking for %d hit%s",
 				node_addr(n), guid_hex_str(muid),
 				wanted, wanted == 1 ? "" : "s");
 		return;
@@ -434,11 +463,19 @@ oob_deliver_hits(struct gnutella_node *n, gchar *muid, guint8 wanted)
 
 	/*
 	 * Fetch the proper servent, create one if none exists yet.
+	 *
+	 * N.B: We assume that for a given host address, UDP deflation support
+	 * will never change: if the host has marked support for deflation in
+	 * the claim message once, we assume that it will always support it.
+	 * Likewise, if it did not request it the first time, no matter what we
+	 * get next, we will never deflate hits for this OOB delivery.
+	 *		--RAM, 2006-08-13
 	 */
 
 	s = g_hash_table_lookup(servent_by_host, &r->dest);
 	if (s == NULL) {
-		s = servent_make(&r->dest);
+		gboolean can_deflate = NODE_CAN_INFLATE(n);	/* Can we deflate? */
+		s = servent_make(&r->dest, can_deflate);
 		g_hash_table_insert(servent_by_host, s->host, s);
 		servent_created = TRUE;
 	}
@@ -452,14 +489,14 @@ oob_deliver_hits(struct gnutella_node *n, gchar *muid, guint8 wanted)
 	deliver_count = (wanted == 255) ? r->count : MIN(wanted, r->count);
 
 	if (query_debug || udp_debug)
-		printf("OOB query %s: host %s wants %d hit%s, delivering %d\n",
+		g_message("OOB query %s: host %s wants %d hit%s, delivering %d",
 			guid_hex_str(r->muid), node_addr(n), wanted, wanted == 1 ? "" : "s",
 			deliver_count);
 
 	if (deliver_count)
 		qhit_build_results(
 			r->files, deliver_count,
-			OOB_MAX_QHIT_SIZE,
+			s->can_deflate ? OOB_MAX_DQHIT_SIZE : OOB_MAX_QHIT_SIZE,
 			oob_record_hit, s,
 			r->muid,
 			r->use_ggep_h);
@@ -511,7 +548,7 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 		} else {
 
 			if (query_debug || udp_debug)
-				printf("OOB query %s, notified %s about %d hit%s\n",
+				g_message("OOB query %s, notified %s about %d hit%s",
 						guid_hex_str(r->muid),
 						host_addr_port_to_string(r->dest.addr, r->dest.port),
 						r->count,
@@ -533,7 +570,7 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 	 */
 
 	if (query_debug)
-		printf("OOB query %s, previous LIME12/v2 #%d was dropped\n",
+		g_message("OOB query %s, previous LIME12/v2 #%d was dropped",
 			guid_hex_str(r->muid), r->notify_requeued);
 
 	if (++r->notify_requeued < OOB_MAX_RETRY)
@@ -548,21 +585,26 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 static void
 oob_send_reply_ind(struct oob_results *r)
 {
+	mqueue_t *q;
 	pmsg_t *mb;
 	pmsg_t *emb;
 
+	q = node_udp_get_outq(host_addr_net(r->dest.addr));
+	if (q == NULL)
+		return;
+
 	mb = vmsg_build_oob_reply_ind(r->muid, MIN(r->count, 255));
 	emb = pmsg_clone_extend(mb, oob_pmsg_free, r);
 	r->refcount++;
 	pmsg_free(mb);
 
 	if (query_debug || udp_debug)
-		printf("OOB query %s, notifying %s about %d hit%s, try #%d\n",
+		g_message("OOB query %s, notifying %s about %d hit%s, try #%d",
 			guid_hex_str(r->muid), host_addr_port_to_string(r->dest.addr,
 				r->dest.port),
 			r->count, r->count == 1 ? "" : "s", r->notify_requeued);
 
-	mq_udp_putq(node_udp_get_outq(), emb, &r->dest);
+	mq_udp_putq(q, emb, &r->dest);
 }
 
 /**
@@ -586,12 +628,13 @@ oob_got_results(
 	g_assert(count > 0);
 	g_assert(files != NULL);
 
-	guid_oob_get_addr_port(n->header.muid, &addr, &port);
+	guid_oob_get_addr_port(gnutella_header_get_muid(&n->header), &addr, &port);
 
 	to.addr = addr;
 	to.port = port;
 
-	r = results_make(n->header.muid, files, count, &to, use_ggep_h);
+	r = results_make(gnutella_header_get_muid(&n->header),
+			files, count, &to, use_ggep_h);
 
 	oob_send_reply_ind(r);
 }
