@@ -1,5 +1,5 @@
 /*
- * $Id: pcache.h,v 1.10 2005/12/15 17:12:47 cbiere Exp $
+ * $Id: pcache.h 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -38,6 +38,7 @@
 
 #include "hcache.h"
 #include "if/core/nodes.h"
+#include "gnutella.h"
 
 struct gnutella_node;
 
@@ -76,7 +77,7 @@ enum {
  * Global Functions.
  */
 
-struct gnutella_msg_init *build_ping_msg(
+gnutella_msg_init_t *build_ping_msg(
 	const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size);
 
 /*
