@@ -1,10 +1,8 @@
 /*
- * $Id: pproxy.c,v 1.7 2004/10/24 01:11:12 cbiere Exp $
+ * $Id: pproxy.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
- * Push proxy HTTP management.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,19 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Push proxy HTTP management.
+ *
+ * @author Raphael Manfredi
+ * @date 2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: pproxy.c,v 1.7 2004/10/24 01:11:12 cbiere Exp $");
+RCSID("$Id: pproxy.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "pproxy.h"
 #include "http.h"
@@ -42,14 +50,17 @@ RCSID("$Id: pproxy.c,v 1.7 2004/10/24 01:11:12 cbiere Exp $");
 #include "routing.h"
 #include "gmsg.h"
 #include "uploads.h"
+#include "ggep.h"
+#include "ggep_type.h"
 #include "gnet_stats.h"
 #include "lib/url.h"
 
 /* Following extra needed for the client-side */
 
-#include "settings.h"		/* For listen_ip() */
+#include "settings.h"			/* For listen_addr() */
 #include "token.h"
 #include "downloads.h"
+#include "features.h"
 
 #include "lib/misc.h"
 #include "lib/atoms.h"
@@ -57,6 +68,7 @@ RCSID("$Id: pproxy.c,v 1.7 2004/10/24 01:11:12 cbiere Exp $");
 #include "lib/header.h"
 #include "lib/glib-missing.h"
 #include "lib/endian.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -67,7 +79,7 @@ RCSID("$Id: pproxy.c,v 1.7 2004/10/24 01:11:12 cbiere Exp $");
  *** Server-side of push-proxy
  ***/
 
-static GSList *pproxies = NULL;	/* Currently active push-proxy requests */
+static GSList *pproxies = NULL;	/**< Currently active push-proxy requests */
 
 static void send_pproxy_error(struct pproxy *pp, int code,
 	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
@@ -95,8 +107,7 @@ pproxy_free_resources(struct pproxy *pp)
 	}
 	if (pp->socket != NULL) {
 		g_assert(pp->socket->resource.pproxy == pp);
-		socket_free(pp->socket);
-		pp->socket = NULL;
+		socket_free_null(&pp->socket);
 	}
 }
 
@@ -116,14 +127,15 @@ send_pproxy_error_v(
 	gint hevcnt = 0;
 
 	if (msg) {
-		gm_vsnprintf(reason, sizeof(reason), msg, ap);
+		gm_vsnprintf(reason, sizeof reason, msg, ap);
 	} else
 		reason[0] = '\0';
 
 	if (pp->error_sent) {
 		g_warning("push-proxy: already sent an error %d to %s, "
 			"not sending %d (%s)",
-			pp->error_sent, ip_to_gchar(pp->socket->ip), code, reason);
+			pp->error_sent,host_addr_to_string(pp->socket->addr),
+			code, reason);
 		return;
 	}
 
@@ -134,9 +146,9 @@ send_pproxy_error_v(
 	 */
 
 	if (ext) {
-		size_t slen = g_strlcpy(extra, ext, sizeof(extra));
-		
-		if (slen < sizeof(extra)) {
+		size_t slen = g_strlcpy(extra, ext, sizeof extra);
+
+		if (slen < sizeof extra) {
 			hev[hevcnt].he_type = HTTP_EXTRA_LINE;
 			hev[hevcnt++].he_msg = extra;
 		} else
@@ -176,11 +188,11 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 	g_assert(pp != NULL);
 
 	if (reason) {
-		gm_vsnprintf(errbuf, sizeof(errbuf), reason, ap);
+		gm_vsnprintf(errbuf, sizeof errbuf , reason, ap);
 		logreason = errbuf;
 	} else {
 		if (pp->error_sent) {
-			gm_snprintf(errbuf, sizeof(errbuf), "HTTP %d", pp->error_sent);
+			gm_snprintf(errbuf, sizeof errbuf, "HTTP %d", pp->error_sent);
 			logreason = errbuf;
 		} else {
 			errbuf[0] = '\0';
@@ -190,7 +202,7 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 
 	if (dbg > 1) {
 		printf("push-proxy: ending request from %s (%s): %s\n",
-			pp->socket ? ip_to_gchar(pp->socket->ip) : "<no socket>",
+			pp->socket ? host_addr_to_string(pp->socket->addr) : "<no socket>",
 			pproxy_vendor_str(pp),
 			logreason);
 	}
@@ -202,9 +214,9 @@ pproxy_remove_v(struct pproxy *pp, const gchar *reason, va_list ap)
 	}
 
 	pproxy_free_resources(pp);
-	wfree(pp, sizeof(*pp));
+	wfree(pp, sizeof *pp);
 
-	pproxies = g_slist_remove(pproxies, (gpointer) pp);
+	pproxies = g_slist_remove(pproxies, pp);
 }
 
 /**
@@ -217,9 +229,9 @@ void
 pproxy_remove(struct pproxy *pp, const gchar *reason, ...)
 {
 	va_list args;
-	
+
 	g_assert(pp != NULL);
-	
+
 	va_start(args, reason);
 	pproxy_remove_v(pp, reason, args);
 	va_end(args);
@@ -255,20 +267,23 @@ pproxy_timer(time_t now)
 	GSList *to_remove = NULL;
 
 	for (sl = pproxies; sl; sl = g_slist_next(sl)) {
-		struct pproxy *pp = (struct pproxy *) sl->data;
-		
+		struct pproxy *pp = sl->data;
+
 		/*
 		 * We can't call pproxy_remove() since it will remove the structure
 		 * from the list we are traversing.
 		 */
 
-		if (delta_time(now, pp->last_update) > upload_connecting_timeout) {
+		if (
+			delta_time(now, pp->last_update) >
+				(time_delta_t) upload_connecting_timeout
+		) {
 			to_remove = g_slist_prepend(to_remove, pp);
 		}
 	}
 
 	for (sl = to_remove; sl; sl = g_slist_next(sl)) {
-		struct pproxy *pp = (struct pproxy *) sl->data;
+		struct pproxy *pp = sl->data;
 		pproxy_error_remove(pp, 408, "Request timeout");
 	}
 
@@ -283,10 +298,11 @@ pproxy_create(struct gnutella_socket *s)
 {
 	struct pproxy *pp;
 
-	pp = walloc0(sizeof(*pp));
+	pp = walloc0(sizeof *pp);
 
 	pp->socket = s;
-	pp->last_update = time(NULL);
+	pp->flags = 0; /* XXX: TLS? */
+	pp->last_update = tm_time();
 	s->resource.pproxy = pp;
 
 	return pp;
@@ -298,32 +314,51 @@ pproxy_create(struct gnutella_socket *s)
  *
  * Fills the GUID atom into `guid_atom' and the file index into `file_idx'.
  *
- * Returns TRUE if OK, FALSE if we could not figure it out, in which case
+ * @returns TRUE if OK, FALSE if we could not figure it out, in which case
  * we also return an error to the calling party.
  */
 static gboolean
-get_params(struct pproxy *pp, gchar *request,
+get_params(struct pproxy *pp, const gchar *request,
 	gchar **guid_atom, guint32 *file_idx)
 {
+	static const struct {
+		const gchar *req;
+		const gchar *attr;
+	} req_types[] = {
+		{ "/gnutella/pushproxy?",	"ServerId" },
+		{ "/gnutella/push-proxy?",	"ServerId" },
+		{ "/gnet/push-proxy?",		"guid" },
+	};
 	gchar *uri;
-	gchar *attr;
+	const gchar *attr;
 	gchar *p;
-	gchar *value;
+	const gchar *value;
 	gint datalen;
 	url_params_t *up;
+	guint i;
 
+	g_assert(pp);
+	g_assert(request);
+	g_assert(guid_atom);
+	g_assert(file_idx);
+	
 	/*
 	 * Move to the start of the requested path.  Note that sizeof("GET")
 	 * accounts for the trailing NUL in the string.
 	 */
 
-	uri = request + ((request[0] == 'G') ? sizeof("GET") : sizeof("HEAD"));
-	while (is_ascii_blank(*uri))
-		uri++;
+	uri = is_strprefix(request, "GET");
+	if (!uri)
+		uri = is_strprefix(request, "HEAD");
+	if (!uri) {
+		pproxy_error_remove(pp, 501, "Not Implemented");
+		return FALSE;
+	}
+	uri = skip_ascii_blanks(uri);
 
 	/*
 	 * Go patch the first space we encounter before HTTP to be a NUL.
-	 * Indeed, the requesst shoud be "GET /get/12/foo.txt HTTP/1.0".
+	 * Indeed, the request should be "GET /get/12/foo.txt HTTP/1.0".
 	 *
 	 * Note that if we don't find HTTP/ after the space, it's not an
 	 * error: they're just sending an HTTP/0.9 request, which is awkward
@@ -331,7 +366,7 @@ get_params(struct pproxy *pp, gchar *request,
 	 */
 
 	p = strrchr(uri, ' ');
-	if (p && p[1]=='H' && p[2]=='T' && p[3]=='T' && p[4]=='P' && p[5]=='/')
+	if (p && is_strprefix(&p[1], "HTTP/"))
 		*p = '\0';
 
 	/*
@@ -342,16 +377,18 @@ get_params(struct pproxy *pp, gchar *request,
 	 *		--RAM, 18/07/2003
 	 */
 
-	if (0 == strncmp(uri, "/gnutella/pushproxy?", 20)) {
-		uri += 20;
-		attr = "ServerId";
-	} else if (0 == strncmp(uri, "/gnutella/push-proxy?", 21)) {
-		uri += 21;
-		attr = "ServerId";
-	} else if (0 == strncmp(uri, "/gnet/push-proxy?", 17)) {
-		uri += 17;
-		attr = "guid";
-	} else {
+	attr = NULL;
+	for (i = 0; i < G_N_ELEMENTS(req_types); i++) {
+		gchar *q;
+
+		if (NULL != (q = is_strprefix(req_types[i].req, uri))) {
+			attr = req_types[i].attr;
+			uri = q;
+			break;
+		}
+	}
+
+	if (!attr) {
 		pproxy_error_remove(pp, 400, "Request not understood");
 		return FALSE;
 	}
@@ -396,7 +433,7 @@ get_params(struct pproxy *pp, gchar *request,
 
 		if (dbg > 4)
 			printf("PUSH-PROXY: decoding %s=%s as base32\n", attr, value);
-		
+
 		guid = base32_to_guid(value);
 		if (guid == NULL) {
 			pproxy_error_remove(pp, 400, "Malformed push-proxy request: "
@@ -406,7 +443,7 @@ get_params(struct pproxy *pp, gchar *request,
 
 		*guid_atom = atom_guid_get(guid);
 	} else if (0 == strcmp(attr, "guid")) {
-		gchar guid[16];
+		gchar guid[GUID_RAW_SIZE];
 
 		/*
 		 * GUID in hexadecimal: valid length is 32.
@@ -429,19 +466,21 @@ get_params(struct pproxy *pp, gchar *request,
 		}
 
 		*guid_atom = atom_guid_get(guid);
-	} else
+	} else {
 		g_error("unhandled parameter \"%s\"", attr);
+	}
 
 	/*
 	 * Extract the optional "file=" parameter.
 	 */
 
 	value = url_params_get(up, "file");
+	{
+		gint error;
 
-	if (value == NULL)
-		*file_idx = 0;
-	else
-		*file_idx = (guint32) atoi(value);		/* Silent about errors */
+		/* Ignore errors; parse_uint32() returns 0 on error. */
+		*file_idx = value ? parse_uint32(value, NULL, 10, &error) : 0;
+	}
 
 	url_params_free(up);
 	return TRUE;
@@ -452,27 +491,107 @@ error:
 }
 
 /**
- * Build a push request to send.  We set TTL=max_ttl-1 and hops=1 since
+ * Builds a push request to send.  We set TTL=max_ttl-1 and hops=1 since
  * it does not come from our node really.  The file ID may be set to 0, but
  * it should be ignored when the GIV is received anyway.
+ *
+ * @param size_ptr no document
+ * @param ttl the TTL to use for the packet header.
+ * @param hops the hops value to use for the packet header.
+ * @param guid the hops value to use for the packet header.
+ * @param addr the host address the receiving peer should connect to.
+ * @param port the port number the receiving peer should connect to.
+ * @param file_idx the file index this push is for.
+ * @return	A pointer to a static buffer holding the created Gnutella PUSH
+ *			packet on success, NULL on failure.
  */
-static void
-build_push(struct gnutella_msg_push_request *m,
-	gchar *guid, guint32 ip, guint16 port, guint32 file_idx)
+const gchar *
+build_push(size_t *size_ptr, guint8 ttl, guint8 hops, const gchar *guid,
+	host_addr_t addr_v4, host_addr_t addr_v6, guint16 port,
+	guint32 file_idx, gboolean supports_tls)
 {
-	message_set_muid(&m->header, GTA_MSG_PUSH_REQUEST);
+	static union {
+		gnutella_msg_push_request_t m;
+		gchar data[1024];
+	} packet;
+	gchar *p = packet.data;
+	size_t len = 0, size = sizeof packet;
+	ggep_stream_t gs;
 
-	m->header.function = GTA_MSG_PUSH_REQUEST;
-	m->header.ttl = max_ttl - 1;
-	m->header.hops = 1;
+	g_assert(size_ptr);
+	g_assert(guid);
+	g_assert(0 != port);
 
-	WRITE_GUINT32_LE(sizeof(struct gnutella_push_request), m->header.size);
+	{
+		gnutella_header_t *header = gnutella_msg_push_request_header(&packet.m);
 
-	memcpy(m->request.guid, guid, 16);
+		message_set_muid(header, GTA_MSG_PUSH_REQUEST);
+		gnutella_header_set_function(header, GTA_MSG_PUSH_REQUEST);
+		gnutella_header_set_ttl(header, ttl);
+		gnutella_header_set_hops(header, hops);
+	}
+	
+	gnutella_msg_push_request_set_guid(&packet.m, guid);
 
-	WRITE_GUINT32_LE(file_idx, m->request.file_id);
-	WRITE_GUINT32_BE(ip, m->request.host_ip);
-	WRITE_GUINT16_LE(port, m->request.host_port);
+	STATIC_ASSERT(49 == sizeof packet.m);
+	p += sizeof packet.m;
+	size -= sizeof packet.m;
+	len += sizeof packet.m - GTA_HEADER_SIZE;	/* Exclude the header size */
+
+	ggep_stream_init(&gs, p, size);
+
+#ifdef HAS_GNUTLS
+	if (is_host_addr(addr_v4)) {
+		supports_tls |= is_my_address(addr_v4, port);
+	}
+	if (is_host_addr(addr_v6)) {
+		supports_tls |= is_my_address(addr_v6, port);
+	}
+#endif /* HAS_GNUTLS */
+	
+	if (supports_tls) {
+		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0)) {
+			g_warning("could not write GGEP \"GTKG.TLS\" extension into PUSH");
+			ggep_stream_close(&gs);
+			return NULL;
+		}
+	}
+
+	if (is_host_addr(addr_v6) && NET_TYPE_IPV6 == host_addr_net(addr_v6)) {
+		const guint8 *ipv6 = host_addr_ipv6(&addr_v6);
+
+		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0)) {
+			g_warning("could not write GGEP \"GTKG.IPV6\" extension into PUSH");
+			ggep_stream_close(&gs);
+			return NULL;
+		}
+	}
+
+	{
+		size_t glen;
+
+		glen = ggep_stream_close(&gs);
+		g_assert(size >= glen);
+
+		size -= glen;
+		len += glen;
+		p += glen;
+	}
+	g_assert(len < size);
+	g_assert(len < sizeof packet);
+
+	gnutella_msg_push_request_set_file_id(&packet.m,
+		file_idx == URN_INDEX ? 0 : file_idx);
+	gnutella_msg_push_request_set_host_ip(&packet.m, host_addr_ipv4(addr_v4));
+	gnutella_msg_push_request_set_host_port(&packet.m, port);
+	gnutella_header_set_size(gnutella_msg_push_request_header(&packet.m), len);
+
+	message_add(
+		gnutella_header_get_muid(gnutella_msg_push_request_header(&packet.m)),
+		GTA_MSG_PUSH_REQUEST, NULL);
+
+	*size_ptr = p - packet.data;
+	return packet.data;
 }
 
 /**
@@ -481,12 +600,12 @@ build_push(struct gnutella_msg_push_request *m,
  * @return atom, or NULL.
  */
 static gchar *
-validate_vendor(gchar *vendor, gchar *token, guint32 ip)
+validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
 {
 	gchar *result = NULL;
 
 	if (vendor) {
-		gboolean faked = !version_check(vendor, token, ip);
+		gboolean faked = !version_check(vendor, token, addr);
 
 		if (faked) {
 			gchar name[1024];
@@ -506,19 +625,20 @@ validate_vendor(gchar *vendor, gchar *token, guint32 ip)
  * the push proxyfication.
  */
 static void
-pproxy_request(struct pproxy *pp, header_t *header) 
+pproxy_request(struct pproxy *pp, header_t *header)
 {
 	struct gnutella_socket *s = pp->socket;
-	gchar *request = getline_str(s->getline);
+	const gchar *request = getline_str(s->getline);
 	struct gnutella_node *n;
 	gchar *buf;
 	gchar *token;
 	gchar *user_agent;
-	struct gnutella_msg_push_request m;
 	GSList *nodes;
+	gboolean supports_tls;
 
 	if (dbg > 2) {
-		printf("----Push-proxy request from %s:\n", ip_to_gchar(s->ip));
+		printf("----Push-proxy request from %s:\n",
+			host_addr_to_string(s->addr));
 		printf("%s\n", request);
 		header_dump(header, stdout);
 		printf("----\n");
@@ -532,7 +652,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	token = header_get(header, "X-Token");
 	user_agent = header_get(header, "User-Agent");
 
-	pp->user_agent = validate_vendor(user_agent, token, s->ip);
+	pp->user_agent = validate_vendor(user_agent, token, s->addr);
 
 	/*
 	 * Determine the servent ID.
@@ -543,8 +663,11 @@ pproxy_request(struct pproxy *pp, header_t *header)
 
 	if (dbg > 2)
 		printf("PUSH-PROXY: %s requesting a push to %s for file #%d\n",
-			ip_to_gchar(s->ip), guid_hex_str(pp->guid), pp->file_idx);
+			host_addr_to_string(s->addr), guid_hex_str(pp->guid),
+			pp->file_idx);
 
+	supports_tls = header_get_feature("tls", header, NULL, NULL);
+			
 	/*
 	 * Make sure they provide an X-Node header so we know whom to set up
 	 * as the originator of the push.  Then validate the address.
@@ -558,24 +681,78 @@ pproxy_request(struct pproxy *pp, header_t *header)
 		return;
 	}
 
-	if (!gchar_to_ip_port(buf, &pp->ip, &pp->port)) {
+	{
+		const gchar *endptr;
+		host_addr_t addr;
+		guint16 port;
+
+		if (!string_to_host_addr_port(buf, &endptr, &addr, &port)) {
+			pproxy_error_remove(pp, 400,
+				"Malformed push-proxy request: cannot parse X-Node");
+			return;
+		}
+
+		pp->addr_v4 = zero_host_addr;
+		pp->addr_v6 = zero_host_addr;
+		pp->port = port;
+
+		switch (host_addr_net(addr)) {
+		case NET_TYPE_IPV4:
+			pp->addr_v4 = addr;
+			break;
+		case NET_TYPE_IPV6:
+			pp->addr_v6 = addr;
+			break;
+		case NET_TYPE_LOCAL:
+		case NET_TYPE_NONE:
+			break;
+		}
+		
+		endptr = skip_ascii_spaces(endptr);
+		if (',' == *endptr) {
+			endptr = skip_ascii_spaces(&endptr[1]);
+			if (
+				string_to_host_addr_port(endptr, NULL, &addr, &port) &&
+				port == pp->port
+			) {
+				switch (host_addr_net(addr)) {
+				case NET_TYPE_IPV4:
+					pp->addr_v4 = addr;
+					break;
+				case NET_TYPE_IPV6:
+					pp->addr_v6 = addr;
+					break;
+				case NET_TYPE_LOCAL:
+				case NET_TYPE_NONE:
+					break;
+				}
+			}
+		}
+	}
+
+	if (!host_is_valid(pp->addr_v4, pp->port)) {
+		pp->addr_v4 = zero_host_addr;
+	}
+	if (!host_is_valid(pp->addr_v6, pp->port)) {
+		pp->addr_v6 = zero_host_addr;
+	}
+	
+	if (!is_host_addr(pp->addr_v4) && !is_host_addr(pp->addr_v6)) {
 		pproxy_error_remove(pp, 400,
-			"Malformed push-proxy request: cannot parse X-Node");
+			"Malformed push-proxy request: supplied address is unreachable");
 		return;
 	}
 
-	if (!host_is_valid(pp->ip, pp->port)) {
-		pproxy_error_remove(pp, 400,
-			"Malformed push-proxy request: supplied address %s unreachable",
-			ip_port_to_gchar(pp->ip, pp->port));
-		return;
+	if (
+		!host_addr_equal(pp->addr_v4, s->addr) &&
+		!host_addr_equal(pp->addr_v6, s->addr)
+	) {
+		g_warning("push-proxy request from %s (%s) said node was at %s/%s",
+			host_addr_to_string(s->addr), pproxy_vendor_str(pp),
+			host_addr_port_to_string(pp->addr_v4, pp->port),
+			host_addr_port_to_string2(pp->addr_v6, pp->port));
 	}
 
-	if (pp->ip != s->ip)
-		g_warning("push-proxy request from %s (%s) said node was at %s",
-			ip_to_gchar(s->ip), pproxy_vendor_str(pp),
-			ip_port_to_gchar(pp->ip, pp->port));
-
 	/*
 	 * Locate a route to that servent.
 	 */
@@ -583,18 +760,34 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	n = route_proxy_find(pp->guid);
 
 	if (n != NULL) {
-		build_push(&m, pp->guid, pp->ip, pp->port, pp->file_idx);
-		message_add(m.header.muid, GTA_MSG_PUSH_REQUEST, NULL);
+		const gchar *packet;
+		size_t size;
 
-		gmsg_sendto_one(n, (gchar *) &m, sizeof(m));
-		gnet_stats_count_general(GNR_PUSH_PROXY_RELAYED, 1);
+		/*
+ 		 * We set TTL=max_ttl-1 and hops=1 since
+		 * it does not come from our node really.
+		 */
 
-		http_send_status(pp->socket, 202, FALSE, NULL, 0,
-			"Push-proxy: message sent to node");
+		packet = build_push(&size, max_ttl - 1, 1, pp->guid,
+					pp->addr_v4, pp->addr_v6, pp->port, pp->file_idx,
+					supports_tls);
 
-		pp->error_sent = 202;
-		pproxy_remove(pp, "Push sent directly to node GUID %s",
-			guid_hex_str(pp->guid));
+		if (NULL == packet) {
+			g_warning("Failed to send push for %s/%s (index=%lu)",
+				host_addr_port_to_string(pp->addr_v4, pp->port),
+				host_addr_port_to_string2(pp->addr_v6, pp->port),
+				(gulong) pp->file_idx);
+		} else {
+			gmsg_sendto_one(n, &packet, size);
+			gnet_stats_count_general(GNR_PUSH_PROXY_RELAYED, 1);
+
+			http_send_status(pp->socket, 202, FALSE, NULL, 0,
+					"Push-proxy: message sent to node");
+
+			pp->error_sent = 202;
+			pproxy_remove(pp, "Push sent directly to node GUID %s",
+					guid_hex_str(pp->guid));
+		}
 
 		return;
 	}
@@ -603,28 +796,43 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 * Bad luck, no direct connection.  Look for a Gnutella route.
 	 */
 
-	nodes = route_towards_guid(pp->guid);
+	if (NULL != (nodes = route_towards_guid(pp->guid))) {
+		const gchar *packet;
+		size_t size;
 
-	if (nodes != NULL) {
-		gint cnt;
+		/*
+ 		 * We set TTL=max_ttl-1 and hops=1 since
+		 * it does not come from our node really.
+		 */
 
-		build_push(&m, pp->guid, pp->ip, pp->port, pp->file_idx);
-		message_add(m.header.muid, GTA_MSG_PUSH_REQUEST, NULL);
+		packet = build_push(&size, max_ttl - 1, 1,
+					pp->guid, pp->addr_v4, pp->addr_v6, pp->port,
+					pp->file_idx, supports_tls);
 
-		gmsg_sendto_all(nodes, (gchar *) &m, sizeof(m));
-		gnet_stats_count_general(GNR_PUSH_PROXY_BROADCASTED, 1);
+		if (NULL == packet) {
+			g_warning("Failed to send push to %s/%s (index=%lu)",
+				host_addr_port_to_string(pp->addr_v4, pp->port),
+				host_addr_port_to_string2(pp->addr_v6, pp->port),
+				(gulong) pp->file_idx);
+		} else {
+			gint cnt;
+
+			gmsg_sendto_all(nodes, packet, size);
+			gnet_stats_count_general(GNR_PUSH_PROXY_BROADCASTED, 1);
+
+			cnt = g_slist_length(nodes);
+
+			http_send_status(pp->socket, 203, FALSE, NULL, 0,
+					"Push-proxy: message sent through Gnutella (via %d node%s)",
+					cnt, cnt == 1 ? "" : "s");
+
+			pp->error_sent = 203;
+			pproxy_remove(pp, "Push sent via Gnutella (%d node%s) for GUID %s",
+					cnt, cnt == 1 ? "" : "s", guid_hex_str(pp->guid));
+		}
 
-		cnt = g_slist_length(nodes);
 		g_slist_free(nodes);
-
-		http_send_status(pp->socket, 203, FALSE, NULL, 0,
-			"Push-proxy: message sent through Gnutella (via %d node%s)",
-			cnt, cnt == 1 ? "" : "s");
-
-		pp->error_sent = 203;
-		pproxy_remove(pp, "Push sent via Gnutella (%d node%s) for GUID %s",
-			cnt, cnt == 1 ? "" : "s", guid_hex_str(pp->guid));
-
+		nodes = NULL;
 		return;
 	}
 
@@ -633,8 +841,9 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 * sending a GIV back.
 	 */
 
-	if (guid_eq(pp->guid, guid)) {
-		upload_send_giv(pp->ip, pp->port, 0, 1, 0, "<from push-proxy>", FALSE);
+	if (guid_eq(pp->guid, servent_guid)) {
+		upload_send_giv(pp->addr_v4, pp->port, 0, 1, 0,
+			"<from push-proxy>", FALSE, pp->flags);
 
 		http_send_status(pp->socket, 202, FALSE, NULL, 0,
 			"Push-proxy: you found the target GUID %s",
@@ -660,7 +869,11 @@ pproxy_request(struct pproxy *pp, header_t *header)
  *** I/O header parsing callbacks.
  ***/
 
-#define PPROXY(x)	((struct pproxy *) (x))
+static inline struct pproxy *
+PPROXY(gpointer obj)
+{
+	return obj;
+}
 
 static void
 err_line_too_long(gpointer obj)
@@ -727,8 +940,6 @@ call_pproxy_request(gpointer obj, header_t *header)
 	pproxy_request(PPROXY(obj), header);
 }
 
-#undef PPROXY
-
 /**
  * Create new push-proxy request and begin reading HTTP headers.
  */
@@ -759,10 +970,10 @@ pproxy_close(void)
 	GSList *l;
 
 	for (l = pproxies; l; l = g_slist_next(l)) {
-		struct pproxy *pp = (struct pproxy *) l->data;
+		struct pproxy *pp = l->data;
 
 		pproxy_free_resources(pp);
-		wfree(pp, sizeof(*pp));
+		wfree(pp, sizeof *pp);
 	}
 
 	g_slist_free(pproxies);
@@ -781,12 +992,6 @@ static void cproxy_http_newstate(gpointer handle, http_state_t newstate);
 
 #define CPROXY_MAGIC	0xc8301U
 
-/*
- * Operating flags.
- */
-
-#define CP_F_SWAPPED_IP	0x00000001		/* Tried to swap IP address */
-
 /**
  * Free the structure and all its dependencies.
  */
@@ -809,7 +1014,7 @@ cproxy_free(struct cproxy *cp)
 	}
 
 	cp->magic = 0;
-	wfree(cp, sizeof(*cp));
+	wfree(cp, sizeof *cp);
 }
 
 /**
@@ -819,64 +1024,12 @@ static void
 cproxy_http_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 {
 	struct cproxy *cp = (struct cproxy *) http_async_get_opaque(handle);
-	guint32 new_ip;
 
 	g_assert(cp != NULL);
 	g_assert(cp->magic == CPROXY_MAGIC);
 
 	http_async_log_error(handle, type, v);
 
-	/*
-	 * Temporary hack.
-	 *
-	 * Old servents used little-endian for encoding IP addresses in PUSH.
-	 * If we were unable to connect, and we have not yet swapped the
-	 * IP address, try doing so now.
-	 *
-	 *	--RAM, 04/08/2003
-	 */
-
-	new_ip = swap_guint32(cp->ip);
-
-	if (
-		!(cp->flags & CP_F_SWAPPED_IP) &&
-		type == HTTP_ASYNC_ERROR &&
-		GPOINTER_TO_INT(v) == HTTP_ASYNC_CONN_TIMEOUT &&
-		host_is_valid(new_ip, cp->port)
-	) {
-		static gchar path[128];
-
-		(void) gm_snprintf(path, sizeof(path),
-			"/gnutella/push-proxy?ServerId=%s", guid_base32_str(cp->guid));
-
-		cp->flags |= CP_F_SWAPPED_IP;
-		cp->http_handle = http_async_get_ip(path, new_ip, cp->port,
-			cproxy_http_header_ind, NULL, cproxy_http_error_ind);
-
-		if (cp->http_handle != NULL) {
-			g_warning("retrying with swapped push-proxy address %s (was %s)",
-				ip_to_gchar(new_ip), ip_port_to_gchar(cp->ip, cp->port));
-			cp->ip = new_ip;
-
-			/*
-			 * Customize async HTTP layer.
-			 */
-
-			http_async_set_opaque(cp->http_handle, cp, NULL);
-			http_async_set_op_request(cp->http_handle, cproxy_build_request);
-			http_async_on_state_change(cp->http_handle, cproxy_http_newstate);
-
-			return;
-		}
-
-		cp->flags &= ~CP_F_SWAPPED_IP;
-
-		/* FALL THROUGH */
-	}
-
-	if (cp->flags & CP_F_SWAPPED_IP)
-		cp->ip = swap_guint32(cp->ip);		/* Restore original IP */
-
 	cp->http_handle = NULL;
 	cp->done = TRUE;
 
@@ -891,13 +1044,13 @@ cproxy_http_error_ind(gpointer handle, http_errtype_t type, gpointer v)
 
 /**
  * HTTP async callback for header reception notification.
- * Returns whether processing can continue.
+ * @returns whether processing can continue.
  */
 static gboolean
 cproxy_http_header_ind(gpointer handle, header_t *header,
 	gint code, const gchar *message)
 {
-	struct cproxy *cp = (struct cproxy *) http_async_get_opaque(handle);
+	struct cproxy *cp = http_async_get_opaque(handle);
 	gchar *token;
 	gchar *server;
 
@@ -914,7 +1067,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 	if (server == NULL)
 		server = header_get(header, "User-Agent");
 
-	cp->server = validate_vendor(server, token, cp->ip);
+	cp->server = validate_vendor(server, token, cp->addr);
 
 	/*
 	 * Don't continue past headers, we don't expect data, and besides the
@@ -945,7 +1098,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 		break;
 	case 400:
 		g_warning("push-proxy at %s (%s) for %s file #%u reported HTTP %d: %s",
-			ip_port_to_gchar(cp->ip, cp->port), cproxy_vendor_str(cp),
+			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx, code, message);
 		/* FALL THROUGH */
 	case 410:
@@ -954,7 +1107,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 	default:
 		g_warning("push-proxy at %s (%s) for %s file #%u "
 			"sent unexpected HTTP %d: %s",
-			ip_port_to_gchar(cp->ip, cp->port), cproxy_vendor_str(cp),
+			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx, code, message);
 		download_proxy_failed(cp->d);
 		break;
@@ -962,7 +1115,7 @@ cproxy_http_header_ind(gpointer handle, header_t *header,
 
 	if (dbg > 2 && cp->sent)
 		printf("PUSH-PROXY at %s (%s) sent PUSH for %s file #%u %s\n",
-			ip_port_to_gchar(cp->ip, cp->port), cproxy_vendor_str(cp),
+			host_addr_port_to_string(cp->addr, cp->port), cproxy_vendor_str(cp),
 			guid_hex_str(cp->guid), cp->file_idx,
 			cp->directly ? "directly" : "via Gnet");
 
@@ -982,6 +1135,10 @@ cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
 	const gchar *verb, const gchar *path, const gchar *unused_host,
 	guint16 unused_port)
 {
+	gchar addr_buf[128];
+	gchar addr_v6_buf[128];
+	host_addr_t addr;
+
 	/*
 	 * Note that we send an HTTP/1.0 request here, hence we need neither
 	 * the Host: nor the Connection: header.
@@ -992,15 +1149,32 @@ cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
 	(void) unused_port;
 	g_assert(len <= INT_MAX);
 
+	addr = listen_addr();
+	if (is_host_addr(addr)) {
+		gm_snprintf(addr_buf, sizeof addr_buf, "X-Node: %s\r\n",
+			host_addr_port_to_string(addr, listen_port));
+	} else {
+		addr_buf[0] = '\0';
+	}
+	addr = listen_addr6();
+	if (is_host_addr(addr)) {
+		gm_snprintf(addr_v6_buf, sizeof addr_v6_buf, "X-Node: %s\r\n",
+			host_addr_port_to_string(addr, listen_port));
+	} else {
+		addr_v6_buf[0] = '\0';
+	}
+	
 	return gm_snprintf(buf, len,
 		"%s %s HTTP/1.0\r\n"
 		"User-Agent: %s\r\n"
 		"X-Token: %s\r\n"
-		"X-Node: %s\r\n"
+		"%s"
+		"%s"
 		"\r\n",
 		verb, path, version_string,
 		tok_version(),
-		ip_port_to_gchar(listen_ip(), listen_port));
+		addr_buf,
+		addr_v6_buf);
 }
 
 /**
@@ -1009,7 +1183,7 @@ cproxy_build_request(gpointer unused_handle, gchar *buf, size_t len,
 static void
 cproxy_http_newstate(gpointer handle, http_state_t newstate)
 {
-	struct cproxy *cp = (struct cproxy *) http_async_get_opaque(handle);
+	struct cproxy *cp = http_async_get_opaque(handle);
 
 	g_assert(cp != NULL);
 	g_assert(cp->d != NULL);
@@ -1025,13 +1199,12 @@ cproxy_http_newstate(gpointer handle, http_state_t newstate)
  * @returns NULL if problem during connection.
  */
 struct cproxy *
-cproxy_create(struct download *d, guint32 ip, guint16 port,
+cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
 	gchar *guid, guint32 file_idx)
 {
 	struct cproxy *cp;
 	gpointer handle;
 	static gchar path[128];
-	gboolean swapped = FALSE;
 
 	(void) gm_snprintf(path, sizeof(path),
 		"/gnutella/push-proxy?ServerId=%s", guid_base32_str(guid));
@@ -1040,54 +1213,26 @@ cproxy_create(struct download *d, guint32 ip, guint16 port,
 	 * Try to connect immediately: if we can't connect, no need to continue.
 	 */
 
-	handle = http_async_get_ip(path, ip, port,
+	handle = http_async_get_addr(path, addr, port,
 		cproxy_http_header_ind, NULL, cproxy_http_error_ind);
 
-	/*
-	 * Temporary hack.
-	 *
-	 * Old servents used little-endian for encoding IP addresses in PUSH.
-	 * If we were unable to connect, try swapping the IP address.
-	 *
-	 *	--RAM, 04/08/2003
-	 */
-
-	if (handle == NULL) {
-		guint32 new_ip;
-
-		new_ip = swap_guint32(ip);
-		if (host_is_valid(new_ip, port)) {
-			g_warning("can't connect to push-proxy %s for GUID %s: %s "
-				"-- swapping to %s",
-				ip_port_to_gchar(ip, port), guid_hex_str(guid),
-				http_async_strerror(http_async_errno),
-				ip_to_gchar(new_ip));
-
-			ip = new_ip;
-			swapped = TRUE;
-
-			handle = http_async_get_ip(path, ip, port,
-				cproxy_http_header_ind, NULL, cproxy_http_error_ind);
-		}
-	}
-
 	if (handle == NULL) {
 		g_warning("can't connect to push-proxy %s for GUID %s: %s",
-			ip_port_to_gchar(ip, port), guid_hex_str(guid),
+			host_addr_port_to_string(addr, port), guid_hex_str(guid),
 			http_async_strerror(http_async_errno));
 		return NULL;
 	}
 
-	cp = walloc0(sizeof(*cp));
+	cp = walloc0(sizeof *cp);
 
 	cp->magic = CPROXY_MAGIC;
 	cp->d = d;
-	cp->ip = ip;
+	cp->addr = addr;
 	cp->port = port;
 	cp->guid = atom_guid_get(guid);
 	cp->file_idx = file_idx;
 	cp->http_handle = handle;
-	cp->flags = swapped ? CP_F_SWAPPED_IP : 0;
+	cp->flags = 0;
 
 	/*
 	 * Customize async HTTP layer.
@@ -1112,10 +1257,10 @@ cproxy_reparent(struct download *d, struct download *cd)
 	g_assert(cd->cproxy != NULL);
 	g_assert(d->cproxy == cd->cproxy);
 	g_assert(d->cproxy->magic == CPROXY_MAGIC);
-	
+
 	cd->cproxy->d = cd;
 	d->cproxy = NULL;
-	
+
 	g_assert(d->cproxy == NULL);
 	g_assert(cd->cproxy != NULL);
 	g_assert(cd == cd->cproxy->d);
