@@ -1,5 +1,5 @@
 /*
- * $Id: rx_link.c,v 1.13 2005/11/30 16:43:41 rmanfredi Exp $
+ * $Id: rx_link.c 11928 2006-09-09 19:42:18Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: rx_link.c,v 1.13 2005/11/30 16:43:41 rmanfredi Exp $");
+RCSID("$Id: rx_link.c 11928 2006-09-09 19:42:18Z rmanfredi $")
 
 #include "sockets.h"
 #include "pmsg.h"
@@ -66,11 +66,14 @@ struct attr {
 static void
 is_readable(gpointer data, gint unused_source, inputevt_cond_t cond)
 {
-	rxdrv_t *rx = (rxdrv_t *) data;
-	struct attr *attr = (struct attr *) rx->opaque;
-	pdata_t *db;
+	rxdrv_t *rx = data;
+	struct attr *attr = rx->opaque;
+	pdata_t *db[32];
+	struct iovec iov[G_N_ELEMENTS(db)];
 	pmsg_t *mb;
 	ssize_t r;
+	guint i, iov_cnt;
+	size_t avail;
 
 	(void) unused_source;
 	g_assert(attr->bio);			/* Input enabled */
@@ -80,38 +83,70 @@ is_readable(gpointer data, gint unused_source, inputevt_cond_t cond)
 		return;
 	}
 
-	/*
-	 * Grab an RX buffer, and try to fill as much as we can.
-	 */
-
-	db = rxbuf_new();
-
-	r = bio_read(attr->bio, pdata_start(db), pdata_len(db));
-	if (r == 0) {
-		attr->cb->got_eof(rx->owner);
-		goto error;
-	} else if ((ssize_t) -1 == r) {
-		if (errno != VAL_EAGAIN)
-			attr->cb->read_error(rx->owner, _("Read error: %s"),
-				g_strerror(errno));
-		goto error;
+	if (!inputevt_data_available(&avail)) {
+		/* buf_size should be equivalent to BUF_SIZE as defined in rxbuf.c.
+		 * If we don't know how much can be read immediately, we make a
+		 * guess. This prevents multiple readv() syscalls when reading from
+		 * a fast source which would occur otherwise. */
+		avail = 64 * 1024;
 	}
 
 	/*
-	 * Got something, build a message and send it to the upper layer.
-	 * NB: `mb' is expected to be freed by the last layer using it.
+	 * Grab RX buffers, and try to fill as much as we can.
 	 */
 
-	if (attr->cb->add_rx_given != NULL)
-		attr->cb->add_rx_given(rx->owner, r);
+	i = 0;
+	for (i = 0; i < G_N_ELEMENTS(db); /* NOTHING */) {
+		size_t len;
+		
+		db[i] = rxbuf_new();
+		len = pdata_len(db[i]);
+		iov[i].iov_base = pdata_start(db[i]);
+		iov[i].iov_len = len;
+		i++;
 
-	mb = pmsg_alloc(PMSG_P_DATA, db, 0, r);
+		if (len >= avail)
+			break;
+		avail -= len;
+	}
+	iov_cnt = i;
 
-	(*rx->data_ind)(rx, mb);
-	return;
+	i = 0;	/* To free all buffers on error */
+	r = bio_readv(attr->bio, iov, iov_cnt);
+	if (r == 0) {
+		attr->cb->got_eof(rx->owner);
+	} else if ((ssize_t) -1 == r) {
+		if (!is_temporary_error(errno))
+			attr->cb->read_error(rx->owner, _("Read error: %s"),
+				g_strerror(errno));
+	} else {
+		/*
+		 * Got something, build a message and send it to the upper layer.
+		 * NB: `mb' is expected to be freed by the last layer using it.
+		 */
 
-error:
-	rxbuf_free(db, NULL);
+		if (attr->cb->add_rx_given != NULL)
+			attr->cb->add_rx_given(rx->owner, r);
+
+		while (r > 0 && i < iov_cnt) {
+			size_t n = pdata_len(db[i]);
+
+			if (n > (size_t) r) {
+				n = (size_t) r;
+				r = 0;
+			} else {
+				r -= n;
+			}
+			mb = pmsg_alloc(PMSG_P_DATA, db[i], 0, n);
+			i++;
+			if (!(*rx->data_ind)(rx, mb))
+				break;
+		}
+	}
+
+	for (/* CONTINUE*/; i < iov_cnt; i++) {
+		rxbuf_free(db[i]);
+	}
 }
 
 /***
@@ -151,14 +186,14 @@ rx_link_init(rxdrv_t *rx, gconstpointer args)
 static void
 rx_link_destroy(rxdrv_t *rx)
 {
-	struct attr *attr = (struct attr *) rx->opaque;
+	struct attr *attr = rx->opaque;
 
 	if (attr->bio) {
 		bsched_source_remove(attr->bio);
 		attr->bio = NULL;					/* Paranoid */
 	}
 
-	wfree(attr, sizeof(*attr));
+	wfree(attr, sizeof *attr);
 }
 
 /**
@@ -166,8 +201,10 @@ rx_link_destroy(rxdrv_t *rx)
  *
  * Since we normally read from the network, we don't have to process those
  * data and can forward them directly to the upper layer.
+ *
+ * @return FALSE if there was an error or the receiver wants no more data.
  */
-static void
+static gboolean
 rx_link_recv(rxdrv_t *rx, pmsg_t *mb)
 {
 	struct attr *attr = rx->opaque;
@@ -183,7 +220,7 @@ rx_link_recv(rxdrv_t *rx, pmsg_t *mb)
 	 * NB: `mb' is expected to be freed by the last layer using it.
 	 */
 
-	(*rx->data_ind)(rx, mb);
+	return (*rx->data_ind)(rx, mb);
 }
 
 /**
@@ -192,7 +229,7 @@ rx_link_recv(rxdrv_t *rx, pmsg_t *mb)
 static void
 rx_link_enable(rxdrv_t *rx)
 {
-	struct attr *attr = (struct attr *) rx->opaque;
+	struct attr *attr = rx->opaque;
 
 	g_assert(attr->bio == NULL);
 
@@ -201,7 +238,7 @@ rx_link_enable(rxdrv_t *rx)
 	 */
 
 	attr->bio = bsched_source_add(attr->bs, attr->wio, BIO_F_READ,
-		is_readable, (gpointer) rx);
+					is_readable, rx);
 
 	g_assert(attr->bio);
 }
@@ -212,7 +249,7 @@ rx_link_enable(rxdrv_t *rx)
 static void
 rx_link_disable(rxdrv_t *rx)
 {
-	struct attr *attr = (struct attr *) rx->opaque;
+	struct attr *attr = rx->opaque;
 
 	/*
 	 * Disabling is blindly called when the RX stack is freed, regardless
@@ -237,7 +274,7 @@ rx_link_disable(rxdrv_t *rx)
 static struct
 bio_source *rx_link_bio_source(rxdrv_t *rx)
 {
-	struct attr *attr = (struct attr *) rx->opaque;
+	struct attr *attr = rx->opaque;
 
 	return attr->bio;
 }
