@@ -1,10 +1,8 @@
 /*
- * $Id: rxbuf.h,v 1.1 2004/08/24 22:53:22 rmanfredi Exp $
+ * $Id: rxbuf.h 11928 2006-09-09 19:42:18Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Network RX buffer allocator.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,6 +23,16 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Network RX buffer allocator.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #ifndef _core_rxbuf_h_
 #define _core_rxbuf_h_
 
@@ -35,7 +43,7 @@
  */
 
 pdata_t *rxbuf_new(void);
-void rxbuf_free(gpointer p, gpointer unused);
+void rxbuf_free(gpointer p);
 
 void rxbuf_init(void);
 void rxbuf_close(void);
