@@ -1,5 +1,5 @@
 /*
- * $Id: settings.h,v 1.2 2004/10/23 04:10:46 cbiere Exp $
+ * $Id: settings.h 11820 2006-08-30 02:10:23Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -23,39 +23,51 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Gtk-Gnutella configuration.
+ *
+ * @author Raphael Manfredi
+ * @author Richard Eckart
+ * @date 2001-2003
+ */
+
 #ifndef _core_settings_h_
 #define _core_settings_h_
 
 #include "common.h"
 
 #include "if/core/settings.h"
+#include "lib/host_addr.h"
 
-/*
- * Global Data
+/**
+ * Global Data.
  */
 
 extern struct in_addr *local_netmasks;
 
-/*
- * Global macros.
- */
-
-#define listen_ip()		(force_local_ip ? forced_local_ip : local_ip)
-
 
 /*
  * Global Functions
  */
 
+gboolean is_my_address(const host_addr_t addr, guint16 port);
+
+void settings_early_init(void);
+gint settings_ensure_unicity(gboolean silent);
 void settings_init(void);
 void settings_save_if_dirty(void);
 void settings_shutdown(void);
-void settings_ip_changed(guint32 new_ip, guint32 peer);
+void settings_addr_changed(const host_addr_t new_addr, const host_addr_t peer);
 guint32 settings_max_msg_size(void);
 void settings_close(void);
-void settings_ask_for_property(gchar *name, gchar *value);
+void settings_ask_for_property(const gchar *name,
+	const gchar *value) G_GNUC_NORETURN;
 
 guint32 get_average_servent_uptime(time_t now);
-guint32 get_average_ip_lifetime(time_t now);
+guint32 get_average_ip_lifetime(time_t now, enum net_type net);
 
 #endif /* _core_settings_h_ */
+/* vi: set ts=4 sw=4 cindent: */
