@@ -1,5 +1,5 @@
 /*
- * $Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $
+ * $Id: shell.c 12194 2006-11-03 01:33:16Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -25,9 +25,7 @@
 
 #include "common.h"
 
-#ifdef USE_REMOTE_CTRL
-
-RCSID("$Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $");
+RCSID("$Id: shell.c 12194 2006-11-03 01:33:16Z cbiere $")
 
 #include "shell.h"
 #include "sockets.h"
@@ -38,6 +36,7 @@ RCSID("$Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $");
 
 #include "if/bridge/c2ui.h"
 #include "if/bridge/ui2c.h"
+#include "if/core/main.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -46,6 +45,7 @@ RCSID("$Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $");
 #include "lib/getline.h"
 #include "lib/glib-missing.h"
 #include "lib/inputevt.h"
+#include "lib/iso3166.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
@@ -54,11 +54,13 @@ RCSID("$Id: shell.c,v 1.30 2005/12/17 10:35:24 cbiere Exp $");
 #define SHELL_BUFFER_SIZE 64000
 
 #define IS_PROCESSING(sh) (sh->outpos > 0)
-#define SHELL_TIMEOUT 60
 
-#define REPLY_READY       100
-#define REPLY_ERROR       400
-#define REPLY_GOOD_BYE    900
+enum shell_reply {
+	REPLY_NONE		= 0,
+	REPLY_READY		= 100,
+	REPLY_ERROR		= 400,
+	REPLY_GOOD_BYE	= 900
+};
 
 static GSList *sl_shells = NULL;
 
@@ -71,7 +73,10 @@ typedef struct gnutella_shell {
 	gboolean shutdown;  /**< In shutdown mode? */
 } gnutella_shell_t;
 
+#ifdef USE_REMOTE_CTRL
 static gchar auth_cookie[SHA1_RAW_SIZE];
+static gboolean shell_auth(const gchar *str);
+#endif	/* USE_REMOTE_CTRL */
 
 static void shell_shutdown(gnutella_shell_t *sh);
 static void shell_destroy(gnutella_shell_t *sh);
@@ -81,39 +86,52 @@ static void print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
 static void shell_handle_data(gpointer data, gint unused_source,
 	inputevt_cond_t cond);
 
-enum {
-	CMD_UNKNOWN,
-	CMD_NOOP,
-	CMD_QUIT,
-	CMD_SEARCH,
-	CMD_NODE,
+enum shell_cmd {
 	CMD_ADD,
 	CMD_HELP,
-	CMD_PRINT,
-	CMD_SET,
-	CMD_WHATIS,
 	CMD_HORIZON,
-	CMD_RESCAN
+	CMD_NODE,
+	CMD_NODES,
+	CMD_NOOP,
+	CMD_OFFLINE,
+	CMD_ONLINE,
+	CMD_PRINT,
+	CMD_PROPS,
+	CMD_QUIT,
+	CMD_RESCAN,
+	CMD_SEARCH,
+	CMD_SET,
+	CMD_SHUTDOWN,
+	CMD_STATUS,
+	CMD_WHATIS,
+
+	CMD_UNKNOWN
 };
 
 static const struct {
 	const gint id;
 	const gchar * const cmd;
 } commands[] = {
-	{	CMD_QUIT,		"QUIT"		},
-	{	CMD_SEARCH,		"SEARCH"	},
-	{	CMD_NODE,		"NODE"		},
-	{	CMD_ADD,		"ADD"		},
-	{	CMD_HELP,		"HELP"		},
-	{	CMD_PRINT,		"PRINT"		},
-	{	CMD_SET,		"SET"		},
-	{	CMD_WHATIS,		"WHATIS"	},
-	{	CMD_HORIZON,	"HORIZON"	},
-	{	CMD_RESCAN,		"RESCAN"	}
+	{	CMD_ADD,		"add"		},
+	{	CMD_HELP,		"help"		},
+	{	CMD_HORIZON,	"horizon"	},
+	{	CMD_NODE,		"node"		},
+	{	CMD_NODES,		"nodes"		},
+	{	CMD_OFFLINE,	"offline"  	},
+	{	CMD_ONLINE,		"online"   	},
+	{	CMD_PRINT,		"print"		},
+	{	CMD_PROPS,		"props"		},
+	{	CMD_QUIT,		"quit"		},
+	{	CMD_RESCAN,		"rescan"	},
+	{	CMD_SEARCH,		"search"	},
+	{	CMD_SET,		"set"		},
+	{	CMD_SHUTDOWN,	"shutdown"	},
+	{	CMD_STATUS,		"status"	},
+	{	CMD_WHATIS,		"whatis"	},
 };
 
 
-static gint
+static enum shell_cmd
 get_command(const gchar *cmd)
 {
 	guint i;
@@ -201,20 +219,26 @@ shell_unescape(gchar *s)
  * The returned string needs to be g_free-ed when no longer needed.
  */
 static gchar *
-shell_get_token(const gchar *s, gint *pos) {
+shell_get_token(const gchar *s, gint *pos)
+{
 	const gchar *start, *end;
 	gchar *retval;
 
 	g_assert(pos);
 	g_assert(s);
+	g_assert(-1 == *pos || *pos >= 0);
 
-	start = s+(*pos);
-
-	if (*start == '\0')
-		*pos = -1;
-
-	if (*pos == -1)
+	if (*pos >= 0) {
+		start = &s[*pos];
+		if (*start == '\0') {
+			*pos = -1;
+		}
+	} else {
+		start = NULL;	/* Suppress compiler warning */
+	}
+	if (*pos < 0) {
 		return NULL; /* nothing more to get */
+	}
 
 	end = shell_token_end(start);
 
@@ -231,7 +255,7 @@ shell_get_token(const gchar *s, gint *pos) {
 	return retval;
 }
 
-static guint
+static enum shell_reply
 shell_exec_node(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar *tok;
@@ -246,33 +270,37 @@ shell_exec_node(gnutella_shell_t *sh, const gchar *cmd)
 
 	switch (get_command(tok)) {
 	case CMD_ADD: {
-		gchar *tok_buf;
-		host_addr_t addr;
+		gchar *tok_buf, *tok_buf2;
+		const gchar *host, *end;
 		guint32 port = GTA_PORT;
+		gint flags = SOCK_F_FORCE;
 
-		tok_buf = shell_get_token(cmd, &pos);
-
-		if (tok_buf) {
-			addr = name_to_host_addr(tok_buf);
-			G_FREE_NULL(tok_buf);
-		} else
+		tok_buf2 = shell_get_token(cmd, &pos);
+		if (!tok_buf2)
 			goto error;
+		
+		host = tok_buf2;
+		end = is_strprefix(host, "tls:");
+		if (end) {
+			host = end;
+			flags |= SOCK_F_TLS;
+		}
 
 		tok_buf = shell_get_token(cmd, &pos);
-
 		if (tok_buf) {
 			gint error;
-			guint32 u;
 			
-			u = parse_uint32(tok_buf, NULL, 10, &error);
-			port = error || u > 65535 ? 0 : u;
+			port = parse_uint16(tok_buf, NULL, 10, &error);
 			G_FREE_NULL(tok_buf);
 		}
 
-		if (is_host_addr(addr) && port) {
-			node_add(addr, port, CONNECT_F_FORCE);
+		if (port) {
+			node_add_by_name(host, port, flags);
 			sh->msg = _("Node added");
-		} else {
+		}
+		G_FREE_NULL(tok_buf2);
+		
+		if (!port) {
 			sh->msg = _("Invalid IP/Port");
 			goto error;
 		}
@@ -295,12 +323,12 @@ error:
 	return REPLY_ERROR;
 }
 
-static guint
+static enum shell_reply
 shell_exec_search(gnutella_shell_t *sh, const gchar *cmd)
 {
+	enum shell_reply reply_code = REPLY_ERROR;
 	gchar *tok;
 	gint pos = 0;
-	guint reply_code = REPLY_ERROR;
 
 	g_assert(sh);
 	g_assert(cmd);
@@ -373,13 +401,12 @@ get_prop_stub_by_name(const gchar *tok_prop, prop_set_stub_t **stub_ptr)
 	return NO_PROP;
 }
 
-
-static guint
+static enum shell_reply
 shell_exec_print(gnutella_shell_t *sh, const gchar *cmd)
 {
+	enum shell_reply reply_code = REPLY_ERROR;
 	gchar *tok_prop;
 	gint pos = 0;
-	guint reply_code = REPLY_ERROR;
 	prop_set_stub_t *stub = NULL;
 	property_t prop;
 
@@ -419,13 +446,13 @@ error:
 	return REPLY_ERROR;
 }
 
-static guint
+static enum shell_reply
 shell_exec_set(gnutella_shell_t *sh, const gchar *cmd)
 {
+	enum shell_reply reply_code = REPLY_ERROR;
 	gchar *tok_prop;
 	gchar *tok_value;
 	gint pos = 0;
-	guint reply_code = REPLY_ERROR;
 	prop_set_stub_t *stub = NULL;
 	property_t prop;
 	prop_def_t *prop_buf = NULL;
@@ -541,12 +568,12 @@ failure:
 /**
  * Takes a whatis command and tries to execute it.
  */
-static guint
+static enum shell_reply
 shell_exec_whatis(gnutella_shell_t *sh, const gchar *cmd)
 {
+	enum shell_reply reply_code = REPLY_ERROR;
 	gchar *tok_prop;
 	gint pos = 0;
-	guint reply_code = REPLY_ERROR;
 	prop_set_stub_t *stub = NULL;
 	prop_def_t *prop_buf = NULL;
 	property_t prop;
@@ -595,7 +622,7 @@ error:
 /**
  * Rescan the shared directories for added/removed files.
  */
-static guint
+static enum shell_reply
 shell_exec_rescan(gnutella_shell_t *sh, const gchar *cmd)
 {
 	g_assert(sh);
@@ -613,7 +640,7 @@ shell_exec_rescan(gnutella_shell_t *sh, const gchar *cmd)
 /**
  * Displays horizon size information.
  */
-static guint
+static enum shell_reply
 shell_exec_horizon(gnutella_shell_t *sh, const gchar *cmd)
 {
 	gchar buf[200];
@@ -631,7 +658,7 @@ shell_exec_horizon(gnutella_shell_t *sh, const gchar *cmd)
     if (tok != NULL) {
 		shell_write(sh, tok);
 		shell_write(sh, "\n");
-		if (0 == ascii_strcasecmp(tok, "ALL")) {
+		if (0 == ascii_strcasecmp(tok, "all")) {
 			all = TRUE;
 		} else {
         	sh->msg = _("Unknown parameter");
@@ -671,8 +698,7 @@ shell_exec_horizon(gnutella_shell_t *sh, const gchar *cmd)
 			gm_snprintf(buf, sizeof buf,
 				_("Horizon size via HSEP node %s (%s):"),
 				node_addr(n),
-				NODE_IS_LEAF(n) ? _("leaf") :
-					(NODE_IS_ULTRA(n) ? _("ultrapeer") : _("normal node")));
+				node_peermode_to_string(n->peermode));
 
 			shell_write(sh, buf);
 			shell_write(sh, "\n\n");
@@ -776,7 +802,8 @@ print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
 
 		s1 = uint64_to_string(t[i][HSEP_IDX_NODES] + non_hsep[HSEP_IDX_NODES]);
 		s2 = uint64_to_string2(t[i][HSEP_IDX_FILES] + non_hsep[HSEP_IDX_FILES]);
-		s3 = short_kb_size(t[i][HSEP_IDX_KIB] + non_hsep[HSEP_IDX_KIB]);
+		s3 = short_kb_size(t[i][HSEP_IDX_KIB] + non_hsep[HSEP_IDX_KIB],
+				display_metric_units);
 
 		gm_snprintf(buf, sizeof buf, "%*d  %*s  %*s  %*s\n",
 			maxlen[0], i + 1,
@@ -789,38 +816,328 @@ print_hsep_table(gnutella_shell_t *sh, hsep_triple *table,
 
 }
 
+static void
+print_node_info(gnutella_shell_t *sh, const struct gnutella_node *n)
+{
+	gnet_node_flags_t flags;
+	time_delta_t up, con;
+	time_t now;
+	gchar buf[1024];
+	gchar vendor_escaped[20];
+	gchar uptime_buf[8];
+	gchar contime_buf[8];
 
+	g_return_if_fail(sh);
+	g_return_if_fail(n);
+	
+	if (!NODE_IS_ESTABLISHED(n)) {
+		return;
+	}
+
+	now = tm_time();
+	con = n->connect_date ? delta_time(now, n->connect_date) : 0;
+	up = n->up_date ? delta_time(now, n->up_date) : 0;
+	node_fill_flags(n->node_handle, &flags);
+
+	{
+		const gchar *vendor;
+		gchar *escaped;
+		
+		vendor = node_vendor(n);
+		escaped = hex_escape(vendor, TRUE);
+		g_strlcpy(vendor_escaped, escaped, sizeof vendor_escaped);
+		if (escaped != vendor) {
+			G_FREE_NULL(escaped);
+		}
+	}
+
+	g_strlcpy(uptime_buf, up > 0 ? compact_time(up) : "?",
+		sizeof uptime_buf);
+	g_strlcpy(contime_buf, con > 0 ? compact_time(con) : "?",
+		sizeof contime_buf);
+
+	gm_snprintf(buf, sizeof buf,
+		"%-21.45s %5.1u %s %2.2s %6.6s %6.6s %.30s",
+		node_addr(n),
+		(guint) n->gnet_port,
+		node_flags_to_string(&flags),
+		iso3166_country_cc(n->country),
+		contime_buf,
+		uptime_buf,
+		vendor_escaped);
+
+	shell_write(sh, buf);
+	shell_write(sh, "\n");	/* Terminate line */
+}
+
+/**
+ * Displays all connected nodes
+ */
+static enum shell_reply
+shell_exec_nodes(gnutella_shell_t *sh, const gchar *cmd)
+{
+	const GSList *sl;
+
+	g_assert(sh);
+	g_assert(cmd);
+	g_assert(!IS_PROCESSING(sh));
+
+	sh->msg = "";
+
+	shell_write(sh,
+	  "100~ \n"
+	  "Node                  Port  Flags       CC Since  Uptime User-Agent\n");
+
+	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
+		const struct gnutella_node *n = sl->data;
+		print_node_info(sh, n);
+	}
+	shell_write(sh, ".\n");	/* Terminate message body */
+
+	return REPLY_READY;
+}
+
+/**
+ * Displays assorted status information
+ */
+static enum shell_reply
+shell_exec_stat(gnutella_shell_t *sh)
+{
+	gchar buf[2048];
+	time_t now;
+
+	g_assert(sh);
+
+	now = tm_time();
+
+	/* General status */ 
+	{
+		const gchar *blackout;
+		short_string_t leaf_switch;
+		short_string_t ultra_check;
+	
+		leaf_switch = timestamp_get_string(node_last_ultra_leaf_switch);
+		ultra_check = timestamp_get_string(node_last_ultra_check);
+
+		if (is_firewalled && is_udp_firewalled)
+			blackout = "TCP,UDP";
+		else if (is_firewalled)
+			blackout = "TCP";
+		else if (is_udp_firewalled)
+			blackout = "UDP";
+		else
+			blackout = "No";
+
+		gm_snprintf(buf, sizeof buf,
+			"+---------------------------------------------------------+\n"
+			"|                      Status                             |\n"
+			"|=========================================================|\n"
+			"|   Mode: %-9s  Last Switch: %-19s     |\n"
+			"| Uptime: %-9s   Last Check: %-19s     |\n"
+			"|   Port: %-5u         Blackout: %-7s                 |\n"
+			"|=========================================================|\n",
+			online_mode
+				? guc_node_peermode_to_string(current_peermode)
+				: "offline",
+			node_last_ultra_leaf_switch ? leaf_switch.str : "never",
+			short_time(delta_time(now, start_stamp)),
+			node_last_ultra_check ? ultra_check.str : "never",
+			socket_listen_port(),
+			blackout);
+		shell_write(sh, buf);
+	}
+
+	/* IPv4 info */ 
+	switch (network_protocol) {
+	case NET_USE_BOTH:
+	case NET_USE_IPV4:
+		gm_snprintf(buf, sizeof buf,
+			"| IPv4 Address: %-17s Last Change: %-9s  |\n",
+			host_addr_to_string(listen_addr()),
+			short_time(delta_time(now, current_ip_stamp)));
+		shell_write(sh, buf);
+	}
+
+	/* IPv6 info */ 
+	switch (network_protocol) {
+	case NET_USE_BOTH:
+		shell_write(sh,
+			"|---------------------------------------------------------|\n");
+		/* FALL THROUGH */
+	case NET_USE_IPV6:
+		gm_snprintf(buf, sizeof buf,
+			"| IPv6 Address: %-39s   |\n"
+			"|                                 Last Change: %-9s  |\n",
+			host_addr_to_string(listen_addr6()),
+			short_time(delta_time(now, current_ip6_stamp)));
+		shell_write(sh, buf);
+	}
+
+	/* Node counts */
+	gm_snprintf(buf, sizeof buf,
+		"|=========================================================|\n"
+		"| Connected Peers: %-4u                                   |\n"
+		"|    Ultra %4u/%-4u   Leaf %4u/%-4u   Legacy %4u/%-4u  |\n"
+		"|=========================================================|\n",
+		node_ultra_count + node_leaf_count + node_normal_count,
+		node_ultra_count,
+		NODE_P_ULTRA == current_peermode ? max_connections : max_ultrapeers,
+		node_leaf_count,
+		max_leaves,
+		node_normal_count,
+		normal_connections);
+	shell_write(sh, buf);
+
+	/* Bandwidths */
+	{	
+		const gboolean metric = display_metric_units;
+		short_string_t gnet_in, http_in, leaf_in, gnet_out, http_out, leaf_out;
+		gnet_bw_stats_t bw_stats;
+
+		gnet_get_bw_stats(BW_GNET_IN, &bw_stats);
+		gnet_in = short_rate_get_string(bw_stats.average, metric);
+
+		gnet_get_bw_stats(BW_GNET_OUT, &bw_stats);
+		gnet_out = short_rate_get_string(bw_stats.average, metric);
+		
+		gnet_get_bw_stats(BW_HTTP_IN, &bw_stats);
+		http_in = short_rate_get_string(bw_stats.average, metric);
+		
+		gnet_get_bw_stats(BW_HTTP_OUT, &bw_stats);
+		http_out = short_rate_get_string(bw_stats.average, metric);
+		
+		gnet_get_bw_stats(BW_LEAF_IN, &bw_stats);
+		leaf_in = short_rate_get_string(bw_stats.average, metric);
+
+		gnet_get_bw_stats(BW_LEAF_OUT, &bw_stats);
+		leaf_out = short_rate_get_string(bw_stats.average, metric);
+
+		gm_snprintf(buf, sizeof buf,
+			"| Bandwidth:           GNet          HTTP          Leaf   |\n"
+			"|---------------------------------------------------------|\n"
+			"|        In:    %11s   %11s   %11s   |\n"
+			"|       Out:    %11s   %11s   %11s   |\n"
+		    "+_________________________________________________________+\n",	
+			gnet_in.str, http_in.str, leaf_in.str,
+			gnet_out.str, http_out.str, leaf_out.str);
+		shell_write(sh, buf);
+	}
+
+	return REPLY_READY;
+}
+
+/**
+ * Close GNet connections
+ */
+static enum shell_reply
+shell_exec_offline(gnutella_shell_t *sh)
+{
+	gnet_prop_set_boolean_val(PROP_ONLINE_MODE, FALSE);
+	shell_write(sh, "Closing GNet connections\n");
+
+	return REPLY_READY;
+}
+
+/**
+ * Open GNet connections
+ */
+static enum shell_reply
+shell_exec_online(gnutella_shell_t *sh)
+{
+	gnet_prop_set_boolean_val(PROP_ONLINE_MODE, TRUE);
+	shell_write(sh, "Opening GNet connections\n");
+
+	return REPLY_READY;
+}
+
+/**
+ * Display all properties
+ */
+static enum shell_reply
+shell_exec_props(gnutella_shell_t *sh, const gchar *args)
+{
+	GSList *props, *sl;
+
+	g_assert(sh);
+	g_assert(args);
+	
+	props = gnet_prop_get_by_regex('\0' == args[0] ? "." : args, NULL);
+	if (!props) {
+		sh->msg = _("No matching property.");
+		return REPLY_ERROR;
+	}
+
+	for (sl = props; NULL != sl; sl = g_slist_next(sl)) {
+		const gchar *name_1, *name_2;
+		property_t prop;
+		gchar buf[80];
+	   
+		prop = GPOINTER_TO_UINT(sl->data);
+		name_1 = gnet_prop_name(prop);
+
+		if (g_slist_next(sl)) {
+			sl = g_slist_next(sl);
+			prop = GPOINTER_TO_UINT(sl->data);
+			name_2 = gnet_prop_name(prop);
+		} else {
+			name_2 = "";
+		}
+
+		gm_snprintf(buf, sizeof buf, "%-34.34s  %-34.34s\n", name_1, name_2);
+		shell_write(sh, buf);
+	}
+	g_slist_free(props);
+	props = NULL;
+
+	return REPLY_READY;
+}
 
 /**
  * Takes a command string and tries to parse and execute it.
  */
-static guint
+static enum shell_reply
 shell_exec(gnutella_shell_t *sh, const gchar *cmd)
 {
+	enum shell_reply reply_code = REPLY_ERROR;
+	const gchar *args;
 	gchar *tok;
 	gint pos = 0;
-	guint reply_code = REPLY_ERROR;
 
 	g_assert(sh);
 	g_assert(cmd);
 
 	tok = shell_get_token(cmd, &pos);
 	if (!tok)
-		return CMD_NOOP;
+		return REPLY_NONE;
+
+	if (pos >= 0) {
+		args = &cmd[pos];
+	} else {
+		args = "";
+	}
 
 	switch (get_command(tok)) {
 	case CMD_HELP:
 		shell_write(sh,
-			"100-Help:\n"
-			"100-SEARCH ADD <query>\n"
-			"100-NODE ADD <ip> [port]\n"
-			"100-PRINT <property>\n"
-			"100-SET <property> <value>\n"
-			"100-WHATIS <property>\n"
-			"100-HORIZON [ALL]\n"
-			"100-RESCAN\n"
-			"100-QUIT\n"
-			"100-HELP\n");
+			"100~ \n"
+			"The following commands are available:\n"
+			"help\n"
+			"horizon [all]\n"
+			"node add <ip> [port]\n"
+			"nodes\n"
+			"offline\n"
+			"online\n"
+			"print <property>\n"
+			"props [<regex>]\n"
+			"quit\n"
+			"rescan\n"
+			"search add <query>\n"
+			"set <property> <value>\n"
+			"shutdown\n"
+			"status\n"
+			"whatis <property>\n"
+		);
+		shell_write(sh, ".\n");
 		reply_code = REPLY_READY;
 		break;
 	case CMD_QUIT:
@@ -828,28 +1145,54 @@ shell_exec(gnutella_shell_t *sh, const gchar *cmd)
 		reply_code = REPLY_GOOD_BYE;
 		shell_shutdown(sh);
 		break;
+	case CMD_SHUTDOWN:
+		sh->msg = _("Shutdown sequence initiated.");
+		reply_code = REPLY_READY;
+		/*
+		 * Don't use gtk_gnutella_exit() because we want at least send
+		 * some feedback before terminating. 
+		 */
+		gtk_gnutella_request_shutdown();
+		break;
 	case CMD_SEARCH:
-		reply_code = shell_exec_search(sh, &cmd[pos]);
+		reply_code = shell_exec_search(sh, args);
 		break;
 	case CMD_NODE:
-		reply_code = shell_exec_node(sh, &cmd[pos]);
+		reply_code = shell_exec_node(sh, args);
 		break;
 	case CMD_PRINT:
-		reply_code = shell_exec_print(sh, &cmd[pos]);
+		reply_code = shell_exec_print(sh, args);
 		break;
 	case CMD_SET:
-		reply_code = shell_exec_set(sh, &cmd[pos]);
+		reply_code = shell_exec_set(sh, args);
 		break;
 	case CMD_WHATIS:
-		reply_code = shell_exec_whatis(sh, &cmd[pos]);
+		reply_code = shell_exec_whatis(sh, args);
 		break;
 	case CMD_HORIZON:
-		reply_code = shell_exec_horizon(sh, &cmd[pos]);
+		reply_code = shell_exec_horizon(sh, args);
 		break;
 	case CMD_RESCAN:
-		reply_code = shell_exec_rescan(sh, &cmd[pos]);
+		reply_code = shell_exec_rescan(sh, args);
 		break;
-	default:
+	case CMD_NODES:
+		reply_code = shell_exec_nodes(sh, args);
+		break;
+	case CMD_STATUS:
+		reply_code = shell_exec_stat(sh);
+		break;
+	case CMD_OFFLINE:
+		reply_code = shell_exec_offline(sh);
+		break;
+	case CMD_ONLINE:
+		reply_code = shell_exec_online(sh);
+		break;
+	case CMD_PROPS:
+		reply_code = shell_exec_props(sh, args);
+		break;
+	case CMD_ADD:
+	case CMD_NOOP:
+	case CMD_UNKNOWN:
 		goto error;
 	}
 
@@ -887,7 +1230,7 @@ shell_write_data(gnutella_shell_t *sh)
 	written = s->wio.write(&s->wio, sh->outbuf, sh->outpos);
 	switch (written) {
 	case (ssize_t) -1:
-		if (errno == VAL_EAGAIN || errno == EINTR)
+		if (is_temporary_error(errno))
 			return;
 
 		/* FALL THRU */
@@ -918,8 +1261,6 @@ static void
 shell_read_data(gnutella_shell_t *sh)
 {
 	struct gnutella_socket *s;
-	gint parsed;
-	ssize_t rc = -1;
 
 	g_assert(sh);
 
@@ -929,50 +1270,48 @@ shell_read_data(gnutella_shell_t *sh)
 	sh->last_update = tm_time();
 	s = sh->socket;
 
-	if (s->pos >= sizeof(s->buffer))
+	if (s->buf_size - s->pos < 1) {
 		g_warning("Remote shell: Read more than buffer size.\n");
-	else {
-		gchar *p = s->buffer + s->pos;
-		size_t size = sizeof(s->buffer) - s->pos - 1;
+	} else {
+		size_t size = s->buf_size - s->pos - 1;
+		ssize_t ret;
 
-		rc = s->wio.read(&s->wio, p, size);
-		if (rc <= 0) {
-			if (rc == 0) {
-				if (s->pos == 0) {
-					g_warning("shell connection closed: EOF");
-					shell_destroy(sh);
-					return;
-				}
-			} else {
-				g_warning("Receiving data failed: %s\n",
-					g_strerror(errno));
+		ret = s->wio.read(&s->wio, &s->buf[s->pos], size);
+		if (0 == ret) {
+			if (0 == s->pos) {
+				g_message("shell connection closed: EOF");
 				shell_destroy(sh);
 				return;
 			}
+		} else if ((ssize_t) -1 == ret) {
+			if (!is_temporary_error(errno)) {
+				g_warning("Receiving data failed: %s\n", g_strerror(errno));
+				shell_destroy(sh);
+				return;
+			}
+		} else {
+			s->pos += ret;
 		}
-		s->pos += rc;
 	}
 
-	while (s->pos) {
-		guint reply_code;
+	while (s->pos > 0) {
+		enum shell_reply reply_code;
+		size_t parsed;
 
-		g_assert (s->pos > 0);
-
-		switch (getline_read(s->getline, s->buffer, s->pos, &parsed)) {
+		switch (getline_read(s->getline, s->buf, s->pos, &parsed)) {
 		case READ_OVERFLOW:
 			g_warning("Line is too long (from shell at %s)\n",
 				host_addr_port_to_string(s->addr, s->port));
 			shell_destroy(sh);
 			return;
 		case READ_DONE:
-			if (s->pos != (size_t) parsed)
-				memmove(s->buffer, &s->buffer[parsed], s->pos - parsed);
+			if (s->pos != parsed)
+				memmove(s->buf, &s->buf[parsed], s->pos - parsed);
 			s->pos -= parsed;
 			break;
 		case READ_MORE:
-		default:
-			g_assert((size_t) parsed == s->pos);
-
+			g_assert(parsed == s->pos);
+			s->pos = 0;
 			return;
 		}
 
@@ -981,7 +1320,7 @@ shell_read_data(gnutella_shell_t *sh)
 		 */
 
 		reply_code = shell_exec(sh, getline_str(s->getline));
-		if (CMD_NOOP != reply_code) {
+		if (REPLY_NONE != reply_code) {
 			gchar *buf = NULL;
 			size_t size;
 
@@ -1011,7 +1350,7 @@ shell_handle_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	g_assert(sh);
 
 	if (cond & INPUT_EVENT_EXCEPTION) {
-		g_warning ("shell connection closed: exception\n");
+		g_warning ("shell connection closed: exception");
 		shell_destroy(sh);
 		return;
 	}
@@ -1064,38 +1403,6 @@ shell_write(gnutella_shell_t *sh, const gchar *s)
 	return TRUE;
 }
 
-/**
- * Takes a HELO command string and checks whether the connection
- * is allowed using the specified credentials.
- *
- * @return TRUE if the connection is allowed.
- */
-static gboolean
-shell_auth(const gchar *str)
-{
-	gboolean ok;
-	gchar *tok_helo;
-	gchar *tok_cookie;
-	gint pos = 0;
-
-	tok_helo = shell_get_token(str, &pos);
-	tok_cookie = shell_get_token(str, &pos);
-
-	g_warning("auth: [%s] [<cookie not displayed>]", tok_helo);
-
-	if (tok_helo && tok_cookie) {
-		ok = strcmp("HELO", tok_helo) == 0 &&
-			strcmp(sha1_base32(auth_cookie), tok_cookie) == 0;
-	} else {
-		ok = FALSE;
-	}
-
-	G_FREE_NULL(tok_helo);
-	G_FREE_NULL(tok_cookie);
-
-	return ok;
-}
-
 /**
  * Create a new gnutella_shell object.
  */
@@ -1150,8 +1457,7 @@ shell_destroy(gnutella_shell_t *sh)
 		sh->outbuf = NULL;
 	}
 
-	socket_free(sh->socket);
-	sh->socket = NULL;
+	socket_free_null(&sh->socket);
 	shell_free(sh);
 }
 
@@ -1171,6 +1477,7 @@ void
 shell_add(struct gnutella_socket *s)
 {
 	gnutella_shell_t *sh;
+	gboolean granted = FALSE;
 
 	g_assert(s);
 	g_assert(0 == s->gdk_tag);
@@ -1189,15 +1496,36 @@ shell_add(struct gnutella_socket *s)
 
 	sl_shells = g_slist_prepend(sl_shells, sh);
 
-	if (!enable_shell) {
-		g_warning("shell control interface disabled");
-		shell_write(sh, "401 Disabled\n");
-		shell_shutdown(sh);
-	} else if (!shell_auth(getline_str(s->getline))) {
-		g_warning("invalid credentials");
-		shell_write(sh, "400 Invalid credentials\n");
-		shell_shutdown(sh);
+	if (socket_is_local(s)) {
+		if (enable_local_socket) {
+			granted = TRUE;
+		} else {
+			g_warning("local shell control interface disabled");
+			shell_write(sh, "401 Disabled\n");
+			shell_shutdown(sh);
+		}
 	} else {
+#ifdef USE_REMOTE_CTRL
+		if (enable_shell) {
+		   	if (shell_auth(getline_str(s->getline))) {
+				granted = TRUE;
+			} else {
+				g_warning("invalid credentials");
+				shell_write(sh, "400 Invalid credentials\n");
+				shell_shutdown(sh);
+			}
+		} else {
+			g_warning("remote shell control interface disabled");
+			shell_write(sh, "401 Disabled\n");
+			shell_shutdown(sh);
+		}
+#else	/* !USE_REMOTE_CTRL */
+		g_warning("remote shell control interface disabled");
+		shell_shutdown(sh);
+#endif	/* USE_REMOTE_CTRL */
+	}
+
+	if (!sh->shutdown && granted) {
 		shell_write(sh, "100 Welcome to ");
 		shell_write(sh, version_short_string);
 		shell_write(sh, "\n");
@@ -1205,10 +1533,63 @@ shell_add(struct gnutella_socket *s)
 
 	getline_reset(s->getline); /* clear AUTH command from buffer */
 
-	if ((sh->outpos == 0) && sh->shutdown) {
+	if (sh->outpos == 0 && sh->shutdown) {
+		shell_destroy(sh);
+	}
+}
+
+void
+shell_timer(time_t now)
+{
+	GSList *sl, *to_remove = NULL;
+
+	for (sl = sl_shells; sl != NULL; sl = g_slist_next(sl)) {
+		gnutella_shell_t *sh = sl->data;
+		time_delta_t timeout = remote_shell_timeout;
+
+		if (timeout > 0 && delta_time(now, sh->last_update) > timeout)
+			to_remove = g_slist_prepend(to_remove, sh);
+	}
+
+	for (sl = to_remove; sl != NULL; sl = g_slist_next(sl)) {
+		gnutella_shell_t *sh = sl->data;
 		shell_destroy(sh);
 	}
 
+	g_slist_free(to_remove);
+}
+
+#ifdef USE_REMOTE_CTRL
+/**
+ * Takes a HELO command string and checks whether the connection
+ * is allowed using the specified credentials.
+ *
+ * @return TRUE if the connection is allowed.
+ */
+static gboolean
+shell_auth(const gchar *str)
+{
+	gboolean ok;
+	gchar *tok_helo;
+	gchar *tok_cookie;
+	gint pos = 0;
+
+	tok_helo = shell_get_token(str, &pos);
+	tok_cookie = shell_get_token(str, &pos);
+
+	g_warning("auth: [%s] [<cookie not displayed>]", tok_helo);
+
+	if (tok_helo && tok_cookie) {
+		ok = strcmp("HELO", tok_helo) == 0 &&
+			strcmp(sha1_base32(auth_cookie), tok_cookie) == 0;
+	} else {
+		ok = FALSE;
+	}
+
+	G_FREE_NULL(tok_helo);
+	G_FREE_NULL(tok_cookie);
+
+	return ok;
 }
 
 static void
@@ -1231,29 +1612,10 @@ shell_dump_cookie(void)
 	file_config_close(out, &fp);
 }
 
-void
-shell_timer(time_t now)
-{
-	GSList *sl, *to_remove = NULL;
-
-	for (sl = sl_shells; sl != NULL; sl = g_slist_next(sl)) {
-		gnutella_shell_t *sh = sl->data;
-
-		if (delta_time(now, sh->last_update) > SHELL_TIMEOUT)
-			to_remove = g_slist_prepend(to_remove, sh);
-	}
-
-	for (sl = to_remove; sl != NULL; sl = g_slist_next(sl)) {
-		gnutella_shell_t *sh = sl->data;
-		shell_destroy(sh);
-	}
-
-	g_slist_free(to_remove);
-}
-
 void
 shell_init(void)
 {
+
 	gint n;
 
 	for (n = 0; n < SHA1_RAW_SIZE; n ++) {
@@ -1265,6 +1627,13 @@ shell_init(void)
 
 	shell_dump_cookie();
 }
+#else	/* !USE_REMOTE_CTRL */
+void
+shell_init(void)
+{
+	/* Nothing to do */
+}
+#endif	/* USE_REMOTE_CTRL */
 
 void
 shell_close(void)
@@ -1282,4 +1651,3 @@ shell_close(void)
 }
 
 /* vi: set ts=4 sw=4 cindent: */
-#endif	/* USE_REMOTE_CTRL */
