@@ -1,5 +1,5 @@
-/* 
- * $Id: sq.c,v 1.4 2004/09/20 20:59:17 rmanfredi Exp $
+/*
+ * $Id: sq.c 12147 2006-10-27 19:55:09Z cbiere $
  *
  * Copyright (c) 2002-2003, Alex Bennee <alex@bennee.com> & Raphael Manfredi
  *
@@ -24,6 +24,7 @@
  */
 
 /**
+ * @ingroup core
  * @file
  *
  * Search queue.
@@ -37,11 +38,15 @@
  * this system of having one search queue per node is not used.  Instead,
  * we have one global search queue, which is used to space launching
  * of dynamic queries.
+ *
+ * @author Alex Bennee <alex@bennee.com>
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: sq.c,v 1.4 2004/09/20 20:59:17 rmanfredi Exp $");
+RCSID("$Id: sq.c 12147 2006-10-27 19:55:09Z cbiere $")
 
 #include "sq.h"					/* search_queue structures */
 #include "pmsg.h"
@@ -52,16 +57,11 @@ RCSID("$Id: sq.c,v 1.4 2004/09/20 20:59:17 rmanfredi Exp $");
 #include "if/gnet_property_priv.h"
 
 #include "lib/atoms.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-/*
- * Compute start of search string (which is NUL terminated) in query.
- * The "+2" skips the "speed" field in the query.
- */
-#define QUERY_TEXT(m)	((m) + sizeof(struct gnutella_header) + 2)
-
-/*
+/**
  * A search queue entry.
  *
  * Each entry references the search that issued the query.  Before sending
@@ -69,21 +69,21 @@ RCSID("$Id: sq.c,v 1.4 2004/09/20 20:59:17 rmanfredi Exp $");
  * over-querying for that particular search.
  */
 typedef struct smsg {
-	pmsg_t *mb;					/* The message block for the query */
-	gnet_search_t shandle;		/* Handle to search that originated query */
-	query_hashvec_t *qhv;		/* The query hash vector for QRP matching */
+	pmsg_t *mb;					/**< The message block for the query */
+	gnet_search_t shandle;		/**< Handle to search that originated query */
+	query_hashvec_t *qhv;		/**< The query hash vector for QRP matching */
 } smsg_t;
 
-/*
+/**
  * Message information for mutated blocks.
  *
  * Records meta-information about the message being queued so that we may
  * react when the message queue informs us it has processed it.
  */
 struct smsg_info {
-	gpointer search;			/* The search object which sends the query */
-	guint32 id;					/* The unique search ID */
-	guint32 node_id;			/* The unique node ID to which we're sending */
+	gpointer search;			/**< The search object which sends the query */
+	guint32 id;					/**< The unique search ID */
+	guint32 node_id;			/**< The unique node ID to which we're sending */
 };
 
 static squeue_t *global_sq = NULL;
@@ -217,7 +217,7 @@ sqh_remove(squeue_t *sq, gnet_search_t sh)
 				GUINT_TO_POINTER(sh), &key, &value);
 
 	g_assert(found);
-	g_assert((gnet_search_t) key == sh);
+	g_assert((gnet_search_t) GPOINTER_TO_UINT(key) == sh);
 
 	g_hash_table_remove(sq->handles, GUINT_TO_POINTER(sh));
 }
@@ -247,7 +247,7 @@ sq_make(struct gnutella_node *node)
 	 */
 
 	sq->count		= 0;
-	sq->last_sent 	= time(NULL);
+	sq->last_sent 	= tm_time();
 	sq->searches 	= NULL;
 	sq->n_sent 		= 0;
 	sq->n_dropped 	= 0;
@@ -269,7 +269,7 @@ sq_clear(squeue_t *sq)
 
 	if (dbg > 3)
 		printf("clearing sq node %s (sent=%d, dropped=%d)\n",
-			sq->node ? node_ip(sq->node) : "GLOBAL",
+			sq->node ? node_addr(sq->node) : "GLOBAL",
 			sq->n_sent, sq->n_dropped);
 
 	for (l = sq->searches; l; l = g_list_next(l)) {
@@ -330,6 +330,7 @@ sq_puthere(squeue_t *sq, gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
  * to remove the queries when a search is closed, and avoid queuing twice
  * the same search.
  *
+ * @param sq DOCUMENT THIS!
  * @param mb the query message
  * @param sh the search handle
  */
@@ -359,6 +360,7 @@ sq_global_putq(gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 void
 sq_process(squeue_t *sq, time_t now)
 {
+    time_delta_t spacing = search_queue_spacing;
 	GList *item;
 	smsg_t *sb;
 	struct gnutella_node *n;
@@ -382,7 +384,7 @@ retry:
 	if (sq->count == 0)
 		return;
 
-    if (delta_time(now, sq->last_sent) < search_queue_spacing)
+    if (delta_time(now, sq->last_sent) < spacing)
 		return;
 
 	n = sq->node;					/* Will be NULL for the global SQ */
@@ -430,7 +432,8 @@ retry:
 
 		if (dbg > 2)
 			printf("sq GLOBAL, queuing \"%s\" (%u left, %d sent)\n",
-				QUERY_TEXT(pmsg_start(sb->mb)), sq->count, sq->n_sent);
+				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
+				sq->count, sq->n_sent);
 
 		dq_launch_local(sb->shandle, sb->mb, sb->qhv);
 
@@ -444,7 +447,7 @@ retry:
 
 		if (dbg > 2)
 			printf("sq for node %s, queuing \"%s\" (%u left, %d sent)\n",
-				node_ip(n), QUERY_TEXT(pmsg_start(sb->mb)),
+				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 
 		/*
@@ -462,7 +465,7 @@ retry:
 	} else {
 		if (dbg > 4)
 			printf("sq for node %s, ignored \"%s\" (%u left, %d sent)\n",
-				node_ip(n), QUERY_TEXT(pmsg_start(sb->mb)),
+				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 		pmsg_free(sb->mb);
 		if (sb->qhv)
@@ -508,7 +511,8 @@ cap_queue(squeue_t *sq)
 
 		if (dbg > 4)
 			printf("sq for node %s, dropped \"%s\" (%u left, %d dropped)\n",
-				node_ip(sq->node), QUERY_TEXT(pmsg_start(sb->mb)),
+				node_addr(sq->node),
+				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_dropped);
 
 		sqh_remove(sq, sb->shandle);
@@ -541,8 +545,8 @@ sq_search_closed(squeue_t *sq, gnet_search_t sh)
 
 		if (dbg > 4)
 			printf("sq for node %s, dropped \"%s\" on search close (%u left)\n",
-				sq->node ? node_ip(sq->node) : "GLOBAL",
-				QUERY_TEXT(pmsg_start(sb->mb)), sq->count);
+				sq->node ? node_addr(sq->node) : "GLOBAL",
+				gnutella_msg_search_get_text(pmsg_start(sb->mb)), sq->count);
 
 		sqh_remove(sq, sb->shandle);
 		smsg_discard(sb);
@@ -572,7 +576,7 @@ sq_set_peermode(node_peer_t mode)
 }
 
 /**
- * Returns global queue.
+ * @returns global queue.
  */
 squeue_t *
 sq_global_queue(void)
