@@ -1,5 +1,5 @@
 /*
- * $Id: token.c,v 1.15 2006/02/22 16:02:31 rmanfredi Exp $
+ * $Id: token.c 12293 2006-11-09 22:55:17Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: token.c,v 1.15 2006/02/22 16:02:31 rmanfredi Exp $");
+RCSID("$Id: token.c 12293 2006-11-09 22:55:17Z cbiere $")
 
 #include "token.h"
 #include "clock.h"
@@ -132,6 +132,56 @@ static const gchar *keys_096_1[] = {
 	"9536 535d 0e1b e017 2f73 a0bc c01a dcf9",
 };
 
+static const gchar *keys_096_2[] = {
+	"fa04 15ab 5f1d 52cc 5a5d 865b 27b5 c5a5",
+	"7f0b 5987 86ec a100 85c5 d0fc f504 b58c",
+	"2b06 f3a7 05a8 2769 2679 1ab9 9ba0 2360",
+	"4c3f 5636 3ba7 471b 43ea 3350 d71f b0fd",
+	"00bf c06b 3703 78e3 87c2 ea7a 3a8c b684",
+	"5103 6580 7be1 9cab fecb 3c46 12a9 4287",
+	"a5c5 a8f0 d10e 2202 526a f62e d615 afe2",
+	"32b8 5d94 5e4b c936 8703 2386 5f67 aa6b",
+	"10f8 e454 5d88 ec00 7e44 636f cda5 e45a",
+	"0409 a3aa b426 c79a 0d17 8895 f595 d744",
+	"9f9f c223 b290 0e7b 0950 d991 1ed9 bae0",
+	"a297 f754 a185 81f1 d57d 7b82 fde8 c7d1",
+	"d3de 3537 3271 0649 c2e8 de86 842e 1b79",
+	"86d1 3b44 b4eb 4842 1736 d42b 8b0b ce30",
+	"3b93 8172 1331 99fe 1423 c188 642b c45b",
+	"f013 188b 833d de3b 49d9 3709 6067 fd88",
+	"42ad 9a84 808e 7638 601a 6d33 9543 0827",
+	"ba4e d65e 1768 320d c88f 7b16 3194 0e08",
+	"ef2b e292 e29f c2ef 96a2 bc3c fbcb bb96",
+	"71a7 a828 ce50 d736 11a5 e687 ad42 ce5c",
+	"266b be05 0d52 f825 37e8 ffe3 b4f1 282c",
+	"e75d b41f 2332 309d c3ee 4dd7 060d fc08",
+};
+
+static const gchar *keys_096_3[] = {
+	"ca43 91c3 d080 c596 5ec2 9e17 5011 ebb6",
+	"4b53 6e49 a6a8 c045 c0c4 bfc7 8c54 4d9a",
+	"411b fc41 6643 ba3a 527a 52c9 4a72 3db0",
+	"13aa 0ae0 54f8 c9b3 2af8 3faa c363 cc4b",
+	"ebd2 69db d313 b7da 2425 9412 1ce3 16be",
+	"faf3 1626 20db 8b5a d448 0887 a22e c156",
+	"e925 4acd 65d3 f64d daea 0766 b260 ae03",
+	"62ff b5f7 02bf 189c a17f 7fec 3845 2d8e",
+	"efe6 4c84 2149 aee7 0e61 9192 b293 8f5d",
+	"47ec defc f33c 698e a8d4 eb06 c5bd d062",
+	"4aab bc51 4caf f6c2 676d f60c 6e6e 6ad6",
+	"afad 3bbc 4981 0a4d a0b9 a247 697f 8052",
+	"c244 2aff d148 3a7b 3da0 f837 7207 6795",
+	"8b8b 3562 29c5 3d4c 3e9a 46ca 47ff 5e18",
+	"4973 9c6e bbb0 6a26 29e9 92d6 300b c4cb",
+	"c9fd b78f 37d5 029d 7e99 ce84 d052 7efb",
+	"1520 f5bd 1a53 8250 ca3e 8ced 4d1f 1b38",
+	"850a 2b7b f978 16fa 3465 a243 c66f 3796",
+	"fdf1 fbdf 54bb 32a0 44b1 3892 30b4 ea5c",
+	"8a44 58ba ddac b04f 9806 1bce 7a7d 486e",
+	"6cc1 768e 0d38 e314 c388 88be 7bf9 2dea",
+	"c994 a380 dde0 baff 8900 c9e5 d94c b9ff",
+};
+
 /**
  * Describes the keys to use depending on the version.
  */
@@ -142,17 +192,25 @@ struct tokkey {
 } token_keys[] = {
 	/* Keep this array sorted by increasing timestamp */
 	{
-		{ 0, 96, 0, 'b', 0, 1132614000 },			/* 2005-11-22 */
+		{ 0, 96, 0, 'b', 0, 0, 1132614000 },		/* 2005-11-22 */
 		keys_096b, G_N_ELEMENTS(keys_096b),
 	},
 	{
-		{ 0, 96, 0, '\0', 0, 1138057200 },			/* 2006-01-24 */
+		{ 0, 96, 0, '\0', 0, 0, 1138057200 },		/* 2006-01-24 */
 		keys_096, G_N_ELEMENTS(keys_096),
 	},
 	{
-		{ 0, 96, 1, '\0', 0, 1140562800 },			/* 2006-02-22 */
+		{ 0, 96, 1, '\0', 0, 0, 1140562800 },		/* 2006-02-22 */
 		keys_096_1, G_N_ELEMENTS(keys_096_1),
 	},
+	{
+		{ 0, 96, 2, '\0', 0, 0, 1161025877 },		/* 2006-10-16 */
+		keys_096_2, G_N_ELEMENTS(keys_096_2),
+	},
+	{
+		{ 0, 96, 3, '\0', 0, 0, 1163108781 },		/* 2006-11-09 */
+		keys_096_3, G_N_ELEMENTS(keys_096_3),
+	},
 };
 
 /**
@@ -174,13 +232,18 @@ static const gchar *tok_errstr[] = {
 	"Level too short",				/**< TOK_SHORT_LEVEL */
 	"Level mismatch",				/**< TOK_INVALID_LEVEL */
 	"Missing level",				/**< TOK_MISSING_LEVEL */
+	"Missing build number",			/**< TOK_MISSING_BUILD */
+	"Wrong build number",			/**< TOK_WRONG_BUILD */
 };
 
 /**
  * @return human-readable error string corresponding to error code `errnum'.
  */
-const gchar *tok_strerror(tok_error_t errnum)
+const gchar *
+tok_strerror(tok_error_t errnum)
 {
+	STATIC_ASSERT(G_N_ELEMENTS(tok_errstr) == TOK_MAX_ERROR);
+
 	if ((gint) errnum < 0 || errnum >= G_N_ELEMENTS(tok_errstr))
 		return "Invalid error code";
 
@@ -192,7 +255,8 @@ const gchar *tok_strerror(tok_error_t errnum)
  *
  * @return NULL if we cannot locate any suitable keys.
  */
-static const struct tokkey *find_tokkey(time_t now)
+static const struct tokkey *
+find_tokkey(time_t now)
 {
 	time_t adjusted = now - VERSION_ANCIENT_BAN;
 	const struct tokkey *tk;
@@ -207,14 +271,34 @@ static const struct tokkey *find_tokkey(time_t now)
 	return NULL;
 }
 
+/**
+ * Find latest token structure that is anterior or equal to the remote version.
+ */
+static const struct tokkey *
+find_latest(const version_t *rver)
+{
+	guint i;
+	const struct tokkey *tk;
+	const struct tokkey *result = NULL;
+
+	for (i = 0; i < G_N_ELEMENTS(token_keys); i++) {
+		tk = &token_keys[i];
+		if (version_build_cmp(&tk->ver, rver) > 0)
+			break;
+		result = tk;
+	}
+
+	return result;
+}
+
 /**
  * Pickup a key randomly.
  *
  * @returns the key string and the index within the key array into `idx'
  * and the token key structure used in `tkused'.
  */
-static const gchar *random_key(
-	time_t now, guint *idx, const struct tokkey **tkused)
+static const gchar *
+random_key(time_t now, guint *idx, const struct tokkey **tkused)
 {
 	static gboolean warned = FALSE;
 	guint random_idx;
@@ -241,7 +325,8 @@ static const gchar *random_key(
 /**
  * Generate new token for given version string.
  */
-static gchar *tok_generate(time_t now, const gchar *version)
+static gchar *
+tok_generate(time_t now, const gchar *version)
 {
 	gchar token[TOKEN_BASE64_SIZE + 1];
 	gchar digest[TOKEN_VERSION_SIZE];
@@ -324,7 +409,8 @@ static gchar *tok_generate(time_t now, const gchar *version)
  * It is not meant to be used for strict authentication management, since
  * the algorithm and the keys are exposed publicly.
  */
-gchar *tok_version(void)
+gchar *
+tok_version(void)
 {
 	static time_t last_generated = 0;
 	static gchar *toklevel = NULL;
@@ -356,7 +442,8 @@ gchar *tok_version(void)
  *
  * @returns a pointer to static data.
  */
-gchar *tok_short_version(void)
+gchar *
+tok_short_version(void)
 {
 	static time_t last_generated = 0;
 	static gchar *toklevel = NULL;
@@ -389,7 +476,8 @@ gchar *tok_short_version(void)
  *
  * @returns error code, or TOK_OK if token is valid.
  */
-tok_error_t tok_version_valid(
+tok_error_t
+tok_version_valid(
 	const gchar *version, const gchar *tokenb64, gint len, host_addr_t addr)
 {
 	time_t now = tm_time();
@@ -397,6 +485,7 @@ tok_error_t tok_version_valid(
 	guint32 stamp32;
 	const struct tokkey *tk;
 	const struct tokkey *rtk;
+	const struct tokkey *latest;
 	guint idx;
 	const gchar *key;
 	SHA1Context ctx;
@@ -460,19 +549,31 @@ tok_error_t tok_version_valid(
 	if (!version_fill(version, &rver))		/* Remote version */
 		return TOK_BAD_VERSION;
 
-	if (version_cmp(&rver, &tk->ver) < 0)
+	if (version_build_cmp(&rver, &tk->ver) < 0)
 		return TOK_OLD_VERSION;
 
+	if (end == NULL)
+		return TOK_MISSING_LEVEL;
+
+	latest = find_latest(&rver);
+	if (latest == NULL)						/* Unknown in our key set */
+		return TOK_OLD_VERSION;
+
+	/*
+	 * Verify build.
+	 */
+
+	if (rver.timestamp >= 1156543200) {		/* 2006-08-26 */
+		if (0 == rver.build)
+			return TOK_MISSING_BUILD;
+		if (rver.build < latest->ver.build)
+			return TOK_WRONG_BUILD;
+	}
+
 	/*
 	 * Verify level.
 	 */
 
-	if (end == NULL) {						/* No level */
-		if (rver.timestamp >= 1046127600)	/* 25/02/2003 */
-			return TOK_MISSING_LEVEL;
-		return TOK_OK;
-	}
-
 	lvllen = len - toklen - 2;				/* Forget about "; " */
 	end += 2;								/* Skip "; " */
 
@@ -536,7 +637,8 @@ tok_error_t tok_version_valid(
  * Check whether the version is too ancient to be able to generate a proper
  * token string identifiable by remote parties.
  */
-gboolean tok_is_ancient(time_t now)
+gboolean
+tok_is_ancient(time_t now)
 {
 	return find_tokkey(now) == NULL;
 }
