@@ -1,10 +1,8 @@
 /*
- * $Id: tx_deflate.h,v 1.2 2004/09/03 15:05:07 cbiere Exp $
+ * $Id: tx_deflate.h 9430 2005-09-07 17:24:05Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Network driver -- compressing layer.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,6 +23,16 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Network driver -- compressing layer.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #ifndef _core_tx_deflate_h_
 #define _core_tx_deflate_h_
 
@@ -33,12 +41,24 @@
 
 const struct txdrv_ops *tx_deflate_get_ops(void);
 
-/*
+/**
+ * Callbacks used by the deflating layer.
+ */
+struct tx_deflate_cb {
+	void (*add_tx_deflated)(gpointer owner, gint amount);
+	void (*shutdown)(gpointer owner, const gchar *reason, ...);
+};
+
+/**
  * Arguments to be passed when the layer is intantiated.
  */
 struct tx_deflate_args {
-	txdrv_t *nd;				/* Network driver underneath us (link) */
-	cqueue_t *cq;				/* Callout queue to use */
+	struct tx_deflate_cb *cb;	/**< Callbacks */
+	cqueue_t *cq;				/**< Callout queue to use */
+	size_t buffer_size;			/**< Internal buffer size to use */
+	size_t buffer_flush;		/**< Flush after that many bytes */
+	gboolean nagle;				/**< Whether to use Nagle or not */
+	gboolean gzip;				/**< Whether to use gzip encapsulation */
 };
 
 #endif	/* _core_tx_deflate_h_ */
