@@ -1,5 +1,5 @@
 /*
- * $Id: verify_tth.h,v 1.2 2004/10/06 05:19:36 cbiere Exp $
+ * $Id: verify_tth.h 12058 2006-09-27 19:33:55Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -22,7 +22,17 @@
  *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *----------------------------------------------------------------------
  */
- 
+
+/**
+ * @ingroup core
+ * @file
+ *
+ * Tigertree hash verification.
+ *
+ * @author Raphael Manfredi
+ * @date 2001-2003
+ */
+
 #ifndef _core_verify_tth_h_
 #define _core_verify_tth_h_
 
@@ -31,7 +41,7 @@ struct shared_file;
 void tt_verify_init(void);
 void tt_verify_close(void);
 
-void request_tigertree(struct shared_file *sf);
+void request_tigertree(const struct shared_file *sf);
 void tt_compute_close(void);
 
 #endif /* _core_verify_tth_h_ */
