@@ -1,5 +1,5 @@
 /*
- * $Id: version.c,v 1.19 2005/12/21 13:43:04 rmanfredi Exp $
+ * $Id: version.c 12111 2006-10-15 17:10:46Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: version.c,v 1.19 2005/12/21 13:43:04 rmanfredi Exp $");
+RCSID("$Id: version.c 12111 2006-10-15 17:10:46Z cbiere $")
 
 #ifdef I_SYS_UTSNAME
 #include <sys/utsname.h>		/* For uname() */
@@ -48,6 +48,8 @@ RCSID("$Id: version.c,v 1.19 2005/12/21 13:43:04 rmanfredi Exp $");
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
+#include "if/core/main.h"
+
 #include "lib/atoms.h"
 #include "lib/getdate.h"
 #include "lib/misc.h"
@@ -66,6 +68,27 @@ static version_t last_rel_version;
 static version_t last_dev_version;
 static guint8 version_code;
 
+/**
+ * Compare the given timestamp against the date 2007-03-22. We don't
+ * care about the exact date here so there might be some offset of a few
+ * hours with respect to the timezone or daylight saving.
+ *
+ * @param now A timestamp.
+ * @return TRUE if 2007-03-22 is still in the future, FALSE otherwise.
+ */
+static gboolean
+before_2007_03_22(time_t now)
+{
+	static gboolean initialized;
+	static time_t x;
+
+	if (!initialized) {
+		initialized = TRUE;
+		x = date2time("2007-03-22", tm_time());
+	}
+	return delta_time(now, x) < 0;
+}
+
 /**
  * Get version string.
  */
@@ -91,9 +114,9 @@ static void
 version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
 {
 	printf("VERSION%s \"%s\":\n"
-		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u\n",
+		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u build=%u\n",
 		cmptag, str, ver->major, ver->minor, ver->patchlevel,
-		ver->tag ? ver->tag : ' ', ver->taglevel);
+		ver->tag ? ver->tag : ' ', ver->taglevel, ver->build);
 }
 
 /**
@@ -117,6 +140,9 @@ version_str(const version_t *ver)
 			rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
 	}
 
+	if (ver->build)
+		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
+
 	if (ver->timestamp) {
 		struct tm *tmp = localtime(&ver->timestamp);
 		rw += gm_snprintf(&str[rw], sizeof(str)-rw, " (%d-%02d-%02d)",
@@ -189,6 +215,7 @@ static gboolean
 version_parse(const gchar *str, version_t *ver)
 {
 	const gchar *v;
+	gint error;
 
 	/*
 	 * Modern version numbers are formatted like this:
@@ -207,61 +234,96 @@ version_parse(const gchar *str, version_t *ver)
 	 *    gtk-gnutella/0.90.1b2 (24/06/2002; X11; Linux 2.4.18-2emi i686)
 	 *
 	 * where the third number is the "patchlevel".
+	 *
+	 * Starting 2006-08-26, the user-agent string includes the SVN revision
+	 * at the time of the build.  To be compatible with the servents out
+	 * there, the version is included as an optional tag following the date.
+	 *
+	 *    gtk-gnutella/0.85 (04/04/2002; r3404; X11; FreeBSD 4.6-STABLE i386)
+	 *
+	 * However, we also start parsing the new format that we shall use when
+	 * all current GTKG out there have expired:
+	 *
+	 *    gtk-gnutella/0.85-3404 (04/04/2002; X11; FreeBSD 4.6-STABLE i386)
+	 *    gtk-gnutella/0.90b2-3404 (04/04/2002; X11; FreeBSD 4.6-STABLE i386)
+	 *
+	 * where the '-3404' introduces the build number.
 	 */
 
 	if (NULL == (v = is_strprefix(str, "gtk-gnutella/")))
 		return FALSE;
 
-	if (
-		5 == sscanf(v, "%u.%u.%u%c%u",
-			&ver->major, &ver->minor, &ver->patchlevel,
-			&ver->tag, &ver->taglevel)
-	)
-		goto ok;
+	/*
+	 * Parse "major.minor", followed by optional ".patchlevel".
+	 */
 
-	ver->taglevel = 0;
+	ver->major = parse_uint32(v, &v, 10, &error);
+	if (error)
+		return FALSE;
 
-	if (
-		4 == sscanf(v, "%u.%u.%u%c",
-			&ver->major, &ver->minor, &ver->patchlevel, &ver->tag)
-	) {
-		if (!isalpha(ver->tag))		/* Not a letter */
-			ver->tag = '\0';
-		goto ok;
+	if (*v++ != '.')
+		return FALSE;
+
+	ver->minor = parse_uint32(v, &v, 10, &error);
+	if (error)
+		return FALSE;
+
+	if ('.' == *v) {
+		v++;
+		ver->patchlevel = parse_uint32(v, &v, 10, &error);
+		if (error)
+			return FALSE;
+	} else
+		ver->patchlevel = 0;
+
+	/*
+	 * Parse optional tag letter "x" followed by optional "taglevel".
+	 */
+
+	if (is_ascii_alpha(*v)) {
+		ver->tag = *v++;
+		if (is_ascii_digit(*v)) {
+			ver->taglevel = parse_uint32(v, &v, 10, &error);
+			if (error)
+				return FALSE;
+		} else
+			ver->taglevel = 0;
+	} else {
+		ver->tag = '\0';
+		ver->taglevel = 0;
 	}
 
-	ver->tag = '\0';
+	/*
+	 * Parse optional "-build".
+	 */
 
-	if (
-		3 == sscanf(v, "%u.%u.%u",
-			&ver->major, &ver->minor, &ver->patchlevel)
-	)
-		goto ok;
+	if ('-' == *v) {
+		v++;
+		ver->build = parse_uint32(v, &v, 10, &error);
+		if (error)
+			return FALSE;
+	} else
+		ver->build = 0;
 
-	ver->patchlevel = 0;
+	/*
+	 * Legacy support: until 2007-03-22, we look at the build revision
+	 * number in the user-agent string if we did not get it so far.
+	 * We expect to find something like "(2006-02-22; r3456"
+	 *
+	 * Remove this code after 2007-03-22!
+	 */
 
-	if (
-		4 == sscanf(v, "%u.%u%c%u",
-			&ver->major, &ver->minor, &ver->tag, &ver->taglevel)
-	)
-		goto ok;
+	if (0 == ver->build) {	/* Useless after 2007-03-22 normally */
+		const gchar *p;
 
-	ver->taglevel = 0;
+		p = strchr(str, '(');		/* ')' for vi */
+		if (p)
+			p = strchr(p, ';');		/* Move past date */
 
-	if (3 == sscanf(v, "%u.%u%c", &ver->major, &ver->minor, &ver->tag)) {
-		if (!isalpha(ver->tag))		/* Not a letter */
-			ver->tag = '\0';
-		goto ok;
+		if (p && NULL != (p = is_strprefix(p, "; r")))
+			ver->build = (guint32) atoi(p);
 	}
 
-	ver->tag = '\0';
-
-	if (2 == sscanf(v, "%u.%u", &ver->major, &ver->minor))
-		goto ok;
-
-	return FALSE;
-
-ok:
 	if (dbg > 6)
 		version_dump(str, ver, "#");
 
@@ -284,17 +346,17 @@ version_tagcmp(guchar a, guchar b)
 	if (b == '\0')			/* Stable release has no tag */
 		return -1;
 
-	if (a == 'u')			/* Unstable from CVS */
+	if (a == 'u')			/* Unstable from SVN */
 		return -1;
 
-	if (b == 'u')			/* Unstable from CVS */
+	if (b == 'u')			/* Unstable from SVN */
 		return +1;
 
 	return a < b ? -1 : +1;	/* 'a' or 'b' for Alpha / Beta */
 }
 
 /**
- * Compare two gtk-gnutella versions, timestamp not withstanding.
+ * Compare two gtk-gnutella versions, timestamp and build not withstanding.
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
 gint
@@ -317,6 +379,20 @@ version_cmp(const version_t *a, const version_t *b)
 	return a->major < b->major ? -1 : +1;
 }
 
+/**
+ * Compare two gtk-gnutella versions, including build number (but not
+ * the version timestamps).  In theory, we could simply compare build numbers
+ * but we can't ensure they won't ever be reset one day.
+ *
+ * @returns -1, 0 or +1 depending on the sign of "a - b".
+ */
+gint
+version_build_cmp(const version_t *a, const version_t *b)
+{
+	gint cmp = version_cmp(a, b);
+	return 0 == cmp ? CMP(a->build, b->build) : cmp;
+}
+
 /**
  * Parse vendor string and fill supplied version structure `vs'.
  *
@@ -350,7 +426,7 @@ version_new_found(const gchar *text, gboolean stable)
 
 	if ('\0' != last_stable[0] && '\0' != last_dev[0]) {
 		gm_snprintf(s, sizeof s,
-			_(" - Newer versions available: release %s / from CVS %s"),
+			_(" - Newer versions available: release %s / from SVN %s"),
 			last_stable, last_dev);
 	} else if ('\0' != last_stable[0]) {
 		gm_snprintf(s, sizeof s,
@@ -358,7 +434,7 @@ version_new_found(const gchar *text, gboolean stable)
 			last_stable);
 	} else if ('\0' != last_dev[0]) {
 		gm_snprintf(s, sizeof s,
-			_(" - Newer version available: from CVS %s"),
+			_(" - Newer version available: from SVN %s"),
 			last_dev);
 	}
 
@@ -462,19 +538,20 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 
 	/*
 	 * If timestamp is greater and we were comparing against a stable
-	 * release, and cmp == 0, then this means an update in CVS about
+	 * release, and cmp == 0, then this means an update in SVN about
 	 * a "released" version, probably alpha/beta.
 	 */
 
 	if (
 		cmp == 0 &&
-		delta_time(their_version.timestamp, target_version->timestamp) > 0 &&
+		(delta_time(their_version.timestamp, target_version->timestamp) > 0
+			|| their_version.build > target_version->build) &&
 		target_version == &last_rel_version
 	) {
 		if (dbg > 6)
-			printf("VERSION is a CVS update of a release\n");
+			printf("VERSION is a SVN update of a release\n");
 
-		if (version_cmp(&last_dev_version, &their_version) > 0) {
+		if (version_build_cmp(&last_dev_version, &their_version) > 0) {
 			if (dbg > 6)
 				printf("VERSION is less recent than latest dev we know\n");
 			return TRUE;
@@ -483,23 +560,23 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 	}
 
 	/*
-	 * Only compare a development version which has the same version
-	 * number with a more up to date version.
-	 * This will also avoid a 0.93.4u to be listed out of date if also a 0.94u
-	 * is available, while both are actually in development.
-	 * 		-- JA 15/04/2004
+	 * Their version is more recent, but is unstable -- only continue if
+	 * our version is also unstable.
 	 */
 
-	/*
-	 * Their version is more recent, but is unstable
-	 */
-	if (their_version.tag == 'u' && cmp < 0)
+	if (cmp < 0 && their_version.tag == 'u' && our_version.tag != 'u')
 		return TRUE;
 
-	if (delta_time(their_version.timestamp, target_version->timestamp) <= 0)
+	if (
+		delta_time(their_version.timestamp, target_version->timestamp) < 0 ||
+		their_version.build <= target_version->build
+	)
 		return TRUE;
 
-	if (delta_time(their_version.timestamp, our_version.timestamp) == 0)
+	if (
+		delta_time(their_version.timestamp, our_version.timestamp) == 0 &&
+		their_version.build <= our_version.build
+	)
 		return TRUE;
 
 	/*
@@ -521,7 +598,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 
 	version =  version_str(&their_version);
 
-	g_warning("more recent %s version of gtk-gnutella: %s",
+	g_message("more recent %s version of gtk-gnutella: %s",
 		target_version == &last_dev_version ? "development" : "released",
 		version);
 
@@ -533,52 +610,102 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 	return TRUE;
 }
 
+/**
+ * Generates the version string. This function does not require any
+ * initialization, thus may be called very early e.g., for showing
+ * version information if the executable was invoked with --version
+ * as argument.
+ *
+ * @return A pointer to a static buffer holding the version string.
+ */
+const gchar *
+version_build_string(void)
+{
+	static gboolean initialized;
+	static gchar buf[128];
+
+	if (!initialized) {
+		time_t now = tm_time_exact();
+		const gchar *sysname = "Unknown";
+		const gchar *machine = NULL;
+
+		initialized = TRUE;
+
+#ifdef HAS_UNAME
+		{
+			static struct utsname un;	/* Must survive this scope */
+
+			if (-1 != uname(&un)) {
+				sysname = un.sysname;
+				machine = un.machine;
+			} else {
+				g_warning("uname() failed: %s:", g_strerror(errno));
+			}
+		}
+#endif /* HAS_UNAME */
+
+		/*
+		 * Until 2007-03-22, we generate the version string using "r<build#>"
+		 * in the optional part.  After that date, the revision format is
+		 * expanded to include the build number right into the string.
+		 */
+
+		if (before_2007_03_22(now))				/* before 2007-03-22 */
+			gm_snprintf(buf, sizeof buf,
+					"gtk-gnutella/%s (%s; r%u; %s; %s%s%s)",
+					GTA_VERSION_NUMBER, GTA_RELEASE, main_get_build(),
+					GTA_INTERFACE,
+					sysname,
+					machine && machine[0] ? " " : "",
+					machine ? machine : "");
+		else								/* after 2007-03-22 */
+			gm_snprintf(buf, sizeof buf,
+					"gtk-gnutella/%s-%u (%s; %s; %s%s%s)",
+					GTA_VERSION_NUMBER, main_get_build(),
+					GTA_RELEASE, GTA_INTERFACE,
+					sysname,
+					machine && machine[0] ? " " : "",
+					machine ? machine : "");
+	}
+	return buf;
+}
+
 /**
  * Initialize version string.
  */
 void
 version_init(void)
 {
-	gchar buf[128];
-	time_t now = tm_time();
-	const gchar *sysname = "Unknown";
-	const gchar *machine = NULL;
-	gboolean ok;
+	time_t now;
+
+	version_string = atom_str_get(version_build_string());
+	now = tm_time();
 
-#ifdef HAS_UNAME
 	{
-		static struct utsname un;	/* Must survive this scope */
-	
-		if (0 == uname(&un)) {
-			sysname = un.sysname;
-			machine = un.machine;
-		} else {
-			g_warning("uname() failed: %s:", g_strerror(errno));
-		}
+		gboolean ok;
+		ok = version_parse(version_string, &our_version);
+		g_assert(ok);
 	}
-#endif /* HAS_UNAME */
-
-	gm_snprintf(buf, sizeof buf,
-		"gtk-gnutella/%s (%s; %s; %s%s%s)",
-		GTA_VERSION_NUMBER, GTA_RELEASE, GTA_INTERFACE,
-		sysname,
-		machine && machine[0] ? " " : "",
-		machine ? machine : "");
-
-	version_string = atom_str_get(buf);
-	ok = version_parse(version_string, &our_version);
-	g_assert(ok);
 
 	g_message("%s", version_string);
 
 	version_stamp(version_string, &our_version);
 	g_assert(our_version.timestamp != 0);
 
-	gm_snprintf(buf, sizeof(buf),
-		"gtk-gnutella/%s (%s)",
-		GTA_VERSION_NUMBER, GTA_RELEASE);
+	{
+		gchar buf[128];
 
-	version_short_string = atom_str_get(buf);
+		if (before_2007_03_22(now))				/* before 2007-03-22 */
+			gm_snprintf(buf, sizeof(buf),
+					"gtk-gnutella/%s (%s; r%u)",
+					GTA_VERSION_NUMBER, GTA_RELEASE, main_get_build());
+		else
+			gm_snprintf(buf, sizeof(buf),
+					"gtk-gnutella/%s-%u (%s)",
+					GTA_VERSION_NUMBER, main_get_build(), GTA_RELEASE);
+
+		version_short_string = atom_str_get(buf);
+	}
 
 	last_rel_version = our_version;		/* struct copy */
 	last_dev_version = our_version;		/* struct copy */
@@ -653,7 +780,7 @@ void
 version_ancient_warn(void)
 {
 	time_t now = tm_time();
-	gint lifetime, remain, elapsed;
+	time_delta_t lifetime, remain, elapsed;
 	time_t s;
 
 	g_assert(our_version.timestamp != 0);	/* version_init() called */
@@ -689,7 +816,7 @@ version_ancient_warn(void)
 	 */
 
 	lifetime = our_version.tag ? VERSION_UNSTABLE_WARN : VERSION_ANCIENT_WARN;
-	remain = lifetime - elapsed;
+	remain = delta_time(lifetime, elapsed);
 
 	g_assert(remain >= 0);		/* None of the checks above have fired */
 
@@ -703,20 +830,22 @@ version_ancient_warn(void)
 	 * that do not introduce significant Gnutella features.
 	 */
 
-	for (s = now + VERSION_ANCIENT_REMIND; s > now; s -= SECS_PER_DAY) {
+	s = time_advance(now, VERSION_ANCIENT_REMIND);
+	for (/* NOTHING */; delta_time(s, now) > 0; s -= SECS_PER_DAY) {
 		if (!tok_is_ancient(s))
 			break;
 	}
 
-	remain = MIN(remain, s - now);
+	remain = MIN(remain, delta_time(s, now));
 
 	/*
 	 * Let them know when version will expire soon...
 	 */
 
-	if (remain < VERSION_ANCIENT_REMIND)
+	if (remain < VERSION_ANCIENT_REMIND) {
         gnet_prop_set_guint32_val(PROP_ANCIENT_VERSION_LEFT_DAYS,
 			remain / SECS_PER_DAY);
+	}
 }
 
 /**
