@@ -1,5 +1,5 @@
 /*
- * $Id: c2ui.c,v 1.10 2005/12/11 13:52:51 rmanfredi Exp $
+ * $Id: c2ui.c 12141 2006-10-27 01:17:39Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -81,13 +81,6 @@
  * Functions the CORE uses to access the UI
  */
 
-/*	upload interface functions (CORE -> UI) */
-void
-gcu_uploads_gui_set_gnet_addr(gnet_upload_t u, host_addr_t addr, guint16 port)
-{
-	uploads_gui_set_gnet_addr(u, addr, port);
-}
-
 /*	download interface functions (CORE -> UI) */
 void
 gcu_download_enable_start_now(guint32 running_downloads, guint32 max_downloads)
@@ -186,8 +179,7 @@ gcu_gtk_main_flush(void)
 #endif /* GTK */
 }
 
-
-/**	dearch interface functions (CORE -> UI) */
+/**	search interface functions (CORE -> UI) */
 gboolean
 gcu_search_gui_new_search(const gchar *query, flag_t flags)
 {
@@ -197,9 +189,9 @@ gcu_search_gui_new_search(const gchar *query, flag_t flags)
 
 /*	upload interface functions (CORE -> UI)*/
 void
-gcu_upload_stats_gui_add(struct ul_stats *stat)
+gcu_upload_stats_gui_add(struct ul_stats *s)
 {
-	upload_stats_gui_add(stat);
+	upload_stats_gui_add(s);
 }
 
 void
