@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.h,v 1.20 2006/02/19 21:23:10 rmanfredi Exp $
+ * $Id: fileinfo.h 10882 2006-05-17 05:45:46Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -79,7 +79,12 @@ typedef struct gnet_fi_chunks {
     gboolean old;
 } gnet_fi_chunks_t;
 
+enum fi_magic {
+	FI_MAGIC = 0xd99892e7
+};
+
 typedef struct dl_file_info {
+	enum fi_magic magic;	
     gnet_fi_t fi_handle;    /**< Handle */
 	gchar *guid;			/**< Unique fileinfo ID */
 	guint32 flags;			/**< Operating flags */
@@ -87,7 +92,7 @@ typedef struct dl_file_info {
 	gchar *path;			/**< Output file path (atom) */
 	GSList *alias;			/**< List of file name aliases (atoms) */
 	filesize_t size;		/**< File size */
-	guint64 *size_atom;		/**< File size (atom -- points to value in memory) */
+	filesize_t *size_atom;	/**< File size (atom -- points to value in memory) */
 	gchar *sha1;			/**< server SHA1 (atom) if known, NULL if not. */
 	gchar *cha1;			/**< computed SHA1 (atom) if known, NULL if not. */
 	gint32 refcount;		/**< Reference count of file (number of sources)*/
@@ -131,10 +136,26 @@ typedef struct dl_file_info {
 	filesize_t copied;		/**< Amount of bytes copied so far */
 } fileinfo_t;
 
-#define FILE_INFO_COMPLETE(x)	((x)->done == (x)->size && (x)->file_size_known)
+static inline void
+file_info_check(const fileinfo_t *fi)
+{
+	g_assert(fi);
+	g_assert(FI_MAGIC == fi->magic);
+}
 
-#define FILE_INFO_COMPLETE_AFTER(x,z)	\
-	((x)->done + (z) >= (x)->size && (x)->file_size_known)
+static inline gboolean
+FILE_INFO_COMPLETE(const fileinfo_t *fi)
+{
+	file_info_check(fi);
+	return fi->file_size_known && fi->done == fi->size;
+}
+
+static inline gboolean
+FILE_INFO_COMPLETE_AFTER(const fileinfo_t *fi, filesize_t off)
+{
+	file_info_check(fi);
+	return fi->file_size_known && off >= fi->size - fi->done;
+}
 
 typedef void (*fi_listener_t) (gnet_fi_t);
 typedef void (*fi_src_listener_t) (gnet_fi_t, gnet_src_t);
@@ -173,6 +194,7 @@ gchar **fi_get_aliases(gnet_fi_t fih);
 void fi_purge_by_handle_list(const GSList *list);
 
 const gchar *file_info_readable_filename(const struct dl_file_info *fi);
+gchar *file_info_build_magnet(gnet_fi_t fih);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_fileinfo_h_ */
