@@ -1,5 +1,5 @@
 /*
- * $Id: main.h,v 1.3 2005/09/16 21:57:23 rmanfredi Exp $
+ * $Id: main.h 11741 2006-08-26 13:43:13Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -26,9 +26,14 @@
 #ifndef _if_core_main_h_
 #define _if_core_main_h_
 
+#include <glib.h>
+
+guint32 main_get_build(void);
+
 #ifdef CORE_SOURCES
 
 void gtk_gnutella_exit(gint n);
+void gtk_gnutella_request_shutdown(void);
 gboolean debugging(guint t);
 
 #endif /* CORE_SOURCES */
