@@ -1,5 +1,5 @@
 /*
- * $Id: net_stats.h,v 1.20 2005/09/11 13:43:29 daichik Exp $
+ * $Id: net_stats.h 11782 2006-08-27 10:04:55Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -39,6 +39,7 @@ enum {
 	MSG_BYE,
 	MSG_QRP,
 	MSG_HSEP,
+	MSG_RUDP,
 	MSG_VENDOR,
 	MSG_STANDARD,
 	MSG_PUSH_REQUEST,
@@ -77,6 +78,10 @@ typedef enum msg_drop_reason {
 	MSG_DROP_BAD_RESULT,
 	MSG_DROP_BAD_RETURN_ADDRESS,
 	MSG_DROP_HOSTILE_IP,
+	MSG_DROP_SPAM,
+	MSG_DROP_EVIL,
+	MSG_DROP_INFLATE_ERROR,
+	MSG_DROP_UNKNOWN_HEADER_FLAGS,
 	
 	MSG_DROP_REASON_COUNT /**< number of known reasons to drop a message */
 } msg_drop_reason_t;
@@ -95,6 +100,8 @@ typedef enum {
 	GNR_OOB_HITS_WITH_ALIEN_IP,
 	GNR_UNCLAIMED_OOB_HITS,
 	GNR_PARTIALLY_CLAIMED_OOB_HITS,
+	GNR_SPURIOUS_OOB_HIT_CLAIM,
+	GNR_UNREQUESTED_OOB_HITS,
 	GNR_QUERY_COMPACT_COUNT,
 	GNR_QUERY_COMPACT_SIZE,
 	GNR_QUERY_UTF8,
@@ -120,6 +127,9 @@ typedef enum {
 	GNR_UDP_BOGUS_SOURCE_IP,
 	GNR_UDP_ALIEN_MESSAGE,
 	GNR_UDP_UNPROCESSED_MESSAGE,
+	GNR_UDP_TX_COMPRESSED,
+	GNR_UDP_RX_COMPRESSED,
+	GNR_UDP_LARGER_HENCE_NOT_COMPRESSED,
 	
 	GNR_TYPE_COUNT /* number of general stats */
 } gnr_stats_t;
