@@ -1,5 +1,5 @@
 /*
- * $Id: settings.h,v 1.1 2004/08/25 05:40:39 rmanfredi Exp $
+ * $Id: settings.h 11856 2006-09-02 03:27:35Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -32,8 +32,14 @@
 
 #ifdef CORE_SOURCES
 
+#include "lib/host_addr.h"
+
+host_addr_t listen_addr(void);
+host_addr_t listen_addr6(void);
 const gchar *settings_config_dir(void);
 const gchar *settings_home_dir(void);
+const gchar *settings_local_socket_path(void);
+enum net_type settings_dns_net(void);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_settings_h */
