@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h,v 1.17 2005/12/11 13:52:51 rmanfredi Exp $
+ * $Id: uploads.h 11727 2006-08-26 04:20:24Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -40,14 +40,15 @@ typedef enum {
 	GTA_UL_COMPLETE         = 2,    /**< The file has been sent completely */
 	GTA_UL_SENDING          = 3,    /**< We are sending data */
 	GTA_UL_HEADERS          = 4,    /**< Receiving the HTTP request headers */
-	GTA_UL_WAITING          = 5,    /**< Waiting new HTTP request */
+	GTA_UL_WAITING          = 5,    /**< Waiting end of follow-up request */
 	GTA_UL_ABORTED          = 6,    /**< Upload removed during operation */
 	GTA_UL_CLOSED           = 7,    /**< Upload removed while waiting */
 	GTA_UL_QUEUED           = 8,    /**< Upload is queued */
 	GTA_UL_QUEUE            = 9,    /**< Send a queue (Similar to push) */
 	GTA_UL_QUEUE_WAITING    = 10,   /**< Connect back with GTA_UL_QUEUE was
 									     success now waiting for a response */
-	GTA_UL_PFSP_WAITING     = 11    /**< Requested range unavailable, retry... */
+	GTA_UL_PFSP_WAITING     = 11,   /**< Requested range unavailable, retry */
+	GTA_UL_EXPECTING        = 12,   /**< Expecting follow-up HTTP request */
 } upload_stage_t;
 
 typedef struct gnet_upload_status {
@@ -83,6 +84,9 @@ typedef struct gnet_upload_info {
 	gboolean push;		/**< Whether we're pushing or not */
 	gboolean partial;	/**< Whether it's a partial file */
 	gboolean encrypted; /**< Whether the connection is (TLS) encrypted */
+	
+	host_addr_t gnet_addr;	/**< Advertised Gnutella address for connecting */
+	guint16 gnet_port;		/**< Advertised Gnutella listening port */
 } gnet_upload_info_t;
 
 /*
@@ -95,6 +99,7 @@ typedef struct gnet_upload_info {
 	||	(u)->status == GTA_UL_QUEUE					\
 	||	(u)->status == GTA_UL_QUEUE_WAITING			\
 	||	(u)->status == GTA_UL_PFSP_WAITING			\
+	||	(u)->status == GTA_UL_EXPECTING				\
 	||	(u)->status == GTA_UL_WAITING	)
 
 #define UPLOAD_IS_COMPLETE(u)	\
@@ -103,9 +108,14 @@ typedef struct gnet_upload_info {
 #define UPLOAD_IS_SENDING(u)	\
 	((u)->status == GTA_UL_SENDING)
 
+#define UPLOAD_WAITING_FOLLOWUP(u) \
+	((u)->status == GTA_UL_WAITING || (u)->status == GTA_UL_EXPECTING)
+
+#define UPLOAD_READING_HEADERS(u) \
+	((u)->status == GTA_UL_HEADERS || (u)->status == GTA_UL_WAITING)
 
 struct ul_stats {
-	gchar  *filename;
+	gchar  *filename;	/**< Atom */
 	filesize_t size;
 	guint32 attempts;
 	guint32 complete;
