@@ -1,5 +1,5 @@
 /*
- * $Id: adns.h,v 1.2 2004/10/21 14:35:28 cbiere Exp $
+ * $Id: adns.h 10736 2006-04-20 05:32:53Z cbiere $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -23,17 +23,30 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Asynchronous DNS lookup.
+ *
+ * @author Christian Biere
+ * @date 2003
+ */
+
 #ifndef _adns_h_
 #define _adns_h_
 
 #include <glib.h>
+#include "host_addr.h"	/* For ``struct host_addr'' */
 
-typedef void (*adns_callback_t)(guint32, gpointer);
+typedef void (*adns_callback_t)(const host_addr_t *, size_t, gpointer);
 typedef void (*adns_reverse_callback_t)(const gchar *, gpointer);
 
 void adns_init(void);
-gboolean adns_resolve(const gchar *, adns_callback_t, gpointer);
-gboolean adns_reverse_lookup(guint32 ip, adns_reverse_callback_t, gpointer);
+gboolean adns_resolve(const gchar *, enum net_type net,
+	adns_callback_t, gpointer);
+gboolean adns_reverse_lookup(const host_addr_t,
+	adns_reverse_callback_t, gpointer);
 void adns_close(void);
 
 #endif /* _adns_h_ */
