@@ -1,5 +1,5 @@
 /*
- * $Id: atoms.h,v 1.7 2005/12/27 16:29:29 cbiere Exp $
+ * $Id: atoms.h 11889 2006-09-06 04:17:33Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -42,10 +42,20 @@
  * Atom types.
  */
 
-#define ATOM_STRING		0		/**< Strings */
-#define ATOM_GUID		1		/**< GUIDs (binary, 16 bytes) */
-#define ATOM_SHA1		2		/**< SHA1 (binary, 20 bytes) */
-#define ATOM_UINT64		3		/**< integers (binary, 8 bytes) */
+enum atom_type {
+	ATOM_STRING,	/**< Strings */
+	ATOM_GUID,		/**< GUIDs (binary, 16 bytes) */
+	ATOM_SHA1,		/**< SHA1 (binary, 20 bytes) */
+	ATOM_UINT64,	/**< integers (binary, 8 bytes) */
+	ATOM_FILESIZE,	/**< filesize_t (binary) */
+
+	NUM_ATOM_TYPES
+};
+
+#if !defined(TRACK_ATOMS) || defined(ATOMS_SOURCE)
+gpointer atom_get(enum atom_type type, gconstpointer key);
+void atom_free(enum atom_type type, gconstpointer key);
+#endif
 
 /*
  * Convenience macros.
@@ -65,6 +75,11 @@
 #define atom_uint64_get(k)	atom_get_track(ATOM_UINT64, (k), _WHERE_, __LINE__)
 #define atom_uint64_free(k)	atom_free_track(ATOM_UINT64, (k), _WHERE_, __LINE__)
 
+#define atom_filesize_get(k) \
+	atom_get_track(ATOM_FILESIZE, (k), _WHERE_, __LINE__)
+#define atom_filesize_free(k) \
+	atom_free_track(ATOM_FILESIZE, (k), _WHERE_, __LINE__)
+
 #ifndef ATOMS_SOURCE
 #define atom_get(t,k)		atom_get_track(t, (k), _WHERE_, __LINE__)
 #define atom_free(t,k)		atom_free_track(t, (k), _WHERE_, __LINE__)
@@ -72,17 +87,65 @@
 
 #else	/* !TRACK_ATOMS */
 
-#define atom_str_get(k)		atom_get(ATOM_STRING, k)
-#define atom_str_free(k)	atom_free(ATOM_STRING, k)
+static inline gchar *
+atom_str_get(const gchar *k)
+{
+	return atom_get(ATOM_STRING, k);
+}
 
-#define atom_guid_get(k)	atom_get(ATOM_GUID, k)
-#define atom_guid_free(k)	atom_free(ATOM_GUID, k)
+static inline void
+atom_str_free(const gchar *k)
+{
+	return atom_free(ATOM_STRING, k);
+}
 
-#define atom_sha1_get(k)	atom_get(ATOM_SHA1, k)
-#define atom_sha1_free(k)	atom_free(ATOM_SHA1, k)
+static inline gchar *
+atom_guid_get(const gchar *k)
+{
+	return atom_get(ATOM_GUID, k);
+}
 
-#define atom_uint64_get(k)	atom_get(ATOM_UINT64, k)
-#define atom_uint64_free(k)	atom_free(ATOM_UINT64, k)
+static inline void
+atom_guid_free(const gchar *k)
+{
+	return atom_free(ATOM_GUID, k);
+}
+
+static inline gchar *
+atom_sha1_get(const gchar *k)
+{
+	return atom_get(ATOM_SHA1, k);
+}
+
+static inline void
+atom_sha1_free(const gchar *k)
+{
+	return atom_free(ATOM_SHA1, k);
+}
+
+static inline guint64 *
+atom_uint64_get(const guint64 *k)
+{
+	return atom_get(ATOM_UINT64, k);
+}
+
+static inline void
+atom_uint64_free(const guint64 *k)
+{
+	return atom_free(ATOM_UINT64, k);
+}
+
+static inline filesize_t *
+atom_filesize_get(const filesize_t *k)
+{
+	return atom_get(ATOM_FILESIZE, k);
+}
+
+static inline void
+atom_filesize_free(const filesize_t *k)
+{
+	return atom_free(ATOM_FILESIZE, k);
+}
 
 #endif	/* TRACK_ATOMS */
 
@@ -96,8 +159,8 @@ void atoms_close(void);
 /*
  * Hash functions and equality checks
  */
-guint uint64_hash(gconstpointer key);
-gint uint64_eq(gconstpointer a, gconstpointer b);
+guint filesize_hash(gconstpointer key);
+gint filesize_eq(gconstpointer a, gconstpointer b);
 guint sha1_hash(gconstpointer key);
 gint sha1_eq(gconstpointer a, gconstpointer b);
 guint guid_hash(gconstpointer key);
@@ -107,14 +170,47 @@ gint uint64_eq(gconstpointer a, gconstpointer b);
 guint binary_hash(const guchar *key, guint len);
 
 #ifdef TRACK_ATOMS
-gpointer atom_get_track(gint type, gconstpointer key, gchar *file, gint line);
-void atom_free_track(gint type, gconstpointer key, gchar *file, gint line);
+gpointer atom_get_track(enum atom_type, gconstpointer key,
+			gchar *file, gint line);
+void atom_free_track(enum atom_type, gconstpointer key, gchar *file, gint line);
 #endif
 
-#if !defined(TRACK_ATOMS) || defined(ATOMS_SOURCE)
-gpointer atom_get(gint type, gconstpointer key);
-void atom_free(gint type, gconstpointer key);
-#endif
+static inline void
+atom_str_free_null(gchar **k_ptr)
+{
+	if (*k_ptr) {
+		atom_free(ATOM_STRING, *k_ptr);
+		*k_ptr = NULL;
+	}
+}
+
+static inline void
+atom_sha1_free_null(gchar **k_ptr)
+{
+	if (*k_ptr) {
+		atom_free(ATOM_SHA1, *k_ptr);
+		*k_ptr = NULL;
+	}
+}
+
+static inline void
+atom_filesize_free_null(filesize_t **k_ptr)
+{
+	if (*k_ptr) {
+		atom_free(ATOM_FILESIZE, *k_ptr);
+		*k_ptr = NULL;
+	}
+}
+
+static inline void
+atom_guid_free_null(gchar **k_ptr)
+{
+	if (*k_ptr) {
+		atom_free(ATOM_GUID, *k_ptr);
+		*k_ptr = NULL;
+	}
+}
+
 
 #endif	/* _atoms_h_ */
 
