@@ -1,5 +1,5 @@
 /*
- * $Id: bg.c,v 1.9 2005/08/31 21:37:32 rmanfredi Exp $
+ * $Id: bg.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: bg.c,v 1.9 2005/08/31 21:37:32 rmanfredi Exp $");
+RCSID("$Id: bg.c 12242 2006-11-06 20:41:55Z cbiere $")
 
 #include "bg.h"
 #include "tm.h"
@@ -70,7 +70,7 @@ static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
 struct bgtask {
 	guint magic;			/**< Magic number */
 	guint32 flags;			/**< Operating flags */
-	gchar *name;			/**< Task name */
+	const gchar *name;		/**< Task name */
 	gint step;				/**< Current processing step */
 	gint seqno;				/**< Number of calls at same step */
 	bgstep_cb_t *stepvec;	/**< Set of steps to run in sequence */
@@ -108,20 +108,26 @@ struct bgtask {
  * Operating flags.
  */
 
-#define TASK_F_EXITED		0x00000001	/**< Exited */
-#define TASK_F_SIGNAL		0x00000002	/**< Signal received */
-#define TASK_F_RUNNING		0x00000004	/**< Task is running */
-#define TASK_F_ZOMBIE		0x00000008	/**< Task waiting status collect */
-#define TASK_F_NOTICK		0x00000010	/**< Do no recompute tick info */
-#define TASK_F_SLEEPING		0x00000020	/**< Task is sleeping */
-#define TASK_F_RUNNABLE		0x00000040	/**< Task is runnable */
-#define TASK_F_DAEMON		0x80000000	/**< Task is a daemon */
+enum {
+	TASK_F_EXITED	=	0x00000001,	/**< Exited */
+	TASK_F_SIGNAL	=	0x00000002,	/**< Signal received */
+	TASK_F_RUNNING	=	0x00000004,	/**< Task is running */
+	TASK_F_ZOMBIE	=	0x00000008,	/**< Task waiting status collect */
+	TASK_F_NOTICK	=	0x00000010,	/**< Do no recompute tick info */
+	TASK_F_SLEEPING	=	0x00000020,	/**< Task is sleeping */
+	TASK_F_RUNNABLE	=	0x00000040,	/**< Task is runnable */
+	TASK_F_DAEMON	=	0x80000000	/**< Task is a daemon */
+};
 
 /*
  * Access routines to internal fields.
  */
 
-#define TASK(x)		((struct bgtask *) (x))
+static inline struct bgtask *
+TASK(gpointer x)
+{
+	return x;
+}
 
 gint bg_task_seqno(gpointer h)			{ return TASK(h)->seqno; }
 gpointer bg_task_context(gpointer h)	{ return TASK(h)->ucontext; }
@@ -132,11 +138,10 @@ static gint runcount = 0;
 static GSList *dead_tasks = NULL;
 
 /**
- * bg_sched_add
- *
  * Add new task to the scheduler (run queue).
  */
-static void bg_sched_add(struct bgtask *bt)
+static void
+bg_sched_add(struct bgtask *bt)
 {
 	g_assert(!(bt->flags & TASK_F_RUNNABLE));	/* Not already in list */
 
@@ -150,11 +155,10 @@ static void bg_sched_add(struct bgtask *bt)
 }
 
 /**
- * bg_sched_remove
- *
  * Remove task from the scheduler (run queue).
  */
-static void bg_sched_remove(struct bgtask *bt)
+static void
+bg_sched_remove(struct bgtask *bt)
 {
 	/*
 	 * We currently have only one run queue: we don't handle priorities.
@@ -165,25 +169,23 @@ static void bg_sched_remove(struct bgtask *bt)
 }
 
 /**
- * bg_sched_pick
- *
  * Pick next task to schedule.
  */
-static struct bgtask *bg_sched_pick(void)
+static struct bgtask *
+bg_sched_pick(void)
 {
 	/*
 	 * All task in run queue have equal priority, pick the first.
 	 */
 
-	return (runq != NULL) ? (struct bgtask *) runq->data : NULL;
+	return runq != NULL ? TASK(runq->data) : NULL;
 }
 
 /**
- * bg_task_suspend
- *
  * Suspend task.
  */
-static void bg_task_suspend(struct bgtask *bt)
+static void
+bg_task_suspend(struct bgtask *bt)
 {
 	tm_t end;
 	gint elapsed;
@@ -240,11 +242,10 @@ static void bg_task_suspend(struct bgtask *bt)
 }
 
 /**
- * bg_task_resume
- *
  * Resume task execution.
  */
-static void bg_task_resume(struct bgtask *bt)
+static void
+bg_task_resume(struct bgtask *bt)
 {
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 
@@ -255,11 +256,10 @@ static void bg_task_resume(struct bgtask *bt)
 }
 
 /**
- * bg_sched_sleep
- *
  * Add task to the sleep queue.
  */
-static void bg_sched_sleep(struct bgtask *bt)
+static void
+bg_sched_sleep(struct bgtask *bt)
 {
 	g_assert(!(bt->flags & TASK_F_SLEEPING));
 	g_assert(!(bt->flags & TASK_F_RUNNING));
@@ -272,11 +272,10 @@ static void bg_sched_sleep(struct bgtask *bt)
 }
 
 /**
- * bg_sched_wakeup
- *
  * Remove task from the sleep queue and insert it to the runqueue.
  */
-static void bg_sched_wakeup(struct bgtask *bt)
+static void
+bg_sched_wakeup(struct bgtask *bt)
 {
 	g_assert(bt->flags & TASK_F_SLEEPING);
 	g_assert(!(bt->flags & TASK_F_RUNNING));
@@ -291,14 +290,13 @@ static void bg_sched_wakeup(struct bgtask *bt)
 static struct bgtask *current_task = NULL;
 
 /**
- * bg_task_switch
- *
  * Switch to new task `bt'.
  * If argument is NULL, suspends current task.
  *
  * @returns previously scheduled task, if any.
  */
-static struct bgtask *bg_task_switch(struct bgtask *bt)
+static struct bgtask *
+bg_task_switch(struct bgtask *bt)
 {
 	struct bgtask *old = current_task;
 
@@ -319,8 +317,6 @@ static struct bgtask *bg_task_switch(struct bgtask *bt)
 }
 
 /**
- * bg_task_create
- *
  * Create a new background task.
  * The `steps' array is cloned, so it can be built on the caller's stack.
  *
@@ -335,8 +331,8 @@ static struct bgtask *bg_task_switch(struct bgtask *bt)
  *
  * @returns an opaque handle.
  */
-gpointer bg_task_create(
-	gchar *name,						/**< Task name (for tracing) */
+gpointer
+bg_task_create(const gchar *name,		/**< Task name (for tracing) */
 	bgstep_cb_t *steps, gint stepcnt,	/**< Work to perform (copied) */
 	gpointer ucontext,					/**< User context */
 	bgclean_cb_t ucontext_free,			/**< Free routine for context */
@@ -370,8 +366,6 @@ gpointer bg_task_create(
 }
 
 /**
- * bg_daemon_create
- *
  * A "daemon" is a task equipped with a work queue.
  *
  * When the daemon is initially created, it has an empty work queue and it is
@@ -390,15 +384,16 @@ gpointer bg_task_create(
  *
  * Use bg_daemon_enqueue() to enqueue more work to the daemon.
  */
-gpointer bg_daemon_create(
-	gchar *name,						/**< Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt,	/**< Work to perform (copied) */
-	gpointer ucontext,					/**< User context */
-	bgclean_cb_t ucontext_free,			/**< Free routine for context */
-	bgstart_cb_t start_cb,				/**< Starting working on an item */
-	bgend_cb_t end_cb,					/**< Done working on an item */
-	bgclean_cb_t item_free,				/**< Free routine for work queue items */
-	bgnotify_cb_t notify)				/**< Start/Stop notify (optional) */
+gpointer
+bg_daemon_create(
+	const gchar *name,				  /**< Task name (for tracing) */
+	bgstep_cb_t *steps, gint stepcnt, /**< Work to perform (copied) */
+	gpointer ucontext,				  /**< User context */
+	bgclean_cb_t ucontext_free,		  /**< Free routine for context */
+	bgstart_cb_t start_cb,			  /**< Starting working on an item */
+	bgend_cb_t end_cb,				  /**< Done working on an item */
+	bgclean_cb_t item_free,			  /**< Free routine for work queue items */
+	bgnotify_cb_t notify)			  /**< Start/Stop notify (optional) */
 {
 	struct bgtask *bt;
 	gint stepsize;
@@ -430,12 +425,11 @@ gpointer bg_daemon_create(
 }
 
 /**
- * bg_daemon_enqueue
- *
  * Enqueue work item to the daemon task.
  * If task was sleeping, wake it up.
  */
-void bg_daemon_enqueue(gpointer h, gpointer item)
+void
+bg_daemon_enqueue(gpointer h, gpointer item)
 {
 	struct bgtask *bt = TASK(h);
 
@@ -456,11 +450,10 @@ void bg_daemon_enqueue(gpointer h, gpointer item)
 }
 
 /**
- * bg_task_free
- *
  * Free task structure.
  */
-static void bg_task_free(struct bgtask *bt)
+static void
+bg_task_free(struct bgtask *bt)
 {
 	GSList *l;
 	gint stepsize;
@@ -487,11 +480,10 @@ static void bg_task_free(struct bgtask *bt)
 }
 
 /**
- * bg_task_terminate
- *
  * Terminate the task, invoking the completion callback if defined.
  */
-static void bg_task_terminate(struct bgtask *bt)
+static void
+bg_task_terminate(struct bgtask *bt)
 {
 	bgstatus_t status;
 
@@ -579,12 +571,11 @@ static void bg_task_terminate(struct bgtask *bt)
 }
 
 /**
- * bg_task_exit
- *
  * Called by user code to "exit" the task.
  * We exit immediately, not returning to the user code.
  */
-void bg_task_exit(gpointer h, gint code)
+void
+bg_task_exit(gpointer h, gint code)
 {
 	struct bgtask *bt = TASK(h);
 
@@ -603,11 +594,10 @@ void bg_task_exit(gpointer h, gint code)
 }
 
 /**
- * bg_task_sendsig
- *
  * Deliver signal via the user's signal handler.
  */
-static void bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
+static void
+bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 {
 	g_assert(bt->flags & TASK_F_RUNNING);
 
@@ -621,13 +611,12 @@ static void bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 }
 
 /**
- * bg_task_kill
- *
  * Send a signal to the given task.
  *
  * @returns -1 if the task could not be signalled.
  */
-static gint bg_task_kill(gpointer h, bgsig_t sig)
+static gint
+bg_task_kill(gpointer h, bgsig_t sig)
 {
 	struct bgtask *bt = TASK(h);
 	bgsig_cb_t sighandler;
@@ -684,13 +673,12 @@ static gint bg_task_kill(gpointer h, bgsig_t sig)
 }
 
 /**
- * bg_task_signal
- *
  * Install user-level signal handler for a task signal.
  *
  * @returns previously installed signal handler.
  */
-bgsig_cb_t bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
+bgsig_cb_t
+bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
 {
 	struct bgtask *bt = TASK(h);
 	bgsig_cb_t oldhandler;
@@ -705,11 +693,10 @@ bgsig_cb_t bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
 }
 
 /**
- * bg_task_deliver_signals
- *
  * Deliver all the signals queued so far for the task.
  */
-static void bg_task_deliver_signals(struct bgtask *bt)
+static void
+bg_task_deliver_signals(struct bgtask *bt)
 {
 	g_assert(bt->flags & TASK_F_RUNNING);
 
@@ -737,11 +724,10 @@ static void bg_task_deliver_signals(struct bgtask *bt)
 }
 
 /**
- * bg_task_cancel
- *
  * Cancel a given task.
  */
-void bg_task_cancel(gpointer h)
+void
+bg_task_cancel(gpointer h)
 {
 	struct bgtask *bt = TASK(h);
 	struct bgtask *old = NULL;
@@ -789,12 +775,11 @@ void bg_task_cancel(gpointer h)
 }
 
 /**
- * bg_task_ticks_used
- *
  * This routine can be called by the task when a single step is not using
  * all its ticks and it matters for the computation of the cost per tick.
  */
-void bg_task_ticks_used(gpointer h, gint used)
+void
+bg_task_ticks_used(gpointer h, gint used)
 {
 	struct bgtask *bt = TASK(h);
 
@@ -811,27 +796,25 @@ void bg_task_ticks_used(gpointer h, gint used)
 }
 
 /**
- * bg_reclaim_dead
- *
  * Reclaim all dead tasks
  */
-static void bg_reclaim_dead(void)
+static void
+bg_reclaim_dead(void)
 {
-	GSList *l;
-
-	for (l = dead_tasks; l; l = l->next)
-		bg_task_free((struct bgtask *) l->data);
+	GSList *sl;
 
+	for (sl = dead_tasks; sl; sl = g_slist_next(sl)) {
+		bg_task_free(TASK(sl->data));
+	}
 	g_slist_free(dead_tasks);
 	dead_tasks = NULL;
 }
 
 /**
- * bg_task_ended
- *
  * Called when a task has ended its processing.
  */
-static void bg_task_ended(struct bgtask *bt)
+static void
+bg_task_ended(struct bgtask *bt)
 {
 	gpointer item;
 
@@ -884,11 +867,12 @@ static void bg_task_ended(struct bgtask *bt)
 }
 
 /**
- * bg_sched_timer
- *
  * Main task scheduling timer, called once per second.
+ *
+ * @param overloaded	when TRUE, CPU is already deemed to be busy enough
  */
-void bg_sched_timer(void)
+void
+bg_sched_timer(gboolean overloaded)
 {
 	struct bgtask * volatile bt;
 	volatile gint remain = MAX_LIFE;
@@ -899,6 +883,9 @@ void bg_sched_timer(void)
 	g_assert(current_task == NULL);
 	g_assert(runcount >= 0);
 
+	if (overloaded)
+		remain /= 2;	/* Use less CPU time when overloaded */
+
 	/*
 	 * Loop as long as there are tasks to be scheduled and we have some
 	 * time left to spend.
@@ -907,9 +894,12 @@ void bg_sched_timer(void)
 	while (runcount > 0 && remain > 0) {
 		/*
 		 * Compute how much time we can spend for this task.
+		 * Slow things down if overloaded.
 		 */
 
 		target = MAX(MIN_LIFE, MAX_LIFE / runcount);
+		if (overloaded)
+			target /= 2;
 
 		bt = bg_sched_pick();
 		g_assert(bt);					/* runcount > 0 => there is a task */
@@ -925,12 +915,21 @@ void bg_sched_timer(void)
 		 */
 
 		if (bt->tick_cost) {
+			g_assert(bt->tick_cost > 0);
+			g_assert(bt->prev_ticks >= 0);
+			g_assert(bt->prev_ticks <= INT_MAX / DELTA_FACTOR);
+
 			ticks = 1 + target / bt->tick_cost;
+
 			if (bt->prev_ticks) {
-				if (ticks > bt->prev_ticks * DELTA_FACTOR)
+				if (ticks > bt->prev_ticks * DELTA_FACTOR) {
 					ticks = bt->prev_ticks * DELTA_FACTOR;
-				else if (DELTA_FACTOR * ticks < bt->prev_ticks)
-					ticks = bt->prev_ticks / DELTA_FACTOR;
+				} else if (ticks < bt->prev_ticks / DELTA_FACTOR) {
+					if (bt->prev_ticks > DELTA_FACTOR)
+						ticks = bt->prev_ticks / DELTA_FACTOR;
+					else
+						ticks = 1;
+				}
 			}
 			g_assert(ticks > 0);
 		} else
@@ -1041,11 +1040,10 @@ void bg_sched_timer(void)
 }
 
 /**
- * bg_close
- *
  * Called at shutdown time.
  */
-void bg_close(void)
+void
+bg_close(void)
 {
 	GSList *l;
 	GSList *c;
@@ -1079,4 +1077,4 @@ void bg_close(void)
 /* bg_task_get_exitcode */
 /* bg_task_get_signal */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
