@@ -1,10 +1,8 @@
 /*
- * $Id: event.c,v 1.2 2004/10/20 15:38:58 cbiere Exp $
+ * $Id: event.c 12270 2006-11-09 06:07:54Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
- * Functions that should be in gtk+-1.2 or gtk+-2.x but are not.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,11 +23,22 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Functions that should be in GTK+ 1.2 or GTK+ 2.x but are not.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: event.c,v 1.2 2004/10/20 15:38:58 cbiere Exp $");
+RCSID("$Id: event.c 12270 2006-11-09 06:07:54Z cbiere $")
 
 #include "event.h"
+#include "misc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -48,18 +57,22 @@ subscriber_new(GCallback cb, enum frequency_type t, guint32 interval)
     return s;
 }
 
-#define subscriber_destroy(s) wfree(s, sizeof(struct subscriber))
+static inline void
+subscriber_destroy(struct subscriber *s)
+{
+	wfree(s, sizeof *s);
+}
 
 inline struct event *
 event_new(const gchar *name)
 {
     struct event *evt;
-    
+
     g_assert(name != NULL);
 
     evt = g_new0(struct event, 1);
     evt->name = name;
-    
+
     return evt;
 }
 
@@ -79,39 +92,40 @@ void
 event_add_subscriber(struct event *evt, GCallback cb,
 	enum frequency_type t, guint32 interval)
 {
-    struct subscriber * s;
+    struct subscriber *s;
+	GSList *sl;
 
     g_assert(evt != NULL);
     g_assert(cb != NULL);
-    g_assert(g_slist_find(evt->subscribers, (gpointer) cb) == NULL);
+
+	for (sl = evt->subscribers; sl; sl = g_slist_next(sl)) {
+		s = sl->data;
+		g_assert(s->cb != cb);
+	}
 
     s = subscriber_new(cb, t, interval);
-
     evt->subscribers = g_slist_append(evt->subscribers, s);
 }
 
-static gint
-cmp_subscriber_callback(struct subscriber *s, GCallback cb)
-{
-    return (s->cb == cb) ? 0 : 1;
-}
-
 void
 event_remove_subscriber(struct event *evt, GCallback cb)
 {
     GSList *sl;
-	struct subscriber *s;
+	struct subscriber *s = NULL;
 
     g_assert(evt != NULL);
     g_assert(cb != NULL);
-    
-    sl = g_slist_find_custom(evt->subscribers, (gpointer) cb, 
-        (GCompareFunc) cmp_subscriber_callback);
 
-    g_assert(sl != NULL);
-    g_assert(sl->data != NULL);
+	for (sl = evt->subscribers; sl; sl = g_slist_next(sl)) {
+			s = sl->data;
+			if (s->cb == cb)
+				break;
+	}
+
+	g_assert(sl != NULL);
+    g_assert(s != NULL);
+	g_assert(s->cb == cb);
 
-	s = sl->data;
     evt->subscribers = g_slist_remove(evt->subscribers, s);
 	subscriber_destroy(s);
 }
@@ -130,7 +144,7 @@ event_subscriber_active(struct event *evt)
 
 
 struct event_table *
-event_table_new(void) 
+event_table_new(void)
 {
     struct event_table *t;
 
@@ -141,7 +155,7 @@ event_table_new(void)
 }
 
 void
-real_event_table_destroy(struct event_table *t, gboolean cleanup) 
+real_event_table_destroy(struct event_table *t, gboolean cleanup)
 {
     if (cleanup)
         event_table_remove_all(t);
