@@ -1,11 +1,25 @@
+#include <stdlib.h>
 #ifndef lint
-static char yysccsid[] = "@(#)yaccpar	1.8 (Berkeley) 01/20/91";
-#endif
+#if 0
+static char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
+#else
+#if defined(__NetBSD__) && defined(__IDSTRING)
+__IDSTRING(yyrcsid, "$NetBSD: skeleton.c,v 1.26 2006/05/24 18:01:43 christos Exp $");
+#endif /* __NetBSD__ && __IDSTRING */
+#endif /* 0 */
+#endif /* lint */
 #define YYBYACC 1
+#define YYMAJOR 1
+#define YYMINOR 9
+#define YYLEX yylex()
+#define YYEMPTY -1
+#define yyclearin (yychar=(YYEMPTY))
+#define yyerrok (yyerrflag=0)
+#define YYRECOVERING (yyerrflag!=0)
+#define YYPREFIX "yy"
 #line 2 "getdate.y"
-
 /*
- * $Id: getdate.c,v 1.6 2005/08/18 20:40:38 cbiere Exp $
+ * $Id: getdate.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Date parsing.
  *
@@ -141,7 +155,7 @@ static char yysccsid[] = "@(#)yaccpar	1.8 (Berkeley) 01/20/91";
 #define yycheck  gd_yycheck
 
 static int yylex (void);
-static int yyerror (char *s);
+static int yyerror (const char *s);
 extern int yyparse (void);
 
 #define EPOCH		1970
@@ -201,7 +215,7 @@ typedef union {
     int			Number;
     enum _MERIDIAN	Meridian;
 } YYSTYPE;
-#line 204 "y.tab.c"
+#line 219 "y.tab.c"
 #define tAGO 257
 #define tDAY 258
 #define tDAY_UNIT 259
@@ -221,7 +235,7 @@ typedef union {
 #define tNUMBER_T 273
 #define tNUMBER_DOT 274
 #define YYERRCODE 256
-short yylhs[] = {                                        -1,
+const short yylhs[] = {                                        -1,
     0,    0,    2,    2,    2,    2,    2,    2,    3,    3,
     3,    3,    3,    3,    3,    3,    4,    4,    4,    9,
     6,    6,    6,    5,    5,    5,    5,    5,    5,    5,
@@ -229,7 +243,7 @@ short yylhs[] = {                                        -1,
    10,   10,   10,   10,   10,   10,   10,   10,   10,   10,
    10,   10,    8,    1,    1,
 };
-short yylen[] = {                                         2,
+const short yylen[] = {                                         2,
     0,    2,    1,    1,    1,    1,    1,    1,    2,    4,
     4,    6,    6,    6,    7,    6,    1,    1,    2,    3,
     1,    2,    2,    3,    5,    3,    3,    3,    2,    4,
@@ -237,7 +251,7 @@ short yylen[] = {                                         2,
     2,    2,    1,    2,    2,    1,    2,    2,    1,    2,
     2,    1,    1,    0,    1,
 };
-short yydefred[] = {                                      1,
+const short yydefred[] = {                                      1,
     0,    0,   43,   18,   46,   49,    0,   40,   52,    0,
     0,   37,    0,    2,    3,    4,    5,    6,    7,    8,
     0,   22,    0,   42,   45,   48,   39,   51,   36,   23,
@@ -246,10 +260,10 @@ short yydefred[] = {                                      1,
    30,   55,   11,    0,   10,    0,    0,    0,   25,    0,
    12,   13,    0,    0,    0,   15,   20,
 };
-short yydgoto[] = {                                       1,
+const short yydgoto[] = {                                       1,
    55,   14,   15,   16,   17,   18,   19,   20,   62,   21,
 };
-short yysindex[] = {                                      0,
+const short yysindex[] = {                                      0,
  -245,  -40,    0,    0,    0,    0, -260,    0,    0, -230,
   -47,    0, -249,    0,    0,    0,    0,    0,    0,    0,
  -239,    0,  -25,    0,    0,    0,    0,    0,    0,    0,
@@ -258,7 +272,7 @@ short yysindex[] = {                                      0,
     0,    0,    0, -240,    0, -228, -261, -227,    0,  -18,
     0,    0, -225, -223,  -18,    0,    0,
 };
-short yyrindex[] = {                                      0,
+const short yyrindex[] = {                                      0,
     0,    1,    0,    0,    0,    0,    0,    0,    0,    0,
   124,    0,   16,    0,    0,    0,    0,    0,    0,    0,
    31,    0,   46,    0,    0,    0,    0,    0,    0,    0,
@@ -267,11 +281,11 @@ short yyrindex[] = {                                      0,
     0,    0,    0,    0,    0,    0,   91,    0,    0,   76,
     0,    0,  106,    0,    0,    0,    0,
 };
-short yygindex[] = {                                      0,
+const short yygindex[] = {                                      0,
   -12,    0,    0,    0,    0,    0,    0,    0,  -15,    0,
 };
 #define YYTABLESIZE 396
-short yytable[] = {                                      41,
+const short yytable[] = {                                      41,
    21,   54,   52,   22,   47,   45,   46,   60,   48,   23,
    40,   42,    2,    3,    4,   17,    5,   43,   44,    6,
     7,    8,    9,   10,   11,   12,   13,   49,   24,   57,
@@ -313,7 +327,7 @@ short yytable[] = {                                      41,
    31,    0,   31,   53,    0,   31,   31,   31,   31,    0,
     0,   31,   31,   53,    0,   53,
 };
-short yycheck[] = {                                      47,
+const short yycheck[] = {                                      47,
     0,   58,  264,   44,  269,  269,  270,  269,  273,  270,
    58,  261,  258,  259,  260,    0,  262,  257,   44,  265,
   266,  267,  268,  269,  270,  271,  272,  270,  259,  270,
@@ -361,7 +375,7 @@ short yycheck[] = {                                      47,
 #endif
 #define YYMAXTOKEN 274
 #if YYDEBUG
-char *yyname[] = {
+const char * const yyname[] = {
 "end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,"','",0,0,"'/'",0,0,0,0,0,0,0,0,0,0,"':'",0,0,0,0,0,0,0,0,0,
 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -373,7 +387,7 @@ char *yyname[] = {
 "tMONTH","tMONTH_UNIT","tSEC_UNIT","tSNUMBER","tUNUMBER","tYEAR_UNIT","tZONE",
 "tNUMBER_T","tNUMBER_DOT",
 };
-char *yyrule[] = {
+const char * const yyrule[] = {
 "$accept : spec",
 "spec :",
 "spec : spec item",
@@ -432,20 +446,18 @@ char *yyrule[] = {
 "o_merid : tMERIDIAN",
 };
 #endif
-#define yyclearin (yychar=(-1))
-#define yyerrok (yyerrflag=0)
 #ifdef YYSTACKSIZE
-#ifndef YYMAXDEPTH
+#undef YYMAXDEPTH
 #define YYMAXDEPTH YYSTACKSIZE
-#endif
 #else
 #ifdef YYMAXDEPTH
 #define YYSTACKSIZE YYMAXDEPTH
 #else
-#define YYSTACKSIZE 500
-#define YYMAXDEPTH 500
+#define YYSTACKSIZE 10000
+#define YYMAXDEPTH 10000
 #endif
 #endif
+#define YYINITSTACKSIZE 200
 int yydebug;
 int yynerrs;
 int yyerrflag;
@@ -454,9 +466,11 @@ short *yyssp;
 YYSTYPE *yyvsp;
 YYSTYPE yyval;
 YYSTYPE yylval;
-short yyss[YYSTACKSIZE];
-YYSTYPE yyvs[YYSTACKSIZE];
-#define yystacksize YYSTACKSIZE
+short *yyss;
+short *yysslim;
+YYSTYPE *yyvs;
+int yystacksize;
+int yyparse(void);
 #line 500 "getdate.y"
 
 /* Month and day table. */
@@ -646,12 +660,9 @@ static TABLE const MilitaryTable[] = {
 
 
 /* ARGSUSED */
-static int yyerror(char *unused_s)
+static int yyerror(const char *unused_s)
 {
 	(void) unused_s;
-#ifndef lint
-	(void) yysccsid;
-#endif
     return 0;
 }
 
@@ -1022,19 +1033,47 @@ int main(int ac, char *av[])
     /* NOTREACHED */
 }
 #endif				/* defined (TEST) */
-#line 1022 "y.tab.c"
+#line 1037 "y.tab.c"
+/* allocate initial stack or double stack size, up to YYMAXDEPTH */
+static int yygrowstack(void);
+static int yygrowstack(void)
+{
+    int newsize, i;
+    short *newss;
+    YYSTYPE *newvs;
+
+    if ((newsize = yystacksize) == 0)
+        newsize = YYINITSTACKSIZE;
+    else if (newsize >= YYMAXDEPTH)
+        return -1;
+    else if ((newsize *= 2) > YYMAXDEPTH)
+        newsize = YYMAXDEPTH;
+    i = yyssp - yyss;
+    if ((newss = (short *)realloc(yyss, newsize * sizeof *newss)) == NULL)
+        return -1;
+    yyss = newss;
+    yyssp = newss + i;
+    if ((newvs = (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs)) == NULL)
+        return -1;
+    yyvs = newvs;
+    yyvsp = newvs + i;
+    yystacksize = newsize;
+    yysslim = yyss + newsize - 1;
+    return 0;
+}
+
 #define YYABORT goto yyabort
+#define YYREJECT goto yyabort
 #define YYACCEPT goto yyaccept
 #define YYERROR goto yyerrlab
 int
-yyparse()
+yyparse(void)
 {
-    register int yym, yyn, yystate;
+    int yym, yyn, yystate;
 #if YYDEBUG
-    register char *yys;
-    extern char *getenv();
+    const char *yys;
 
-    if (yys = getenv("YYDEBUG"))
+    if ((yys = getenv("YYDEBUG")) != NULL)
     {
         yyn = *yys;
         if (yyn >= '0' && yyn <= '9')
@@ -1046,12 +1085,13 @@ yyparse()
     yyerrflag = 0;
     yychar = (-1);
 
+    if (yyss == NULL && yygrowstack()) goto yyoverflow;
     yyssp = yyss;
     yyvsp = yyvs;
     *yyssp = yystate = 0;
 
 yyloop:
-    if (0 != (yyn = yydefred[yystate])) goto yyreduce;
+    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
     if (yychar < 0)
     {
         if ((yychar = yylex()) < 0) yychar = 0;
@@ -1061,8 +1101,8 @@ yyloop:
             yys = 0;
             if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
             if (!yys) yys = "illegal-symbol";
-            printf("yydebug: state %d, reading %d (%s)\n", yystate,
-                    yychar, yys);
+            printf("%sdebug: state %d, reading %d (%s)\n",
+                    YYPREFIX, yystate, yychar, yys);
         }
 #endif
     }
@@ -1071,10 +1111,10 @@ yyloop:
     {
 #if YYDEBUG
         if (yydebug)
-            printf("yydebug: state %d, shifting to state %d\n",
-                    yystate, yytable[yyn]);
+            printf("%sdebug: state %d, shifting to state %d\n",
+                    YYPREFIX, yystate, yytable[yyn]);
 #endif
-        if (yyssp >= yyss + yystacksize - 1)
+        if (yyssp >= yysslim && yygrowstack())
         {
             goto yyoverflow;
         }
@@ -1091,19 +1131,11 @@ yyloop:
         goto yyreduce;
     }
     if (yyerrflag) goto yyinrecovery;
-#ifdef lint
     goto yynewerror;
-#endif
-#if lint 
 yynewerror:
-#endif
     yyerror("syntax error");
-#ifdef lint
     goto yyerrlab;
-#endif
-#ifdef lint
 yyerrlab:
-#endif
     ++yynerrs;
 yyinrecovery:
     if (yyerrflag < 3)
@@ -1116,10 +1148,10 @@ yyinrecovery:
             {
 #if YYDEBUG
                 if (yydebug)
-                    printf("yydebug: state %d, error recovery shifting\
- to state %d\n", *yyssp, yytable[yyn]);
+                    printf("%sdebug: state %d, error recovery shifting\
+ to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
 #endif
-                if (yyssp >= yyss + yystacksize - 1)
+                if (yyssp >= yysslim && yygrowstack())
                 {
                     goto yyoverflow;
                 }
@@ -1131,8 +1163,8 @@ yyinrecovery:
             {
 #if YYDEBUG
                 if (yydebug)
-                    printf("yydebug: error recovery discarding state %d\n",
-                            *yyssp);
+                    printf("%sdebug: error recovery discarding state %d\n",
+                            YYPREFIX, *yyssp);
 #endif
                 if (yyssp <= yyss) goto yyabort;
                 --yyssp;
@@ -1149,8 +1181,8 @@ yyinrecovery:
             yys = 0;
             if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
             if (!yys) yys = "illegal-symbol";
-            printf("yydebug: state %d, error recovery discards token %d (%s)\n",
-                    yystate, yychar, yys);
+            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
+                    YYPREFIX, yystate, yychar, yys);
         }
 #endif
         yychar = (-1);
@@ -1159,8 +1191,8 @@ yyinrecovery:
 yyreduce:
 #if YYDEBUG
     if (yydebug)
-        printf("yydebug: state %d, reducing by rule %d (%s)\n",
-                yystate, yyn, yyrule[yyn]);
+        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
+                YYPREFIX, yystate, yyn, yyrule[yyn]);
 #endif
     yym = yylen[yyn];
     yyval = yyvsp[1-yym];
@@ -1574,7 +1606,7 @@ case 55:
 	    yyval.Meridian = yyvsp[0].Meridian;
 	  }
 break;
-#line 1570 "y.tab.c"
+#line 1610 "y.tab.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
@@ -1584,8 +1616,8 @@ break;
     {
 #if YYDEBUG
         if (yydebug)
-            printf("yydebug: after reduction, shifting from state 0 to\
- state %d\n", YYFINAL);
+            printf("%sdebug: after reduction, shifting from state 0 to\
+ state %d\n", YYPREFIX, YYFINAL);
 #endif
         yystate = YYFINAL;
         *++yyssp = YYFINAL;
@@ -1599,8 +1631,8 @@ break;
                 yys = 0;
                 if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                 if (!yys) yys = "illegal-symbol";
-                printf("yydebug: state %d, reading %d (%s)\n",
-                        YYFINAL, yychar, yys);
+                printf("%sdebug: state %d, reading %d (%s)\n",
+                        YYPREFIX, YYFINAL, yychar, yys);
             }
 #endif
         }
@@ -1614,10 +1646,10 @@ break;
         yystate = yydgoto[yym];
 #if YYDEBUG
     if (yydebug)
-        printf("yydebug: after reduction, shifting from state %d \
-to state %d\n", *yyssp, yystate);
+        printf("%sdebug: after reduction, shifting from state %d \
+to state %d\n", YYPREFIX, *yyssp, yystate);
 #endif
-    if (yyssp >= yyss + yystacksize - 1)
+    if (yyssp >= yysslim && yygrowstack())
     {
         goto yyoverflow;
     }
