@@ -1,5 +1,5 @@
 /*
- * $Id: glib-missing.c,v 1.25 2005/11/02 09:25:23 cbiere Exp $
+ * $Id: glib-missing.c 12019 2006-09-21 15:53:49Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -41,13 +41,154 @@
 
 #include "common.h"
 
-RCSID("$Id: glib-missing.c,v 1.25 2005/11/02 09:25:23 cbiere Exp $");
+RCSID("$Id: glib-missing.c 12019 2006-09-21 15:53:49Z cbiere $")
 
 #include "glib-missing.h"
-#include "utf8.h"
+#include "iovec.h"
 #include "misc.h"
+#include "utf8.h"
+
 #include "override.h"		/* Must be the last header included */
 
+#ifdef USE_GLIB1
+static GMemVTable gm_vtable = {
+	malloc,			/* malloc */
+	realloc,		/* realloc */
+	free,			/* free */
+	calloc,			/* calloc */
+	malloc,			/* try_malloc */
+	realloc,		/* try_realloc */
+};
+
+/***
+ *** Remap g_malloc() and friends to be able to emulate g_mem_set_vtable()
+ *** with GTK1.  Fortunately, glib1.x placed the allocation routines in
+ *** a dedicated mem.o file, so we may safely redefine them here.
+ ***/
+
+gpointer
+g_malloc(gulong size)
+{
+	gpointer p;
+
+	if (size == 0)
+		return NULL;
+
+	p = gm_vtable.malloc(size);
+
+	if (p)
+		return p;
+
+	g_error("allocation of %lu bytes failed", size);
+	return NULL;
+}
+
+gpointer
+g_malloc0(gulong size)
+{
+	gpointer p;
+
+	if (size == 0)
+		return NULL;
+
+	p = gm_vtable.malloc(size);
+
+	if (p) {
+		memset(p, 0, size);
+		return p;
+	}
+
+	g_error("allocation of %lu bytes failed", size);
+	return NULL;
+}
+
+gpointer
+g_realloc(gpointer p, gulong size)
+{
+	gpointer n;
+
+	if (size == 0) {
+		gm_vtable.free(p);
+		return NULL;
+	}
+
+	n = gm_vtable.realloc(p, size);
+
+	if (n)
+		return n;
+
+	g_error("re-allocation of %lu bytes failed", size);
+	return NULL;
+}
+
+void
+g_free(gpointer p)
+{
+	gm_vtable.free(p);
+}
+
+gpointer
+g_try_malloc(gulong size)
+{
+	return size > 0 ? gm_vtable.try_malloc(size) : NULL;
+}
+
+gpointer
+g_try_realloc(gpointer p, gulong size)
+{
+	return size > 0 ? gm_vtable.try_realloc(p, size) : NULL;
+}
+
+/**
+ * Emulates a calloc().
+ */
+static gpointer
+emulate_calloc(gsize n, gsize m)
+{
+	gpointer p;
+
+	if (n > 0 && m > 0 && m < ((size_t) -1) / n) {
+		size_t size = n * m;
+		p = gm_vtable.malloc(size);
+		memset(p, 0, size);
+	} else {
+		p = NULL;
+	}
+	return p;
+}
+
+/**
+ * Sets the GMemVTable to use for memory allocation.
+ * This function must be called before using any other GLib functions.
+ *
+ * The vtable only needs to provide malloc(), realloc(), and free() functions;
+ * GLib can provide default implementations of the others.
+ * The malloc() and realloc() implementations should return NULL on failure, 
+ */
+void
+g_mem_set_vtable(GMemVTable *vtable)
+{
+	gm_vtable.malloc = vtable->malloc;
+	gm_vtable.realloc = vtable->realloc;
+	gm_vtable.free = vtable->free;
+
+	gm_vtable.calloc = vtable->calloc ? vtable->calloc : emulate_calloc;
+	gm_vtable.try_malloc = vtable->try_malloc ?
+		vtable->try_malloc : vtable->malloc;
+	gm_vtable.try_realloc = vtable->try_realloc ?
+		vtable->try_realloc : vtable->realloc;
+}
+
+/**
+ * Are we using system's malloc?
+ */
+gboolean
+g_mem_is_system_malloc(void)
+{
+	return gm_vtable.malloc == malloc;
+}
+#endif	/* USE_GLIB1 */
+
 #ifndef TRACK_MALLOC
 
 /**
@@ -106,14 +247,26 @@ gm_list_insert_after(GList *list, GList *lnk, gpointer data)
 }
 
 #ifdef USE_GLIB1
+#undef g_list_delete_link		/* Remaped under -DTRACK_MALLOC */
+#undef g_slist_delete_link
 GList *
 g_list_delete_link(GList *l, GList *lnk)
 {
-	GList *new;
+	GList *head;
 
-	new = g_list_remove_link(l, lnk);
+	head = g_list_remove_link(l, lnk);
 	g_list_free_1(lnk);
-	return new;
+	return head;
+}
+
+GSList *
+g_slist_delete_link(GSList *sl, GSList *lnk)
+{
+	GSList *head;
+
+	head = g_slist_remove_link(sl, lnk);
+	g_slist_free_1(lnk);
+	return head;
 }
 #endif /* USE_GLIB1 */
 
@@ -250,78 +403,114 @@ gm_savemain(gint argc, gchar **argv, gchar **env)
 	orig_env = env;
 }
 
-/**
- * Change the process title as seen by "ps".
- */
-void
-gm_setproctitle(gchar *title)
+static inline size_t
+str_vec_count(gchar *strv[])
 {
-	static gint sysarglen = 0;		/* Length of the exec() arguments */
-	gint tlen;
-	gint i;
+	size_t i = 0;
+
+	while (strv[i]) {
+		i++;
+	}
+	return i;
+}
+
+#if !defined(HAS_SETPROCTITLE)
+/**
+ * Compute the length of the exec() arguments that were given to us.
+ *
+ * @param argc The original ``argc'' argument from main().
+ * @param argv The original ``argv'' argument from main().
+ * @param env_ptr The original ``env'' variable.
+ */
+static struct iovec
+gm_setproctitle_init(gint argc, gchar *argv[], gchar *env_ptr[])
+{
+	size_t env_count, n;
+	struct iovec *iov;
+
+	g_assert(argc > 0);
+	g_assert(argv);
+	g_assert(env_ptr);
+
+	env_count = str_vec_count(env_ptr);
+	n = argc + env_count;
+	iov = iov_alloc_n(n);
+
+	iov_reset_n(iov, n);
+
+	iov_init_from_string_vector(&iov[0], n, argv, argc);
+	iov_init_from_string_vector(&iov[argc], n - argc, env_ptr, env_count);
 
 	/*
-	 * Compute the length of the exec() arguments that were given to us.
+	 * Let's see how many argv[] arguments were contiguous.
 	 */
-
-	if (sysarglen == 0) {
-		gchar *s = orig_argv[0];
-
-		s += strlen(s) + 1;			/* Go past trailing NUL */
-
-		/*
-		 * Let's see whether all the argv[] arguments were contiguous.
-		 */
-
-		for (i = 1; i < orig_argc; i++) {
-			if (orig_argv[i] != s)
-				break;
-			s += strlen(s) + 1;		/* Yes, still contiguous */
-		}
-
-		/*
-		 * Maybe the environment is contiguous as well...
-		 */
-
-		for (i = 0; orig_env[i] != NULL; i++) {
-			if (orig_env[i] != s)
-				break;
-			s += strlen(s) + 1;		/* Yes, still contiguous */
-		}
-
-		sysarglen = s - orig_argv[0] - 1;	/* -1: leave room for NUL */
-
-#if 0
-		g_message("exec() args used %d contiguous bytes", sysarglen + 1);
-#endif
-	}
-
-	tlen = strlen(title);
-
-	if (tlen >= sysarglen) {		/* If too large, needs truncation */
-		memcpy(orig_argv[0], title, sysarglen);
-		(orig_argv[0])[sysarglen] = '\0';
-	} else {
-		memcpy(orig_argv[0], title, tlen + 1);	/* Copy trailing NUL */
-		if (tlen + 1 < sysarglen)
-			memset(orig_argv[0] + tlen + 1, ' ', sysarglen - tlen - 1);
+	{
+		size_t size;
+		
+		size = iov_contiguous_size(iov, n);
+		g_message("%lu bytes available for gm_setproctitle().", (gulong) size);
 	}
 
 	/*
 	 * Scrap references to the arguments.
 	 */
+	{
+		gint i;
 
-	for (i = 1; i < orig_argc; i++)
-		orig_argv[i] = NULL;
+		for (i = 1; i < argc; i++)
+			argv[i] = NULL;
+	}
+	
+	
+	return iov_get(iov, n);
+}
+#endif /* !HAS_SETPROCTITLE */
+
+/**
+ * Change the process title as seen by "ps".
+ */
+void
+gm_setproctitle(const gchar *title)
+#if defined(HAS_SETPROCTITLE)
+{
+	setproctitle("%s", title);
+}
+#else /* !HAS_SETPROCTITLE */
+{
+	static struct iovec *args;
+	static size_t n;
+
+	if (!args) {
+		struct iovec iov;
+		
+		iov = gm_setproctitle_init(orig_argc, orig_argv, orig_env);
+		args = cast_to_gpointer(iov.iov_base); /* Solaris has caddr_t */
+		n = iov.iov_len;
+	}
+
+	/* Scatter the title over the argv[] and env[] elements */
+	iov_scatter_string(args, n, title);
+}
+#endif /* HAS_SETPROCTITLE */
+
+/**
+ * Return the process title as seen by "ps"
+ */
+const gchar *
+gm_getproctitle(void)
+{
+	return orig_argv[0];
 }
 
 #ifdef USE_GLIB1
+#undef g_string_append_len		/* Macro when -DTRACK_MALLOC */
+#undef g_string_append_c		/* Macro when -DTRACK_MALLOC */
 /**
  * Appends len bytes of val to string. Because len is provided, val may
  * contain embedded nuls and need not be nul-terminated.
  */
 GString *
-g_string_append_len(GString *gs, const gchar *val,  gssize len)
+g_string_append_len(GString *gs, const gchar *val, gssize len)
 {
 	const gchar *p = val;
 
@@ -415,4 +604,45 @@ gm_sanitize_filename(const gchar *filename,
 	return q ? q : deconstify_gchar(s);
 }
 
+/**
+ * Frees the GString context but keeps the string data itself and returns
+ * it. With Gtk+ 2.x g_string_free(gs, FALSE) would do the job but the
+ * variant in Gtk+ 1.2 returns nothing.
+ *
+ * @return The string data.
+ */
+gchar *
+gm_string_finalize(GString *gs)
+{
+	gchar *s;
+
+	g_return_val_if_fail(gs, NULL);
+	g_return_val_if_fail(gs->str, NULL);
+	s = gs->str;
+	g_string_free(gs, FALSE);
+	return s;
+}
+
+/**
+ * Detects a loop in a singly-linked list.
+ *
+ * @return TRUE if the given slist contains a loop; FALSE otherwise.
+ */
+gboolean
+gm_slist_is_looping(const GSList *slist)
+{
+	const GSList *sl, *p;
+
+	p = slist;
+	sl = slist;
+	for (sl = slist; /* NOTHING */; sl = g_slist_next(sl)) {
+		p = g_slist_next(g_slist_next(p));
+		if (p == sl || p == g_slist_next(sl)) {
+			break;
+		}
+	}
+
+	return NULL != p;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
