@@ -1,5 +1,5 @@
 /*
- * $Id: inputevt.h,v 1.17 2005/09/27 20:11:30 rmanfredi Exp $
+ * $Id: inputevt.h 11860 2006-09-02 17:01:33Z cbiere $
  *
  * Copyright (c) 2002, ko (ko-@wanadoo.fr)
  * Copyright (c) 2005, Christian Biere
@@ -79,13 +79,8 @@ guint inputevt_add(gint source, inputevt_cond_t condition,
 	inputevt_handler_t handler, gpointer data);
 
 const gchar *inputevt_cond_to_string(inputevt_cond_t cond);
-
-#if defined(HAS_EPOLL) || defined(HAS_KQUEUE)
+gboolean inputevt_data_available(size_t *avail_ptr);
 void inputevt_remove(guint id);
-#else /* !(HAS_EPOLL || HAS_KQUEUE) */
-#define inputevt_remove(source) (g_source_remove(source))
-#define inputevt_timer()
-#endif /* HAS_EPOLL || HAS_KQUEUE */
 
 #endif  /* _inputevt_h_ */
 
