@@ -1,5 +1,5 @@
 /*
- * $Id: malloc.c,v 1.15 2005/11/15 14:37:25 cbiere Exp $
+ * $Id: malloc.c 11605 2006-08-18 21:01:35Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -44,7 +44,7 @@
  */
 
 #if defined(TRACK_MALLOC) || defined(TRACK_ZALLOC)
-RCSID("$Id: malloc.c,v 1.15 2005/11/15 14:37:25 cbiere Exp $");
+RCSID("$Id: malloc.c 11605 2006-08-18 21:01:35Z cbiere $")
 
 /*
  * When MALLOC_FRAMES is supplied, we keep information about the allocation
@@ -134,8 +134,7 @@ frame_hash(gconstpointer key)
 static gint
 frame_eq(gconstpointer a, gconstpointer b)
 {
-	struct frame *fa = (struct frame *) a;
-	struct frame *fb = (struct frame *) b;
+	const struct frame *fa = a, *fb = b;
 
 	return fa->len == fb->len &&
 		0 == memcmp(fa->stack, fb->stack, fa->len * sizeof(void *));
@@ -183,7 +182,7 @@ static GHashTable *stats = NULL; /**< maps stats(file, line) -> stats */
 static guint
 stats_hash(gconstpointer key)
 {
-	struct stats *s = (struct stats *) key;
+	const struct stats *s = key;
 
 	return g_str_hash(s->file) ^ s->line;
 }
@@ -195,8 +194,7 @@ stats_hash(gconstpointer key)
 static gint
 stats_eq(gconstpointer a, gconstpointer b)
 {
-	struct stats *sa = (struct stats *) a;
-	struct stats *sb = (struct stats *) b;
+	const struct stats *sa = a, *sb = b;
 
 	return  sa->line == sb->line && 0 == strcmp(sa->file, sb->file);
 }
@@ -210,7 +208,7 @@ stats_eq(gconstpointer a, gconstpointer b)
 static void
 malloc_init(void)
 {
-	blocks = g_hash_table_new(g_direct_hash, g_direct_equal);
+	blocks = g_hash_table_new(NULL, NULL);
 
 #ifdef MALLOC_STATS
 	stats = g_hash_table_new(stats_hash, stats_eq);
@@ -227,7 +225,7 @@ malloc_init(void)
 static void
 malloc_log_block(gpointer k, gpointer v, gpointer leaksort)
 {
-	struct block *b = (struct block *) v;
+	struct block *b = v;
 
 	g_warning("leaked block 0x%lx (%u bytes) from \"%s:%d\"",
 		(gulong) k, b->size, b->file, b->line);
@@ -235,7 +233,7 @@ malloc_log_block(gpointer k, gpointer v, gpointer leaksort)
 	leak_add(leaksort, b->size, b->file, b->line);
 
 	if (b->realloc) {
-		struct block *r = (struct block *) b->realloc->data;
+		struct block *r = b->realloc->data;
 		gint cnt = g_slist_length(b->realloc);
 
 		g_warning("   (realloc'ed %d time%s, lastly from \"%s:%d\")",
@@ -416,7 +414,7 @@ free_record(gpointer o, gchar *file, gint line)
 		return;
 	}
 
-	b = (struct block *) v;
+	b = v;
 	g_assert(o == k);
 
 #ifdef MALLOC_STATS
@@ -803,9 +801,13 @@ hashtable_destroy_track(GHashTable *h, gchar *file, gint line)
  * Wrapper over hash_list_new().
  */
 hash_list_t *
-hash_list_new_track(gchar *file, gint line)
+hash_list_new_track(
+	GHashFunc hash_func, GEqualFunc eq_func, gchar *file, gint line)
 {
-	return malloc_record(hash_list_new(), 28, file, line);	/* Approx. size */
+	return malloc_record(
+		hash_list_new(hash_func, eq_func),
+		28,				/* Approx. size */
+		file, line);
 }
 
 /**
@@ -826,17 +828,17 @@ hash_list_free_track(hash_list_t *h, gchar *file, gint line)
 #define GLIST_LINK_SIZE		12		/* Random size */
 
 GSList *
-slist_alloc_track(gchar *file, gint line)
+track_slist_alloc(gchar *file, gint line)
 {
 	return malloc_record(g_slist_alloc(), GSLIST_LINK_SIZE, file, line);
 }
 
 GSList *
-slist_append_track(GSList *l, gpointer data, gchar *file, gint line)
+track_slist_append(GSList *l, gpointer data, gchar *file, gint line)
 {
 	GSList *new;
 
-	new = slist_alloc_track(file, line);
+	new = track_slist_alloc(file, line);
 	new->data = data;
 
 	if (l) {
@@ -848,11 +850,11 @@ slist_append_track(GSList *l, gpointer data, gchar *file, gint line)
 }
 
 GSList *
-slist_prepend_track(GSList *l, gpointer data, gchar *file, gint line)
+track_slist_prepend(GSList *l, gpointer data, gchar *file, gint line)
 {
 	GSList *new;
 
-	new = slist_alloc_track(file, line);
+	new = track_slist_alloc(file, line);
 	new->data = data;
 	new->next = l;
 
@@ -860,7 +862,7 @@ slist_prepend_track(GSList *l, gpointer data, gchar *file, gint line)
 }
 
 GSList *
-slist_copy_track(GSList *list, gchar *file, gint line)
+track_slist_copy(GSList *list, gchar *file, gint line)
 {
 	GSList *new;
 	GSList *l;
@@ -874,7 +876,7 @@ slist_copy_track(GSList *list, gchar *file, gint line)
 }
 
 void
-slist_free_track(GSList *l, gchar *file, gint line)
+track_slist_free(GSList *l, gchar *file, gint line)
 {
 	GSList *lk;
 
@@ -885,7 +887,7 @@ slist_free_track(GSList *l, gchar *file, gint line)
 }
 
 void
-slist_free1_track(GSList *l, gchar *file, gint line)
+track_slist_free1(GSList *l, gchar *file, gint line)
 {
 	if (l == NULL)
 		return;
@@ -895,7 +897,7 @@ slist_free1_track(GSList *l, gchar *file, gint line)
 }
 
 GSList *
-slist_remove_track(GSList *l, gpointer data, gchar *file, gint line)
+track_slist_remove(GSList *l, gpointer data, gchar *file, gint line)
 {
 	GSList *lk;
 
@@ -903,29 +905,39 @@ slist_remove_track(GSList *l, gpointer data, gchar *file, gint line)
 	if (lk == NULL)
 		return l;
 
-	free_record(lk, file, line);
-	return g_slist_remove(l, data);
+	return track_slist_delete_link(l, lk, file, line);
 }
 
 GSList *
-slist_insert_track(GSList *l, gpointer data, gint pos, gchar *file, gint line)
+track_slist_delete_link(GSList *l, GSList *lk, gchar *file, gint line)
+{
+	GSList *new;
+
+	new = g_slist_remove_link(l, lk);
+	track_slist_free1(lk, file, line);
+
+	return new;
+}
+
+GSList *
+track_slist_insert(GSList *l, gpointer data, gint pos, gchar *file, gint line)
 {
 	GSList *lk;
 
 	if (pos < 0)
-		return slist_append_track(l, data, file, line);
+		return track_slist_append(l, data, file, line);
 	else if (pos == 0)
-		return slist_prepend_track(l, data, file, line);
+		return track_slist_prepend(l, data, file, line);
 
 	lk = g_slist_nth(l, pos - 1);
 	if (lk == NULL)
-		return slist_append_track(l, data, file, line);
+		return track_slist_append(l, data, file, line);
 	else
-		return slist_insert_after_track(l, lk, data, file, line);
+		return track_slist_insert_after(l, lk, data, file, line);
 }
 
 GSList *
-slist_insert_sorted_track(GSList *l, gpointer d, GCompareFunc c,
+track_slist_insert_sorted(GSList *l, gpointer d, GCompareFunc c,
 	gchar *file, gint line)
 {
 	gint cmp;
@@ -934,7 +946,7 @@ slist_insert_sorted_track(GSList *l, gpointer d, GCompareFunc c,
 	GSList *new;
 
 	if (l == NULL)
-		return slist_prepend_track(l, d, file, line);
+		return track_slist_prepend(l, d, file, line);
 
 	cmp = (*c)(d, tmp->data);
 	while (tmp->next != NULL && cmp > 0) {
@@ -943,7 +955,7 @@ slist_insert_sorted_track(GSList *l, gpointer d, GCompareFunc c,
 		cmp = (*c)(d, tmp->data);
 	}
 
-	new = slist_alloc_track(file, line);
+	new = track_slist_alloc(file, line);
 	new->data = d;
 
 	if (tmp->next == NULL && cmp > 0) {
@@ -962,15 +974,15 @@ slist_insert_sorted_track(GSList *l, gpointer d, GCompareFunc c,
 }
 
 GSList *
-slist_insert_after_track(GSList *l, GSList *lk, gpointer data,
+track_slist_insert_after(GSList *l, GSList *lk, gpointer data,
 	gchar *file, gint line)
 {
 	GSList *new;
 
 	if (lk == NULL)
-		return slist_prepend_track(l, data, file, line);
+		return track_slist_prepend(l, data, file, line);
 
-	new = slist_alloc_track(file, line);
+	new = track_slist_alloc(file, line);
 	new->data = data;
 
 	new->next = lk->next;
@@ -980,17 +992,17 @@ slist_insert_after_track(GSList *l, GSList *lk, gpointer data,
 }
 
 GList *
-list_alloc_track(gchar *file, gint line)
+track_list_alloc(gchar *file, gint line)
 {
 	return malloc_record(g_list_alloc(), GLIST_LINK_SIZE, file, line);
 }
 
 GList *
-list_append_track(GList *l, gpointer data, gchar *file, gint line)
+track_list_append(GList *l, gpointer data, gchar *file, gint line)
 {
 	GList *new;
 
-	new = list_alloc_track(file, line);
+	new = track_list_alloc(file, line);
 	new->data = data;
 
 	if (l) {
@@ -1003,11 +1015,11 @@ list_append_track(GList *l, gpointer data, gchar *file, gint line)
 }
 
 GList *
-list_prepend_track(GList *l, gpointer data, gchar *file, gint line)
+track_list_prepend(GList *l, gpointer data, gchar *file, gint line)
 {
 	GList *new;
 
-	new = list_alloc_track(file, line);
+	new = track_list_alloc(file, line);
 	new->data = data;
 
 	if (l) {
@@ -1023,7 +1035,7 @@ list_prepend_track(GList *l, gpointer data, gchar *file, gint line)
 }
 
 GList *
-list_copy_track(GList *list, gchar *file, gint line)
+track_list_copy(GList *list, gchar *file, gint line)
 {
 	GList *new;
 	GList *l;
@@ -1037,7 +1049,7 @@ list_copy_track(GList *list, gchar *file, gint line)
 }
 
 void
-list_free_track(GList *l, gchar *file, gint line)
+track_list_free(GList *l, gchar *file, gint line)
 {
 	GList *lk;
 
@@ -1048,7 +1060,7 @@ list_free_track(GList *l, gchar *file, gint line)
 }
 
 void
-list_free1_track(GList *l, gchar *file, gint line)
+track_list_free1(GList *l, gchar *file, gint line)
 {
 	if (l == NULL)
 		return;
@@ -1058,7 +1070,7 @@ list_free1_track(GList *l, gchar *file, gint line)
 }
 
 GList *
-list_remove_track(GList *l, gpointer data, gchar *file, gint line)
+track_list_remove(GList *l, gpointer data, gchar *file, gint line)
 {
 	GList *lk;
 
@@ -1066,28 +1078,28 @@ list_remove_track(GList *l, gpointer data, gchar *file, gint line)
 	if (lk == NULL)
 		return l;
 
-	return list_delete_link_track(l, lk, file, line);
+	return track_list_delete_link(l, lk, file, line);
 }
 
 GList *
-list_insert_track(GList *l, gpointer data, gint pos, gchar *file, gint line)
+track_list_insert(GList *l, gpointer data, gint pos, gchar *file, gint line)
 {
 	GList *lk;
 
 	if (pos < 0)
-		return list_append_track(l, data, file, line);
+		return track_list_append(l, data, file, line);
 	else if (pos == 0)
-		return list_prepend_track(l, data, file, line);
+		return track_list_prepend(l, data, file, line);
 
 	lk = g_list_nth(l, pos - 1);
 	if (lk == NULL)
-		return list_append_track(l, data, file, line);
+		return track_list_append(l, data, file, line);
 	else
-		return list_insert_after_track(l, lk, data, file, line);
+		return track_list_insert_after(l, lk, data, file, line);
 }
 
 GList *
-list_insert_sorted_track(GList *l, gpointer d, GCompareFunc c,
+track_list_insert_sorted(GList *l, gpointer d, GCompareFunc c,
 	gchar *file, gint line)
 {
 	gint cmp;
@@ -1095,7 +1107,7 @@ list_insert_sorted_track(GList *l, gpointer d, GCompareFunc c,
 	GList *new;
 
 	if (l == NULL)
-		return list_prepend_track(l, d, file, line);
+		return track_list_prepend(l, d, file, line);
 
 	cmp = (*c)(d, tmp->data);
 	while (tmp->next != NULL && cmp > 0) {
@@ -1103,7 +1115,7 @@ list_insert_sorted_track(GList *l, gpointer d, GCompareFunc c,
 		cmp = (*c)(d, tmp->data);
 	}
 
-	new = list_alloc_track(file, line);
+	new = track_list_alloc(file, line);
 	new->data = d;
 
 	if (tmp->next == NULL && cmp > 0) {
@@ -1126,15 +1138,15 @@ list_insert_sorted_track(GList *l, gpointer d, GCompareFunc c,
 }
 
 GList *
-list_insert_after_track(GList *l, GList *lk, gpointer data,
+track_list_insert_after(GList *l, GList *lk, gpointer data,
 	gchar *file, gint line)
 {
 	GList *new;
 
 	if (lk == NULL)
-		return list_prepend_track(l, data, file, line);
+		return track_list_prepend(l, data, file, line);
 
-	new = list_alloc_track(file, line);
+	new = track_list_alloc(file, line);
 	new->data = data;
 
 	new->prev = lk;
@@ -1149,12 +1161,12 @@ list_insert_after_track(GList *l, GList *lk, gpointer data,
 }
 
 GList *
-list_delete_link_track(GList *l, GList *lk, gchar *file, gint line)
+track_list_delete_link(GList *l, GList *lk, gchar *file, gint line)
 {
 	GList *new;
 
 	new = g_list_remove_link(l, lk);
-	list_free1_track(lk, file, line);
+	track_list_free1(lk, file, line);
 
 	return new;
 }
@@ -1370,7 +1382,7 @@ leak_free_kv(gpointer key, gpointer value, gpointer unused_user)
 void
 leak_close(gpointer o)
 {
-	struct leak_set *ls = (struct leak_set *) o;
+	struct leak_set *ls = o;
 
 	g_hash_table_foreach_remove(ls->places, leak_free_kv, NULL);
 	g_hash_table_destroy(ls->places);
@@ -1399,7 +1411,7 @@ leak_add(gpointer o, guint32 size, gchar *file, gint line)
 	found = g_hash_table_lookup_extended(ls->places, key, &k, &v);
 
 	if (found) {
-		lr = (struct leak_record *) v;
+		lr = v;
 		lr->size += size;
 		lr->count++;
 	} else {
@@ -1424,12 +1436,10 @@ struct leak {			/* A memory leak, for sorting purposes */
 static gint
 leak_size_cmp(const void *p1, const void *p2)
 {
-	guint32 i1 = ((struct leak *) p1)->lr->size;
-	guint32 i2 = ((struct leak *) p2)->lr->size;
+	const struct leak *leak1 = p1, *leak2 = p2;
+	guint32 i1 = leak1->lr->size, i2 = leak2->lr->size;
 
-	return
-		i1 == i2 ?  0 :
-		i1 < i2  ? +1 : -1;		/* Reverse order: largest first */
+	return CMP(i2, i1); /* Reverse order: largest first */
 }
 
 struct filler {			/* Used by hash table iterator to fill leak array */
@@ -1463,7 +1473,7 @@ fill_array(gpointer key, gpointer value, gpointer user)
 void
 leak_dump(gpointer o)
 {
-	struct leak_set *ls = (struct leak_set *) o;
+	struct leak_set *ls =  o;
 	gint count;
 	struct filler filler;
 	gint i;
@@ -1526,12 +1536,10 @@ struct afiller {		/* Used by hash table iterator to fill alloc array */
 static gint
 stats_allocated_cmp(const void *p1, const void *p2)
 {
-	guint32 i1 = (*(struct stats **) p1)->allocated;
-	guint32 i2 = (*(struct stats **) p2)->allocated;
+	const struct stats * const *s1 = p1, * const *s2 = p2;
+	guint32 i1 = (*s1)->allocated, i2 = (*s2)->allocated;
 
-	return
-		i1 == i2 ?  0 :
-		i1 < i2  ? +1 : -1;		/* Reverse order: largest first */
+	return CMP(i2, i1);	/* Reverse order: largest first */
 }
 
 /**
@@ -1541,12 +1549,10 @@ stats_allocated_cmp(const void *p1, const void *p2)
 static gint
 stats_total_allocated_cmp(const void *p1, const void *p2)
 {
-	guint32 i1 = (*(struct stats **) p1)->total_allocated;
-	guint32 i2 = (*(struct stats **) p2)->total_allocated;
+	const struct stats * const *s1 = p1, * const *s2 = p2;
+	guint32 i1 = (*s1)->total_allocated, i2 = (*s2)->total_allocated;
 
-	return
-		i1 == i2 ?  0 :
-		i1 < i2  ? +1 : -1;		/* Reverse order: largest first */
+	return CMP(i2, i1);	/* Reverse order: largest first */
 }
 
 /**
@@ -1556,14 +1562,14 @@ stats_total_allocated_cmp(const void *p1, const void *p2)
 static gint
 stats_residual_cmp(const void *p1, const void *p2)
 {
-	struct stats *s1 = *(struct stats **) p1;
-	struct stats *s2 = *(struct stats **) p2;
+	const struct stats * const *s1_ptr = p1, * const *s2_ptr = p2;
+	const struct stats *s1 = *s1_ptr, *s2 = *s2_ptr;
 	gint32 i1 = s1->allocated + s1->reallocated - s1->freed;
 	gint32 i2 = s2->allocated + s2->reallocated - s2->freed;
 
-	return
-		i1 == i2 ?  stats_allocated_cmp(p1, p2) :
-		i1 < i2  ? +1 : -1;		/* Reverse order: largest first */
+	return i1 == i2
+		? stats_allocated_cmp(p1, p2)
+		: CMP(i2, i1); /* Reverse order: largest first */
 }
 
 /**
@@ -1573,14 +1579,14 @@ stats_residual_cmp(const void *p1, const void *p2)
 static gint
 stats_total_residual_cmp(const void *p1, const void *p2)
 {
-	struct stats *s1 = *(struct stats **) p1;
-	struct stats *s2 = *(struct stats **) p2;
+	const struct stats * const *s1_ptr = p1, * const *s2_ptr = p2;
+	const struct stats *s1 = *s1_ptr, *s2 = *s2_ptr;
 	gint32 i1 = s1->total_allocated + s1->total_reallocated - s1->total_freed;
 	gint32 i2 = s2->total_allocated + s2->total_reallocated - s2->total_freed;
 
-	return
-		i1 == i2 ?  stats_total_allocated_cmp(p1, p2) :
-		i1 < i2  ? +1 : -1;		/* Reverse order: largest first */
+	return i1 == i2
+		? stats_total_allocated_cmp(p1, p2)
+		: CMP(i2, i1); /* Reverse order: largest first */
 }
 
 /**
@@ -1590,8 +1596,8 @@ stats_total_residual_cmp(const void *p1, const void *p2)
 static void
 stats_fill_array(gpointer unused_key, gpointer value, gpointer user)
 {
-	struct afiller *filler = (struct afiller *) user;
-	struct stats *st = (struct stats *) value;
+	struct afiller *filler = user;
+	struct stats *st = value;
 	struct stats **e;
 
 	(void) unused_key;
@@ -1622,15 +1628,15 @@ stats_array_dump(FILE *f, struct afiller *filler)
 		gint remains = st->allocated + st->reallocated - st->freed;
 		gint total_remains =
 			st->total_allocated + st->total_reallocated - st->total_freed;
-		gchar *c_allocated = strdup(compact_size(st->allocated));
-		gchar *c_freed = strdup(compact_size(st->freed));
-		gchar *c_reallocated = strdup(compact_size(ABS(st->reallocated)));
-		gchar *c_remains = strdup(compact_size(ABS(remains)));
-		gchar *c_tallocated = strdup(compact_size(st->total_allocated));
-		gchar *c_tfreed = strdup(compact_size(st->total_freed));
+		gchar *c_allocated = strdup(compact_size(st->allocated, TRUE));
+		gchar *c_freed = strdup(compact_size(st->freed, TRUE));
+		gchar *c_reallocated = strdup(compact_size(ABS(st->reallocated), TRUE));
+		gchar *c_remains = strdup(compact_size(ABS(remains), TRUE));
+		gchar *c_tallocated = strdup(compact_size(st->total_allocated, TRUE));
+		gchar *c_tfreed = strdup(compact_size(st->total_freed, TRUE));
 		gchar *c_treallocated =
-			strdup(compact_size(ABS(st->total_reallocated)));
-		gchar *c_tremains = strdup(compact_size(ABS(total_remains)));
+			strdup(compact_size(ABS(st->total_reallocated), TRUE));
+		gchar *c_tremains = strdup(compact_size(ABS(total_remains), TRUE));
 
 #ifdef MALLOC_FRAMES
 		alloc_stacks = st->alloc_frames == NULL ?
@@ -1690,7 +1696,7 @@ alloc_dump(FILE *f, gboolean total)
 	fprintf(f, "--- distinct allocation spots found: %d at %s\n",
 		count, short_time(now - init_time));
 
-	filler.stats = (struct stats **) malloc(sizeof(struct stats *) * count);
+	filler.stats = malloc(sizeof(struct stats *) * count);
 	filler.count = count;
 	filler.idx = 0;
 
@@ -1738,7 +1744,7 @@ alloc_dump(FILE *f, gboolean total)
 static void
 stats_reset(gpointer uu_key, gpointer value, gpointer uu_user)
 {
-	struct stats *st = (struct stats *) value;
+	struct stats *st = value;
 
 	(void) uu_key;
 	(void) uu_user;
