@@ -1,10 +1,8 @@
 /*
- * $Id: tiger.h,v 1.1 2004/08/24 23:09:57 rmanfredi Exp $
+ * $Id: tiger.h 11145 2006-06-22 19:32:48Z cbiere $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
- * Tiger hash.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,13 +23,23 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Tiger hash.
+ *
+ * @author Jeroen Asselman
+ * @date 2003
+ */
 
 #ifndef _tiger_h_
 #define _tiger_h_
 
 #include <glib.h>
 
-void tiger(guint64 *str, guint64 length, guint64 res[3]);
+void tiger_check(void);
+void tiger(gconstpointer data, guint64 length, guchar hash[24]);
 
 #endif /* _tiger_h_ */
-
+/* vi: set ts=4 sw=4 cindent: */
