@@ -1,5 +1,5 @@
 /*
- * $Id: tm.c,v 1.12 2005/10/01 13:56:52 cbiere Exp $
+ * $Id: tm.c 11916 2006-09-08 10:28:52Z cbiere $
  *
  * Copyright (c) 2003-2005, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tm.c,v 1.12 2005/10/01 13:56:52 cbiere Exp $");
+RCSID("$Id: tm.c 11916 2006-09-08 10:28:52Z cbiere $")
 
 #ifdef I_SYS_TIMES
 #include <sys/times.h>
@@ -123,12 +123,23 @@ tm_now(tm_t *tm)
 
 /**
  * Fill supplied structure with current time (recomputed).
+ * If the time jumps backward the previously recorded timestamp
+ * is used instead to enforce a monotonic flow of time.
  */
 void
 tm_now_exact(tm_t *tm)
 {
+	const tm_t past = now;
+	
 	g_get_current_time(&now);
-	*tm = now;
+	if (now.tv_sec < past.tv_sec) {
+		now = past;
+	} else if (now.tv_sec == past.tv_sec) {
+		if (now.tv_usec < past.tv_usec)
+			now.tv_usec = past.tv_usec;
+	}
+	if (tm)
+		*tm = now;
 }
 
 /**
@@ -146,7 +157,7 @@ tm_time(void)
 time_t
 tm_time_exact(void)
 {
-	g_get_current_time(&now);
+	tm_now_exact(NULL);
 	return (time_t) now.tv_sec;
 }
 
@@ -231,7 +242,8 @@ tm_cputime(gdouble *user, gdouble *sys)
 
 		errno = 0;
 		if (-1 == getrusage(RUSAGE_SELF, &usage)) {
-			u = s = 0;
+			u = 0;
+			s = 0;
 			g_warning("getrusage(RUSAGE_SELF, ...) failed: %s",
 				errno ? g_strerror(errno) : "unsupported");
 		} else {
@@ -241,6 +253,10 @@ tm_cputime(gdouble *user, gdouble *sys)
 #else
 		getrusage_failed = TRUE;
 #endif /* HAS_GETRUSAGE */
+	} else {
+		/* For stupid compilers */
+		u = 0;
+		s = 0;
 	}
 
 	if (getrusage_failed) {	
