@@ -1,5 +1,5 @@
 /*
- * $Id: urn.h,v 1.1 2004/08/27 15:27:06 rmanfredi Exp $
+ * $Id: urn.h 8551 2005-07-07 12:59:23Z daichik $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -23,6 +23,16 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * URN handling of specific formats.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2004
+ */
+
 #ifndef _urn_h_
 #define _urn_h_
 
@@ -32,11 +42,10 @@
  * Public interface.
  */
 
-gboolean urn_get_http_sha1(gchar *buf, gchar *retval);
-gboolean urn_get_sha1(gchar *buf, gchar *digest);
-gboolean urn_get_sha1_no_prefix(gchar *buf, gchar *digest);
+gboolean urn_get_http_sha1(const gchar *buf, gchar *retval);
+gboolean urn_get_sha1(const gchar *buf, gchar *digest);
+gboolean urn_get_sha1_no_prefix(const gchar *buf, gchar *digest);
 
 #endif	/* _urn_h_ */
 
-/* vi: set ts=4: */
-
+/* vi: set ts=4 sw=4 cindent: */
