@@ -1,5 +1,5 @@
 /*
- * $Id: utf8.h,v 1.41 2006/02/02 23:58:20 cbiere Exp $
+ * $Id: utf8.h 11951 2006-09-13 05:13:29Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _utf8_h_
 #define _utf8_h_
 
-#include <glib.h>
+#include "common.h"
 
 
 #if 0  /* xxxUSE_ICU */
@@ -85,6 +85,8 @@ gboolean utf8_is_valid_string(const gchar *s);
 gboolean utf8_is_valid_data(const gchar *s, size_t n);
 size_t utf8_char_count(const gchar *s);
 size_t utf8_strlcpy(gchar *dst, const gchar *src, size_t dst_size);
+size_t utf8_strcpy_max(gchar *dst, size_t dst_size,
+			const gchar *src, size_t max_chars);
 guint32 utf8_decode_char_fast(const gchar *s, guint *retlen)
 	NON_NULL_PARAM((1,2));
 gint utf8_to_iso8859(gchar *s, gint len, gboolean space);
@@ -95,6 +97,7 @@ gchar *utf8_strupper_copy(const gchar *src);
 gchar *utf8_canonize(const gchar *src);
 gchar *utf8_normalize(const gchar *src, uni_norm_t norm);
 
+guint NON_NULL_PARAM((2)) utf8_encode_char(guint32 uc, gchar *buf, size_t size);
 size_t utf32_to_utf8(const guint32 *in, gchar *out, size_t size);
 guint32 utf32_lowercase(guint32 uc);
 
@@ -146,6 +149,27 @@ utf16_encode_char_compact(guint32 uc)
 	return (guint32) -1;
 }
 
+static inline gboolean
+utf8_byte_is_allowed(guchar c)
+{
+	switch (c) {
+	case 0xC0:
+	case 0xC1:
+	case 0xF5:
+	case 0xF6:
+	case 0xF7:
+	case 0xF8:
+	case 0xFA:
+	case 0xFB:
+	case 0xFC:
+	case 0xFD:
+	case 0xFE:
+	case 0xFF:
+		return FALSE;
+	}
+	return TRUE;
+}
+
 /**
  * Lazy converters either return a pointer to a static buffer or manage
  * the allocated memory themselves. They may also return the original
@@ -163,6 +187,8 @@ const gchar *lazy_locale_to_utf8(const gchar *src);
 const gchar *lazy_locale_to_ui_string(const gchar *src);
 const gchar *lazy_locale_to_ui_string2(const gchar *src);
 
+const gchar *lazy_filename_to_ui_string(const gchar *src);
+
 const gchar *lazy_locale_to_utf8_normalized(const gchar *src, uni_norm_t norm);
 const gchar *lazy_unknown_to_utf8_normalized(const gchar *src, uni_norm_t norm,
 				const gchar **charset_ptr);
