@@ -1,5 +1,5 @@
 /*
- * $Id: vendors.c,v 1.14 2005/11/13 10:36:12 cbiere Exp $
+ * $Id: vendors.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,16 +35,16 @@
 
 #include "common.h"
 
-RCSID("$Id: vendors.c,v 1.14 2005/11/13 10:36:12 cbiere Exp $");
+RCSID("$Id: vendors.c 12242 2006-11-06 20:41:55Z cbiere $")
 
 #include "vendors.h"
 #include "endian.h"
 #include "misc.h"
 #include "override.h"	/* Must be the last header included */
 
-struct vendor {
+static const struct vendor {
     guint32 code;
-    gchar *name;
+    const gchar *name;
 } vendor_map[] = {
 	/* This array MUST be sorted, because it is searched dichotomically */
 
@@ -74,6 +74,7 @@ struct vendor {
     { T_GNUC, "Gnucleus" },
     { T_GNUM, "Gnuminous" },
     { T_GNUT, "Gnut" },
+    { T_GNZL, "Gnoozle" },
     { T_GTKG, "gtk-gnutella" },
     { T_HSLG, "Hagelslag" },
     { T_HUIT, "Huitella" },
@@ -104,6 +105,7 @@ struct vendor {
     { T_QTEL, "Qtella" },
     { T_RASP, "Rasputin" },
     { T_RAZA, "Shareaza" },
+    { T_RAZB, "Shareaza-v2" },
     { T_SHNB, "Shinobu" },
     { T_SNOW, "FrostWire" },
     { T_SNUT, "SwapNut" },
@@ -128,7 +130,7 @@ struct vendor {
  *
  * @returns vendor string if found, NULL otherwise.
  */
-static gchar *
+static const gchar *
 find_vendor(guint32 code)
 {
 #define GET_KEY(i) (vendor_map[(i)].code)
@@ -195,8 +197,8 @@ vendor_code_str(guint32 code)
 const gchar *
 lookup_vendor_name(union vendor_code code)
 {
-	static gchar temp[1 + G_N_ELEMENTS(code.b)];
-	gchar *name;
+	static gchar temp[5];
+	const gchar *name;
     guint i;
 
 	STATIC_ASSERT(5 == G_N_ELEMENTS(temp));
@@ -210,9 +212,11 @@ lookup_vendor_name(union vendor_code code)
 
 	
 	/* Unknown type, look whether we have all printable ASCII */
-	for (i = 0; i < G_N_ELEMENTS(code.b); i++) {
-		if (is_ascii_print(code.b[i]))
-            temp[i] = code.b[i];
+	for (i = 0; i < G_N_ELEMENTS(temp) - 1; i++) {
+		const guint8 *u8 = cast_to_gconstpointer(&code.be32);
+
+		if (is_ascii_print(u8[i]))
+            temp[i] = u8[i];
 		else {
             temp[0] = '\0';
 			break;
