@@ -1,5 +1,5 @@
 /*
- * $Id: zalloc.c,v 1.8 2005/09/10 10:10:36 daichik Exp $
+ * $Id: zalloc.c 12061 2006-09-27 19:42:46Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,11 +35,45 @@
 
 #include "common.h"
 
-RCSID("$Id: zalloc.c,v 1.8 2005/09/10 10:10:36 daichik Exp $");
+RCSID("$Id: zalloc.c 12061 2006-09-27 19:42:46Z cbiere $")
 
 #include "zalloc.h"
+#include "hashtable.h"
+#include "misc.h"
+
 #include "override.h"		/* Must be the last header included */
 
+/**
+ * Extra allocated zones.
+ */
+struct subzone {
+	struct subzone *sz_next;	/**< Next allocated zone chunk, NULL if last */
+	gpointer sz_base;			/**< Base address of zone arena */
+	size_t  sz_size;			/**< Size of zone arena */
+};
+
+
+/**
+ * @struct zone
+ *
+ * Zone structure.
+ *
+ * Zone structures can be linked together to form one big happy chain.
+ * During allocation/free, only the first zone structure of the list is
+ * updated. The other zone structures are updated only during the garbage
+ * collecting phase, if any.
+ */
+
+struct zone {			/* Zone descriptor */
+	gchar **zn_free;	/**< Pointer to first free block */
+	struct subzone *zn_next;/**< Next allocated zone chunk, null if none */
+    struct subzone zn_arena;
+	gint zn_refcnt;		/**< How many references to that zone? */
+	gint zn_size;		/**< Size of blocks in zone */
+	gint zn_hint;		/**< Hint size, for next zone extension */
+	gint zn_cnt;		/**< Amount of used blocks in zone */
+};
+
 /*
  * Define ZONE_SAFE to allow detection of duplicate frees on a zone object.
  *
@@ -76,20 +110,23 @@ RCSID("$Id: zalloc.c,v 1.8 2005/09/10 10:10:36 daichik Exp $");
 #define DEFAULT_HINT		128		/**< Default amount of blocks in a zone */
 #define MAX_ZONE_SIZE		16384	/**< Maximum zone size */
 
-/**
- * Extra allocated zones.
- */
-struct subzone {
-	struct subzone *zn_next;	/**< Next allocated zone chunk, null if last */
-	gpointer zn_arena;			/**< Base address of zone arena */
-};
+/* Under REMAP_ZALLOC, map zalloc() and zfree() to g_malloc() and g_free() */
 
-static void zn_cram(zone_t *, gchar *, gint);
-static struct zone *zn_create(zone_t *, gint, gint);
+#ifdef REMAP_ZALLOC
+gpointer
+zalloc(zone_t *zone)
+{
+	return g_malloc(zone->zn_size);
+}
 
-/* Under REMAP_ZALLOC, do not define zalloc() and zfree(). */
+void
+zfree(zone_t *zone, gpointer ptr)
+{
+	(void) zone;
+	g_free(ptr);
+}
 
-#ifndef REMAP_ZALLOC
+#else	/* !REMAP_ZALLOC */
 
 static gchar **zn_extend(zone_t *);
 
@@ -122,7 +159,8 @@ static gchar **zn_extend(zone_t *);
  * Moreover, periodic calls to the zone gc are needed to collect unused chunks
  * when peak allocations are infrequent or occur at random.
  */
-gpointer zalloc(zone_t *zone)
+gpointer
+zalloc(zone_t *zone)
 {
 	gchar **blk;		/**< Allocated block */
 
@@ -159,7 +197,7 @@ gpointer zalloc(zone_t *zone)
 	 * Use first block from new extended zone.
 	 */
 
-	zone->zn_free = (char **) *blk;
+	zone->zn_free = (gchar **) *blk;
 	zone->zn_cnt++;
 
 #ifdef ZONE_SAFE
@@ -176,7 +214,8 @@ gpointer zalloc(zone_t *zone)
 /**
  * Tracking version of zalloc().
  */
-gpointer zalloc_track(zone_t *zone, gchar *file, gint line)
+gpointer
+zalloc_track(zone_t *zone, gchar *file, gint line)
 {
 	gchar *blk = zalloc(zone);
 	gchar *p;
@@ -194,7 +233,8 @@ gpointer zalloc_track(zone_t *zone, gchar *file, gint line)
  * the user part of it.  The block is known to be of size `size' and should
  * be also recorded in the `leakset' for summarizing of all the leaks.
  */
-static void zblock_log(gchar *p, gint size, gpointer leakset)
+static void
+zblock_log(gchar *p, gint size, gpointer leakset)
 {
 	gchar *uptr;			/* User pointer */
 	gchar *file;
@@ -214,14 +254,15 @@ static void zblock_log(gchar *p, gint size, gpointer leakset)
 /**
  * Go through the whole zone and dump all the used blocks.
  */
-static void zdump_used(zone_t *zone)
+static void
+zdump_used(zone_t *zone)
 {
 	gint used = 0;
 	struct subzone *next;
 	gchar *p;
 	gpointer leakset = leak_init();
 
-	p = (gchar *) zone->zn_arena;
+	p = zone->zn_base;
 	next = zone->zn_next;
 
 	for (;;) {
@@ -238,7 +279,7 @@ static void zdump_used(zone_t *zone)
 		if (next == NULL)
 			break;
 
-		p = (gchar *) next->zn_arena;
+		p = next->zn_base;
 		next = next->zn_next;
 	}
 
@@ -261,7 +302,8 @@ static void zdump_used(zone_t *zone)
  *
  * Warning: returning a block to the wrong zone may lead to disasters.
  */
-void zfree(zone_t *zone, gpointer ptr)
+void
+zfree(zone_t *zone, gpointer ptr)
 {
 	g_assert(ptr);
 	g_assert(zone);
@@ -282,36 +324,68 @@ void zfree(zone_t *zone, gpointer ptr)
 	g_assert(zone->zn_cnt > 0);		/* There must be something to free! */
 
 	*(gchar **) ptr = (gchar *) zone->zn_free;	/* Will precede old head */
-	zone->zn_free = (gchar **) ptr;				/* New free list head */
+	zone->zn_free = ptr;						/* New free list head */
 	zone->zn_cnt--;								/* To make zone gc easier */
 }
 #endif	/* !REMAP_ZALLOC */
 
 /**
- * Create a new zone able to hold items of 'size' bytes. Returns
- * NULL if no new zone can be created.
+ * Cram a new zone in chunk.
  *
- * The hint argument is to be construed as the average amount of objects
- * that are to be created per zone chunks. That is not the total amount of
- * expected objects of a given type. Leaving it a 0 selects the default hint
- * value.
+ * A zone consists of linked blocks, where the address of the next free block
+ * is written in the first bytes of each free block.
  */
-zone_t *zcreate(gint size, gint hint)
+static void
+zn_cram(zone_t *zone, gpointer arena)
 {
-	zone_t *zone;			/* Zone descriptor */
+	gint i;
+	gchar **next = arena, *p = arena;
 
-	zone = g_malloc(sizeof(*zone));
-
-	return zn_create(zone, size, hint);
+	for (i = 1; i < zone->zn_hint; i++) {
+		next = (gpointer) p;
+		*next = &p[zone->zn_size];
+		p = *next;
+	}
+	*next = NULL;
 }
 
+static void
+subzone_alloc_arena(struct subzone *sz, size_t size)
+{
+	size_t threshold = (compat_pagesize() / 32) * 31;
+
+	if (size < threshold) {
+		sz->sz_size = size;
+		sz->sz_base = malloc(sz->sz_size);
+	} else {
+		sz->sz_size = MAX(size, compat_pagesize());
+		sz->sz_base = alloc_pages(sz->sz_size);
+	}
+}
+
+static void
+subzone_free_arena(struct subzone *sz)
+{
+	if (sz->sz_size < compat_pagesize()) {
+		free(sz->sz_base);
+	} else {
+		free_pages(sz->sz_base, sz->sz_size);
+	}
+	sz->sz_base = NULL;
+	sz->sz_size = 0;
+}
+
+
 /**
  * Create a new zone able to hold items of 'size' bytes.
  */
-static zone_t *zn_create(zone_t *zone, gint size, gint hint)
+static zone_t *
+zn_create(zone_t *zone, gint size, gint hint)
 {
-	gint asked;		/* Amount of bytes requested */
-	gchar *arena;	/* Zone arena we got */
+	size_t arena_size;		/* Amount of bytes requested */
+
+	g_assert(size > 0);
+	g_assert(hint >= 0);
 
 	/*
 	 * Make sure size is big enough to store the free-list pointer used to
@@ -319,9 +393,7 @@ static zone_t *zn_create(zone_t *zone, gint size, gint hint)
 	 * the correct boundary.
 	 */
 
-	g_assert(size > 0);
-	if (size < (gint) sizeof(gchar *))
-		size = sizeof(gchar *);
+	size = MAX((gint) sizeof(gchar *), size);
 
 #ifdef ZONE_SAFE
 	/*
@@ -352,20 +424,17 @@ static zone_t *zn_create(zone_t *zone, gint size, gint hint)
 	 */
 
 	hint = (hint == 0) ? DEFAULT_HINT : hint;
-	asked = size * hint;
-
-	if (asked > MAX_ZONE_SIZE) {
+	if (hint > MAX_ZONE_SIZE / size) {
 		hint = MAX_ZONE_SIZE / size;
-		asked = size * hint;
 	}
-
-	g_assert(asked <= MAX_ZONE_SIZE);
+	arena_size = size * hint;
+	g_assert(arena_size <= MAX_ZONE_SIZE);
 
 	/*
 	 * Allocate the arena.
 	 */
 
-	arena = g_malloc(asked);
+	subzone_alloc_arena(&zone->zn_arena, arena_size);
 
 	/*
 	 * Initialize zone descriptor.
@@ -375,24 +444,41 @@ static zone_t *zn_create(zone_t *zone, gint size, gint hint)
 	zone->zn_size = size;
 	zone->zn_next = NULL;			/* Link zones to keep track of arenas */
 	zone->zn_cnt = 0;
-	zone->zn_free = (gchar **) arena;	/* First free block available */
-	zone->zn_arena = arena;				/* For GC, later on */
+	zone->zn_free = zone->zn_arena.sz_base;	/* First free block available */
 	zone->zn_refcnt = 1;
 
-	zn_cram(zone, arena, size);
+	zn_cram(zone, zone->zn_arena.sz_base);
 
 	return zone;
 }
 
+
+/**
+ * Create a new zone able to hold items of 'size' bytes. Returns
+ * NULL if no new zone can be created.
+ *
+ * The hint argument is to be construed as the average amount of objects
+ * that are to be created per zone chunks. That is not the total amount of
+ * expected objects of a given type. Leaving it a 0 selects the default hint
+ * value.
+ */
+zone_t *
+zcreate(gint size, gint hint)
+{
+	zone_t *zone;			/* Zone descriptor */
+
+	zone = malloc(sizeof(*zone));
+
+	return zn_create(zone, size, hint);
+}
+
 /**
  * Destroy a zone chunk by releasing its memory to the system if possible,
  * converting it into a malloc chunk otherwise.
  */
-void zdestroy(zone_t *zone)
+void
+zdestroy(zone_t *zone)
 {
-	struct subzone *sz;
-	struct subzone *next;
-
 	/*
 	 * A zone can be used by many different parts of the code, through
 	 * calls to zget().  Therefore, only destroy the zone when all references
@@ -412,14 +498,19 @@ void zdestroy(zone_t *zone)
 #endif
 	}
 
-	for (sz = zone->zn_next, next = NULL; sz; sz = next) {
-		next = sz->zn_next;
-		G_FREE_NULL(sz->zn_arena);
-		g_free(sz);
+	{
+		struct subzone *sz = zone->zn_next;
+
+		while (sz) {
+			struct subzone *next = sz->sz_next;
+			subzone_free_arena(sz);
+			free(sz);
+			sz = next;
+		}
 	}
 
-	G_FREE_NULL(zone->zn_arena);
-	g_free(zone);
+	subzone_free_arena(&zone->zn_arena);
+	free(zone);
 }
 
 /**
@@ -431,9 +522,10 @@ void zdestroy(zone_t *zone)
  * For instance, walloc() requests zones for power-of-two sizes and uses
  * zget() to get the zone, instead of zcreate() to maximize sharing.
  */
-zone_t *zget(gint size, gint hint)
+zone_t *
+zget(gint size, gint hint)
 {
-	static GHashTable *zt;	/* Keeps size (modulo ZALLOC_ALIGNBYTES) -> zone */
+	static hash_table_t *zt;/* Keeps size (modulo ZALLOC_ALIGNBYTES) -> zone */
 	zone_t *zone;
 
 	/*
@@ -441,7 +533,7 @@ zone_t *zget(gint size, gint hint)
 	 */
 
 	if (zt == NULL)
-		zt = g_hash_table_new(g_direct_hash, 0);
+		zt = hash_table_new();
 
 	/*
 	 * Make sure size is big enough to store the free-list pointer used to
@@ -456,7 +548,7 @@ zone_t *zget(gint size, gint hint)
 		size = sizeof(gchar *);
 	size = zalloc_round(size);
 
-	zone = (zone_t *) g_hash_table_lookup(zt, GINT_TO_POINTER(size));
+	zone = hash_table_lookup(zt, GINT_TO_POINTER(size));
 
 	if (zone) {
 		if (zone->zn_hint < hint)
@@ -478,28 +570,11 @@ zone_t *zget(gint size, gint hint)
 	 * time!
 	 */
 
-	g_hash_table_insert(zt, GINT_TO_POINTER(size), zone);
+	hash_table_insert(zt, GINT_TO_POINTER(size), zone);
 
 	return zone;
 }
 
-/**
- * Cram a new zone in chunk.
- *
- * A zone consists of linked blocks, where the address of the next free block
- * is written in the first bytes of each free block.
- */
-static void zn_cram(zone_t *zone, gchar *arena, gint size)
-{
-	gchar *end;		/* End address (first address beyond zone scope) */
-	gchar *next;	/* Next free block in arena */
-
-	end = arena + zone->zn_hint * zone->zn_size;
-
-	for (next = arena + size; arena < end; next += size, arena += size)
-		*(gchar **) arena = (next < end) ? next : (gchar *) 0;
-}
-
 #ifndef REMAP_ZALLOC
 /**
  * Extend zone by allocating a new zone chunk.
@@ -507,19 +582,20 @@ static void zn_cram(zone_t *zone, gchar *arena, gint size)
  * @return the address of the first new free block within the extended
  *         chunk arena.
  */
-static gchar **zn_extend(zone_t *zone)
+static gchar **
+zn_extend(zone_t *zone)
 {
-	struct subzone *new;		/* New sub-zone */
+	struct subzone *sz;		/* New sub-zone */
 
-	new = g_malloc(sizeof(*new));
+	sz = malloc(sizeof *sz);
 
-	new->zn_arena = g_malloc(zone->zn_size * zone->zn_hint);
-	zone->zn_free = (gchar **) new->zn_arena;
+	subzone_alloc_arena(sz, zone->zn_size * zone->zn_hint);
+	zone->zn_free = sz->sz_base;
 
-	new->zn_next = zone->zn_next;
-	zone->zn_next = new;				/* New subzone at head of list */
+	sz->sz_next = zone->zn_next;
+	zone->zn_next = sz;				/* New subzone at head of list */
 
-	zn_cram(zone, new->zn_arena, zone->zn_size);
+	zn_cram(zone, sz->sz_base);
 
 	return zone->zn_free;
 }
