@@ -1,5 +1,5 @@
 /*
- * $Id: downloads_common.h,v 1.2 2004/08/27 15:53:57 rmanfredi Exp $
+ * $Id: downloads_common.h 11050 2006-06-11 21:31:42Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -28,11 +28,14 @@
 
 #include "if/ui/gtk/downloads.h"
 
-void gui_update_c_downloads(gint, gint);
+#define IO_STALLED	60 /**< If nothing exchanged after that many secs */
 
 void on_button_downloads_clear_stopped_clicked(
 	GtkButton *button, gpointer user_data);
 void on_togglebutton_queue_freeze_toggled(
 	GtkToggleButton *togglebutton, gpointer user_data);
 
+const gchar *download_progress_to_string(const struct download *d);
+const gchar *source_progress_to_string(const struct download *d);
+
 #endif /* _gtk_downloads_common_h_ */
