@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats_common.c,v 1.21 2005/08/11 06:32:14 cbiere Exp $
+ * $Id: gnet_stats_common.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,10 +25,11 @@
 
 #include "gui.h"
 
-RCSID("$Id: gnet_stats_common.c,v 1.21 2005/08/11 06:32:14 cbiere Exp $");
+RCSID("$Id: gnet_stats_common.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gnet_stats_common.h"
 #include "gtk-missing.h"
+#include "settings.h"
 
 #include "if/core/net_stats.h"
 #include "if/bridge/ui2c.h"
@@ -49,6 +50,7 @@ msg_type_str(gint value)
 		N_("Bye"),
 		N_("QRP"),
 		N_("HSEP"),
+		N_("RUDP"),
 		N_("Vendor spec."),
 		N_("Vendor std."),
 		N_("Push"),
@@ -107,6 +109,10 @@ msg_drop_str(gint value)
 		N_("Malformed Query Hit"),
 		N_("Bad return address"),
 		N_("Hostile IP address"),
+		N_("Spam"),
+		N_("Evil filename"),
+		N_("Payload inflating error"),
+		N_("Unknown header flags present"),
 	};
 
 	STATIC_ASSERT(G_N_ELEMENTS(strs) == MSG_DROP_REASON_COUNT);
@@ -139,6 +145,8 @@ general_type_str(gint value)
 		N_("OOB hits bearing alien IP address"),
 		N_("Unclaimed locally-generated OOB hits"),
 		N_("Partially claimed locally-generated OOB hits"),
+		N_("Unrequested OOB hits received"),
+		N_("Spurious OOB hit claiming received"),
 		N_("Compacted queries"),
 		N_("Bytes saved by compacting"),
 		N_("UTF8 queries"),
@@ -164,6 +172,9 @@ general_type_str(gint value)
 		N_("UDP messages with bogus source IP"),
 		N_("Alien UDP messages (non-Gnutella)"),
 		N_("Unprocessed UDP Gnutella messages"),
+		N_("Compressed UDP messages enqueued"),
+		N_("Compressed UDP messages received"),
+		N_("Uncompressed UDP messages due to no gain"),
 	};
 
 	STATIC_ASSERT(G_N_ELEMENTS(strs) == GNR_TYPE_COUNT);
@@ -240,18 +251,18 @@ gnet_stats_gui_horizon_update(hsep_triple *table, guint32 triples)
 
 	val = table[hops][HSEP_IDX_NODES] + other[HSEP_IDX_NODES];
 	gtk_label_printf(GTK_LABEL(
-			lookup_widget(main_window, "label_statusbar_horizon_node_count")),
+			gui_main_window_lookup("label_statusbar_horizon_node_count")),
 		"%s %s", uint64_to_string(val), NG_("node", "nodes", val));
 
 	val = table[hops][HSEP_IDX_FILES] + other[HSEP_IDX_FILES];
 	gtk_label_printf(GTK_LABEL(
-			lookup_widget(main_window, "label_statusbar_horizon_file_count")),
+			gui_main_window_lookup("label_statusbar_horizon_file_count")),
 		"%s %s", uint64_to_string(val), NG_("file", "files", val));
 
 	val = table[hops][HSEP_IDX_KIB] + other[HSEP_IDX_KIB];
 	gtk_label_printf(GTK_LABEL(
-			lookup_widget(main_window, "label_statusbar_horizon_kb_count")),
-		"%s", short_kb_size(val));
+			gui_main_window_lookup("label_statusbar_horizon_kb_count")),
+		"%s", short_kb_size(val, show_metric_units()));
 }
 
 /* vi: set ts=4 sw=4 cindent: */
