@@ -1,10 +1,8 @@
 /*
- * $Id: gtk-missing.c,v 1.8 2004/11/09 19:28:20 cbiere Exp $
+ * $Id: gtk-missing.c 11437 2006-08-05 00:24:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
- * Functions that should be in gtk+-1.2 or gtk+-2.x but are not.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,9 +23,21 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Missing functions in the GTK+.
+ *
+ * Functions that should be in GTK+ 1.2 or GTK+ 2.x but are not.
+ *
+ * @author Richard Eckart
+ * @date 2002-2003
+ */
+
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c,v 1.8 2004/11/09 19:28:20 cbiere Exp $");
+RCSID("$Id: gtk-missing.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "gtk-missing.h"
 #include "columns.h"
@@ -36,17 +46,18 @@ RCSID("$Id: gtk-missing.c,v 1.8 2004/11/09 19:28:20 cbiere Exp $");
 #include "if/gui_property_priv.h"
 
 #include "lib/glib-missing.h"
+#include "lib/misc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-/*
- * gtk_paned_get_position:
+/**
+ * @fn gtk_paned_get_position(GtkPaned *paned)
  *
  * Get position of divider in a GtkPaned. (in GTK2)
  */
 #ifndef USE_GTK2
 gint
-gtk_paned_get_position(GtkPaned *paned) 
+gtk_paned_get_position(GtkPaned *paned)
 {
     g_return_val_if_fail(paned != NULL, -1);
     g_return_val_if_fail(GTK_IS_PANED (paned), -1);
@@ -60,7 +71,8 @@ gtk_paned_get_position(GtkPaned *paned)
  * Set the internal name of the column without changing the
  * column header widget. (Copy paste internal column_title_new
  * from gtkclist.c)
- * BEWARE: EVIL HACK
+ *
+ * @warning EVIL HACK
  */
 void
 gtk_clist_set_column_name(GtkCList * clist, gint col, gchar * t)
@@ -73,142 +85,100 @@ gtk_clist_set_column_name(GtkCList * clist, gint col, gchar * t)
 
     clist->column[col].title = g_strdup(t);
 }
-
-/**
- * Save visibility of columns in given property which must by a boolean array 
- * property with at least as many elements as there are columns.
- */
-void
-gtk_clist_save_visibility(GtkCList *clist, property_t prop)
-{
-	gint i;
-	guint32 val;
-
-	g_assert(clist);
-
-    for (i = 0; i < clist->columns; i++) {
-		val = clist->column[i].visible;
-		gui_prop_set_boolean(prop, &val, i, 1);
-	}
-}
-
-/**
- * Restore visibility of columns from given property which must by a boolean
- * array property with at least as many elements as there are columns.
- */
-void
-gtk_clist_restore_visibility(GtkCList *clist, property_t prop)
-{
-	gint i;
-	gboolean val;
-
-	g_assert(clist);
-
-    for (i = 0; i < clist->columns; i++) {
-		gui_prop_get_boolean(prop, &val, i, 1);
-    	gtk_clist_set_column_visibility(clist, i, val);
-	}
-}
 #endif /* USE_GTK1 */
 
+#define GTK_ITERATION_MAX	100		/* Don't spend too much time in GUI */
 
-/*
- * gtk_main_flush:
+/**
+ * Process all pending gtk events (id est draw now!).
  *
- * Process all pending gtk events (i.e. draw now!)
- * Returns TRUE if gtk_main_quit has been called 
+ * @returns TRUE if gtk_main_quit has been called
  * for the innermost mainloop. Aborts flush if
  * gtk_main_quit has been called.
  */
 gint
-gtk_main_flush(void) 
+gtk_main_flush(void)
 {
     gint val = FALSE;
+	gint i = 0;
 
-    while (gtk_events_pending()) {
+    while (gtk_events_pending() && i++ < GTK_ITERATION_MAX) {
         val = gtk_main_iteration_do(FALSE);
 		if (val)
 			break;
 	}
 
+	if (i > GTK_ITERATION_MAX && !val)
+		g_warning("gtk_main_flush: too much work");
+
     return val;
 }
 
-
-
 /**
  * Select the menu item which has given data attached to it.
  */
 void
-option_menu_select_item_by_data(GtkWidget *m, gpointer *d)
+option_menu_select_item_by_data(GtkOptionMenu *option_menu, gconstpointer data)
 {
-    GList *l;
-    gint n = 0;
-    GtkWidget *menu;
+    GList *iter;
+    gint i;
 
-    g_assert(m != NULL);
-    g_assert(GTK_IS_OPTION_MENU(m));
+    g_assert(option_menu);
+    g_assert(GTK_IS_OPTION_MENU(option_menu));
 
-    menu = GTK_WIDGET(gtk_option_menu_get_menu(GTK_OPTION_MENU(m)));
-
-    for (l = GTK_MENU_SHELL(menu)->children; l != NULL; l = g_list_next(l)) {
-        if (l->data != NULL) {
-            if (gtk_object_get_user_data((GtkObject *) l->data) == d) {
-                gtk_option_menu_set_history(GTK_OPTION_MENU(m), n); 
-                break;
-            }
+    iter = GTK_MENU_SHELL(gtk_option_menu_get_menu(option_menu))->children;
+    for (i = 0; iter != NULL; iter = g_list_next(iter), i++) {
+        if (iter->data && gtk_object_get_user_data(iter->data) == data) {
+			gtk_option_menu_set_history(option_menu, i);
+			return;
         }
-
-        n ++;
     }
 
-    if (l == NULL)
-        g_warning("option_menu_select_item_by_data: no item with data %p", d);
+	g_warning("option_menu_select_item_by_data: no item with data %p", data);
 }
 
 
 
 /**
- * Add a new menu item to the GtkMenu m, with the label l and
- * the data d.
+ * Add a new menu item to the GtkMenu "menu" with the label "label_text" and
+ * the data "data".
  */
 GtkWidget *
-menu_new_item_with_data(GtkMenu * m, gchar * l, gpointer d )
+menu_new_item_with_data(GtkMenu *menu, const gchar *label_text, gpointer data)
 {
-    GtkWidget *w;                                                     
-                         
-    g_assert(l != NULL);
-                                                 
-    w = gtk_menu_item_new_with_label(l);                              
-    gtk_object_set_user_data((GtkObject *)w, (gpointer)d);            
-    gtk_widget_show(w);                                               
-    gtk_menu_append(m, w);    
-    return w;
+    GtkWidget *widget;
+
+    g_assert(menu);
+    g_assert(label_text);
+
+    widget = gtk_menu_item_new_with_label(label_text);
+    gtk_object_set_user_data(GTK_OBJECT(widget), data);
+    gtk_widget_show(widget);
+    gtk_menu_append(menu, widget);
+    return widget;
 }
 
 
 /**
- * Fetches the data set associated with the selected menu item in 
+ * Fetches the data set associated with the selected menu item in
  * the GtkOptionMenu m.
  */
 gpointer
-option_menu_get_selected_data(GtkWidget *m)
+option_menu_get_selected_data(GtkOptionMenu *option_menu)
 {
     GtkWidget *menu;
 
-    g_assert(GTK_IS_OPTION_MENU(m));
+    g_assert(GTK_IS_OPTION_MENU(option_menu));
 
-    menu = gtk_menu_get_active
-        (GTK_MENU(gtk_option_menu_get_menu(GTK_OPTION_MENU(m))));
-
-    return (menu != NULL) ? gtk_object_get_user_data(GTK_OBJECT(menu)) : NULL;
+    menu = gtk_menu_get_active(GTK_MENU(gtk_option_menu_get_menu(option_menu)));
+    return menu ? gtk_object_get_user_data(GTK_OBJECT(menu)) : NULL;
 }
 
-
 /**
  * Given a radio button it returns a pointer to the active radio
- * button in the group the given button is in. 
- * Returns NULL if there is no active button.
+ * button in the group the given button is in.
+ *
+ * @returns NULL if there is no active button.
  */
 GtkWidget *
 radiobutton_get_active_in_group(GtkRadioButton *rb)
@@ -262,7 +232,7 @@ void gtk_label_printf(GtkLabel *label, const gchar *format, ...)
     g_assert(label != NULL);
 
     va_start(args, format);
-    
+
     if (format != NULL)
         gm_vsnprintf(buf, sizeof(buf), format, args);
     else
@@ -274,22 +244,19 @@ void gtk_label_printf(GtkLabel *label, const gchar *format, ...)
 }
 
 /**
- * Takes a NULL terminated array of strings which are supposed to
- * be widgets names found with the given top-level widget. Sets all
- * to the given sentitivity state.
+ * Takes an array of strings which are supposed to be widgets names found with
+ * the given top-level widget. Sets all to the given sentitivity state.
  */
 void
-gtk_mass_widget_set_sensitive(GtkWidget *toplevel, gchar *list[], gboolean b)
+gtk_mass_widget_set_sensitive(GtkWidget *toplevel,
+	const gchar * const list[], guint n, gboolean b)
 {
-    guint n;
-    GtkWidget *w;
-    
+    guint i;
+
     g_assert(toplevel != NULL);
 
-    for (n = 0; list[n] != NULL; n ++) {
-        w = lookup_widget(toplevel, list[n]);
-        gtk_widget_set_sensitive(w, b);
-    }
+    for (i = 0; i < n; i++)
+        gtk_widget_set_sensitive(lookup_widget(toplevel, list[i]), b);
 }
 
 /**
@@ -308,22 +275,22 @@ clist_collect_data(GtkCList *clist, gboolean allow_null, GCompareFunc cfn)
     GSList *to_unselect = NULL;
 
     g_assert(clist != NULL);
-    
+
     /*
      * Collect the data of the selected rows.
      */
     for (l = clist->selection; l != NULL; l = g_list_next(l)) {
         gpointer data;
         gint row;
-         
+
         row = GPOINTER_TO_INT(l->data);
         data = gtk_clist_get_row_data(clist, row);
- 
+
         if ((data != NULL) || allow_null) {
             if (cfn != NULL) {
                 if (g_slist_find_custom(result_list, data, cfn) != NULL) {
                     if (gui_debug >= 3) {
-                        const gchar *name = 
+                        const gchar *name =
                             gtk_widget_get_name(GTK_WIDGET(clist));
                         printf("%s has duplicate data: %p\n",
                             (name != NULL) ? name : "<UNKNOWN>", data);
@@ -360,12 +327,15 @@ clist_collect_data(GtkCList *clist, gboolean allow_null, GCompareFunc cfn)
 
 /**
  * @return a pointer to a newly allocated GtkTreeIter. Must be freed
- * with w_tree_iter_free().  
+ * with w_tree_iter_free().
  */
 GtkTreeIter *
 w_tree_iter_new(void)
 {
-	return walloc(sizeof(GtkTreeIter));
+	GtkTreeIter *iter;
+
+	iter = walloc(sizeof *iter);
+	return iter;
 }
 
 /**
@@ -377,7 +347,7 @@ w_tree_iter_copy(GtkTreeIter *iter)
 {
 	GtkTreeIter *copy;
 
-	copy = walloc(sizeof(*copy));
+	copy = walloc(sizeof *copy);
 	*copy = *iter;
 	return copy;
 }
@@ -388,28 +358,40 @@ w_tree_iter_copy(GtkTreeIter *iter)
 void
 w_tree_iter_free(GtkTreeIter *iter)
 {
-	wfree(iter, sizeof(*iter));
+	wfree(iter, sizeof *iter);
 }
 
+/**
+ * For use as GDestroyNotify function.
+ */
+void
+ht_w_tree_iter_free(gpointer p)
+{
+	GtkTreeIter *iter = p;
+	wfree(iter, sizeof *iter);
+}
+
+
 typedef struct collect_data_struct {
     GSList *results;
     GSList *to_unselect;
 	GCompareFunc cfn;
+	tree_selection_get_data_func gdf;	
 	GtkTreeView *tv;
 	const gchar *name; /* name of the treeview widget (for debugging) */
+	guint column;
 } collect_data_struct_t;
 
 static inline void
 tree_selection_collect_data_record(GtkTreeModel *model, GtkTreeIter *iter,
 		collect_data_struct_t *cdata, gboolean unselect)
 {
-	gpointer data = NULL;
-	
+	gpointer data;
+
 	g_assert(NULL != cdata);
 	g_assert(NULL != iter);
-	
-	gtk_tree_model_get(model, iter, c_sr_record, &data, (-1));
-	g_assert(NULL != data);
+
+	data = cdata->gdf(model, iter);
 
 	if (unselect) {
 		cdata->to_unselect = g_slist_prepend(cdata->to_unselect,
@@ -433,17 +415,17 @@ tree_selection_collect_data_helper(GtkTreeModel *model,
 
 	/* Collect the data of the parent row */
 	tree_selection_collect_data_record(model, iter, cdata, TRUE);
-	
+
 	/* If the row is not expanded and there are any children, collect their
 	 * data as well. This is not recursive and descends only one level */
-	
+
 	if (
 			gtk_tree_model_iter_has_child(model, iter) &&
 			!gtk_tree_view_row_expanded(cdata->tv, path)
 	) {
 		GtkTreeIter child;
 		gint i = 0;
-		
+
 		while (gtk_tree_model_iter_nth_child(model, &child, iter, i)) {
 			tree_selection_collect_data_record(model, &child, cdata, FALSE);
 			i++;
@@ -454,9 +436,11 @@ tree_selection_collect_data_helper(GtkTreeModel *model,
 static void
 tree_selection_unselect_helper(gpointer data, gpointer user_data)
 {
-	gtk_tree_selection_unselect_iter((GtkTreeSelection *) user_data,
-		(GtkTreeIter *) data);
-	w_tree_iter_free((GtkTreeIter *) data);
+	GtkTreeSelection *s = user_data;
+	GtkTreeIter *iter = data;
+
+	gtk_tree_selection_unselect_iter(s, iter);
+	w_tree_iter_free(iter);
 }
 
 /**
@@ -466,7 +450,8 @@ tree_selection_unselect_helper(gpointer data, gpointer user_data)
  * items from the result list. Using cfn will significantly increase runtime.
  */
 GSList *
-tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
+tree_selection_collect_data(GtkTreeSelection *selection,
+	tree_selection_get_data_func gdf, GCompareFunc cfn)
 {
 	collect_data_struct_t cdata;
 
@@ -475,6 +460,7 @@ tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
 	cdata.results = NULL;
 	cdata.to_unselect = NULL;
 	cdata.cfn = cfn;
+	cdata.gdf = gdf;
 	cdata.tv = gtk_tree_selection_get_tree_view(selection);
 	if (gui_debug >= 3) {
 		cdata.name = gtk_widget_get_name(
@@ -488,7 +474,7 @@ tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
      * Browse selected rows and gather data.
      */
 	gtk_tree_selection_selected_foreach(selection,
-		tree_selection_collect_data_helper, (gpointer) &cdata);
+		tree_selection_collect_data_helper, &cdata);
 
     /*
      * Now unselect the rows from which we got data.
@@ -504,78 +490,143 @@ tree_selection_collect_data(GtkTreeSelection *selection, GCompareFunc cfn)
     return cdata.results;
 }
 
-void
-tree_view_save_widths(GtkTreeView *treeview, property_t prop)
+struct tree_view_motion {
+	tree_view_motion_callback cb;
+	GtkTreeView *tv;
+	guint signal_id;
+	guint timeout_id;
+	guint x, y;
+	gboolean ready;
+};
+
+static gboolean
+on_tree_view_motion_notify(GtkWidget *widget,
+	GdkEventMotion *event, gpointer udata)
 {
-	gint i;
+	tree_view_motion_t *tvm;
 
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		guint32 width;
+	g_assert(widget != NULL);
+	g_assert(event != NULL);
 
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
+	tvm = udata;
+	g_assert(tvm != NULL);
+	g_assert(tvm->cb != NULL);
 
-		width = gtk_tree_view_column_get_width(c);
-		if ((gint) width > 0)
-			gui_prop_set_guint32(prop, &width, i, 1);
+#if 0 
+	{
+		gchar type[32];
+#define EVENT_TYPE(x) case x: gm_snprintf(type, sizeof type, "%s", #x); break;
+		switch (event->type) {
+		EVENT_TYPE(GDK_NOTHING)
+		EVENT_TYPE(GDK_DESTROY)
+		EVENT_TYPE(GDK_EXPOSE)
+		EVENT_TYPE(GDK_MOTION_NOTIFY)
+		EVENT_TYPE(GDK_BUTTON_PRESS)
+		EVENT_TYPE(GDK_2BUTTON_PRESS)
+		EVENT_TYPE(GDK_3BUTTON_PRESS)
+		EVENT_TYPE(GDK_BUTTON_RELEASE)
+		EVENT_TYPE(GDK_KEY_PRESS)
+		EVENT_TYPE(GDK_KEY_RELEASE)
+		EVENT_TYPE(GDK_ENTER_NOTIFY)
+		EVENT_TYPE(GDK_LEAVE_NOTIFY)
+		EVENT_TYPE(GDK_FOCUS_CHANGE)
+		EVENT_TYPE(GDK_CONFIGURE)
+		EVENT_TYPE(GDK_MAP)
+		EVENT_TYPE(GDK_UNMAP)
+		EVENT_TYPE(GDK_PROPERTY_NOTIFY)
+		EVENT_TYPE(GDK_SELECTION_CLEAR)
+		EVENT_TYPE(GDK_SELECTION_REQUEST)
+		EVENT_TYPE(GDK_SELECTION_NOTIFY)
+		EVENT_TYPE(GDK_PROXIMITY_IN)
+		EVENT_TYPE(GDK_PROXIMITY_OUT)
+		EVENT_TYPE(GDK_DRAG_ENTER)
+		EVENT_TYPE(GDK_DRAG_LEAVE)
+		EVENT_TYPE(GDK_DRAG_MOTION)
+		EVENT_TYPE(GDK_DRAG_STATUS)
+		EVENT_TYPE(GDK_DROP_START)
+		EVENT_TYPE(GDK_DROP_FINISHED)
+		EVENT_TYPE(GDK_CLIENT_EVENT)
+		EVENT_TYPE(GDK_VISIBILITY_NOTIFY)
+		EVENT_TYPE(GDK_NO_EXPOSE)
+		EVENT_TYPE(GDK_SCROLL)
+		EVENT_TYPE(GDK_WINDOW_STATE)
+		EVENT_TYPE(GDK_SETTING)
+		default:
+			gm_snprintf(type, sizeof type, "%ld", (gulong) event->type);
+		}
+#undef EVENT_TYPE
+
+		g_message("on_tree_view_motion_notify(): type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
+				type,
+				(gint) event->x, (gint) event->y, event->axes,
+				(gint) event->x_root, (gint) event->y_root);
 	}
+#endif /* 0 */
+
+	tvm->x = event->x;
+	tvm->y = event->y;
+	tvm->ready = TRUE;
+	return FALSE;
+}
+
+static gboolean
+tree_view_motion_timeout(gpointer data)
+{
+	tree_view_motion_t *tvm = data;
+
+	g_assert(tvm != NULL);
+	g_assert(tvm->tv != NULL);
+	g_assert(tvm->cb != NULL);
+
+	if (tvm->ready && GTK_WIDGET_REALIZED(GTK_WIDGET(tvm->tv))) {
+		GtkTreePath *path = NULL;
+
+		tvm->ready = FALSE;
+		gtk_tree_view_get_path_at_pos(tvm->tv, tvm->x, tvm->y, &path,
+			NULL, NULL, NULL);
+		tvm->cb(tvm->tv, path);
+		if (path)
+			gtk_tree_path_free(path);
+	}
+
+	return TRUE;
+}
+
+/**
+ * Registers the callback for the motion-notify-event. Only the last event
+ * seen during any interval will invoke the callback.
+ *
+ * @param tv The GtkTreeView.
+ * @param cb The callback.
+ * @param interval The interval in milliseconds.
+ */
+tree_view_motion_t *
+tree_view_motion_set_callback(GtkTreeView *tv,
+	tree_view_motion_callback cb, guint interval)
+{
+	tree_view_motion_t *tvm;
+
+	g_assert(tv != NULL);
+	g_assert(cb != NULL);
+
+	tvm = g_malloc(sizeof *tvm);
+	tvm->tv = tv;
+	tvm->cb = cb;
+	tvm->timeout_id = g_timeout_add(interval, tree_view_motion_timeout, tvm);
+	tvm->signal_id = g_signal_connect(GTK_OBJECT(tv),
+		"motion-notify-event", G_CALLBACK(on_tree_view_motion_notify), tvm);
+	return tvm;
 }
 
 void
-tree_view_restore_widths(GtkTreeView *treeview, property_t prop)
+tree_view_motion_clear_callback(GtkTreeView *tv, tree_view_motion_t *tvm)
 {
-	gint i;
+	g_assert(tv != NULL);
+	g_assert(tvm != NULL);
 
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		guint32 width;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-		gui_prop_get_guint32(prop, &width, i, 1);
-		g_object_set(G_OBJECT(c), "fixed-width", MAX(1, width), NULL);
-	}
-}
-
-void
-tree_view_save_visibility(GtkTreeView *treeview, property_t prop)
-{
-	guint i;
-
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		guint32 val;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-		val = gtk_tree_view_column_get_visible(c);
-		gui_prop_set_boolean(prop, &val, i, 1);
-	}
-}
-
-void
-tree_view_restore_visibility(GtkTreeView *treeview, property_t prop)
-{
-	guint i;
-
-	g_assert(treeview);
-	for (i = 0; i < INT_MAX; i++) {
-		GtkTreeViewColumn *c;
-		gboolean val;
-
-		c = gtk_tree_view_get_column(treeview, i);
-		if (!c)
-			break;
-		gui_prop_get_boolean(prop, &val, i, 1);
-		gtk_tree_view_column_set_visible(c, val);
-	}
+	g_signal_handler_disconnect(GTK_OBJECT(tv), tvm->signal_id);
+	g_source_remove(tvm->timeout_id);
+	G_FREE_NULL(tvm);
 }
 
 #endif /* USE_GTK2 */
@@ -594,7 +645,7 @@ _gtk_spin_button_get_value(GtkSpinButton *spinbutton)
 }
 
 guint32
-gtk_editable_get_value_as_uint(GtkEditable *editable)
+gtk_editable_get_value_as_uint32(GtkEditable *editable)
 {
     gchar *e;
     guint32 result;
@@ -611,47 +662,75 @@ gtk_editable_get_value_as_uint(GtkEditable *editable)
  * choice number set as user_data.
  */
 void
-gtk_combo_init_choices(GtkCombo *combo, GtkSignalFunc func,
+widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 		prop_def_t *def, gpointer user_data)
 {
-    guint n;
     guint32 original;
 
     g_assert(def != NULL);
-    g_assert(combo != NULL);
+    g_assert(widget != NULL);
+    g_assert(GTK_IS_COMBO(widget) || GTK_IS_OPTION_MENU(widget));
     g_assert(def->type == PROP_TYPE_MULTICHOICE);
     g_assert(def->data.guint32.choices != NULL);
 
     original = *def->data.guint32.value;
 
-    n = 0;
-    while (def->data.guint32.choices[n].title != NULL) {
-        GtkWidget *list_item;
-        GList *l;
+	if (GTK_IS_COMBO(widget)) {
+		GtkCombo *combo = GTK_COMBO(widget);
+		const gchar *title;
+    	guint i;
+		
+		for (i = 0; (title = def->data.guint32.choices[i].title) != NULL; i++) {
+			GtkWidget *list_item;
+			GList *l;
 
-        list_item = gtk_list_item_new_with_label(
-            def->data.guint32.choices[n].title);
+			list_item = gtk_list_item_new_with_label(_(title));
 
-        gtk_object_set_user_data(GTK_OBJECT(list_item),
-            GINT_TO_POINTER(def->data.guint32.choices[n].value));
+			gtk_object_set_user_data(GTK_OBJECT(list_item),
+					GINT_TO_POINTER(def->data.guint32.choices[i].value));
 
-        gtk_widget_show(list_item);
-        
-        gtk_signal_connect_after(
-            GTK_OBJECT(list_item), "select", func, user_data);
+			gtk_widget_show(list_item);
 
-        l = g_list_prepend(NULL, (gpointer) list_item);
-        gtk_list_append_items(GTK_LIST(combo->list), l);
+			gtk_signal_connect_after(GTK_OBJECT(list_item),
+					"select", func, user_data);
 
-        if (def->data.guint32.choices[n].value == original)
-            gtk_list_select_child(GTK_LIST(combo->list), list_item);
-        n ++;
-    }
+			l = g_list_prepend(NULL, (gpointer) list_item);
+			gtk_list_append_items(GTK_LIST(combo->list), l);
+
+			if (def->data.guint32.choices[i].value == original)
+				gtk_list_select_child(GTK_LIST(combo->list), list_item);
+		}
+	} else if (GTK_IS_OPTION_MENU(widget)) {
+		GtkOptionMenu *option_menu = GTK_OPTION_MENU(widget);
+		GtkMenu *menu;
+		const gchar *title;
+		guint i;
+
+		menu = GTK_MENU(gtk_menu_new());
+		for (i = 0; (title = def->data.guint32.choices[i].title) != NULL; i++) {
+			GtkWidget *item;
+
+			item = gtk_menu_item_new_with_label(_(title));
+			gtk_widget_show(item);
+			gtk_object_set_user_data(GTK_OBJECT(item),
+				GUINT_TO_POINTER(def->data.guint32.choices[i].value));
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+#ifdef USE_GTK1
+			gtk_signal_connect_after(GTK_OBJECT(item), "activate",
+				func, user_data);
+#endif /* USE_GTK1 */
+#ifdef USE_GTK2
+			g_signal_connect_after(GTK_OBJECT(item), "activate",
+				G_CALLBACK(func), user_data);
+#endif /* USE_GTK2 */
+		}
+		gtk_option_menu_set_menu(option_menu, GTK_WIDGET(menu));
+	}
 }
 
 #ifdef USE_GTK1
 /**
- * Functions like gtk_ctree_unlink for *Top level parent nodes only*. O(1)  
+ * Functions like gtk_ctree_unlink for *Top level parent nodes only*. O(1)
  */
 static void
 gtk_ctree_fast_unlink (GtkCTree *ctree, GtkCTreeNode *node)
@@ -665,16 +744,16 @@ gtk_ctree_fast_unlink (GtkCTree *ctree, GtkCTreeNode *node)
 	GtkCTreeRow *prev_row;
 	GtkCTreeNode *sibling;
 	GList *list;
-	
+
 	g_assert(NULL == GTK_CTREE_ROW(node)->parent); /* Not a child node */
-	
+
 	clist = GTK_CLIST(ctree);
  	visible = gtk_ctree_is_viewable(ctree, node);
 
 	/* clist->row_list_end unlinked ? */
 	if (visible && (
             GTK_CTREE_NODE_NEXT(node) == NULL || (
-                GTK_CTREE_ROW(node)->children && 
+                GTK_CTREE_ROW(node)->children &&
                 gtk_ctree_is_ancestor(
                     ctree, node, GTK_CTREE_NODE(clist->row_list_end))
             )
@@ -691,7 +770,7 @@ gtk_ctree_fast_unlink (GtkCTree *ctree, GtkCTreeNode *node)
 		work = GTK_CTREE_NODE_NEXT(work);
 		rows++;
     }
-	
+
 	/* Subtract removed node and children from row count */
 	if (visible) {
 		clist->rows -= (rows + 1);
@@ -704,8 +783,8 @@ gtk_ctree_fast_unlink (GtkCTree *ctree, GtkCTreeNode *node)
 		list->prev = (GList *)GTK_CTREE_NODE_PREV(node);
 	}
 
-	prev = GTK_CTREE_NODE_PREV(node); 
-	
+	prev = GTK_CTREE_NODE_PREV(node);
+
 	if (prev && GTK_CTREE_NODE_NEXT(prev) == node) {
 		list = (GList *)GTK_CTREE_NODE_PREV(node);
 		list->next = (GList *)work;
@@ -714,17 +793,17 @@ gtk_ctree_fast_unlink (GtkCTree *ctree, GtkCTreeNode *node)
 	if (clist->row_list == (GList *)node)
 		clist->row_list = (GList *) (GTK_CTREE_ROW(node)->sibling);
 	else {
-		/* We need to hook the previous node up to the node's sibling */	
+		/* We need to hook the previous node up to the node's sibling */
 		if (prev) {
-	
+
 			prev_row = GTK_CTREE_ROW(prev);
 			while (NULL != prev_row->parent) {
 				prev = GTK_CTREE_NODE_PREV(prev);
 				prev_row = GTK_CTREE_ROW(prev);
 			}
-			sibling = prev;			
-			g_assert(GTK_CTREE_ROW(sibling)->sibling == node);		
-			
+			sibling = prev;
+			g_assert(GTK_CTREE_ROW(sibling)->sibling == node);
+
 			GTK_CTREE_ROW(sibling)->sibling = GTK_CTREE_ROW(node)->sibling;
 		}
 	}
@@ -732,8 +811,8 @@ gtk_ctree_fast_unlink (GtkCTree *ctree, GtkCTreeNode *node)
 
 
 /**
- * Functions like gtk_ctree_link for *Top level parent nodes only*.  This is 
- * optimized for data being linked at the beginning of the tree.  
+ * Functions like gtk_ctree_link for *Top level parent nodes only*.  This is
+ * optimized for data being linked at the beginning of the tree.
  * O(1) if linking to beginning, O(n) otherwise.
  */
 static void
@@ -743,21 +822,19 @@ gtk_ctree_fast_link(GtkCTree *ctree, GtkCTreeNode *node, GtkCTreeNode *sibling)
 	GList *list_end;
 	GList *list;
 	GList *work;
-	gboolean visible = FALSE;
 	gint rows = 1;
 
 	g_assert(NULL == GTK_CTREE_ROW(node)->parent); /* Not a child node */
-	
+
 	clist = GTK_CLIST (ctree);
 
 	/* Counts node and children */
 	for (list_end = (GList *)node; list_end->next; list_end = list_end->next)
     	rows++;
-	
+
 	GTK_CTREE_ROW(node)->parent = NULL;
 	GTK_CTREE_ROW(node)->sibling = sibling;
 
-	visible = TRUE;
 	clist->rows += rows;
 	work = clist->row_list;
 
@@ -780,7 +857,7 @@ gtk_ctree_fast_link(GtkCTree *ctree, GtkCTreeNode *node, GtkCTreeNode *sibling)
 			list = (GList *)GTK_CTREE_NODE_PREV(sibling);
 			list->next = (GList *)node;
 		}
-      
+
 		list = (GList *)node;
 		list->prev = (GList *)GTK_CTREE_NODE_PREV(sibling);
 		list_end->next = (GList *)sibling;
@@ -788,19 +865,19 @@ gtk_ctree_fast_link(GtkCTree *ctree, GtkCTreeNode *node, GtkCTreeNode *sibling)
 		list->prev = list_end;
 
 	} else {
-		
+
 		if (work) {
 
 			/* Look from beginning of list/parent all the way to the end. */
 			while (GTK_CTREE_ROW(work)->sibling)
 				work = (GList *)(GTK_CTREE_ROW(work)->sibling);
 			GTK_CTREE_ROW(work)->sibling = node;
-	  
+
 			/* find last child of sibling */
 			work = (GList *) gtk_ctree_last(ctree, GTK_CTREE_NODE(work));
-	  
+
 			list_end->next = work->next;
-			
+
 			if (work->next)
 				list = work->next->prev = list_end;
 			work->next = (GList *)node;
@@ -825,8 +902,8 @@ gtk_ctree_fast_link(GtkCTree *ctree, GtkCTreeNode *node, GtkCTreeNode *sibling)
 
 
 /**
- * Functions like gtk_ctree_move for *Top level parent nodes only*.  This is 
- * optimized for data being moved to the beginning of the tree and assumes 
+ * Functions like gtk_ctree_move for *Top level parent nodes only*.  This is
+ * optimized for data being moved to the beginning of the tree and assumes
  * ctree != NULL and node != NULL.  O(1) as opposed to gtk's which is O(n).
  */
 void
@@ -835,29 +912,27 @@ gtk_ctree_fast_move(GtkCTree *ctree, GtkCTreeNode *node,
 {
 	GtkCList *clist;
 	GtkCTreeNode *work;
-	gboolean visible = FALSE;
-	
+
 	g_assert(NULL == GTK_CTREE_ROW(node)->parent); /* Not a child node */
-	
+
 	clist = GTK_CLIST (ctree);
-	visible = gtk_ctree_is_viewable (ctree, node);
 
 	/* return if it's already the right place */
 	if (new_sibling == GTK_CTREE_ROW(node)->sibling || new_sibling == node) {
 		return;
 	}
-	
+
 	work = NULL;
 	if (gtk_ctree_is_viewable (ctree, node))
     	work = GTK_CTREE_NODE(g_list_nth (clist->row_list, clist->focus_row));
-    
+
 	gtk_ctree_fast_unlink(ctree, node);
 	gtk_ctree_fast_link(ctree, node, new_sibling);
-	
+
 	if (!work) {
 		return;
 	}
-	
+
 	while (work && !gtk_ctree_is_viewable(ctree, work))
 		work = GTK_CTREE_ROW(work)->parent;
 
@@ -874,27 +949,27 @@ gtk_ctree_count_node_children(GtkCTree *unused_ctree, GtkCTreeNode *parent)
 	GtkCTreeRow *current_row;
 	GtkCTreeNode *current_node;
 	gint num_children = 0;
-	
+
 	(void) unused_ctree;
 	current_row = GTK_CTREE_ROW(parent);
 	current_node = current_row->children;
-	
+
 	for (; NULL != current_node; current_node = current_row->sibling) {
 		current_row = GTK_CTREE_ROW(current_node);
 		num_children++;
-	}	
-	
-	return num_children;	
+	}
+
+	return num_children;
 }
 #endif /* USE_GTK1 */
 
 /**
  * Adjust the width of one widget based on the font information of another
- * widget. 
+ * widget.
  *
  * Currently this function sets the width of the target widget so it can
  * accomodate a string of the given length which contains only digits.
- * 
+ *
  * @param extra additional number of pixels to add
  */
 #ifdef USE_GTK1
@@ -921,7 +996,7 @@ gtk_widget_fix_width(GtkWidget *w, GtkWidget *l, guint chars, guint extra)
     }
 
     max_width = max_width * chars + extra;
- 
+
     gtk_widget_set_usize(w, max_width, -1);
 }
 #endif
@@ -949,27 +1024,6 @@ gtk_widget_fix_width(GtkWidget *w, GtkWidget *l, guint chars, guint extra)
     pango_font_metrics_unref(pfm);
 }
 
-void
-set_tooltips_keyboard_mode(GtkWidget *widget, gboolean on)
-{
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel(widget);
-	if (!GTK_IS_WINDOW(toplevel))
-		return;
-	
-	g_object_set_data(G_OBJECT(toplevel),
-			"gtk-tooltips-keyboard-mode", GUINT_TO_POINTER(on));
-}
-
-void
-widget_force_tooltip(GtkWidget *widget)
-{
-	g_return_if_fail(widget != NULL);
-	set_tooltips_keyboard_mode(widget, TRUE);
-	set_tooltips_keyboard_mode(widget, FALSE);
-}
-
 #endif
 
 #ifdef USE_GTK1
@@ -979,7 +1033,7 @@ gtk_progress_bar_set_text(GtkProgressBar *pb, const gchar *text)
 	const gchar *p = text;
 	gchar buf[1024], *q = buf, c;
 
-	/* Replace '%' with %% */	
+	/* Replace '%' with %% */
 	for (p = text; (c = *p) != '\0'; p++) {
 		/* We need room for %% plus \0, truncation is OK */
 		if (&buf[sizeof buf] - q < 3)
@@ -996,4 +1050,20 @@ gtk_progress_bar_set_text(GtkProgressBar *pb, const gchar *text)
 #endif
 
 
+#if !GTK_CHECK_VERSION(2, 4, 0)
+GtkWidget *
+gtk_paned_get_child1(GtkPaned *paned)
+{
+	g_return_val_if_fail(paned != NULL, NULL);
+	return paned->child1;
+}
+
+GtkWidget *
+gtk_paned_get_child2(GtkPaned *paned)
+{
+	g_return_val_if_fail(paned != NULL, NULL);
+	return paned->child2;
+}
+#endif
+
 /* vi: set ts=4 sw=4 cindent: */
