@@ -1,5 +1,5 @@
 /*
- * $Id: callbacks.c,v 1.8 2005/11/15 21:25:28 rmanfredi Exp $
+ * $Id: callbacks.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: callbacks.c,v 1.8 2005/11/15 21:25:28 rmanfredi Exp $");
+RCSID("$Id: callbacks.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/misc.h"
 #include "gtk/search.h"
@@ -73,8 +73,8 @@ on_ctree_menu_tree_select_row(GtkCTree *ctree, GList *node,
     tab = GPOINTER_TO_INT(
 		gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node)));
 
-	gtk_notebook_set_page
-        (GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main")), tab);
+	gtk_notebook_set_page(
+		GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")), tab);
 }
 
 
