@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.c,v 1.33 2006/02/22 12:38:19 rmanfredi Exp $
+ * $Id: downloads.c 12242 2006-11-06 20:41:55Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: downloads.c,v 1.33 2006/02/22 12:38:19 rmanfredi Exp $");
+RCSID("$Id: downloads.c 12242 2006-11-06 20:41:55Z cbiere $")
 
 #include "gtk/downloads.h"
 #include "gtk/downloads_common.h"
@@ -33,6 +33,7 @@ RCSID("$Id: downloads.c,v 1.33 2006/02/22 12:38:19 rmanfredi Exp $");
 #include "gtk/statusbar.h"
 #include "gtk/columns.h"
 #include "gtk/notebooks.h"
+#include "gtk/settings.h"
 
 #include "if/core/pproxy.h"
 #include "if/core/bsched.h"
@@ -77,33 +78,12 @@ static GtkNotebook *notebook = NULL;
 static gboolean ctree_downloads_frozen = FALSE;
 static gboolean ctree_downloads_queue_frozen = FALSE;
 
-#define IO_STALLED			60	/**< If nothing exchanged after that many secs */
 #define DL_GUI_TREE_SPACE	5	/**< Space between a child node and a parent */
 
 /***
  *** Private functions
  ***/
 
-static inline const gchar *
-download_progress_to_string(const struct download *d)
-{
-	static gchar buf[32];
-
-	gm_snprintf(buf, sizeof buf, "%5.2f%%",
-		100.0 * guc_download_total_progress(d));
-	return buf;
-}
-
-static inline const gchar *
-source_progress_to_string(const struct download *d)
-{
-	static gchar buf[32];
-
-	gm_snprintf(buf, sizeof buf, "%5.2f%%",
-		100.0 * guc_download_source_progress(d));
-	return buf;
-}
-
 /**
  * Add the given tree node to the hashtable.
  * The key is an int ref on the fi_handle for a given download.
@@ -493,13 +473,13 @@ void downloads_gui_init(void)
     GtkCList *clist;
 
 	ctree_downloads = GTK_CTREE
-		(lookup_widget(main_window, "ctree_downloads"));
+		(gui_main_window_lookup("ctree_downloads"));
 	ctree_downloads_queue = GTK_CTREE
-		(lookup_widget(main_window, "ctree_downloads_queue"));
+		(gui_main_window_lookup("ctree_downloads_queue"));
 
 	clist = GTK_CLIST(ctree_downloads);
 	clist_queue = GTK_CLIST(ctree_downloads_queue);
-	notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
+	notebook = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
 
     gtk_clist_column_titles_passive(clist);
     gtk_clist_column_titles_passive(clist_queue);
@@ -582,7 +562,8 @@ download_gui_add(struct download *d)
 	titles[c_queue_status] = "";
 
 	if (d->file_info->file_size_known)
-		titles[c_queue_size] = short_size(d->file_info->size);
+		titles[c_queue_size] = short_size(d->file_info->size,
+									show_metric_units());
 	else
 		titles[c_queue_size] = UNKNOWN_SIZE_STR;
 
@@ -716,7 +697,8 @@ download_gui_add(struct download *d)
 		titles[c_dl_status] = "";
 
 		if (d->file_info->file_size_known)
-			titles[c_dl_size] = short_size(d->file_info->size);
+			titles[c_dl_size] = short_size(d->file_info->size,
+									show_metric_units());
 		else
 			titles[c_dl_size] = UNKNOWN_SIZE_STR;
 		titles[c_dl_range] = "";
@@ -889,10 +871,10 @@ gui_update_download_server(struct download *d)
 void
 gui_update_download_range(struct download *d)
 {
-	filesize_t len;
-	gchar *and_more = "";
-	gint rw;
+	const gchar *and_more = "";
 	GtkCTreeNode *node;
+	filesize_t len;
+	gint rw;
 
 	g_assert(d);
 	g_assert(d->status != GTA_DL_QUEUED);
@@ -912,11 +894,11 @@ gui_update_download_range(struct download *d)
 	len += d->overlap_size;
 
 	rw = gm_snprintf(tmpstr, sizeof(tmpstr), "%s%s",
-			compact_size(len), and_more);
+			compact_size(len, show_metric_units()), and_more);
 
 	if (d->skip)
 		gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw, " @ %s",
-			compact_size(d->skip));
+			compact_size(d->skip, show_metric_units()));
 
 	node = gtk_ctree_find_by_row_data(ctree_downloads, NULL, (gpointer) d);
 	if (NULL != node)
@@ -937,7 +919,8 @@ gui_update_download_size(struct download *d)
 	if (!d->file_info->file_size_known)
 		return;
 
-	gm_snprintf(tmpstr, sizeof(tmpstr), "%s", short_size(d->size));
+	gm_snprintf(tmpstr, sizeof tmpstr, "%s",
+		short_size(d->size, show_metric_units()));
 
 	node = gtk_ctree_find_by_row_data(ctree_downloads, NULL, (gpointer) d);
 	if (NULL != node)
@@ -975,7 +958,6 @@ gui_update_download(struct download *d, gboolean force)
 	fileinfo_t *fi;
 	gpointer key;
 	gint active_src, tot_src;
-	guint32 s = 0;
 	gboolean copy_status_to_parent = FALSE;
 
 	gint rw;
@@ -1187,7 +1169,8 @@ gui_update_download(struct download *d, gboolean force)
 			
 			gm_snprintf(tmpstr, sizeof(tmpstr), "%s (%s) %s",
 				FILE_INFO_COMPLETE(fi) ? _("Completed") : _("Chunk done"),
-				short_rate((d->range_end - d->skip + d->overlap_size) / t),
+				short_rate((d->range_end - d->skip + d->overlap_size) / t,
+					show_metric_units()),
 				short_time(t));
 		} else {
 			gm_snprintf(tmpstr, sizeof(tmpstr), "%s (< 1s)",
@@ -1232,7 +1215,8 @@ gui_update_download(struct download *d, gboolean force)
 			
 				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
 					" (%s) %s",
-					short_rate(fi->cha1_hashed / (elapsed ? elapsed : 1)),
+					short_rate(fi->cha1_hashed / (elapsed ? elapsed : 1),
+						show_metric_units()),
 					short_time(fi->cha1_elapsed));
 			}
 
@@ -1249,7 +1233,8 @@ gui_update_download(struct download *d, gboolean force)
 				if (fi->copy_elapsed) {
 					gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
 						_("; Moved (%s) %s"),
-						short_rate(fi->copied / fi->copy_elapsed),
+						short_rate(fi->copied / fi->copy_elapsed,
+							show_metric_units()),
 						short_time(fi->copy_elapsed));
 				}
 				break;
@@ -1270,8 +1255,13 @@ gui_update_download(struct download *d, gboolean force)
 			downloaded = d->pos - d->skip + download_buffered(d);
 			p = guc_download_source_progress(d);
 
-			bps = bio_bps(d->bio);
-			avg_bps = bio_avg_bps(d->bio);
+			if (d->bio) {
+				bps = bio_bps(d->bio);
+				avg_bps = bio_avg_bps(d->bio);
+			} else {
+				avg_bps = 0;
+				bps = 0;
+			}
 
 			if (avg_bps <= 10 && d->last_update != d->start_date)
 				avg_bps = downloaded / (d->last_update - d->start_date);
@@ -1292,7 +1282,7 @@ gui_update_download(struct download *d, gboolean force)
 						_("(stalled)"));
 				else
 					rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-						" (%s)", short_rate(bps));
+						" (%s)", short_rate(bps, show_metric_units()));
 
 				rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
 					_(" [%d/%d] TR: %s"), fi->recvcount, fi->lifecount,
@@ -1306,7 +1296,8 @@ gui_update_download(struct download *d, gboolean force)
 
 					if (fi->recvcount > 1) {
 						rw += gm_snprintf(&tmpstr[rw], sizeof(tmpstr)-rw,
-							" (%s)", short_rate(fi->recv_last_rate));
+							" (%s)", short_rate(fi->recv_last_rate,
+										show_metric_units()));
 					}
 				}
 			} else {
@@ -1442,22 +1433,27 @@ gui_update_download(struct download *d, gboolean force)
 						record_parent_gui_update(key, now);
 
 					} else if (GTA_DL_RECEIVING == d->status) {
-						s = 0;
+						guint32 s;
+
 						active_src = fi->recvcount;
 						tot_src = fi->lifecount;
 
 						if (fi->recv_last_rate)
 							s = (fi->size - fi->done) / fi->recv_last_rate;
+						else
+							s = 0;
 
 						if (s) {
 							gm_snprintf(tmpstr, sizeof(tmpstr),
 								_("(%s)  [%d/%d]  TR:  %s"),
-								short_rate(fi->recv_last_rate),
+								short_rate(fi->recv_last_rate,
+									show_metric_units()),
 								active_src, tot_src, short_time(s));
 						} else {
 							gm_snprintf(tmpstr, sizeof(tmpstr),
 								_("(%s)  [%d/%d]  TR:  -"),
-								short_rate(fi->recv_last_rate),
+								short_rate(fi->recv_last_rate,
+									show_metric_units()),
 								active_src, tot_src);
 						}
 
@@ -1552,27 +1548,27 @@ gui_update_download_abort_resume(void)
 	g_list_free(node_list);
 
 	gtk_widget_set_sensitive
-        (lookup_widget(main_window, "button_downloads_abort"), do_abort);
+        (gui_main_window_lookup("button_downloads_abort"), do_abort);
 	gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_abort"), do_abort);
+        (gui_popup_downloads_lookup("popup_downloads_abort"), do_abort);
     gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_abort_named"),
+        (gui_popup_downloads_lookup("popup_downloads_abort_named"),
 		do_abort);
     gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_abort_host"),
+        (gui_popup_downloads_lookup("popup_downloads_abort_host"),
 		do_abort);
     gtk_widget_set_sensitive(
-        lookup_widget(popup_downloads, "popup_downloads_abort_sha1"),
+        gui_popup_downloads_lookup("popup_downloads_abort_sha1"),
         abort_sha1);
 	gtk_widget_set_sensitive
-        (lookup_widget(main_window, "button_downloads_resume"), do_resume);
+        (gui_main_window_lookup("button_downloads_resume"), do_resume);
 	gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_resume"), do_resume);
+        (gui_popup_downloads_lookup("popup_downloads_resume"), do_resume);
     gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_remove_file"),
+        (gui_popup_downloads_lookup("popup_downloads_remove_file"),
 		do_remove);
     gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_queue"), do_queue);
+        (gui_popup_downloads_lookup("popup_downloads_queue"), do_queue);
 }
 
 /**
