@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c,v 1.13 2005/11/12 11:29:52 cbiere Exp $
+ * $Id: gnet_stats.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,10 +25,11 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c,v 1.13 2005/11/12 11:29:52 cbiere Exp $");
+RCSID("$Id: gnet_stats.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
+#include "gtk/settings.h"
 
 #include "if/core/hsep.h"
 #include "if/core/gnutella.h"
@@ -86,7 +87,8 @@ on_clist_gnet_stats_fc_ttl_resize_column(GtkCList *unused_clist,
     for (n = 0; n < G_N_ELEMENTS(buf); n ++)
         buf[n] = width;
 
-    gui_prop_set_guint32(PROP_GNET_STATS_FC_TTL_COL_WIDTHS, buf, 1, 9);
+    gui_prop_set_guint32(PROP_GNET_STATS_FC_TTL_COL_WIDTHS,
+		buf, 1, G_N_ELEMENTS(buf));
 
     lock = FALSE;
 }
@@ -112,7 +114,8 @@ on_clist_gnet_stats_fc_hops_resize_column(GtkCList *unused_clist,
     for (n = 0; n < G_N_ELEMENTS(buf); n ++)
         buf[n] = width;
 
-    gui_prop_set_guint32(PROP_GNET_STATS_FC_HOPS_COL_WIDTHS, buf, 1, 9);
+    gui_prop_set_guint32(PROP_GNET_STATS_FC_HOPS_COL_WIDTHS,
+		buf, 1, G_N_ELEMENTS(buf));
 
     lock = FALSE;
 }
@@ -204,11 +207,11 @@ byte_stat_str(const guint64 *val_tbl, const guint64 *nb_packets, gint type)
 		guint64 total_size = val_tbl[MSG_TOTAL];
 		if (!gnet_stats_with_headers)
 			size -= nb_packets[MSG_TOTAL] * GTA_HEADER_SIZE;
-        gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%",
+        gm_snprintf(strbuf, sizeof strbuf, "%.2f%%",
             (gfloat) size / total_size * 100.0);
         return strbuf;
     } else
-        return compact_size(size);
+        return compact_size(size, show_metric_units());
 }
 
 const gchar *
@@ -240,7 +243,7 @@ general_stat_str(const gnet_stats_t *stats, gint type)
         return "-";
 
     if (type == GNR_QUERY_COMPACT_SIZE) {
-        return compact_size(stats->general[type]);
+        return compact_size(stats->general[type], show_metric_units());
     } else {
         uint64_to_string_buf(stats->general[type], strbuf, sizeof strbuf);
         return strbuf;
@@ -274,10 +277,10 @@ flowc_stat_str_byte(const guint64 *val_tbl, gint type)
         return gnet_stats_perc ? "-  " : "-";
 
 	if (gnet_stats_perc) {
-		gm_snprintf(strbuf, sizeof(strbuf), "%.2f%%",
+		gm_snprintf(strbuf, sizeof strbuf, "%.2f%%",
             (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
     } else {
-       	return compact_size(val_tbl[type]);
+       	return compact_size(val_tbl[type], show_metric_units());
     }
 
     return strbuf;
@@ -303,17 +306,17 @@ gnet_stats_gui_init(void)
         titles[n] = "-";
 
     clist_stats_msg = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_msg"));
+        gui_main_window_lookup("clist_gnet_stats_msg"));
     clist_stats_fc_ttl = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_fc_ttl"));
+        gui_main_window_lookup("clist_gnet_stats_fc_ttl"));
     clist_stats_fc_hops = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_fc_hops"));
+        gui_main_window_lookup("clist_gnet_stats_fc_hops"));
     clist_reason = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_drop_reasons"));
+        gui_main_window_lookup("clist_gnet_stats_drop_reasons"));
     clist_general = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_general"));
+        gui_main_window_lookup("clist_gnet_stats_general"));
     clist_horizon = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_horizon"));
+        gui_main_window_lookup("clist_gnet_stats_horizon"));
 
     /*
      * Set column justification for numeric columns to GTK_JUSTIFY_RIGHT.
@@ -422,7 +425,7 @@ gnet_stats_gui_update(time_t now)
 		return;
 	last_update = now;
     current_page = gtk_notebook_get_current_page(
-        GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main")));
+        GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")));
 
     if (current_page != nb_main_page_gnet_stats)
         return;
@@ -430,17 +433,17 @@ gnet_stats_gui_update(time_t now)
     guc_gnet_stats_get(&stats);
 
     clist_stats_msg = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_msg"));
+        gui_main_window_lookup("clist_gnet_stats_msg"));
     clist_reason = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_drop_reasons"));
+        gui_main_window_lookup("clist_gnet_stats_drop_reasons"));
     clist_general = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_general"));
+        gui_main_window_lookup("clist_gnet_stats_general"));
     clist_horizon = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_horizon"));
+        gui_main_window_lookup("clist_gnet_stats_horizon"));
     clist_stats_fc_ttl = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_fc_ttl"));
+        gui_main_window_lookup("clist_gnet_stats_fc_ttl"));
     clist_stats_fc_hops = GTK_CLIST(
-        lookup_widget(main_window, "clist_gnet_stats_fc_hops"));
+        gui_main_window_lookup("clist_gnet_stats_fc_hops"));
 
     gtk_clist_freeze(clist_reason);
     gtk_clist_freeze(clist_general);
