@@ -1,5 +1,5 @@
 /*
- * $Id: search_cb.c,v 1.46 2006/01/21 18:17:51 rmanfredi Exp $
+ * $Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $
  *
  * Copyright (c) 2001-2005, Raphael Manfredi, Richard Eckart
  *
@@ -39,7 +39,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: search_cb.c,v 1.46 2006/01/21 18:17:51 rmanfredi Exp $");
+RCSID("$Id: search_cb.c 12268 2006-11-09 00:55:57Z cbiere $")
 
 #include <gdk/gdkkeysyms.h>
 
@@ -50,6 +50,7 @@ RCSID("$Id: search_cb.c,v 1.46 2006/01/21 18:17:51 rmanfredi Exp $");
 #include "gtk/bitzi.h"			/* Bitzi GTK functions */
 #include "gtk/columns.h"
 #include "gtk/misc.h"
+#include "gtk/settings.h"
 #include "search_cb.h"
 
 #include "if/gui_property.h"
@@ -58,6 +59,7 @@ RCSID("$Id: search_cb.c,v 1.46 2006/01/21 18:17:51 rmanfredi Exp $");
 #include "if/bridge/ui2c.h"
 
 #include "if/core/search.h"
+#include "if/core/sockets.h"
 
 #include "lib/atoms.h"
 #include "lib/glib-missing.h"
@@ -66,8 +68,6 @@ RCSID("$Id: search_cb.c,v 1.46 2006/01/21 18:17:51 rmanfredi Exp $");
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-extern search_t *search_selected;
-
 /***
  *** Private functions
  ***/
@@ -75,64 +75,6 @@ extern search_t *search_selected;
 /**
  *	Activates/deactivates buttons and popups based on what is selected
  */
-static void
-refresh_popup(void)
-{
-	/*
-	 * The following popup items are set insensitive if nothing is currently
-	 * selected.
-	 */
-	static const struct {
-		const gchar *name;
-	} menu[] = {
-		{	"popup_search_drop" },
-		{	"popup_search_drop_global" },
-		{	"popup_search_autodownload" },
-		{	"popup_search_new_from_selected" },
-		{	"popup_search_metadata" },
-	};
-	search_t *search = search_gui_get_current_search();
-	gboolean sensitive;
-	guint i;
-
-	sensitive = NULL != search && NULL != GTK_CLIST(search->ctree)->selection;
-
-	gtk_widget_set_sensitive(
-			lookup_widget(main_window, "button_search_download"),
-			sensitive);
-    gtk_widget_set_sensitive(
-        lookup_widget(popup_search, "popup_search_browse_host"), sensitive);
-
-	for (i = 0; i < G_N_ELEMENTS(menu); i++)
-		gtk_widget_set_sensitive(lookup_widget(popup_search, menu[i].name),
-			sensitive);
-
-	sensitive = NULL != search;	
-    gtk_widget_set_sensitive(
-        lookup_widget(popup_search, "popup_search_restart"), sensitive);
-    gtk_widget_set_sensitive(
-        lookup_widget(popup_search, "popup_search_duplicate"), sensitive);
-
-    if (search) {
-        gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_stop"),
-			!guc_search_is_frozen(search->search_handle));
-		gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_resume"),
-			guc_search_is_frozen(search->search_handle)
-				&& !search_gui_is_expired(search));
-		if (search->passive)
-            gtk_widget_set_sensitive(
-                lookup_widget(popup_search, "popup_search_restart"),
-                FALSE);
-    } else {
-        gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_stop"), FALSE);
-	    gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_resume"), FALSE);
-    }
-}
-
 /**
  * Set or clear (when rc == NULL) the information about the record.
  */
@@ -185,9 +127,9 @@ search_gui_set_details(const record_t *rc)
 		initialized = TRUE;
 
 		for (i = 0; i < G_N_ELEMENTS(entry); i++)
-			entry[i] = GTK_ENTRY(lookup_widget(main_window, widgets[i].name));
+			entry[i] = GTK_ENTRY(gui_main_window_lookup(widgets[i].name));
 
-		xml = GTK_TEXT(lookup_widget(main_window,
+		xml = GTK_TEXT(gui_main_window_lookup(
 							STRINGIFY(CAT2(text_result_,info_xml))));
 	}
 
@@ -240,7 +182,7 @@ search_gui_set_details(const record_t *rc)
 				
 				uint64_to_string_buf(rc->size, bytes, sizeof bytes);
 				gtk_entry_printf(e, _("%s (%s bytes)"),
-					short_size(rc->size), bytes);
+					short_size(rc->size, show_metric_units()), bytes);
 			}
 			break;
 
@@ -268,11 +210,11 @@ search_gui_set_details(const record_t *rc)
 			break;
 
 		case info_index:
-			gtk_entry_printf(e, "%lu", (gulong) rc->index);
+			gtk_entry_printf(e, "%lu", (gulong) rc->file_index);
 			break;
 
 		case info_tag:
-			gtk_entry_set_text(e, EMPTY_STRING(rc->tag));
+			gtk_entry_set_text(e, EMPTY_STRING(rc->results_set->query));
 			break;
 			
 		case info_speed:
@@ -370,16 +312,6 @@ search_cb_autoselect(GtkCTree *ctree, GtkCTreeNode *node)
  *** Glade callbacks
  ***/
 
-void
-on_search_popdown_switch(GtkWidget *unused_w, gpointer unused_data)
-{
-	(void) unused_w;
-	(void) unused_data;
-
-	if (search_selected != NULL)
-        search_gui_set_current_search(search_selected);
-}
-
 
 /**
  *	When the user switches notebook tabs, update the rest of GUI
@@ -403,39 +335,6 @@ on_search_notebook_switch(GtkNotebook *notebook, GtkNotebookPage *unused_page,
     search_gui_set_current_search(sch);
 }
 
-
-/**
- *	Changes current search and updates GUI
- */
-void
-on_clist_search_select_row(GtkCList *clist, gint row, gint unused_column,
-	GdkEvent *unused_event, gpointer unused_udata)
-{
-    search_t *sch;
-
-	(void) unused_column;
-	(void) unused_event;
-	(void) unused_udata;
-    g_assert(clist != NULL);
-
-    sch = gtk_clist_get_row_data(clist, row);
-    if (sch == NULL)
-        return;
-
-    search_gui_set_current_search(sch);
-}
-
-
-/**
- */
-void
-on_search_selected(GtkItem *unused_item, gpointer data)
-{
-	(void) unused_item;
-	search_selected = (search_t *) data;
-}
-
-
 /**
  *	Create a search based on query entered
  */
@@ -475,7 +374,7 @@ on_entry_search_changed(GtkEditable *editable, gpointer unused_udata)
 	(void) unused_udata;
 	
 	g_strstrip(s);
-	gtk_widget_set_sensitive(lookup_widget(main_window, "button_search"),
+	gtk_widget_set_sensitive(gui_main_window_lookup("button_search"),
 		s[0] != '\0');
 	G_FREE_NULL(s);
 }
@@ -492,7 +391,7 @@ on_button_search_clear_clicked(GtkButton *unused_button, gpointer unused_udata)
 
 	gui_search_clear_results();
 	gtk_widget_set_sensitive
-        (lookup_widget(main_window, "button_search_clear"), FALSE);
+        (gui_main_window_lookup("button_search_clear"), FALSE);
 }
 
 
@@ -550,6 +449,16 @@ on_button_search_expand_all_clicked(GtkButton *unused_button,
     search_gui_expand_all();
 }
 
+/**
+ */
+void
+on_button_search_filter_clicked(GtkButton *unused_button, gpointer unused_udata)
+{
+	(void) unused_button;
+	(void) unused_udata;
+
+	filter_open_dialog();
+}
 
 /**
  */
@@ -596,6 +505,8 @@ on_clist_search_results_button_press_event(GtkWidget *widget,
 		if (event->type == GDK_BUTTON_PRESS) {
 			static guint click_time = 0;
 
+			search_gui_set_cursor_position(event->x, event->y);
+
 			if ((event->time - click_time) <= 250) {
 				gint row = 0;
 				gint column = 0;
@@ -624,19 +535,16 @@ on_clist_search_results_button_press_event(GtkWidget *widget,
         /* right click section (popup menu) */
     	if (search_gui_get_current_search()) {
 			GtkMenuItem *item;
-            gboolean search_results_show_tabs;
 
-        	refresh_popup();
-            gui_prop_get_boolean_val(PROP_SEARCH_RESULTS_SHOW_TABS,
-                &search_results_show_tabs);
+        	search_gui_refresh_popup();
 
-			item = GTK_MENU_ITEM(lookup_widget(popup_search,
-									"popup_search_toggle_tabs"));
+			item = GTK_MENU_ITEM(
+					gui_popup_search_lookup("popup_search_toggle_tabs"));
         	gtk_label_set(GTK_LABEL(item->item.bin.child),
 				search_results_show_tabs
 					? _("Show search list")
 					: _("Show tabs"));
-			gtk_menu_popup(GTK_MENU(popup_search), NULL, NULL, NULL, NULL,
+			gtk_menu_popup(GTK_MENU(gui_popup_search()), NULL, NULL, NULL, NULL,
                      event->button, event->time);
         }
 		return TRUE;
@@ -646,18 +554,6 @@ on_clist_search_results_button_press_event(GtkWidget *widget,
 }
 
 
-/**
- */
-void
-on_button_search_filter_clicked(GtkButton *unused_button, gpointer unused_udata)
-{
-	(void) unused_button;
-	(void) unused_udata;
-
-	filter_open_dialog();
-}
-
-
 /**
  *	Sort search according to selected column
  */
@@ -729,7 +625,7 @@ on_ctree_search_results_select_row(GtkCTree *ctree,
 		GTK_SIGNAL_FUNC(on_ctree_search_results_select_row),
 		NULL);
 
-    refresh_popup();
+    search_gui_refresh_popup();
 	search_cb_autoselect(ctree, GTK_CTREE_NODE(node));
 
     gtk_signal_handler_unblock_by_func(GTK_OBJECT(ctree),
@@ -752,7 +648,7 @@ on_ctree_search_results_unselect_row(GtkCTree *unused_ctree, GList *unused_node,
 	(void) unused_udata;
 
 	search_gui_set_details(NULL);	/* Clear details about results */
-    refresh_popup();
+    search_gui_refresh_popup();
 }
 
 
@@ -791,7 +687,7 @@ on_button_search_passive_clicked(GtkButton *unused_button,
      * side effect.
      */
     default_filter = option_menu_get_selected_data(GTK_OPTION_MENU(
-				lookup_widget(main_window, "optionmenu_search_filter")));
+				gui_main_window_lookup("optionmenu_search_filter")));
 
 	search_gui_new_search(_("Passive"), SEARCH_F_PASSIVE, &search);
 
@@ -846,6 +742,7 @@ search_gui_browse_selected(void)
 		gui_record_t *grc;
 		results_set_t *rs;
 		record_t *rc;
+		guint32 flags = 0;
 
 		if (node == NULL)
 			break;
@@ -857,10 +754,12 @@ search_gui_browse_selected(void)
 			continue;
 
 		rs = rc->results_set;
+		flags |= 0 != (rs->status & ST_FIREWALL) ? SOCK_F_PUSH : 0;
+		flags |= 0 != (rs->status & ST_TLS) ? SOCK_F_TLS : 0;
 
 		(void) search_gui_new_browse_host(
-			rs->hostname, rs->addr, rs->port,
-			rs->guid, (rs->status & ST_FIREWALL) != 0, rs->proxies);
+				rs->hostname, rs->addr, rs->port,
+				rs->guid, rs->proxies, flags);
 	}
 }
 
@@ -1009,167 +908,6 @@ void on_popup_search_drop_host_global_activate(GtkMenuItem *unused_menuitem,
     global_add_filter((GFunc) filter_add_drop_host_rule, gui_record_host_eq);
 }
 
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_autodownload_name_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search_add_filter((GFunc) filter_add_download_name_rule,
-		gui_record_name_eq);
-}
-
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_autodownload_sha1_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search_add_filter((GFunc) filter_add_download_sha1_rule,
-		gui_record_sha1_eq);
-}
-
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_new_from_selected_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-    GList *node_list;
-	GSList *data_list;
-    search_t *search;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search = search_gui_get_current_search();
-    g_assert(search != NULL);
-
-    gtk_clist_freeze(GTK_CLIST(search->ctree));
-
-	node_list = g_list_copy(GTK_CLIST(search->ctree)->selection);
-	data_list = search_cb_collect_ctree_data(search->ctree,
-					node_list, gui_record_name_eq);
-
-    g_slist_foreach(data_list, (GFunc) search_gui_add_targetted_search, NULL);
-
-	gtk_clist_thaw(GTK_CLIST(search->ctree));
-	g_slist_free(data_list);
-	g_list_free(node_list);
-}
-
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_edit_filter_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    filter_open_dialog();
-}
-
-
-/**
- *	Create a new search identical to the current search.
- *
- * 	@note Doesn't duplicate filters or passive searches yet.
- */
-void
-on_popup_search_duplicate_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-    search_t *search;
-    guint32 timeout;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    gnet_prop_get_guint32_val
-		(PROP_SEARCH_REISSUE_TIMEOUT, &timeout);
-
-    search = search_gui_get_current_search();
-    /* FIXME: should also duplicate filters! */
-    /* FIXME: should call search_duplicate which has to be written. */
-    /* FIXME: should properly duplicate passive searches. */
-	if (search)
-		search_gui_new_search_full(search->query, tm_time(), search_lifetime,
-			timeout, search->sort_col, search->sort_order,
-			search->enabled ? SEARCH_F_ENABLED : 0, NULL);
-}
-
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_restart_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-    search_t *search;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search = search_gui_get_current_search();
-	if (search)
-		search_gui_restart_search(search);
-}
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_resume_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-    search_t *search;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search = search_gui_get_current_search();
-	if (search && !search_gui_is_expired(search)) {
-		gui_search_set_enabled(search, TRUE);
-		search_gui_update_expiry(search);
-	}
-}
-
-/**
- *	Stop current search
- */
-void
-on_popup_search_stop_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-    search_t *search;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search = search_gui_get_current_search();
-	if (search) {
-		gui_search_set_enabled(search, FALSE);
-		search_gui_update_expiry(search);
-	}
-}
-
-
 /**
  *	Please add comment
  */
@@ -1193,34 +931,6 @@ on_popup_search_config_cols_activate(GtkMenuItem *unused_menuitem,
     /* GtkColumnChooser takes care of cleaning up itself */
 }
 
-/**
- *	Please add comment
- */
-void
-on_popup_search_expand_all_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search_gui_expand_all();
-
-}
-
-/**
- *	Please add comment
- */
-void
-on_popup_search_collapse_all_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    search_gui_collapse_all();
-
-}
-
 /**
  * Queue a bitzi queries from the search context menu
  */
@@ -1262,7 +972,7 @@ on_popup_search_metadata_activate(GtkMenuItem *unused_menuitem,
 			  g_slist_position(data_list, g_slist_last(data_list)) + 1,
 			  cast_to_gconstpointer(data_list));
 
-	G_SLIST_FOREACH(data_list, search_gui_queue_bitzi_by_sha1, NULL);
+	G_SLIST_FOREACH(data_list, search_gui_queue_bitzi_by_sha1);
 
 	gtk_clist_thaw(GTK_CLIST(search->ctree));
 	g_slist_free(data_list);
