@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c,v 1.21 2005/12/19 07:12:17 cbiere Exp $
+ * $Id: gnet_stats.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,12 +25,13 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c,v 1.21 2005/12/19 07:12:17 cbiere Exp $");
+RCSID("$Id: gnet_stats.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
 #include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
+#include "gtk/settings.h"
 
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
@@ -71,14 +72,13 @@ enum gnet_stats_nb_page {
 	NUM_GNET_STATS_NB_PAGES
 };
 
-static void hide_column_by_title(GtkTreeView *, const gchar *, gboolean);
 static void gnet_stats_update_drop_reasons(const gnet_stats_t *);
 
 /***
  *** Private functions
  ***/
 
-static void
+void
 hide_column_by_title(GtkTreeView *treeview, const gchar *header_title,
 	gboolean hidden)
 {
@@ -127,7 +127,7 @@ byte_stat_str(gchar *dst, gulong n, const guint64 *val_tbl,
 	if (0 == val_tbl[type])
 		g_strlcpy(dst, "-", n);
 	else if (!perc)
-		g_strlcpy(dst, compact_size(val_tbl[type]), n);
+		g_strlcpy(dst, compact_size(val_tbl[type], show_metric_units()), n);
 	else
 		gm_snprintf(dst, n, "%.2f%%",
 		    (gfloat) val_tbl[type] / val_tbl[MSG_TOTAL] * 100.0);
@@ -159,31 +159,14 @@ general_stat_str(gchar *dst, size_t size, const gnet_stats_t *stats, gint type)
 	if (stats->general[type] == 0)
 		g_strlcpy(dst, "-", size);
 	else if (type == GNR_QUERY_COMPACT_SIZE)
-		g_strlcpy(dst, compact_size(stats->general[type]), size);
+		g_strlcpy(dst,
+			compact_size(stats->general[type], show_metric_units()), size);
 	else
 		uint64_to_string_buf(stats->general[type], dst, size);
 
 	return dst;
 }
 
-static const gchar *
-type_stat_str(gchar *dst, size_t size, gulong value, gulong total,
-	gboolean perc, gboolean bytes)
-{
-	if (value == 0 || total == 0)
-		g_strlcpy(dst, "-", size);
-	else if (perc)
-		gm_snprintf(dst, size, "%.2f%%", (gfloat) value / total * 100.0);
-	else {
-		if (bytes)
-			g_strlcpy(dst, compact_size(value), size);
-		else
-			gm_snprintf(dst, size, "%lu", (gulong) value);
-	}
-
-	return dst;
-}
-
 static void
 add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 	const gchar *label)
@@ -220,14 +203,16 @@ gnet_stats_update_general(const gnet_stats_t *stats)
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(
 				treeview_gnet_stats_general));
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
+		return;
 
 	for (n = 0; n < GNR_TYPE_COUNT; n++) {
 		gchar buf[32];
 
 		general_stat_str(buf, sizeof buf, stats, n);
 		gtk_list_store_set(store, &iter, 1, buf, (-1));
-		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+			break;
 	}
 }
 
@@ -240,7 +225,8 @@ gnet_stats_update_drop_reasons(const gnet_stats_t *stats)
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(
 				treeview_gnet_stats_drop_reasons));
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
+		return;
 
 	for (n = 0; n < MSG_DROP_REASON_COUNT; n++) {
 		gchar buf[32];
@@ -248,7 +234,8 @@ gnet_stats_update_drop_reasons(const gnet_stats_t *stats)
 		drop_stat_str(buf, sizeof buf, stats, n,
 			gnet_stats_drop_reasons_type, gnet_stats_bytes);
 		gtk_list_store_set(store, &iter, 1, buf, (-1));
-		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+			break;
 	}
 }
 
@@ -270,7 +257,8 @@ gnet_stats_update_messages(const gnet_stats_t *stats)
 	gui_prop_get_boolean_val(PROP_GNET_STATS_BYTES, &bytes);
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
+		return;
 
 	for (n = 0; n < msg_type_str_size(); n++) {
 		if (!bytes) {
@@ -308,65 +296,8 @@ gnet_stats_update_messages(const gnet_stats_t *stats)
 									len, stats->byte.generated, n, perc),
 				(-1));
 		}
-		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
-	}
-
-}
-
-typedef guint64 stat_counters_t[MSG_TYPE_COUNT];
-
-static void
-gnet_stats_update_types(
-	const gnet_stats_t *unused_stats,
-	GtkTreeView *treeview,
-	gint columns,
-	const stat_counters_t *byte_counters,
-	const stat_counters_t *pkg_counters)
-{
-	static gchar str[MSG_TYPE_COUNT][32];
-	GtkListStore *store;
-	GtkTreeIter iter;
-	gboolean perc = FALSE;
-	gboolean bytes = FALSE;
-	gboolean with_headers = FALSE;
-	gint n;
-
-	(void) unused_stats;
-
-	gui_prop_get_boolean_val(PROP_GNET_STATS_PERC, &perc);
-	gui_prop_get_boolean_val(PROP_GNET_STATS_BYTES, &bytes);
-	gui_prop_get_boolean_val(PROP_GNET_STATS_WITH_HEADERS, &with_headers);
-
-	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(treeview)));
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
-
-	for (n = 0; n < MSG_TYPE_COUNT; n++) {
-		gint i;
-
-		if (!bytes)
-			for (i = 0; i < columns; i++)
-				type_stat_str(str[i], sizeof(str[0]),
-					(gulong) pkg_counters[i][n],
-					(gulong) pkg_counters[i][MSG_TOTAL],
-					perc, FALSE);
-		else
-			for (i = 0; i < columns; i++) {
-				gulong	value;
-				gulong	total;
-
-				value = byte_counters[i][n];
-				total = byte_counters[i][MSG_TOTAL];
-				if (!with_headers) {
-					value -= pkg_counters[i][n] * GTA_HEADER_SIZE;
-					total -= pkg_counters[i][MSG_TOTAL] * GTA_HEADER_SIZE;
-				}
-				type_stat_str(str[i], sizeof(str[0]), value, total, perc, TRUE);
-			}
-
-		gtk_list_store_set(store, &iter,
-			1, str[0], 2, str[1], 3, str[2], 4, str[3], 5, str[4],
-			6, str[5], 7, str[6], 8, str[7], 9, str[8], (-1));
-		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+			break;
 	}
 
 }
@@ -374,41 +305,15 @@ gnet_stats_update_types(
 static void
 gnet_stats_update_flowc(const gnet_stats_t *stats)
 {
-	const stat_counters_t *byte_counters, *pkg_counters;
-	GtkTreeView *treeview = treeview_gnet_stats_flowc;
-	gboolean hops = FALSE;
-
-	gui_prop_get_boolean_val(PROP_GNET_STATS_HOPS, &hops);
-	if (hops) {
-		pkg_counters = stats->pkg.flowc_hops;
-		byte_counters = stats->byte.flowc_hops;
-	} else {
-		pkg_counters = stats->pkg.flowc_ttl;
-		byte_counters = stats->byte.flowc_ttl;
-	}
-	hide_column_by_title(treeview, "0", !hops);
-	gnet_stats_update_types(stats, treeview, STATS_FLOWC_COLUMNS,
-		byte_counters, pkg_counters);
+	/* FIXME: Implement this.  */
+	(void) stats;
 }
 
 static void
 gnet_stats_update_recv(const gnet_stats_t *stats)
 {
-	const stat_counters_t *byte_counters, *pkg_counters;
-	GtkTreeView *treeview = treeview_gnet_stats_recv;
-	gboolean hops = FALSE;
-
-	gui_prop_get_boolean_val(PROP_GNET_STATS_HOPS, &hops);
-	if (hops) {
-		pkg_counters = stats->pkg.received_hops;
-		byte_counters = stats->byte.received_hops;
-	} else {
-		pkg_counters = stats->pkg.received_ttl;
-		byte_counters = stats->byte.received_ttl;
-	}
-	hide_column_by_title(treeview, "0", !hops);
-	gnet_stats_update_types(stats, treeview, STATS_FLOWC_COLUMNS,
-		byte_counters, pkg_counters);
+	/* FIXME: Implement this. */
+	(void) stats;
 }
 
 static void
@@ -436,7 +341,8 @@ gnet_stats_update_horizon(void)
 		return;
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview));
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
+		return;
 
 	global_table_size = guc_hsep_get_table_size();
 	/* Skip the first element */
@@ -446,7 +352,8 @@ gnet_stats_update_horizon(void)
 		    c_horizon_files, horizon_stat_str(i, c_horizon_files),
 		    c_horizon_size,	 horizon_stat_str(i, c_horizon_size),
 			(-1));
-		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+			break;
 	}
 
 	last_horizon_update = now;
@@ -475,7 +382,7 @@ gnet_stats_gui_horizon_init(void)
 	STATIC_ASSERT(num_c_horizon == G_N_ELEMENTS(types));
 	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
 	treeview = treeview_gnet_stats_horizon = GTK_TREE_VIEW(
-	    lookup_widget(main_window, "treeview_gnet_stats_horizon"));
+	    gui_main_window_lookup("treeview_gnet_stats_horizon"));
 
 	gui_prop_get_guint32(PROP_GNET_STATS_HORIZON_COL_WIDTHS,
 		width, 0, G_N_ELEMENTS(width));
@@ -522,7 +429,7 @@ gnet_stats_gui_flowc_init(void)
 	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
 
 	treeview = treeview_gnet_stats_flowc = GTK_TREE_VIEW(
-	    lookup_widget(main_window, "treeview_gnet_stats_flowc"));
+	    gui_main_window_lookup("treeview_gnet_stats_flowc"));
 
 	for (n = 0; (gint) n < msg_type_str_size(); n++) {
 		GtkTreeIter iter;
@@ -565,7 +472,7 @@ gnet_stats_gui_drop_reasons_init(void)
 	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
 
 	treeview = treeview_gnet_stats_drop_reasons = GTK_TREE_VIEW(
-	    lookup_widget(main_window, "treeview_gnet_stats_drop_reasons"));
+	    gui_main_window_lookup("treeview_gnet_stats_drop_reasons"));
 	gui_prop_get_guint32(PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
 		width, 0, G_N_ELEMENTS(width));
 
@@ -602,7 +509,7 @@ gnet_stats_gui_general_init(void)
 	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
 
 	treeview = treeview_gnet_stats_general = GTK_TREE_VIEW(
-	    lookup_widget(main_window, "treeview_gnet_stats_general"));
+	    gui_main_window_lookup("treeview_gnet_stats_general"));
 	gui_prop_get_guint32(PROP_GNET_STATS_GENERAL_COL_WIDTHS,
 		width, 0, G_N_ELEMENTS(width));
 
@@ -645,7 +552,7 @@ gnet_stats_gui_messages_init(void)
 	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
 
 	treeview = treeview_gnet_stats_messages = GTK_TREE_VIEW(
-	    lookup_widget(main_window, "treeview_gnet_stats_messages"));
+	    gui_main_window_lookup("treeview_gnet_stats_messages"));
 
 	for (n = 0; (gint) n < msg_type_str_size(); n++) {
 		GtkTreeIter iter;
@@ -696,7 +603,7 @@ gnet_stats_gui_recv_init(void)
 	model = GTK_TREE_MODEL(gtk_list_store_newv(G_N_ELEMENTS(types), types));
 
 	treeview = treeview_gnet_stats_recv = GTK_TREE_VIEW(
-	    lookup_widget(main_window, "treeview_gnet_stats_recv"));
+	    gui_main_window_lookup("treeview_gnet_stats_recv"));
 	gui_prop_get_guint32(PROP_GNET_STATS_RECV_COL_WIDTHS,
 		width, 0, G_N_ELEMENTS(width));
 
@@ -732,10 +639,9 @@ gnet_stats_gui_recv_init(void)
 void
 gnet_stats_gui_init(void)
 {
-	notebook_main = GTK_NOTEBOOK(
-		lookup_widget(main_window, "notebook_main"));
+	notebook_main = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
 	notebook_gnet_stats = GTK_NOTEBOOK(
-		lookup_widget(main_window, "gnet_stats_notebook"));
+							gui_main_window_lookup("gnet_stats_notebook"));
 
 	/*
 	 * Initialize stats tables.
