@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.c,v 1.8 2005/11/16 13:35:14 cbiere Exp $
+ * $Id: hcache.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: hcache.c,v 1.8 2005/11/16 13:35:14 cbiere Exp $");
+RCSID("$Id: hcache.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/hcache.h"
 #include "gtk/columns.h"
@@ -95,9 +95,9 @@ hcache_gui_init(void)
 		HCACHE_STATS_VISIBLE_COLUMNS);
 
 	notebook_main = GTK_NOTEBOOK(
-		lookup_widget(main_window, "notebook_main"));
+		gui_main_window_lookup("notebook_main"));
     treeview_hcache = GTK_TREE_VIEW(
-        lookup_widget(main_window, "treeview_hcache"));
+        gui_main_window_lookup("treeview_hcache"));
 	model = GTK_TREE_MODEL(gtk_list_store_new(4,
 							G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT,
 							G_TYPE_UINT));
@@ -155,7 +155,8 @@ hcache_gui_update(time_t now)
     guc_hcache_get_stats(stats);
 
 	store = GTK_LIST_STORE(gtk_tree_view_get_model(treeview_hcache));
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
+		goto cleanup;
 
 	for (n = 0; n < HCACHE_MAX; n++) {
 		if (n == HCACHE_NONE)
@@ -167,7 +168,8 @@ hcache_gui_update(time_t now)
             c_hcs_misses,     stats[n].misses,
             (-1));
 
-		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+		if (!gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter))
+			break;
 	}
 
 	gtk_tree_view_set_model(treeview_hcache, GTK_TREE_MODEL(store));
