@@ -1,5 +1,5 @@
 /*
- * $Id: search.c,v 1.82 2006/02/01 23:21:35 cbiere Exp $
+ * $Id: search.c 12268 2006-11-09 00:55:57Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -54,15 +54,18 @@
 #include "if/core/sockets.h"
 
 #include "lib/atoms.h"
+#include "lib/magnet.h"
 #include "lib/misc.h"
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
 #include "lib/tm.h"
+#include "lib/url.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: search.c,v 1.82 2006/02/01 23:21:35 cbiere Exp $");
+RCSID("$Id: search.c 12268 2006-11-09 00:55:57Z cbiere $")
 
 #define MAX_TAG_SHOWN	60		/**< Show only first chars of tag */
 
@@ -77,8 +80,9 @@ static gboolean search_gui_shutting_down = FALSE;
 /*
  * Private function prototypes.
  */
-static void
-gui_search_create_tree_view(GtkWidget ** sw, GtkWidget ** tv, gpointer udata);
+static void gui_search_create_tree_view(GtkWidget ** sw,
+				GtkWidget ** tv, gpointer udata);
+static void search_gui_init_dnd(GtkTreeView *tv);
 
 /*
  * If no search are currently allocated
@@ -98,10 +102,10 @@ struct result_data {
 	const GdkColor *bg;
 
 	record_t *record;
-	gchar *ext;			/**< Atom */
-	gchar *meta;		/**< g_malloc()ed */
-	gchar *info;		/**< g_malloc()ed */
+	gchar *meta;		/**< Atom */
+	gchar *info;		/**< g_strdup()ed */
 	guint count;		/**< count of children */
+	guint32 rank;		/**< for stable sorting */
 };
 
 static inline struct result_data *
@@ -117,14 +121,9 @@ get_result_data(GtkTreeModel *model, GtkTreeIter *iter)
 gpointer
 search_gui_get_record(GtkTreeModel *model, GtkTreeIter *iter)
 {
-	static const GValue zero_value;
-	GValue value = zero_value;
-	struct result_data *data;
-
-	gtk_tree_model_get_value(model, iter, 0, &value);
-	data = g_value_get_pointer(&value);
-	return data->record;
+	return get_result_data(model, iter)->record;
 }
+
 void
 search_gui_set_data(GtkTreeModel *model, struct result_data *rd)
 {
@@ -136,46 +135,112 @@ search_gui_set_data(GtkTreeModel *model, struct result_data *rd)
 	gtk_tree_store_set_value(GTK_TREE_STORE(model), &rd->iter, 0, &value);
 }
 
+/**
+ * Callback handler used with gtk_tree_model_foreach() to record the current
+ * rank/position in tree enabling stable sorting. 
+ */
+gboolean
+search_gui_update_rank(
+	GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer udata)
+{
+	guint32 *rank_ptr = udata;
+	struct result_data *data;
+
+	(void) path;
+	
+	data = get_result_data(model, iter);
+	data->rank = *rank_ptr;
+	(*rank_ptr)++;
+	return FALSE;
+}
 
 static void
-cell_renderer(GtkTreeViewColumn *unused_column, GtkCellRenderer *cell, 
+cell_renderer(GtkTreeViewColumn *column, GtkCellRenderer *cell, 
 	GtkTreeModel *model, GtkTreeIter *iter, gpointer udata)
 {
 	const struct result_data *data;
+    const struct results_set *rs;
 	const gchar *text;
-	guint id;
+	enum c_sr_columns id;
 
-	(void) unused_column;
-	
+	if (!gtk_tree_view_column_get_visible(column))
+		return;
+
+	text = NULL;	/* default to nothing */
 	id = GPOINTER_TO_UINT(udata);
 	data = get_result_data(model, iter);
+    rs = data->record->results_set;
+
 	switch (id) {
 	case c_sr_filename:
 		text = data->record->utf8_name;
 		break;
 	case c_sr_ext:
-		text = data->ext;
+		text = data->record->ext;
 		break;
 	case c_sr_meta:
 		text = data->meta;
 		break;
 	case c_sr_info:
-		text = data->info;
+		text = data->info ? data->info : data->record->info;
 		break;
 	case c_sr_size:
-		text = compact_size(data->record->size);
+		text = compact_size(data->record->size, show_metric_units());
 		break;
 	case c_sr_count:
 		text = data->count ? uint32_to_string(1 + data->count) : NULL;
 		break;
 	case c_sr_loc:
-		text = iso3166_country_cc(data->record->results_set->country);
+		if (-1 != rs->country)
+			text = iso3166_country_cc(rs->country);
 		break;
 	case c_sr_charset:
 		text = data->record->charset;
 		break;
-	default:
-		text = NULL;
+	case c_sr_route:
+		text = search_gui_get_route(data->record);
+		break;
+	case c_sr_protocol:
+		if (!(ST_LOCAL & rs->status))
+			text = ST_UDP & rs->status ? _("UDP") : _("TCP");
+		break;
+	case c_sr_hops:
+		if (!(ST_LOCAL & rs->status))
+			text = uint32_to_string(rs->hops);
+		break;
+	case c_sr_ttl:
+		if (!(ST_LOCAL & rs->status))
+			text = uint32_to_string(rs->ttl);
+		break;
+	case c_sr_spam:
+		if (SR_SPAM & data->record->flags) {
+			text = "S";	/* Spam */
+		} else if (ST_SPAM & rs->status) {
+			text = "s";	/* maybe spam */
+		}
+		break;
+	case c_sr_owned:
+		if (SR_OWNED & data->record->flags) {
+			text = _("owned");
+		} else if (SR_PARTIAL & data->record->flags) {
+			text = _("partial");
+		} else if (SR_SHARED & data->record->flags) {
+			text = _("shared");
+		}
+		break;
+	case c_sr_hostile:
+		if (ST_HOSTILE & rs->status) {
+			text = "H";
+		}
+		break;
+	case c_sr_sha1:
+		if (data->record->sha1) {
+			text = sha1_base32(data->record->sha1);
+		}
+		break;
+	case c_sr_num:
+		g_assert_not_reached();
+		break;
 	}
 	g_object_set(cell,
 		"text", text,
@@ -300,11 +365,7 @@ unref_record(GtkTreeModel *model, GtkTreePath *unused_path, GtkTreeIter *iter,
 	 * rd->record may point to freed memory now if this was the last reference
 	 */
 
-	if (rd->ext) {
-		atom_str_free(rd->ext);
-		rd->ext = NULL;
-	}
-	G_FREE_NULL(rd->meta);
+	atom_str_free_null(&rd->meta);
 	G_FREE_NULL(rd->info);
 	WFREE_NULL(rd, sizeof *rd);
 
@@ -421,39 +482,85 @@ search_gui_close_search(search_t *sch)
     guc_search_close(sch->search_handle);
 	atom_str_free(sch->query);
 
-	G_FREE_NULL(sch);
+	wfree(sch, sizeof *sch);
 }
 
+static void
+search_gui_clear_sorting(search_t *sch)
+{
+	gint id;
+		
+	g_return_if_fail(sch);
+	g_return_if_fail(sch->model);
+
+#if GTK_CHECK_VERSION(2,6,0)
+	id = GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID;
+#else
+	id = c_sr_count;
+#endif /* Gtk+ >= 2.6.0 */
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sch->model),
+		id, GTK_SORT_ASCENDING);
+}
+
+static void
+search_gui_restore_sorting(search_t *sch)
+{
+	g_return_if_fail(sch);
+	g_return_if_fail(sch->model);
+
+	if (
+		SORT_NONE != sch->sort_order &&
+		sch->sort_col >= 0 &&
+		(guint) sch->sort_col < SEARCH_RESULTS_VISIBLE_COLUMNS
+	) {
+		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sch->model),
+			sch->sort_col, SORT_ASC == sch->sort_order
+							? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
+	} else {
+		search_gui_clear_sorting(sch);
+	}
+}
+
+
 /**
  * @returns TRUE if search was sucessfully created and FALSE if an error
  * happened. If the "search" argument is not NULL a pointer to the new
  * search is stored there.
  */
 gboolean
-search_gui_new_search_full(const gchar *querystr,
+search_gui_new_search_full(const gchar *query_str,
 	time_t create_time, guint lifetime, guint32 reissue_timeout,
 	gint sort_col, gint sort_order, flag_t flags, search_t **search)
 {
 	static const search_t zero_sch;
-	const gchar *query, *error;
+	const gchar *error_str;
+	struct query *query;
 	search_t *sch;
-	GList *rules;
 	gnet_search_t sch_id;
 	GtkListStore *model;
 	GtkTreeIter iter;
 	gboolean is_only_search;
 	
-	query = search_gui_parse_query(querystr, &rules, &error);
+	query = search_gui_handle_query(query_str, flags, &error_str);
+	if (query || !error_str) {
+		gtk_entry_set_text(
+			GTK_ENTRY(gui_main_window_lookup("entry_search")), "");
+	}
 	if (!query) {
-		statusbar_gui_warning(5, "%s", error);
+		if (error_str) {
+			statusbar_gui_warning(5, "%s", error_str);
+		}
 		return FALSE;
 	}
-	sch_id = guc_search_new(query, create_time, lifetime, reissue_timeout,
-				flags);
+	g_assert(query);
+	g_assert(query->text);
+	
+	sch_id = guc_search_new(query->text, create_time, lifetime,
+				reissue_timeout, flags);
 	if ((gnet_search_t) -1 == sch_id) {
 		/*
 		 * An invalidly encoded SHA1 is already detected by
-		 * search_gui_parse_query(), so a too short query is the only reason
+		 * search_gui_query_parse(), so a too short query is the only reason
 		 * this may fail at the moment.
 		 */
 		statusbar_gui_warning(5, "%s",
@@ -461,7 +568,7 @@ search_gui_new_search_full(const gchar *querystr,
 		return FALSE;
 	}
 
-	sch = g_malloc(sizeof *sch);
+	sch = walloc(sizeof *sch);
 	*sch = zero_sch;
 
 	if (sort_col >= 0 && (guint) sort_col < SEARCH_RESULTS_VISIBLE_COLUMNS)
@@ -478,20 +585,18 @@ search_gui_new_search_full(const gchar *querystr,
 		sch->sort_order = SORT_NONE;
 	}
  
-	sch->query = atom_str_get(query);
+	sch->query = atom_str_get(query->text);
 	sch->enabled = (flags & SEARCH_F_ENABLED) ? TRUE : FALSE;
 	sch->browse = (flags & SEARCH_F_BROWSE) ? TRUE : FALSE;
+	sch->local = (flags & SEARCH_F_LOCAL) ? TRUE : FALSE;
 	sch->search_handle = sch_id;
 	sch->passive = (flags & SEARCH_F_PASSIVE) ? TRUE : FALSE;
-	sch->massive_update = FALSE;
 	sch->dups = g_hash_table_new_full(search_gui_hash_func,
 					search_gui_hash_key_compare, ht_unref_record, NULL);
 
 	sch->parents = g_hash_table_new(sha1_hash, sha1_eq);
 
-	search_gui_filter_new(sch, rules);
-	g_list_free(rules);
-	rules = NULL;
+	search_gui_filter_new(sch, query->rules);
 
 	/* Create a new TreeView if needed, or use the default TreeView */
 
@@ -499,9 +604,7 @@ search_gui_new_search_full(const gchar *querystr,
 		/* We have to create a new TreeView for this search */
 		gui_search_create_tree_view(&sch->scrolled_window,
 			&sch->tree_view, sch);
-
 		gtk_object_set_user_data(GTK_OBJECT(sch->scrolled_window), sch);
-
 		gtk_notebook_append_page(GTK_NOTEBOOK(notebook_search_results),
 			 sch->scrolled_window, NULL);
 	} else {
@@ -520,15 +623,7 @@ search_gui_new_search_full(const gchar *querystr,
 	}
 	sch->model = gtk_tree_view_get_model(GTK_TREE_VIEW(sch->tree_view));
 
-	if (
-		SORT_NONE != sch->sort_order &&
-		sch->sort_col >= 0 &&
-		(guint) sch->sort_col < SEARCH_RESULTS_VISIBLE_COLUMNS
-	) {
-		gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(sch->model),
-			sch->sort_col, SORT_ASC == sch->sort_order
-							? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
-	}
+	search_gui_restore_sorting(sch);
 
 	/* Add the search to the TreeView in pane on the left */
 	model = GTK_LIST_STORE(gtk_tree_view_get_model(tree_view_search));
@@ -553,10 +648,8 @@ search_gui_new_search_full(const gchar *querystr,
 				GTK_NOTEBOOK(notebook_search_results), 0), _("(no search)"));
     }
 	
-	gtk_widget_set_sensitive(lookup_widget(main_window, "button_search_close"),
+	gtk_widget_set_sensitive(gui_main_window_lookup("button_search_close"),
 		TRUE);
-	gtk_entry_set_text(GTK_ENTRY(lookup_widget(main_window, "entry_search")),
-		"");
 	
 	is_only_search = NULL == searches;
 	searches = g_list_append(searches, sch);
@@ -578,13 +671,19 @@ search_gui_new_search_full(const gchar *querystr,
 	 * set when the list of searches is not empty.
 	 */
 	
-	if (sch->browse && !is_only_search)
-		gui_search_force_update_tab_label(sch, tm_time());
-	else
+	if (is_only_search || (!sch->browse && search_jump_to_created)) {
 		search_gui_set_current_search(sch);
-
-	if (NULL != search)
+	} else {
+		gui_search_force_update_tab_label(sch, tm_time());
+	}
+	if (search) {
 		*search = sch;
+	}
+	search_gui_query_free(&query);
+	if (sch->local) {
+		search_gui_init_dnd(GTK_TREE_VIEW(sch->tree_view));
+	}
+
 	return TRUE;
 }
 
@@ -596,13 +695,14 @@ search_gui_cmp_size(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 	
 	(void) unused_udata;
 	
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-
-	return CMP(d1->record->size, d2->record->size);
+	ret = CMP(d1->record->size, d2->record->size);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 static gint
@@ -610,25 +710,24 @@ search_gui_cmp_count(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-
-	if (d1->count == d2->count)
-		return CMP(d1->record->size, d2->record->size);
-	else
-		return CMP(d1->count, d2->count);
+	ret = CMP(d1->count, d2->count);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 static inline gint
 search_gui_cmp_strings(const gchar *a, const gchar *b)
 {
-	if (a && b)
+	if (a && b) {
 		return a == b ? 0 : strcmp(a, b);
-	
-	return a ? 1 : (b ? -1 : 0);
+	} else {
+		return a ? 1 : (b ? -1 : 0);
+	}
 }
 
 static gint
@@ -636,12 +735,29 @@ search_gui_cmp_filename(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-	return search_gui_cmp_strings(d1->record->utf8_name, d2->record->utf8_name);
+	ret = search_gui_cmp_strings(d1->record->utf8_name, d2->record->utf8_name);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_sha1(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	ret = search_gui_cmp_sha1s(d1->record->sha1, d2->record->sha1);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 static gint
@@ -649,12 +765,14 @@ search_gui_cmp_charset(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-	return search_gui_cmp_strings(d1->record->charset, d2->record->charset);
+	ret = search_gui_cmp_strings(d1->record->charset, d2->record->charset);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 
@@ -663,12 +781,14 @@ search_gui_cmp_ext(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-	return search_gui_cmp_strings(d1->ext, d2->ext);
+	ret = search_gui_cmp_strings(d1->record->ext, d2->record->ext);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 static gint
@@ -676,12 +796,14 @@ search_gui_cmp_meta(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-	return search_gui_cmp_strings(d1->meta, d2->meta);
+	ret = search_gui_cmp_strings(d1->meta, d2->meta);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 
@@ -690,14 +812,16 @@ search_gui_cmp_country(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
 
-	return CMP(d1->record->results_set->country,
+	ret = CMP(d1->record->results_set->country,
 					d2->record->results_set->country);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 static gint
@@ -705,12 +829,151 @@ search_gui_cmp_info(
     GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
 {
 	const struct result_data *d1, *d2;
+	const gchar *s1, *s2;
+	gint ret;
 
 	(void) unused_udata;
 
 	d1 = get_result_data(model, a);
 	d2 = get_result_data(model, b);
-	return search_gui_cmp_strings(d1->info, d2->info);
+	s1 = d1->info ? d1->info : d1->record->info;
+	s2 = d2->info ? d2->info : d2->record->info;
+	ret = search_gui_cmp_strings(s1, s2);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_route(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	const struct results_set *rs1, *rs2;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	rs1 = d1->record->results_set;
+	rs2 = d2->record->results_set;
+	ret = host_addr_cmp(rs1->last_hop, rs2->last_hop);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_hops(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	const struct results_set *rs1, *rs2;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	rs1 = d1->record->results_set;
+	rs2 = d2->record->results_set;
+	ret = CMP(rs1->hops, rs2->hops);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_ttl(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	const struct results_set *rs1, *rs2;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	rs1 = d1->record->results_set;
+	rs2 = d2->record->results_set;
+	ret = CMP(rs1->ttl, rs2->ttl);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_protocol(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	const struct results_set *rs1, *rs2;
+	const guint32 mask = ST_UDP;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	rs1 = d1->record->results_set;
+	rs2 = d2->record->results_set;
+	ret = CMP(mask & rs1->status, mask & rs2->status);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_owned(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	const struct results_set *rs1, *rs2;
+	const guint32 mask = SR_OWNED | SR_SHARED;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	rs1 = d1->record->results_set;
+	rs2 = d2->record->results_set;
+	ret = CMP(mask & d1->record->flags, mask & d2->record->flags);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_hostile(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	const struct results_set *rs1, *rs2;
+	const guint32 mask = ST_HOSTILE;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	rs1 = d1->record->results_set;
+	rs2 = d2->record->results_set;
+	ret = CMP(mask & rs1->status, mask & rs2->status);
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
+}
+
+static gint
+search_gui_cmp_spam(
+    GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer unused_udata)
+{
+	const struct result_data *d1, *d2;
+	guint32 mask = SR_SPAM;
+	gint ret;
+
+	(void) unused_udata;
+
+	d1 = get_result_data(model, a);
+	d2 = get_result_data(model, b);
+	ret = CMP(mask & d1->record->flags, mask & d2->record->flags);
+	if (0 == ret) {
+		const struct results_set *rs1, *rs2;
+		rs1 = d1->record->results_set;
+		rs2 = d2->record->results_set;
+		mask = ST_SPAM;
+		ret = CMP(mask & rs1->status, mask & rs2->status);
+	}
+	return 0 != ret ? ret : CMP(d1->rank, d2->rank);
 }
 
 static gchar *
@@ -722,7 +985,7 @@ search_gui_get_info(const record_t *rc, const gchar *vinfo)
 	info[0] = '\0';
 
 	if (rc->tag) {
-		size_t len = strlen(rc->tag);
+		size_t size;
 
 		/*
 		 * We want to limit the length of the tag shown, but we don't
@@ -732,11 +995,12 @@ search_gui_get_info(const record_t *rc, const gchar *vinfo)
 		 *				--RAM, 09/09/2001
 		 */
 
-		len = MIN(len, MAX_TAG_SHOWN);
-		len = MIN(len, sizeof info);
+		size = 1 + strlen(rc->tag);
+		size = MIN(size, MAX_TAG_SHOWN + 1);
+		size = MIN(size, sizeof info);
 		rw = utf8_strlcpy(info,
 				lazy_unknown_to_utf8_normalized(rc->tag, UNI_NORM_GUI, NULL),
-				len);
+				size);
 	}
 	if (vinfo) {
 		g_assert(rw < sizeof info);
@@ -784,11 +1048,11 @@ search_gui_add_record(
 
 	data->record = rc;
 
-	data->ext = search_gui_get_filename_extension(rc->utf8_name);
-	if (data->ext)
-		data->ext = atom_str_get(data->ext);
-
-	data->info = search_gui_get_info(rc, vinfo->len ? vinfo->str : NULL);
+	if (rc->info) {
+		data->info = NULL;
+	} else {
+		data->info = search_gui_get_info(rc, vinfo->len ? vinfo->str : NULL);
+	}
 	data->fg = fg;
 	data->bg = bg;
 
@@ -821,7 +1085,7 @@ search_gui_add_record(
 	 * cache. If so lets update the GUI to reflect this.
 	 */
 	if (NULL != rc->sha1) {
-		bitzi_data_t *bd = guc_querycache_bitzi_by_urn(rc->sha1);
+		bitzi_data_t *bd = guc_query_cache_bitzi_by_sha1(rc->sha1);
 
 		if (bd)
 			search_gui_metadata_update(bd);
@@ -846,7 +1110,8 @@ search_gui_get_record_at_path(GtkTreeView *tv, GtkTreePath *path)
 	g_return_val_if_fail(NULL != sch, NULL);
 
 	model = GTK_TREE_MODEL(sch->model);
-	gtk_tree_model_get_iter(model, &iter, path);
+	if (!gtk_tree_model_get_iter(model, &iter, path))
+		return NULL;
 	data = get_result_data(model, &iter);
 
 	return data->record;
@@ -861,14 +1126,76 @@ search_gui_set_clear_button_sensitive(gboolean flag)
 /* ----------------------------------------- */
 
 
+gchar *
+search_gui_get_magnet(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	GtkTreeIter parent_iter;
+	struct result_data *parent;
+	struct magnet_resource *magnet;
+	gchar *url;
+
+	magnet = magnet_resource_new();
+
+	if (gtk_tree_model_iter_parent(model, &parent_iter, iter)) {
+		iter = &parent_iter;
+	}
+	parent = get_result_data(model, iter);
+
+	magnet_set_display_name(magnet, parent->record->utf8_name);
+
+	if (parent->record->sha1) {
+		GtkTreeIter child;
+
+		magnet_set_sha1(magnet, parent->record->sha1);
+		magnet_set_filesize(magnet, parent->record->size);
+		
+		if (0 == (ST_FIREWALL & parent->record->results_set->status)) {
+			magnet_add_sha1_source(magnet, parent->record->sha1,
+				parent->record->results_set->addr,
+				parent->record->results_set->port);
+		}
+
+		if (gtk_tree_model_iter_children(model, &child, &parent->iter)) {
+			do {	
+				struct result_data *data;
+
+				data = get_result_data(model, &child);
+				g_assert(data);
+
+				if (0 != (ST_FIREWALL & data->record->results_set->status))
+					continue;
+
+				magnet_add_sha1_source(magnet, data->record->sha1,
+					data->record->results_set->addr,
+					data->record->results_set->port);
+			} while (gtk_tree_model_iter_next(model, &child));
+		}
+
+		if (parent->record->alt_locs) {
+			const gnet_host_vec_t *alt = parent->record->alt_locs;
+			gint i, n;
+
+			n = MIN(10, alt->hvcnt);
+			for (i = 0; i < n; i++) {
+				magnet_add_sha1_source(magnet, parent->record->sha1,
+						parent->record->results_set->addr,
+						parent->record->results_set->port);
+			}
+		}
+	}
+
+	url = magnet_to_string(magnet);
+	magnet_resource_free(magnet);	
+	return url;
+}
+
 static void
 download_selected_file(GtkTreeModel *model, GtkTreeIter *iter, GSList **sl)
 {
 	struct result_data *rd;
 	struct results_set *rs;
 	struct record *rc;
-	guint32 flags;
-	gboolean need_push;
+	guint32 flags = 0;
 
 	g_assert(model != NULL);
 	g_assert(iter != NULL);
@@ -882,12 +1209,12 @@ download_selected_file(GtkTreeModel *model, GtkTreeIter *iter, GSList **sl)
 	g_assert(rc->refcount > 0);
 
 	rs = rc->results_set;
-	need_push = 0 != (rs->status & ST_FIREWALL);
-	flags = (rs->status & ST_TLS) ? CONNECT_F_TLS : 0;
+	flags |= (rs->status & ST_FIREWALL) ? SOCK_F_PUSH : 0;
+	flags |= (rs->status & ST_TLS) ? SOCK_F_TLS : 0;
 
-	guc_download_new(rc->name, rc->size, rc->index, rs->addr,
+	guc_download_new(rc->name, rc->size, rc->file_index, rs->addr,
 		rs->port, rs->guid, rs->hostname, rc->sha1, rs->stamp,
-		need_push, NULL, rs->proxies, flags);
+		NULL, rs->proxies, flags);
 
 	if (rs->proxies != NULL)
 		search_gui_free_proxies(rs);
@@ -927,15 +1254,10 @@ remove_selected_file(gpointer iter_ptr, gpointer model_ptr)
 		child_data = get_result_data(model, &child);
 		g_assert(child_data->record->refcount > 0);
 
-		if (rd->ext) {
-			atom_str_free(rd->ext);
-			rd->ext = NULL;
-		}
-		G_FREE_NULL(rd->meta);
+		atom_str_free_null(&rd->meta);
 		G_FREE_NULL(rd->info);
 		
 		rd->record = child_data->record;
-		rd->ext = atom_str_get(child_data->ext);
 		rd->meta = child_data->meta;
 		rd->info = child_data->info;
 		
@@ -1041,11 +1363,13 @@ static gboolean
 search_gui_menu_select_helper(
 	GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
-	gint page = -1;
 	struct menu_helper *mh = data;
+	gpointer value;
+	gint page;
 
 	(void) path;
-	gtk_tree_model_get(model, iter, 1, &page, (-1));
+	gtk_tree_model_get(model, iter, 1, &value, (-1));
+	page = GPOINTER_TO_INT(value);
 	if (page == mh->page) {
 		mh->iter = *iter;
 		return TRUE;
@@ -1063,11 +1387,9 @@ search_gui_menu_select(gint page)
 
 	mh.page = page;
 
-	treeview = GTK_TREE_VIEW(
-		lookup_widget(main_window, "treeview_menu"));
+	treeview = GTK_TREE_VIEW(gui_main_window_lookup("treeview_menu"));
 	model = GTK_TREE_MODEL(gtk_tree_view_get_model(treeview));
-	gtk_tree_model_foreach(
-		model, search_gui_menu_select_helper, &mh);
+	gtk_tree_model_foreach(model, search_gui_menu_select_helper, &mh);
 	selection = gtk_tree_view_get_selection(treeview);
 	gtk_tree_selection_select_iter(selection, &mh.iter);
 }
@@ -1222,6 +1544,105 @@ create_searches_model(void)
 	return GTK_TREE_MODEL(store);
 }
 
+static void
+drag_begin(GtkWidget *widget, GdkDragContext *unused_drag_ctx, gpointer udata)
+{
+	GtkTreeView *tv;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	gchar **url_ptr = udata;
+
+	(void) unused_drag_ctx;
+
+	g_signal_stop_emission_by_name(G_OBJECT(widget), "drag-begin");
+
+	g_assert(url_ptr != NULL);
+	G_FREE_NULL(*url_ptr);
+
+	tv = GTK_TREE_VIEW(widget);
+	gtk_tree_view_get_cursor(tv, &path, NULL);
+	if (!path)
+		return;
+
+	model = gtk_tree_view_get_model(tv);
+	if (gtk_tree_model_get_iter(model, &iter, path)) {
+		const struct result_data *data;
+		const gchar *pathname;
+		gchar *escaped;
+		
+		data = get_result_data(model, &iter);
+		pathname = data->record->path;
+		if (!pathname) {
+			pathname = "(null)";
+		}
+		escaped = url_escape(pathname);
+		*url_ptr = g_strconcat("file://", escaped, (void *) 0);
+		if (escaped != pathname) {
+			G_FREE_NULL(escaped);
+		}
+	}
+
+	gtk_tree_path_free(path);
+}
+
+static void
+drag_data_get(GtkWidget *widget, GdkDragContext *unused_drag_ctx,
+	GtkSelectionData *data, guint unused_info, guint unused_stamp,
+	gpointer udata)
+{
+	gchar **url_ptr = udata;
+
+	(void) unused_drag_ctx;
+	(void) unused_info;
+	(void) unused_stamp;
+
+	g_signal_stop_emission_by_name(G_OBJECT(widget), "drag-data-get");
+
+	g_assert(url_ptr != NULL);
+	if (NULL == *url_ptr)
+		return;
+
+	gtk_selection_data_set_text(data, *url_ptr, -1);
+	G_FREE_NULL(*url_ptr);
+}
+
+static void
+drag_end(GtkWidget *widget, GdkDragContext *unused_drag_ctx, gpointer udata)
+{
+	gchar **url_ptr = udata;
+
+	(void) unused_drag_ctx;
+
+	g_signal_stop_emission_by_name(G_OBJECT(widget), "drag-end");
+
+	g_assert(url_ptr != NULL);
+	G_FREE_NULL(*url_ptr);
+}
+
+static void
+search_gui_init_dnd(GtkTreeView *tv)
+{
+    static const GtkTargetEntry targets[] = {
+        { "STRING", 0, 23 },
+        { "text/plain", 0, 23 },
+    };
+	static gchar *dnd_url; /* Holds the URL to set the drag data */
+
+	g_return_if_fail(tv);
+
+	/* Initialize drag support */
+	gtk_drag_source_set(GTK_WIDGET(tv),
+		GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, targets, G_N_ELEMENTS(targets),
+		GDK_ACTION_DEFAULT | GDK_ACTION_COPY | GDK_ACTION_ASK);
+
+    g_signal_connect(G_OBJECT(tv), "drag-data-get",
+        G_CALLBACK(drag_data_get), &dnd_url);
+    g_signal_connect(G_OBJECT(tv), "drag-begin",
+        G_CALLBACK(drag_begin), &dnd_url);
+    g_signal_connect(G_OBJECT(tv), "drag-end",
+        G_CALLBACK(drag_end), &dnd_url);
+}
 
 /***
  *** Public functions
@@ -1233,23 +1654,25 @@ search_gui_init(void)
     GtkTreeView *tv;
 	search_t *current_search;
 
-    tree_view_search = GTK_TREE_VIEW(lookup_widget(main_window,
-							"tree_view_search"));
-    button_search_clear = GTK_BUTTON(lookup_widget(main_window,
-							"button_search_clear"));
-    notebook_search_results = GTK_NOTEBOOK(lookup_widget(main_window,
-								"notebook_search_results"));
+    tree_view_search =
+		GTK_TREE_VIEW(gui_main_window_lookup("tree_view_search"));
+    button_search_clear =
+		GTK_BUTTON(gui_main_window_lookup("button_search_clear"));
+    notebook_search_results =
+		GTK_NOTEBOOK(gui_main_window_lookup("notebook_search_results"));
 	gtk_notebook_popup_enable(notebook_search_results);
 
 	search_gui_common_init();
 	
+	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(tree_view_search),
+		GTK_SELECTION_MULTIPLE);
 	g_signal_connect(GTK_OBJECT(tree_view_search), "button_press_event",
-		G_CALLBACK(on_tree_view_search_button_press_event), NULL);
+		G_CALLBACK(on_search_list_button_press_event), NULL);
 
 	gtk_tree_view_set_model(tree_view_search, create_searches_model());
 	add_list_columns(tree_view_search);
 	g_signal_connect(G_OBJECT(tree_view_search), "cursor-changed",
-		G_CALLBACK(on_tree_view_search_select_row), NULL);
+		G_CALLBACK(on_tree_view_search_cursor_changed), NULL);
 
 	gui_search_create_tree_view(&default_scrolled_window,
 		&default_search_tree_view, NULL);
@@ -1287,7 +1710,7 @@ search_gui_shutdown(void)
 	search_gui_store_searches();
 
 	pos = gtk_paned_get_position(
-			GTK_PANED(lookup_widget(main_window, "vpaned_results")));
+			GTK_PANED(gui_main_window_lookup("vpaned_results")));
 
 	gui_prop_set_guint32(PROP_RESULTS_DIVIDER_POS, &pos, 0, 1);
 
@@ -1383,11 +1806,10 @@ search_gui_remove_search(search_t *sch)
 
     sensitive = searches != NULL;
 	gtk_widget_set_sensitive(
-        lookup_widget(main_window, "button_search_close"), sensitive);
+        gui_main_window_lookup("button_search_close"), sensitive);
 
 	{
-		search_t *current_search;
-		current_search = search_gui_get_current_search();
+		search_t *current_search = search_gui_get_current_search();
 
 		if (current_search != NULL)
 			sensitive = sensitive &&
@@ -1395,7 +1817,7 @@ search_gui_remove_search(search_t *sch)
 	}
 
     gtk_widget_set_sensitive(
-		lookup_widget(popup_search, "popup_search_download"), sensitive);
+		gui_popup_search_lookup("popup_search_download"), sensitive);
 }
 
 void
@@ -1442,16 +1864,20 @@ search_gui_set_current_search(search_t *sch)
 		tree_view_save_widths(tv_old, PROP_SEARCH_RESULTS_COL_WIDTHS);
 		tree_view_save_visibility(tv_old, PROP_SEARCH_RESULTS_COL_VISIBLE);
 		tree_view_restore_visibility(tv_new, PROP_SEARCH_RESULTS_COL_VISIBLE);
+		search_gui_clear_sorting(current_search);
     } else if (default_search_tree_view) {
 		tree_view_save_widths(GTK_TREE_VIEW(default_search_tree_view),
 			PROP_SEARCH_RESULTS_COL_WIDTHS);
 	}
 
 
-	search_gui_current_search(sch);
+	/* This prevents side-effects otherwise caused by  changing the value of
+	 * spinbutton_reissue_timeout.
+	 */
+	search_gui_forget_current_search();
 	sch->unseen_items = 0;
 
-    spinbutton_reissue_timeout = lookup_widget(main_window,
+    spinbutton_reissue_timeout = gui_main_window_lookup(
 									"spinbutton_search_reissue_timeout");
 
     if (sch != NULL) {
@@ -1461,24 +1887,21 @@ search_gui_set_current_search(search_t *sch)
         gui_search_force_update_tab_label(sch, tm_time());
         search_gui_update_items(sch);
 
-        gtk_spin_button_set_value
-            (GTK_SPIN_BUTTON(spinbutton_reissue_timeout), reissue_timeout);
+        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton_reissue_timeout),
+			reissue_timeout);
         gtk_widget_set_sensitive(spinbutton_reissue_timeout, active);
         gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_download"),
+            gui_popup_search_lookup("popup_search_download"),
 			selection_counter(GTK_TREE_VIEW(sch->tree_view)) > 0);
         gtk_widget_set_sensitive(GTK_WIDGET(button_search_clear),
             sch->items != 0);
         gtk_widget_set_sensitive(
-            lookup_widget(popup_search_list, "popup_search_restart"),
+            gui_popup_search_lookup("popup_search_restart"),
 		   	active);
         gtk_widget_set_sensitive(
-            lookup_widget(popup_search_list, "popup_search_duplicate"),
-			active);
+            gui_popup_search_lookup("popup_search_stop"), !frozen);
         gtk_widget_set_sensitive(
-            lookup_widget(popup_search_list, "popup_search_stop"), !frozen);
-        gtk_widget_set_sensitive(
-            lookup_widget(popup_search_list, "popup_search_resume"), frozen);
+            gui_popup_search_lookup("popup_search_resume"), frozen);
 
 		model = gtk_tree_view_get_model(tree_view_search);
 		if (tree_find_iter_by_data(model, c_sl_sch, sch, &iter)) {
@@ -1488,10 +1911,10 @@ search_gui_set_current_search(search_t *sch)
 			gtk_tree_view_set_cursor(tree_view_search, path, NULL, FALSE);
 			gtk_tree_path_free(path);
 		}
+		search_gui_restore_sorting(sch);
     } else {
 		static const gchar * const popup_items[] = {
 			"popup_search_restart",
-			"popup_search_duplicate",
 			"popup_search_stop",
 			"popup_search_resume",
 		};
@@ -1501,13 +1924,14 @@ search_gui_set_current_search(search_t *sch)
 			gtk_tree_view_get_selection(tree_view_search));
         gtk_widget_set_sensitive(spinbutton_reissue_timeout, FALSE);
        	gtk_widget_set_sensitive(
-            lookup_widget(popup_search, "popup_search_download"), FALSE);
+            gui_popup_search_lookup("popup_search_download"), FALSE);
 
 		for (i = 0; i < G_N_ELEMENTS(popup_items); i++) {
        		gtk_widget_set_sensitive(
-            	lookup_widget(popup_search_list, popup_items[i]), FALSE);
+            	gui_popup_search_lookup(popup_items[i]), FALSE);
 		}
     }
+	search_gui_current_search(sch);
 
 	tree_view_restore_widths(GTK_TREE_VIEW(sch->tree_view),
 		PROP_SEARCH_RESULTS_COL_WIDTHS);
@@ -1554,7 +1978,15 @@ add_results_columns(GtkTreeView *treeview, gpointer udata)
 		{ N_("#"),		   c_sr_count,	  1.0, search_gui_cmp_count },
 		{ N_("Loc"),	   c_sr_loc,	  0.0, search_gui_cmp_country },
 		{ N_("Metadata"),  c_sr_meta,	  0.0, search_gui_cmp_meta },
-		{ N_("Info"),	   c_sr_info,	  0.0, search_gui_cmp_info }
+		{ N_("Info"),	   c_sr_info,	  0.0, search_gui_cmp_info },
+		{ N_("Route"),	   c_sr_route,	  0.0, search_gui_cmp_route },
+		{ N_("Protocol"),  c_sr_protocol, 0.0, search_gui_cmp_protocol },
+		{ N_("Hops"),  	   c_sr_hops,	  0.0, search_gui_cmp_hops },
+		{ N_("TTL"),  	   c_sr_ttl,	  0.0, search_gui_cmp_ttl },
+		{ N_("Owned"),     c_sr_owned,	  0.0, search_gui_cmp_owned },
+		{ N_("Spam"),      c_sr_spam,	  0.0, search_gui_cmp_spam },
+		{ N_("Hostile"),   c_sr_hostile,  0.0, search_gui_cmp_hostile },
+		{ N_("SHA-1"),     c_sr_sha1,     0.0, search_gui_cmp_sha1 },
 	};
 	guint32 width[G_N_ELEMENTS(columns)];
 	guint i;
@@ -1664,17 +2096,19 @@ gui_search_force_update_tab_label(search_t *sch, time_t now)
 
     search = search_gui_get_current_search();
 
-	if (sch == search || sch->unseen_items == 0)
+	if (sch == search || sch->unseen_items == 0) {
 		gm_snprintf(buf, sizeof buf, "%s\n(%d)", sch->query, sch->items);
-	else
+	} else {
 		gm_snprintf(buf, sizeof buf, "%s\n(%d, %d)", sch->query,
 		   sch->items, sch->unseen_items);
+	}
 	sch->last_update_items = sch->items;
 	gtk_notebook_set_tab_label_text(notebook_search_results,
 		sch->scrolled_window, buf);
 	model = gtk_tree_view_get_model(tree_view_search);
 	gtk_tree_model_foreach(model, tree_view_search_update, sch);
 	sch->last_update_time = now;
+
 }
 
 /**
@@ -1707,24 +2141,6 @@ gui_search_clear_results(void)
 	search_gui_update_items(search);
 }
 
-/**
- * Extract the mark/ignore/download color.
- */
-void
-gui_search_get_colors(
-	search_t *sch,
-	GdkColor **mark_color, GdkColor **ignore_color, GdkColor **download_color)
-{
-    *mark_color = &(gtk_widget_get_style(GTK_WIDGET(sch->tree_view))
-        ->bg[GTK_STATE_INSENSITIVE]);
-
-    *ignore_color = &(gtk_widget_get_style(GTK_WIDGET(sch->tree_view))
-        ->fg[GTK_STATE_INSENSITIVE]);
-
-    *download_color =  &(gtk_widget_get_style(GTK_WIDGET(sch->tree_view))
-        ->fg[GTK_STATE_ACTIVE]);
-}
-
 /**
  * Flag whether search is enabled.
  */
@@ -1774,12 +2190,7 @@ search_gui_start_massive_update(search_t *sch)
 {
 	g_assert(sch);
 
-   	if (sch == search_gui_get_current_search() || sch->massive_update)
-		return;
-
-	g_object_ref(sch->model);
-	gtk_tree_view_set_model(GTK_TREE_VIEW(sch->tree_view), NULL);
-	sch->massive_update = TRUE;
+	g_object_freeze_notify(G_OBJECT(sch->tree_view));
 }
 
 void
@@ -1787,14 +2198,8 @@ search_gui_end_massive_update(search_t *sch)
 {
 	g_assert(sch);
 
-	if (!sch->massive_update)
-		return;
-
     gui_search_force_update_tab_label(sch, tm_time());
-	gtk_tree_view_set_model(GTK_TREE_VIEW(sch->tree_view),
-		GTK_TREE_MODEL(sch->model));
-	g_object_unref(GTK_TREE_MODEL(sch->model));
-	sch->massive_update = FALSE;
+	g_object_thaw_notify(G_OBJECT(sch->tree_view));
 }
 
 
@@ -1846,10 +2251,10 @@ search_gui_request_bitzi_data(void)
 			g_assert(rd);
 			
 			/* set the feedback */
-			rd->meta = g_strdup(_("Query queued..."));
+			rd->meta = atom_str_get(_("Query queued..."));
 
 			/* then send the query... */
-	    	guc_query_bitzi_by_urn(rec->sha1);
+	    	guc_query_bitzi_by_sha1(rec->sha1);
 		}
     }
 
@@ -1875,9 +2280,9 @@ search_gui_metadata_update(const bitzi_data_t *data)
 		struct result_data *rd;
 		search_t *search = iter->data;
 
-	   	rd = find_parent_with_sha1(search->parents, data->urnsha1);
+	   	rd = find_parent_with_sha1(search->parents, data->sha1);
 		if (rd) {
-			rd->meta = text ? text : g_strdup(_("Not in database"));
+			rd->meta = atom_str_get(text ? text : _("Not in database"));
 			text = NULL;
 			
 			/* Re-store the parent to refresh the display/sorting */
@@ -1934,5 +2339,81 @@ gui_search_create_tree_view(GtkWidget ** sw, GtkWidget ** tv, gpointer udata)
 		G_CALLBACK(on_leave_notify), NULL);
 	g_object_freeze_notify(G_OBJECT(treeview));
 }
+
+static void
+search_gui_get_selected_searches_helper(GtkTreeModel *model,
+	GtkTreePath *unused_path, GtkTreeIter *iter, gpointer data)
+{
+	gpointer p = NULL;
+
+	(void) unused_path;
+	g_assert(data);
+
+	gtk_tree_model_get(model, iter, c_sl_sch, &p, (-1));
+	if (p) {
+		GSList **sl_ptr = data;
+		*sl_ptr = g_slist_prepend(*sl_ptr, p);
+	}
+}
+
+GSList *
+search_gui_get_selected_searches(void)
+{
+	GSList *sl = NULL;
+	GtkTreeView *tv;
+
+    tv = GTK_TREE_VIEW(gui_main_window_lookup("tree_view_search"));
+	gtk_tree_selection_selected_foreach(gtk_tree_view_get_selection(tv),
+		search_gui_get_selected_searches_helper, &sl);
+	return sl;
+}
+
+gboolean
+search_gui_has_selected_item(search_t *search)
+{
+	GtkTreePath *path = NULL;
+	gboolean ret = FALSE;
+
+	g_return_val_if_fail(search, FALSE);
+
+	gtk_tree_view_get_cursor(GTK_TREE_VIEW(search->tree_view), &path, NULL);
+	if (path) {
+		ret = TRUE;
+		gtk_tree_path_free(path);
+	}
+	return ret;
+}
+
+void
+search_gui_search_list_clicked(GtkWidget *widget, GdkEventButton *event)
+{
+	GtkTreeView *tv = GTK_TREE_VIEW(widget);
+	GtkTreePath *path = NULL;
+	GtkTreeViewColumn *column = NULL;
+
+	if (
+		gtk_tree_view_get_path_at_pos(tv, event->x, event->y,
+			&path, &column, NULL, NULL)
+	) {
+		GtkTreeModel *model;
+		GtkTreeIter iter;
+
+		model = gtk_tree_view_get_model(tv);
+		if (gtk_tree_model_get_iter(model, &iter, path)) {
+			gpointer p = NULL; 
+			gtk_tree_model_get(model, &iter, c_sl_sch, &p, (-1));
+			if (p) {
+				search_t *search = p;
+				search_gui_set_current_search(search);
+#if 0
+				gtk_tree_view_set_cursor(tv, path, column, FALSE);
+				gtk_widget_grab_focus(GTK_WIDGET(tv));
+#endif
+			}
+		}
+		gtk_tree_path_free(path);
+	}
+}
+
 /* -*- mode: cc-mode; tab-width:4; -*- */
 /* vi: set ts=4 sw=4 cindent: */
