@@ -1,5 +1,5 @@
 /*
- * $Id: search_stats.c,v 1.21 2005/12/17 09:25:45 daichik Exp $
+ * $Id: search_stats.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -46,7 +46,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: search_stats.c,v 1.21 2005/12/17 09:25:45 daichik Exp $");
+RCSID("$Id: search_stats.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "gtk/search_stats.h"
 #include "gtk/gtk-missing.h"
@@ -224,8 +224,9 @@ stats_hash_to_treeview(gpointer key, gpointer value, gpointer unused_udata)
 		2, (gulong) val->total_cnt,
 		(-1));
 
-	if (key != s)
+	if (key != s) {
 		G_FREE_NULL(s);
+	}
 
 	/* new period begins */
 	val->period_cnt = 0;
@@ -389,9 +390,9 @@ search_stats_gui_init(void)
 	STATIC_ASSERT(G_N_ELEMENTS(cols) == G_N_ELEMENTS(types));
 
 	treeview_search_stats =
-        GTK_TREE_VIEW(lookup_widget(main_window, "treeview_search_stats"));
+        GTK_TREE_VIEW(gui_main_window_lookup("treeview_search_stats"));
 	label_search_stats_count =
-		GTK_LABEL(lookup_widget(main_window, "label_search_stats_count"));
+		GTK_LABEL(gui_main_window_lookup("label_search_stats_count"));
 
 	treeview = treeview_search_stats;
 
@@ -426,8 +427,9 @@ void
 search_stats_gui_update(time_t now)
 {
 	static time_t last_update = 0;
+	const time_delta_t interval = search_stats_update_interval;
 
-    if (delta_time(now, last_update) < search_stats_update_interval)
+    if (delta_time(now, last_update) < interval)
         return;
 
     last_update = now;
