@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.c,v 1.13 2005/11/16 13:35:14 cbiere Exp $
+ * $Id: upload_stats.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -56,13 +56,14 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: upload_stats.c,v 1.13 2005/11/16 13:35:14 cbiere Exp $");
+RCSID("$Id: upload_stats.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "interface-glade.h" /* for create_popup_upload_stats */
 
 #include "gtk/columns.h"
 #include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
+#include "gtk/settings.h"
 
 #include "if/gui_property.h"
 #include "if/bridge/ui2c.h"
@@ -120,7 +121,8 @@ cell_render_size_func(GtkTreeViewColumn *column, GtkCellRenderer *cell,
 	g_assert(iter != NULL);
 
 	gtk_tree_model_get(model, iter, c_us_size, &val, (-1));
-	g_object_set(cell, "text", short_size(val), (void *) 0);
+	g_object_set(cell, "text",
+		short_size(val, show_metric_units()), (void *) 0);
 }
 
 /**
@@ -288,7 +290,7 @@ upload_stats_gui_init_intern(gboolean intern)
 			G_TYPE_FLOAT,		/* Normalized */
 			G_TYPE_POINTER)); 	/* struct ul_stats */
 		upload_stats_treeview = GTK_TREE_VIEW(
-			lookup_widget(main_window, "treeview_ul_stats"));
+			gui_main_window_lookup("treeview_ul_stats"));
 		gtk_tree_view_set_model(upload_stats_treeview, model);
 		g_object_unref(model);
 
