@@ -1,5 +1,5 @@
 /*
- * $Id: icon.c,v 1.8 2005/11/16 21:08:47 cbiere Exp $
+ * $Id: icon.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2003, Michael Gray
  *
@@ -35,7 +35,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: icon.c,v 1.8 2005/11/16 21:08:47 cbiere Exp $");
+RCSID("$Id: icon.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "icon.h"
 
@@ -337,6 +337,21 @@ icon_timer(void)
     gdk_window_invalidate_rect(canvas->window, &rect, FALSE);
 }
 
+#if GTK_CHECK_VERSION(2, 10, 0)
+static void
+on_status_icon_activate(GtkStatusIcon *status_icon, gpointer unused_udata)
+{
+	(void) status_icon;
+	(void) unused_udata;
+
+	if (GTK_WIDGET_VISIBLE(gui_main_window())) {
+		gtk_widget_hide(gui_main_window());
+	} else {
+		gtk_widget_show(gui_main_window());
+	}
+}
+#endif	/* Gtk+ >= 2.10.0 */
+
 /**
  * For details of what is expected from an icon window and what it
  * should expect.
@@ -363,7 +378,7 @@ icon_init(void)
     gtk_widget_map(canvas);
 	/* FIXME: This causes a crash with twm when iconizing the main window. */
 #if 0
-    gdk_window_set_icon(main_window->window, icon->window, NULL, NULL);
+    gdk_window_set_icon(gui_main_window()->window, icon->window, NULL, NULL);
 #endif
     icon_just_mapped_fg = icon_visible_fg = icon_close_fg = FALSE;
 
@@ -371,6 +386,24 @@ icon_init(void)
     con_pixbuf = create_pixbuf("smallserver.xpm");
     up_pixbuf = create_pixbuf("upload.xpm");
     down_pixbuf = create_pixbuf("download.xpm");
+
+#if GTK_CHECK_VERSION(2, 10, 0)
+	{
+		GtkStatusIcon *status_icon;
+		GdkPixbuf *icon_pixbuf;
+		
+		/*
+		 * Create an status so that Gtk-Gnutella can be minimized to a
+		 * so-called "system tray" if supported by the window manager.
+		 */
+
+    	icon_pixbuf = create_pixbuf("icon.xpm");
+		status_icon = gtk_status_icon_new_from_pixbuf(icon_pixbuf);
+		gtk_status_icon_set_visible(status_icon, TRUE);
+    	g_signal_connect(G_OBJECT(status_icon), "activate",
+			G_CALLBACK(on_status_icon_activate), NULL);
+	}
+#endif	/* Gtk+ >= 2.10.0 */
 }
 
 void
@@ -404,9 +437,9 @@ void
 icon_init(void)
 {
     GtkPixmap *pixmap;
-    pixmap = (GtkPixmap *) create_pixmap(main_window, "icon.xpm");
+    pixmap = (GtkPixmap *) create_pixmap(gui_main_window(), "icon.xpm");
     gtk_pixmap_get(pixmap, &icon_map, &icon_mask);
-    gdk_window_set_icon(main_window->window, NULL, icon_map, icon_mask);
+    gdk_window_set_icon(gui_main_window()->window, NULL, icon_map, icon_mask);
 }
 
 void
