@@ -1,5 +1,5 @@
 /*
- * $Id: main.c,v 1.9 2004/11/21 21:12:42 wyldfire Exp $
+ * $Id: main.c 12268 2006-11-09 00:55:57Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -23,11 +23,24 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Main functions for GTK+ interface.
+ *
+ * @author Raphael Manfredi
+ * @author Richard Eckart
+ * @date 2001-2003
+ */
+
 #include "gui.h"
 
-RCSID("$Id: main.c,v 1.9 2004/11/21 21:12:42 wyldfire Exp $");
+RCSID("$Id: main.c 12268 2006-11-09 00:55:57Z cbiere $")
 
+#ifdef I_PWD
 #include <pwd.h>
+#endif /* I_PWD */
 
 #ifdef USE_GTK1
 #include "gtk1/interface-glade.h"
@@ -60,128 +73,181 @@ RCSID("$Id: main.c,v 1.9 2004/11/21 21:12:42 wyldfire Exp $");
 
 #include "gtk-missing.h"
 
+#include "gtk/misc.h"
+
 #include "if/bridge/ui2c.h"
 
+#include "lib/file.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
 #include "lib/utf8.h"
 #include "lib/override.h"			/* Must be the last header included */
 
-static gchar tmpstr[1024];
-
 /***
  *** Windows
  ***/
-GtkWidget *main_window = NULL;
-GtkWidget *shutdown_window = NULL;
-GtkWidget *dlg_about = NULL;
-GtkWidget *dlg_prefs = NULL;
-GtkWidget *dlg_quit = NULL;
-GtkWidget *popup_downloads = NULL;
-GtkWidget *popup_uploads = NULL;
-GtkWidget *popup_search = NULL;
-GtkWidget *popup_nodes = NULL;
-GtkWidget *popup_monitor = NULL;
-GtkWidget *popup_queue = NULL;
 
+#define WIDGET(name) \
+static GtkWidget * name ## _protected_ ; \
+ \
+GtkWidget *gui_ ## name (void) \
+{ \
+	return name ## _protected_ ; \
+} \
+ \
+static inline void \
+gui_ ## name ## _set (GtkWidget *w) \
+{ \
+	name ## _protected_ = w; \
+} \
+ \
+GtkWidget * \
+gui_ ## name ## _lookup(const gchar *id) \
+{ \
+	return lookup_widget(name ## _protected_ , id); \
+}
+
+WIDGET(dlg_about)
+WIDGET(dlg_faq)
+WIDGET(dlg_prefs)
+WIDGET(dlg_quit)
+WIDGET(main_window)
+WIDGET(popup_downloads)
+WIDGET(popup_monitor)
+WIDGET(popup_nodes)
+WIDGET(popup_queue)
+WIDGET(popup_search)
+WIDGET(popup_search_list)
+WIDGET(popup_uploads)
+WIDGET(shutdown_window)
+#undef WIDGET
 
 /***
  *** Private functions
  ***/
 
-static void gui_init_window_title(void)
+static void
+gui_init_window_title(void)
 {
+	gchar title[256];
+
 #ifdef GTA_REVISION
-	gm_snprintf(tmpstr, sizeof(tmpstr), "gtk-gnutella %s %s",
-		GTA_VERSION_NUMBER,
-		GTA_REVISION);
+	gm_snprintf(title, sizeof(title), "gtk-gnutella %s %s",
+		GTA_VERSION_NUMBER, GTA_REVISION);
 #else
-	gm_snprintf(tmpstr, sizeof(tmpstr), "gtk-gnutella %s",
-		GTA_VERSION_NUMBER);
+	gm_snprintf(title, sizeof(title), "gtk-gnutella %s", GTA_VERSION_NUMBER);
 #endif
 
-	gtk_window_set_title(GTK_WINDOW(main_window), tmpstr);
+	gtk_window_set_title(GTK_WINDOW(gui_main_window()), title);
 }
 
-/*
- * The contents of the navigation tree menu in exact order
+/**
+ * The contents of the navigation tree menu in exact order.
  */
 static const struct {
-	gboolean	parent;	/* Children have the last "TRUE" node as parent */
-	const gchar *title; /* Translatable title for the node */
-	gint		page;	/* Page reference ("the target") for the node */
+	const gint	depth;	/**< Depth in tree */
+	const gchar *title; /**< Translatable title for the node */
+	const gint	page;	/**< Page reference ("the target") for the node */
 } menu[] = {
-	{ TRUE,	 N_("GnutellaNet"),		nb_main_page_gnet },
-	{ FALSE, N_("Stats"),			nb_main_page_gnet_stats },
-	{ FALSE, N_("Hostcache"),		nb_main_page_hostcache },
-	{ TRUE,	 N_("Uploads"),			nb_main_page_uploads },
-	{ FALSE, N_("History"), 		nb_main_page_uploads_stats },
-	{ TRUE,	 N_("Downloads"),		nb_main_page_downloads },
-	{ TRUE,	 N_("Search"),			nb_main_page_search },
-	{ FALSE, N_("Monitor"),			nb_main_page_monitor },
-	{ FALSE, N_("Stats"),			nb_main_page_search_stats },
+	{   0,	 N_("GnutellaNet"),		nb_main_page_gnet },
+	{     1, N_("Stats"),			nb_main_page_gnet_stats },
+	{     1, N_("Hostcache"),		nb_main_page_hostcache },
+	{   0,	 N_("Uploads"),			nb_main_page_uploads },
+	{     1, N_("History"), 		nb_main_page_uploads_stats },
+#ifdef USE_GTK1
+	{   0,	 N_("Downloads"),		nb_main_page_dl_files },
+	{     1, N_("Active"),			nb_main_page_dl_active },
+	{     1, N_("Queue"),			nb_main_page_dl_queue },
+#endif /* USE_GTK1 */
+#ifdef USE_GTK2
+	{   0,	 N_("Downloads"),		nb_main_page_downloads },
+#endif /* USE_GTK1 */
+	{   0,	 N_("Search"),			nb_main_page_search },
+	{     1, N_("Monitor"),			nb_main_page_monitor },
+	{     1, N_("Stats"),			nb_main_page_search_stats },
 };
 
 
 #ifdef USE_GTK2
 
-static gboolean gui_init_menu_helper(
-	GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
+static gboolean
+gui_init_menu_helper(GtkTreeModel *model, GtkTreePath *path,
+	GtkTreeIter *iter, gpointer data)
 {
+	static const GValue zero_value;
+	GValue value;
 	guint32 expanded;
 	gint id;
+
+	value = zero_value;
+	gtk_tree_model_get_value(model, iter, 1, &value);
+	id = GPOINTER_TO_UINT(g_value_get_pointer(&value));
 	
-	gtk_tree_model_get(model, iter, 2, &id, (-1));
 	gui_prop_get_guint32(PROP_TREEMENU_NODES_EXPANDED, &expanded, id, 1);
 	if (expanded)
 		gtk_tree_view_expand_row(GTK_TREE_VIEW(data), path, FALSE);
 	return FALSE;
 }
 
-static void gui_init_menu(void) 
+static void
+gui_init_menu(void)
 {
 	static GType types[] = {
 		G_TYPE_STRING,	/* Label */
-		G_TYPE_INT,		/* Notebook page */
-		G_TYPE_INT		/* Menu entry ID (persistent between releases) */
+		G_TYPE_POINTER,	/* Notebook page number (casted to a pointer) */
 	};
 	GtkTreeView	*treeview;
-	GtkTreeIter	parent;
+	GtkTreeIter	parent, iter;
 	GtkTreeStore *store;
 	GtkTreeViewColumn *column;
     GtkCellRenderer *renderer;
 	guint i;
+	gint depth = -1;
+
+	STATIC_ASSERT(G_N_ELEMENTS(types) == 2);
 
     renderer = gtk_cell_renderer_text_new();
-    g_object_set(renderer, "ypad", GUI_CELL_RENDERER_YPAD, NULL);
-	treeview = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_menu"));
+    g_object_set(renderer, "ypad", GUI_CELL_RENDERER_YPAD, (void *) 0);
+	treeview = GTK_TREE_VIEW(gui_main_window_lookup("treeview_menu"));
 	store = gtk_tree_store_newv(G_N_ELEMENTS(types), types);
 
 	for (i = 0; i < G_N_ELEMENTS(menu); i++) {
-		GtkTreeIter	iter;
-
-		gtk_tree_store_append(store, &iter, menu[i].parent ? NULL : &parent);
-		if (menu[i].parent)
+		if (depth < menu[i].depth) {
+			g_assert(menu[i].depth == depth + 1);
+	
 			parent = iter;
-
+			depth = menu[i].depth;
+		} else if (depth > menu[i].depth) {
+			do {
+				gboolean valid;
+				
+				valid = gtk_tree_model_iter_parent(GTK_TREE_MODEL(store),
+							&parent, &iter);	
+				g_assert(valid);
+				iter = parent;
+			} while (--depth > menu[i].depth);
+			depth = menu[i].depth;
+		}
+		
+		gtk_tree_store_append(store, &iter, depth > 0 ? &parent : NULL);
 		gtk_tree_store_set(store, &iter,
 				0, _(menu[i].title),
-				1, menu[i].page,
-				2, i,
+				1, GUINT_TO_POINTER(menu[i].page),
 				(-1));
-		STATIC_ASSERT(G_N_ELEMENTS(types) == 3);
 	}
 
 	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
 
 	column = gtk_tree_view_column_new_with_attributes(
-		NULL, renderer, "text", 0, NULL);
+				NULL, renderer, "text", 0, (void *) 0);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_append_column(treeview, column);
 	gtk_tree_view_columns_autosize(treeview);
-	
+
 	gtk_tree_model_foreach(GTK_TREE_MODEL(store),
-		(GtkTreeModelForeachFunc) gui_init_menu_helper, treeview);
+		gui_init_menu_helper, treeview);
+
 	g_object_unref(store);
 
 	g_signal_connect(G_OBJECT(treeview), "cursor-changed",
@@ -192,26 +258,77 @@ static void gui_init_menu(void)
 		G_CALLBACK(on_main_gui_treeview_menu_row_expanded), NULL);
 }
 
-/*
- * gui_create_main_window:
- *
+static void
+gui_menu_shutdown(void)
+{
+	g_signal_handlers_disconnect_by_func(
+		G_OBJECT(gui_main_window_lookup("treeview_menu")),
+		on_main_gui_treeview_menu_cursor_changed,
+		NULL);
+	g_signal_handlers_disconnect_by_func(
+		G_OBJECT(gui_main_window_lookup("treeview_menu")),
+		on_main_gui_treeview_menu_row_collapsed,
+		NULL);
+	g_signal_handlers_disconnect_by_func(
+		G_OBJECT(gui_main_window_lookup("treeview_menu")),
+		on_main_gui_treeview_menu_row_expanded,
+		NULL);
+}
+
+#else
+
+static void
+gui_init_menu(void)
+{
+    GtkCTree *ctree_menu = GTK_CTREE(gui_main_window_lookup("ctree_menu"));
+	GtkCTreeNode *parent_node = NULL;
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS(menu); i++) {
+		GtkCTreeNode *node;
+		const gchar *title[1];
+
+		title[0] = _(menu[i].title);
+    	node = gtk_ctree_insert_node(ctree_menu,
+					menu[i].depth == 0 ? NULL : parent_node, NULL,
+					(gchar **) title, /* Override const */
+					0, NULL, NULL, NULL, NULL, FALSE, TRUE);
+		if (i == 0 || menu[i].depth < menu[i - 1].depth)
+			parent_node = node;
+
+    	gtk_ctree_node_set_row_data(ctree_menu, node,
+			GINT_TO_POINTER(menu[i].page));
+	}
+
+	gtk_clist_select_row(GTK_CLIST(ctree_menu), 0, 0);
+}
+
+static void
+gui_menu_shutdown(void)
+{
+	/* NOTHING */
+}
+	
+#endif /* USE_GTK2 */
+
+/**
  * Handles main window UI joining.
+ *
  * Creates all dependent "tab" windows and merges them into
  * the main notebook.
- *
  */
-static GtkWidget *gui_create_main_window(void)
+static void
+gui_init_main_window(void)
 {
-	GtkWidget *window;
+#ifdef USE_GTK2
 	GtkWidget *notebook;
 	GtkWidget *tab_window[nb_main_page_num];
-	gint i ;
+	gint i;
 
 	/*
 	 * First create the main window without the tab contents.
 	 */
-	window = create_main_window();
-	notebook = lookup_widget(window, "notebook_main");
+	notebook = gui_main_window_lookup("notebook_main");
 
 	/*
 	 * Then create all the tabs in their own window.
@@ -220,7 +337,9 @@ static GtkWidget *gui_create_main_window(void)
 	tab_window[nb_main_page_uploads] = create_main_window_uploads_tab();
 	tab_window[nb_main_page_uploads_stats] =
 		create_main_window_upload_stats_tab();
+
 	tab_window[nb_main_page_downloads] = create_main_window_downloads_tab();
+
 	tab_window[nb_main_page_search] = create_main_window_search_tab();
 	tab_window[nb_main_page_monitor] = create_main_window_monitor_tab();
 	tab_window[nb_main_page_search_stats] =
@@ -233,7 +352,7 @@ static GtkWidget *gui_create_main_window(void)
 	 */
 	for (i = 0; i < nb_main_page_num; i++) {
 		GtkWidget *w = tab_window[i];
-		gui_merge_window_as_tab(window, notebook, w);
+		gui_merge_window_as_tab(gui_main_window(), notebook, w);
 		gtk_object_destroy(GTK_OBJECT(w));
 	}
 
@@ -245,53 +364,18 @@ static GtkWidget *gui_create_main_window(void)
 	gtk_container_remove(GTK_CONTAINER(notebook),
 		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 0));
 
-	return window;
+#endif	/* USE_GTK2 */
 }
 
-#else
-
-#define gui_create_main_window() create_main_window()
-
-static void gui_init_menu(void) 
+static void 
+gui_init_dlg_prefs(void)
 {
-    GtkCTree *ctree_menu = GTK_CTREE(lookup_widget(main_window, "ctree_menu"));
-	GtkCTreeNode *parent_node = NULL;    
-	guint i;
-
-	for (i = 0; i < G_N_ELEMENTS(menu); i++) {
-		GtkCTreeNode *node;
-		const gchar *title[] = { _(menu[i].title) };
-
-    	node = gtk_ctree_insert_node(ctree_menu,
-					menu[i].parent ? NULL : parent_node, NULL,
-					(gchar **) title, /* Override const */
-					0, NULL, NULL, NULL, NULL, FALSE, TRUE);
-		if (menu[i].parent)
-			parent_node = node;
-		
-    	gtk_ctree_node_set_row_data(ctree_menu, node,
-			GINT_TO_POINTER(menu[i].page));
-	}
-
-	gtk_clist_select_row(GTK_CLIST(ctree_menu), 0, 0);
-}
-#endif /* USE_GTK2 */
-
-
-
-static GtkWidget *gui_create_dlg_prefs(void)
-{
-	GtkWidget *dialog;
 #ifdef USE_GTK2
     GtkWidget *notebook;
     GtkWidget *tab_window[nb_prefs_num];
 	gint i;
-#endif
-	
-    dialog = create_dlg_prefs();
-#ifdef USE_GTK2
 
-    notebook = lookup_widget(dialog, "notebook_prefs");
+    notebook = gui_dlg_prefs_lookup("notebook_prefs");
 
     /*
      * Then create all the tabs in their own window.
@@ -309,7 +393,7 @@ static GtkWidget *gui_create_dlg_prefs(void)
      */
     for (i = 0; i < nb_prefs_num; i++) {
         GtkWidget *w = tab_window[i];
-        gui_merge_window_as_tab(dialog, notebook, w);
+        gui_merge_window_as_tab(gui_dlg_prefs(), notebook, w);
         gtk_object_destroy(GTK_OBJECT(w));
     }
 
@@ -318,12 +402,26 @@ static GtkWidget *gui_create_dlg_prefs(void)
      */
     gtk_container_remove(GTK_CONTAINER(notebook),
         gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 0));
-#endif /* USE_GTK2 */
 
-	return dialog;
+#endif	/* USE_GTK2 */
 }
 
-static GtkWidget *gui_create_dlg_about(void)
+static void
+text_widget_append(GtkWidget *widget, const gchar *line)
+#ifdef USE_GTK2
+{
+	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
+
+	gtk_text_buffer_insert_at_cursor(textbuf, line, (-1));
+}
+#else /* !USE_GTK2 */
+{
+	gtk_text_insert(GTK_TEXT(widget), NULL, NULL, NULL, line, (-1));
+}
+#endif /* USE_GTK2 */
+
+static void
+gui_init_dlg_about(void)
 {
     /* NB: These strings are UTF-8 encoded. */
     static const char * const contributors[] = {
@@ -374,53 +472,141 @@ static GtkWidget *gui_create_dlg_about(void)
         "Hans de Graaff <hans@degraaff.org>",
         "Globuz",
 		"Daichi Kawahata <daichik@users.sourceforge.net>",
-        NULL
+		"Dave Rutherford <polymath69@users.sourceforge.net>",
+		"Ian Sheldon <shellgeekorguk@users.sourceforge.net>",
+		"Bill Pringlemeir <bpringle@sympatico.ca>",
+		"Bouklis Panos <panos@echidna-band.com>",
+		"Sulyok P\303\251ter <sp@elte.hu>",
+		"Alexander N. S\303\270rnes <alex@thehandofagony.com>",
+		"Vitaliy Buyar <vetal18@users.sourceforge.net>",
+		"Wu Xiaoguang <wxgnj@yahoo.com.cn>",
+		"Lorenzo Gaifas <lorenzo@artiemestieri.tn.it>",
+		"FlashCode <flashcode@flashtux.org>",
+		"U\304\237ur \303\207etin <ugur.jnmbk@gmail.com>",
+		"Lloyd Bryant <lloydbaz@msn.com>",
     };
-    GtkWidget *dlg = create_dlg_about();
+	GtkWidget *text;
     guint i;
-#ifdef USE_GTK2
-    GtkTextBuffer *textbuf;
-
-    textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(
-        lookup_widget(dlg, "textview_about_contributors")));
-    
-    for (i = 0; NULL != contributors[i]; i++) {
-        if (i > 0)
-            gtk_text_buffer_insert_at_cursor(textbuf, "\n", (-1));
-        gtk_text_buffer_insert_at_cursor(textbuf, contributors[i], (-1));
-    }
-#else
-    GtkText *text = GTK_TEXT(lookup_widget(dlg, "text_about_contributors"));
-    static char s[256];
-
-    for (i = 0; NULL != contributors[i]; i++) {
-        if (i > 0)
-            gtk_text_insert(text, NULL, NULL, NULL, "\n", (-1));
-        g_strlcpy(s, contributors[i], sizeof(s));
-        gtk_text_insert(text, NULL, NULL, NULL,
-            lazy_utf8_to_locale(s, 0), (-1));
-    }
-#endif
-
+	
+	text = gui_dlg_about_lookup("textview_about_contributors");
     gtk_label_set_text(
-        GTK_LABEL(lookup_widget(dlg, "label_about_title")), 
+        GTK_LABEL(gui_dlg_about_lookup("label_about_title")),
         guc_version_get_version_string());
 
-    return dlg;
+  	for (i = 0; i < G_N_ELEMENTS(contributors); i++) {
+		if (i > 0)
+			text_widget_append(GTK_WIDGET(text), "\n");
+		text_widget_append(GTK_WIDGET(text),
+			lazy_utf8_to_ui_string(contributors[i]));
+	}
+
+	gtk_label_set_text(
+		GTK_LABEL(gui_dlg_about_lookup("label_about_translation")),
+		/* TRANSLATORS: Translate this as "Translation provided by" or similar
+   	   	   and append your name to the list. */
+    	Q_("translation_credit|"));
 }
 
-/*
- * main_gui_gtkrc_init
- *
+static void
+gui_init_dlg_faq(void)
+{
+	static const gchar faq_file[] = "FAQ";
+	static file_path_t fp[4];
+	GtkWidget *text;
+	const gchar *lang;
+	guint i = 0;
+	FILE *f;
+
+	text = gui_dlg_faq_lookup("textview_faq");
+	lang = locale_get_language();
+
+	file_path_set(&fp[i++], make_pathname(PRIVLIB_EXP, lang), faq_file);
+	file_path_set(&fp[i++], PRIVLIB_EXP G_DIR_SEPARATOR_S "en", faq_file);
+	
+#ifndef OFFICIAL_BUILD
+	file_path_set(&fp[i++],
+		make_pathname(PACKAGE_EXTRA_SOURCE_DIR, lang), faq_file);
+
+	file_path_set(&fp[i++],
+		PACKAGE_EXTRA_SOURCE_DIR G_DIR_SEPARATOR_S "en", faq_file);
+#endif /* !OFFICIAL_BUILD */
+
+	g_assert(i <= G_N_ELEMENTS(fp));
+
+	f = file_config_open_read_norename("FAQ", fp, i);
+	if (f) {
+		gchar buf[4096];
+		gboolean tag = FALSE;
+
+		while (fgets(buf, sizeof buf, f)) {
+			const gchar *s;
+			gchar *p;
+
+			if (!utf8_is_valid_string(buf)) {
+				text_widget_append(GTK_WIDGET(text),
+					_("\nThe FAQ document is damaged.\n"));
+				break;
+			}
+
+			/* Strip HTML tags */
+			for (s = buf, p = buf; *s != '\0'; s++) {
+				if (tag) {
+					if (*s == '>')
+						tag = FALSE;
+				} else if (*s != '<') {
+					*p++ = *s;
+				} else {
+					tag = TRUE;
+				}
+			}
+			*p = '\0';
+			
+			/* Convert entities */
+			p = buf;
+			for (s = buf; '\0' != *s; s++) {
+				if ('&' == *s) {
+					const gchar *endptr;
+					guint32 uc;
+
+					uc = html_decode_entity(s, &endptr);
+					if ((guint32) -1 != uc) {
+						guint n;
+
+						n = utf8_encode_char(uc, p, (endptr - s) + 1);
+						p += n;
+						s = endptr;
+						continue;
+					}
+				}
+				*p++ = *s;
+			}
+			*p = '\0';
+			
+			text_widget_append(GTK_WIDGET(text), lazy_utf8_to_ui_string(buf));
+		}
+
+		fclose(f);
+	} else {
+		static const gchar msg[] =
+		N_(	"The FAQ document could not be loaded. Please read the online FAQ "
+			"at http://gtk-gnutella.sourceforge.net/?page=faq instead.");
+		text_widget_append(GTK_WIDGET(text), _(msg));
+	}
+}
+
+/**
  * Searches for the gktrc file to use. Order in which they are scanned:
- * - $HOME/.gtkrc
- * - $HOME/.gtk/gtkrc
- * - $HOME/.gtk1/gtkrc ($HOME/.gtk2/gtkrc if GTK2 interface is used)
- * - <GTK_GNUTELLA_SETTINGS_DIR>/gtkrc
- * - ./gtkrc
+ *
+ *	- $HOME/.gtkrc
+ *	- $HOME/.gtk/gtkrc
+ *	- $HOME/.gtk1/gtkrc ($HOME/.gtk2/gtkrc if GTK2 interface is used)
+ *	- $GTK_GNUTELLA_DIR/gtkrc
+ *	- ./gtkrc
+ *
  * Where the last one can overrule settings from earlier resource files.
  */
-void main_gui_gtkrc_init(void)
+void
+main_gui_gtkrc_init(void)
 {
 #ifdef USE_GTK2
     const gchar rcfn[] = "gtkrc-2.0";
@@ -436,17 +622,17 @@ void main_gui_gtkrc_init(void)
 	gtk_rc_parse(userrc);
 	G_FREE_NULL(userrc);
 
-	userrc = g_strconcat(guc_settings_home_dir(), 
-		G_DIR_SEPARATOR_S, ".gtk", G_DIR_SEPARATOR_S, "gtkrc", NULL);
+	userrc = g_strconcat(guc_settings_home_dir(),
+		G_DIR_SEPARATOR_S, ".gtk", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
 	gtk_rc_parse(userrc);
 	G_FREE_NULL(userrc);
 
 #ifdef USE_GTK2
-	userrc = g_strconcat(guc_settings_home_dir(), 
-		G_DIR_SEPARATOR_S, ".gtk2", G_DIR_SEPARATOR_S, "gtkrc", NULL);
+	userrc = g_strconcat(guc_settings_home_dir(),
+		G_DIR_SEPARATOR_S, ".gtk2", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
 #else
-	userrc = g_strconcat(guc_settings_home_dir(), 
-		G_DIR_SEPARATOR_S, ".gtk1", G_DIR_SEPARATOR_S, "gtkrc", NULL);
+	userrc = g_strconcat(guc_settings_home_dir(),
+		G_DIR_SEPARATOR_S, ".gtk1", G_DIR_SEPARATOR_S, "gtkrc", (void *) 0);
 #endif
 	gtk_rc_parse(userrc);
 	G_FREE_NULL(userrc);
@@ -458,21 +644,19 @@ void main_gui_gtkrc_init(void)
 	gtk_rc_parse("." G_DIR_SEPARATOR_S "gtkrc");
 }
 
-
 /***
  *** Public functions
  ***/
 
-/*
- * main_gui_early_init:
- *
+/**
  * Some setup of the gui side which I wanted out of main.c but must be done
  * before the backend can be initialized since the core code is not free of
  * GTK yet.
  *      -- Richard, 6/9/2002
  */
-void main_gui_early_init(gint argc, gchar **argv)
-{	
+void
+main_gui_early_init(gint argc, gchar **argv)
+{
 	/* Glade inits */
 
 	gtk_set_locale();
@@ -483,81 +667,103 @@ void main_gui_early_init(gint argc, gchar **argv)
 	add_pixmap_directory(PACKAGE_SOURCE_DIR G_DIR_SEPARATOR_S "pixmaps");
 #endif
 
-    main_window = gui_create_main_window();
-    shutdown_window = create_shutdown_window();
-    dlg_about = gui_create_dlg_about();
-    dlg_prefs = gui_create_dlg_prefs();
-    dlg_quit = create_dlg_quit();
+    gui_main_window_set(create_main_window());
+	gui_init_main_window();
+
+    gui_shutdown_window_set(create_shutdown_window());
+    gui_dlg_quit_set(create_dlg_quit());
+	
+    gui_dlg_about_set(create_dlg_about());
+	gui_init_dlg_about();
+
+    gui_dlg_faq_set(create_dlg_faq());
+    gui_init_dlg_faq();
+
+    gui_dlg_prefs_set(create_dlg_prefs());
+	gui_init_dlg_prefs();
 
 	/* popup menus */
-	popup_search = create_popup_search();
-	popup_monitor = create_popup_monitor();
-	popup_downloads = create_popup_dl_active();
-	popup_queue = create_popup_dl_queued();	
+	gui_popup_search_set(create_popup_search());
+	gui_popup_search_list_set(create_popup_search_list());
 
+#if defined(USE_GTK2)
+	gui_popup_downloads_set(create_popup_downloads());
+#elif defined(USE_GTK1)
+	gui_popup_downloads_set(create_popup_dl_active());
+	gui_popup_queue_set(create_popup_dl_queued());
+#endif /* USE_GTK1 */
+
+	gui_popup_monitor_set(create_popup_monitor());
+
+	gui_popup_nodes_set(create_popup_nodes());
     nodes_gui_early_init();
+
+    gui_popup_uploads_set(create_popup_uploads());
     uploads_gui_early_init();
     statusbar_gui_init();
 
 	gui_init_window_title();
 
     /* search history combo stuff */
-    gtk_combo_disable_activate
-        (GTK_COMBO(lookup_widget(main_window, "combo_search")));
+    gtk_combo_disable_activate(GTK_COMBO(
+		gui_main_window_lookup("combo_search")));
 
     /* copy url selection stuff */
-    gtk_selection_add_target
-        (popup_downloads, GDK_SELECTION_PRIMARY, GDK_SELECTION_TYPE_STRING, 1);
+    gtk_selection_add_target(gui_popup_downloads(),
+		GDK_SELECTION_PRIMARY, GDK_SELECTION_TYPE_STRING, 1);
 }
 
-void main_gui_init(void)
+void
+main_gui_init(void)
 {
 	main_gui_gtkrc_init();
-	
+
 #ifdef USE_GTK1
     gtk_clist_set_column_justification(
-        GTK_CLIST(lookup_widget(main_window, "clist_search_stats")),
+        GTK_CLIST(gui_main_window_lookup("clist_search_stats")),
         c_st_period, GTK_JUSTIFY_RIGHT);
     gtk_clist_set_column_justification(
-        GTK_CLIST(lookup_widget(main_window, "clist_search_stats")),
+        GTK_CLIST(gui_main_window_lookup("clist_search_stats")),
         c_st_total, GTK_JUSTIFY_RIGHT);
     gtk_clist_column_titles_passive(
-        GTK_CLIST(lookup_widget(main_window, "clist_search_stats")));
+        GTK_CLIST(gui_main_window_lookup("clist_search_stats")));
 
     gtk_clist_column_titles_passive(
-        GTK_CLIST(lookup_widget(main_window, "clist_search")));
+        GTK_CLIST(gui_main_window_lookup("clist_search")));
     gtk_clist_set_compare_func(
-        GTK_CLIST(lookup_widget(main_window, "clist_ul_stats")), 
+        GTK_CLIST(gui_main_window_lookup("clist_ul_stats")),
         compare_ul_norm);
-#endif
+#endif /* USE_GTK1 */
 
 #ifdef USE_GTK2
-	GTK_WINDOW(main_window)->allow_shrink = TRUE;
-#endif
+	GTK_WINDOW(gui_main_window())->allow_shrink = TRUE;
+#endif /* USE_GTK2 */
 
+#ifdef USE_GTK1
     /* FIXME: those gtk_widget_set_sensitive should become obsolete when
      * all property-change callbacks are set up properly
      */
 	gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_remove_file"), FALSE);
+        (gui_popup_downloads_lookup("popup_downloads_remove_file"), FALSE);
     gtk_widget_set_sensitive
-        (lookup_widget(popup_downloads, "popup_downloads_copy_url"), FALSE);
+        (gui_popup_downloads_lookup("popup_downloads_copy_url"), FALSE);
 	gtk_widget_set_sensitive
-        (lookup_widget(popup_queue, "popup_queue_abort"), FALSE); 
+        (gui_popup_queue_lookup("popup_queue_abort"), FALSE);
 	gtk_widget_set_sensitive
-        (lookup_widget(popup_queue, "popup_queue_abort_named"), FALSE);
+        (gui_popup_queue_lookup("popup_queue_abort_named"), FALSE);
 	gtk_widget_set_sensitive
-        (lookup_widget(popup_queue, "popup_queue_abort_host"), FALSE);
+        (gui_popup_queue_lookup("popup_queue_abort_host"), FALSE);
     gtk_widget_set_sensitive(
-        lookup_widget(popup_downloads, "popup_downloads_push"),
+        gui_popup_downloads_lookup("popup_downloads_push"),
     	!gtk_toggle_button_get_active(
             GTK_TOGGLE_BUTTON
-                (lookup_widget(main_window, 
+                (lookup_widget(gui_main_window(),
                                "checkbutton_downloads_never_push"))));
+#endif /* USE_GTK1 */
 
     settings_gui_init();
-	downloads_gui_init();
     fi_gui_init();
+	downloads_gui_init();
     vp_gui_init();
     nodes_gui_init();
     gui_init_menu();
@@ -567,7 +773,7 @@ void main_gui_init(void)
     uploads_gui_init();
     upload_stats_gui_init();
     /* Must come before search_init() so searches/filters can be loaded.*/
-	filter_init(); 
+	filter_init();
     search_gui_init();
     filter_update_targets(); /* Make sure the default filters are ok */
     monitor_gui_init();
@@ -575,92 +781,52 @@ void main_gui_init(void)
     gui_update_stats_frames();
 }
 
-void main_gui_run(void)
-{	
-    guint32 coord[4] = { 0, 0, 0, 0 };
+void
+main_gui_run(const gchar *geometry_spec)
+{
+	time_t now = tm_time_exact();
 
-    gui_prop_get_guint32(PROP_WINDOW_COORDS, coord, 0, G_N_ELEMENTS(coord));
-    main_gui_timer();
-    gtk_widget_show(main_window);		/* Display the main window */
-    
-    /*
-     * We need to tell Gtk the size of the window, otherwise we'll get
-     * strange side effects when the window is shown (like implicitly
-     * resized widgets).
-     *      -- Richard, 8/9/2002
-     */
-    if (coord[2] != 0 && coord[3] != 0)
-        gtk_window_set_default_size(
-            GTK_WINDOW(main_window), coord[2], coord[3]);
+    gtk_widget_show(gui_main_window());		/* Display the main window */
 
-    icon_init();
+	if (geometry_spec) {
+		guint32 coord[4] = { 0, 0, 0, 0 };
 
-#ifdef USE_GTK2
-    if (coord[2] != 0 && coord[3] != 0) {
-		gint x, y, dx, dy;
-		gint i;
-
-		/* First, move the window to the supposed location. Next make the
-		 * window visible by gtk_window_get_position()... */
-
-       	gtk_window_move(GTK_WINDOW(main_window), coord[0], coord[1]);
-
-		/* The first call to gtk_window_get_position() makes the window
-		 * visible but x and y are always set to zero. The second call
-		 * yields the *real* values. */
-
-		for (i = 0; i < 2; i++)
-			gtk_window_get_position(GTK_WINDOW(main_window), &x, &y);
-
-		gtk_window_resize(GTK_WINDOW(main_window), coord[2], coord[3]);
-
-		/* (At least) FVWM2 doesn't take the window decoration into account
-		 * when handling positions requests. Readjust the window position
-		 * if we detect that the window manager added an offset. */
-
-		dx = (gint) coord[0] - x;
-		dy = (gint) coord[1] - y;
-		if (dx || dy) {
-        	gtk_window_move(GTK_WINDOW(main_window),
-				coord[0] + dx, coord[1] + dy);
+    	gui_prop_get_guint32(PROP_WINDOW_COORDS, coord, 0, G_N_ELEMENTS(coord));
+		if (0 == gui_parse_geometry_spec(geometry_spec, coord)) {
+    		gui_prop_set_guint32(PROP_WINDOW_COORDS,
+				coord, 0, G_N_ELEMENTS(coord));
 		}
 	}
-#else
-    if (coord[2] != 0 && coord[3] != 0) {
-		gint x, y, dx, dy;
+	gui_restore_window(gui_main_window(), PROP_WINDOW_COORDS);
+ 
+    icon_init();
+    main_gui_timer(now);
 
-        gdk_window_move_resize(main_window->window,
-			coord[0], coord[1], coord[2], coord[3]);
-
-		/* (At least) FVWM2 doesn't take the window decoration into account
-		 * when handling positions requests. Readjust the window position
-		 * if we detect that the window manager added an offset. */
-
-		gdk_window_get_root_origin(main_window->window, &x, &y);
-		dx = (gint) coord[0] - x;
-		dy = (gint) coord[1] - y;
-		if (dx || dy)
-        	gdk_window_move(main_window->window, coord[0] + dx, coord[1] + dy);
-	}
-
-    gtk_widget_fix_width(
-        lookup_widget(main_window, "frame_statusbar_uptime"),
-        lookup_widget(main_window, "label_statusbar_uptime"),
+ 	gtk_widget_fix_width(
+        gui_main_window_lookup("frame_statusbar_uptime"),
+        gui_main_window_lookup("label_statusbar_uptime"),
         8, 8);
-#endif
+
+#ifdef USE_GTK2
+	g_signal_connect(GTK_OBJECT(gui_main_window_lookup("notebook_main")),
+		"switch-page", G_CALLBACK(on_notebook_main_switch_page), NULL);
+#else
+	gtk_signal_connect(GTK_OBJECT(gui_main_window_lookup("notebook_main")),
+		"switch-page", on_notebook_main_switch_page, NULL);
+#endif /* USE_GTK2 */
 
 	/*
 	 * Make sure the application starts in the Gnet pane.
 	 */
 
 	gtk_notebook_set_page(
-		GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main")),
+		GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")),
 		nb_main_page_gnet);
 
 #ifdef USE_GTK1
 	{
 		GtkCTree *ctree_menu =
-			GTK_CTREE(lookup_widget(main_window, "ctree_menu"));
+			GTK_CTREE(gui_main_window_lookup("ctree_menu"));
 		GtkCTreeNode *node;
 
 		node = gtk_ctree_find_by_row_data(ctree_menu,
@@ -670,14 +836,14 @@ void main_gui_run(void)
 		if (node != NULL)
 			gtk_ctree_select(ctree_menu, node);
 	}
-#endif
-
-    settings_gui_init_late();
+#endif /* USE_GTK1 */
 
+	settings_gui_restore_panes();
     gtk_main();
 }
 
-void main_gui_shutdown(void)
+void
+main_gui_shutdown(void)
 {
 	icon_close();
 
@@ -686,12 +852,15 @@ void main_gui_shutdown(void)
      */
 
     filter_close_dialog(FALSE);
-	gtk_widget_hide(main_window);
+	gtk_widget_hide(gui_main_window());
+	if (gui_dlg_prefs())
+		gtk_widget_hide(gui_dlg_prefs());
 
+	gui_menu_shutdown();
     search_stats_gui_shutdown();
     filter_cb_close();
     monitor_gui_shutdown();
-    search_gui_flush(0);
+    search_gui_flush(tm_time(), TRUE);
     search_gui_shutdown(); /* must be done before filter_shutdown! */
  	downloads_gui_shutdown();
 	filter_shutdown();
@@ -704,65 +873,70 @@ void main_gui_shutdown(void)
     hcache_gui_shutdown();
 }
 
-void main_gui_update_coords(void)
+void
+main_gui_update_coords(void)
 {
-    guint32 coord[4] = { 0, 0, 0, 0};
-	gint x, y, w, h;
-
-#ifdef USE_GTK1
-	gdk_window_get_root_origin(main_window->window, &x, &y);
-	gdk_window_get_size(main_window->window, &w, &h);
-#else
-	gtk_window_get_position(GTK_WINDOW(main_window), &x, &y);
-	gtk_window_get_size(GTK_WINDOW(main_window), &w, &h);
-#endif
-	coord[0] = x;
-	coord[1] = y;
-	coord[2] = w;
-	coord[3] = h;
-    gui_prop_set_guint32(PROP_WINDOW_COORDS, coord, 0, G_N_ELEMENTS(coord));
+	gui_save_window(gui_main_window(), PROP_WINDOW_COORDS);
 }
 
 /**
  * Main gui timer. This is called once a second.
  */
-void main_gui_timer(void)
+void
+main_gui_timer(time_t now)
 {
-	time_t now = time((time_t *) NULL);
+	static const gint num_states = 10;
+	gboolean overloaded;
+	gint i;
+
+	gnet_prop_get_boolean_val(PROP_OVERLOADED_CPU, &overloaded);
 
     gui_general_timer(now);
-
-    hcache_gui_update(now);
-    gnet_stats_gui_update(now);
-    search_stats_gui_update(now);
-    nodes_gui_update_nodes_display(now);
-    uploads_gui_update_display(now);
-	fi_gui_update_display(now);
-    statusbar_gui_clear_timeouts(now);
-    search_gui_flush(now);
-
     gui_update_traffic_stats();
-    filter_timer();					/* Update the filter stats */
+
+	/*
+	 * When the CPU is overloaded, non-essential GUI information is not
+	 * updated every second.
+	 */
+
+	for (i = 0; i < num_states; i++) {
+		static guint counter = 0;
+
+		switch (counter) {
+		case 0: hcache_gui_update(now);					break;
+		case 1: gnet_stats_gui_update(now);				break;
+		case 2: search_stats_gui_update(now);			break;
+		case 3: nodes_gui_update_nodes_display(now);	break;
+		case 4: uploads_gui_update_display(now);		break;
+		case 5: fi_gui_update_display(now);				break;
+		case 6: statusbar_gui_clear_timeouts(now);		break;
+		case 7: filter_timer();							break;
+		case 8: downloads_gui_update_display(now);		break;
+		case 9: search_gui_flush(now, FALSE);			break;
+		default:
+			g_error("bad modulus computation (counter is %d)", counter);
+			break;
+		}
+		counter = (counter + 1) % num_states;
+
+		if (overloaded)
+			break;
+	}
 }
 
-void main_gui_shutdown_tick(guint left)
+void
+main_gui_shutdown_tick(guint left)
 {
     static gboolean notice_visible = FALSE;
-	gchar tmp[256];
+    GtkLabel *label;
 
-    GtkLabel *label_shutdown_count;
- 
     if (!notice_visible) {
-        gtk_widget_show(shutdown_window);
+        gtk_widget_show(gui_shutdown_window());
         notice_visible = TRUE;
     }
 
-    label_shutdown_count = GTK_LABEL
-        (lookup_widget(shutdown_window, "label_shutdown_count"));
-
-	gm_snprintf(tmp, sizeof(tmp), "%d seconds", left);
-
-	gtk_label_set(label_shutdown_count,tmp);
+    label = GTK_LABEL(gui_shutdown_window_lookup("label_shutdown_count"));
+	gtk_label_printf(label, NG_("%d second", "%d seconds", left), left);
     gtk_main_flush();
 }
 
