@@ -1,5 +1,5 @@
-/* -*- mode: cc-mode; tab-width:4; -*-
- * $Id: search.h,v 1.9 2004/11/01 15:46:31 rmanfredi Exp $
+/*
+ * $Id: search.h 11712 2006-08-23 22:06:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -31,78 +31,81 @@
 
 #include "if/core/search.h"
 #include "if/core/bitzi.h"
-#include "if/ui/gtk/search.h"
 #include "lib/hashlist.h"
 #include "lib/prop.h"
 
-/* 
- * Structure for search results 
+/**
+ * Structure for search results.
  */
 struct search {
-    gnet_search_t search_handle; /* Search handle */
+    gnet_search_t search_handle;	/**< Search handle */
 
-	gchar      *query;			   /* The search query */
+	gchar      *query;				/**< The query string; always UTF-8 */
 	gboolean    enabled;
 
 #ifdef USE_GTK2
-	GtkWidget  *tree_view;			/* GtkTreeView for this search */
-	GtkTreeModel	*model;			/* GtkTreeModel for the GtkTreeView
-									 * so it can be detached from it */
+	GtkWidget  *tree_view;			/**< GtkTreeView for this search */
+	GtkTreeModel	*model;			/**< GtkTreeModel for the GtkTreeView
+										 so it can be detached from it */
 #else
-	GtkCTree   *ctree;			   	/* GtkCTree for this search */
-#endif
+	GtkCTree   *ctree;			   	/**< GtkCTree for this search */
+#endif /* USE_GTK2 */
 
-	GHashTable *parents;	/* table of mount iterators for any seen SHA1 */
-	GtkWidget  *scrolled_window;   /* GtkScrolledWindow, contains the GtkCList */
-	GtkWidget  *list_item;		   /* The GtkListItem in combo for this search */
-    GtkWidget  *arrow;             /* The arrow displaying sort order  */
+	GHashTable *parents;			/**< table of mount iterators for
+										 any seen SHA1 */
+	GtkWidget  *scrolled_window;	/**< GtkScrolledWindow, contains
+										 the GtkCList */
+    GtkWidget  *arrow;				/**< The arrow displaying sort order */
 
     gint        sort_col;
     gint        sort_order;
     gboolean    sort;
 
-	time_t      last_update_time;  /* last time the notebook tab was updated */
-	guint32     last_update_items; /* Number of items included in last update */
-	gint        tab_updating;	   /* token for timeout func. to be cancelled */
-	guint32     unseen_items;	   /* How many items haven't been seen yet. */
+	time_t      last_update_time;	/**< last time notebook tab was updated */
+	guint32     last_update_items;	/**< # of items included in last update */
+	gint        tab_updating;		/**< ID for timeout func. to be cancelled */
+	guint32     unseen_items;		/**< How many items haven't been seen yet */
 
-	gboolean    passive;		   /* Is this a passive search? */
-	gboolean    massive_update;	   /* massive update in process */
+	gboolean    passive;			/**< Is this a passive search? */
+	gboolean    massive_update;		/**< massive update in process */
+	gboolean    browse;				/**< Is this a browse-host search? */
+	gboolean    local;				/**< Is this a local search? */
 
-	hash_list_t *r_sets;		   /* The results sets of this search */
+	hash_list_t *r_sets;			/**< The results sets of this search */
 
-	GHashTable *dups;			   /* keep a record of dups. */
+	GHashTable *dups;				/**< keep a record of dups. */
 
-    filter_t   *filter;				/* filter ruleset bound to this search */
+    filter_t   *filter;				/**< filter ruleset bound to this search */
 
 	/*
 	 * Search stats.
 	 */
 
-	guint32     items;				/* Total number of items for this search */
-	guint32		tcp_qhits;			/* Query hits received from TCP */
-	guint32		udp_qhits;			/* Query hits received from UDP */
-	guint32		skipped;			/* Ignored hits (skipped over) */
-	guint32		ignored;			/* Filtered out hits */
-	guint32		hidden;				/* Hidden hits, never shown */
-	guint32		auto_downloaded;	/* Auto-downloaded hits */
-	guint32		duplicates;			/* Duplicate hits ignored */
+	guint32     items;				/**< Total number of items for the search */
+	guint32		tcp_qhits;			/**< Query hits received from TCP */
+	guint32		udp_qhits;			/**< Query hits received from UDP */
+	guint32		skipped;			/**< Ignored hits (skipped over) */
+	guint32		ignored;			/**< Filtered out hits */
+	guint32		hidden;				/**< Hidden hits, never shown */
+	guint32		auto_downloaded;	/**< Auto-downloaded hits */
+	guint32		duplicates;			/**< Duplicate hits ignored */
 };
 
 #include "search_common.h"
 
 #ifdef USE_GTK1
 
-/*
+/**
  *	Record associated with each gui node in the search results ctree.
  */
 typedef struct gui_record {
-	record_t *shared_record;	/* Common record data, shared between searches*/
+	record_t *shared_record;		/**< Common record data, shared between
+										 searches */
 
-	gint num_children;			/* Number of children under this node */
+	gint num_children;				/**< Number of children under this node */
 } gui_record_t;
 
-#endif
+#endif /* USE_GTK1 */
 
 
 /*
@@ -113,8 +116,8 @@ void search_gui_init(void);
 void search_gui_shutdown(void);
 
 gboolean search_gui_new_search_full(const gchar *query,
-	guint32 reissue_timeout, gint sort_col, 
-	gint sort_order, flag_t flags, search_t **search);
+	time_t create_time, guint lifetime, guint32 reissue_timeout,
+	gint sort_col, gint sort_order, flag_t flags, search_t **search);
 struct search *search_new_full(const gchar *, guint32, flag_t flags);
 void search_gui_close_search(search_t *sch);
 
@@ -131,17 +134,16 @@ void search_gui_add_record(
 
 gboolean gui_search_update_tab_label(struct search *);
 void gui_search_clear_results(void);
-void gui_search_history_add(gchar *s);
 
 #ifdef USE_GTK2
-void gui_search_create_tree_view(GtkWidget **sw, GtkWidget **tree_view);
 void gui_search_force_update_tab_label(struct search *, time_t now);
-GtkTreeIter * find_parent_with_sha1(GHashTable *ht, gpointer key);
+void search_gui_request_bitzi_data(void);
+const record_t *search_gui_get_record_at_path(GtkTreeView *tv,
+					GtkTreePath *path);
 #else
-void gui_search_create_ctree(GtkWidget **sw, GtkCTree **ctree);
 void gui_search_force_update_tab_label(struct search *);
 GtkCTreeNode *find_parent_with_sha1(GHashTable *ht, gpointer key);
-#endif
+#endif /* USE_GTK2 */
 
 void search_gui_expand_all(void);
 void search_gui_collapse_all(void);
@@ -150,15 +152,12 @@ void gui_search_set_enabled(struct search *sch, gboolean enabled);
 const GList *search_gui_get_searches(void);
 
 void search_gui_set_clear_button_sensitive(gboolean flag);
-void gui_search_get_colors(
-	search_t *sch,
-	GdkColor **mark_color, GdkColor **ignore_color, GdkColor **download_color);
 
 void search_gui_start_massive_update(search_t *sch);
 void search_gui_end_massive_update(search_t *sch);
 
-/*
- * Metadata Update
+/**
+ * Metadata Update.
  */
 
 void search_gui_metadata_update(const bitzi_data_t *data);
@@ -169,8 +168,9 @@ void search_gui_metadata_update(const bitzi_data_t *data);
 gboolean search_gui_search_results_col_widths_changed(property_t prop);
 gboolean search_gui_search_results_col_visible_changed(property_t prop);
 
-void search_gui_queue_bitzi_by_sha1(record_t *rec, void *nothing);
+void search_gui_queue_bitzi_by_sha1(const record_t *rec);
 
 #endif /* _gtk_search_h_ */
 
-/* vi: set ts=4: */
+/* -*- mode: cc-mode; tab-width:4; -*- */
+/* vi: set ts=4 sw=4 cindent: */
