@@ -1,5 +1,5 @@
 /*
- * $Id: statusbar.c,v 1.8 2005/08/31 21:37:33 rmanfredi Exp $
+ * $Id: statusbar.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -37,7 +37,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: statusbar.c,v 1.8 2005/08/31 21:37:33 rmanfredi Exp $");
+RCSID("$Id: statusbar.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "statusbar.h"
 
@@ -89,7 +89,7 @@ statusbar_get(void)
     static GtkStatusbar *sb;
 
 	if (!sb)
-		sb = GTK_STATUSBAR(lookup_widget(main_window, "statusbar"));
+		sb = GTK_STATUSBAR(gui_main_window_lookup("statusbar"));
 
 	return sb;
 }
@@ -142,10 +142,10 @@ statusbar_gui_push(sb_types_t type, guint scid, guint timeout,
 	const gchar *format, ...)
 {
     va_list args;
-    statusbar_msgid_t id = zero_msgid;
+    statusbar_msgid_t id;
 
 	va_start(args, format);
-	statusbar_gui_push_v(type, scid, timeout, format, args);
+	id = statusbar_gui_push_v(type, scid, timeout, format, args);
     va_end(args);
 
     return id;
@@ -162,10 +162,10 @@ statusbar_msgid_t
 statusbar_gui_message(guint timeout, const gchar *format, ...)
 {
     va_list args;
-    statusbar_msgid_t id = zero_msgid;
+    statusbar_msgid_t id;
 
 	va_start(args, format);
-	statusbar_gui_push_v(SB_MESSAGE, scid_info, timeout, format, args);
+	id = statusbar_gui_push_v(SB_MESSAGE, scid_info, timeout, format, args);
     va_end(args);
 
     return id;
@@ -182,10 +182,10 @@ statusbar_msgid_t
 statusbar_gui_warning(guint timeout, const gchar *format, ...)
 {
     va_list args;
-    statusbar_msgid_t id = zero_msgid;
+    statusbar_msgid_t id;
 
 	va_start(args, format);
-	statusbar_gui_push_v(SB_WARNING, scid_warn, timeout, format, args);
+	id = statusbar_gui_push_v(SB_WARNING, scid_warn, timeout, format, args);
     va_end(args);
 
     return id;
@@ -237,8 +237,7 @@ statusbar_gui_set_default(const char *format, ...)
 
     va_start(args, format);
 
-    if (statbar_botstr_new != NULL)
-        G_FREE_NULL(statbar_botstr_new);
+    G_FREE_NULL(statbar_botstr_new);
 
     if (format != NULL) {
         gm_vsnprintf(buf, sizeof(buf), format, args);
@@ -298,8 +297,9 @@ statusbar_gui_clear_timeouts(time_t now)
 
 	for (sl = sl_statusbar_timeouts; sl; sl = g_slist_next(sl)) {
 		struct statusbar_timeout *t = sl->data;
+		const time_delta_t timeout = t->timeout;
 
-		if (delta_time(now, t->stamp) > t->timeout)
+		if (delta_time(now, t->stamp) > timeout)
 			to_remove = g_slist_prepend(to_remove, t);
 	}
 
