@@ -1,5 +1,5 @@
 /*
- * $Id: statusbar.h,v 1.2 2004/09/19 09:26:06 rmanfredi Exp $
+ * $Id: statusbar.h 8104 2005-04-27 16:27:47Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -30,14 +30,6 @@
 
 #include "if/ui/gtk/statusbar.h"
 
-/* 
- * Context ids for the status bar 
- */
-extern guint scid_hostsfile;
-extern guint scid_search_autoselected;
-extern guint scid_queue_freezed;
-extern guint scid_ip_changed;
-
 /*
  * Public interface.
  */
@@ -52,4 +44,4 @@ void statusbar_gui_remove(statusbar_msgid_t);
 
 #endif /* _gtk_statusbar_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
