@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_cb.c,v 1.2 2004/10/25 14:36:16 cbiere Exp $
+ * $Id: uploads_cb.c 12195 2006-11-03 02:20:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,17 +25,21 @@
 
 #include "gui.h"
 
-RCSID("$Id: uploads_cb.c,v 1.2 2004/10/25 14:36:16 cbiere Exp $");
+RCSID("$Id: uploads_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
 
 #include "uploads_cb.h"
 #include "uploads.h"
+#include "uploads_common.h"
 #include "gtkcolumnchooser.h"
 #include "gtk-missing.h"
 #include "columns.h"
 
+#include "gtk/statusbar.h"		/* XXX: whilst we have a FIXME for BH */
+
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/host_addr.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 /***
@@ -46,14 +50,30 @@ RCSID("$Id: uploads_cb.c,v 1.2 2004/10/25 14:36:16 cbiere Exp $");
  * Suited for use as a GFunc in a g_list_for_each.
  */
 static void
-kill_upload(upload_row_data_t *d, gpointer unused_udata)
+kill_upload(gpointer data, gpointer unused_udata)
 {
+	upload_row_data_t *d = data;
+
 	(void) unused_udata;
 
     if (d->valid)
         guc_upload_kill(d->handle);
 }
 
+/**
+ * Suited for use as a GFunc in a g_list_for_each.
+ */
+static void
+browse_uploading_host(gpointer data, gpointer unused_udata)
+{
+	upload_row_data_t *d = data;
+
+	(void) unused_udata;
+
+    if (host_addr_initialized(d->gnet_addr))
+		uploads_gui_browse_host(d->gnet_addr, d->gnet_port);
+}
+
 /***
  *** Public functions
  ***/
@@ -79,11 +99,12 @@ on_clist_uploads_select_row(GtkCList *clist, gint unused_row,
 	(void) unused_event;
 	(void) unused_udata;
 
-    button = lookup_widget(main_window, "button_uploads_kill");
+    button = gui_main_window_lookup("button_uploads_kill");
     gtk_widget_set_sensitive(button, clist->selection != NULL);
 }
 
-void on_clist_uploads_unselect_row(GtkCList *clist, 
+void
+on_clist_uploads_unselect_row(GtkCList *clist,
     gint unused_row, gint unused_column, GdkEvent *unused_event,
 	gpointer unused_udata)
 {
@@ -93,17 +114,17 @@ void on_clist_uploads_unselect_row(GtkCList *clist,
 	(void) unused_column;
 	(void) unused_event;
 	(void) unused_udata;
-    button = lookup_widget(main_window, "button_uploads_kill");
+    button = gui_main_window_lookup("button_uploads_kill");
     gtk_widget_set_sensitive(button, clist->selection != NULL);
 }
 
 void
-on_clist_uploads_resize_column(GtkCList *unused_clist, 
+on_clist_uploads_resize_column(GtkCList *unused_clist,
     gint column, gint width, gpointer unused_udata)
 {
 	(void) unused_clist;
 	(void) unused_udata;
-	
+
     /* FIXME: use properties */
 	*(gint *) &uploads_col_widths[column] = width;
 }
@@ -117,12 +138,12 @@ on_button_uploads_kill_clicked(GtkButton *unused_button, gpointer unused_udata)
 	(void) unused_button;
 	(void) unused_udata;
 
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_uploads"));
+    clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
 
     gtk_clist_freeze(clist);
 
     sl = clist_collect_data(clist, FALSE, NULL);
-    g_slist_foreach(sl, (GFunc) kill_upload, NULL);
+    g_slist_foreach(sl, kill_upload, NULL);
     g_slist_free(sl);
 
     gtk_clist_thaw(clist);
@@ -136,7 +157,7 @@ on_clist_uploads_button_press_event(GtkWidget *unused_widget,
 {
 	gint row, col;
     GtkCList *clist_uploads = GTK_CLIST
-        (lookup_widget(main_window, "clist_uploads"));
+        (gui_main_window_lookup("clist_uploads"));
 
 	(void) unused_widget;
 	(void) unused_udata;
@@ -151,23 +172,33 @@ on_clist_uploads_button_press_event(GtkWidget *unused_widget,
 		(GTK_CLIST(clist_uploads), event->x, event->y, &row, &col))
 		return FALSE;
 
-    gtk_menu_popup(GTK_MENU(popup_uploads), NULL, NULL, NULL, NULL, 
+    gtk_menu_popup(GTK_MENU(gui_popup_uploads()), NULL, NULL, NULL, NULL,
                   event->button, event->time);
 
 	return TRUE;
 }
 
+/**
+ * Initiates a browse host request to the currently selected host.
+ */
 void
-on_popup_uploads_title_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata) 
+on_popup_uploads_browse_host_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
 {
+	GtkCList *clist;
+	GSList *sl;
+
 	(void) unused_menuitem;
 	(void) unused_udata;
-	/* FIXME */
+	
+	clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
+
+	sl = clist_collect_data(clist, FALSE, NULL);
+	g_slist_foreach(sl, browse_uploading_host, NULL);
+	g_slist_free(sl);
 }
 #endif /* USE_GTK1 */
 
-
 #ifdef USE_GTK2
 void
 on_popup_uploads_config_cols_activate(GtkMenuItem *unused_menuitem,
@@ -178,22 +209,22 @@ on_popup_uploads_config_cols_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_menuitem;
 	(void) unused_udata;
 
-    cc = gtk_column_chooser_new(lookup_widget(main_window, "treeview_uploads"));
+    cc = gtk_column_chooser_new(gui_main_window_lookup("treeview_uploads"));
     gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
 }
 
 static void
-uploads_kill_helper(GtkTreeModel *model, GtkTreePath *unused_path,
-	GtkTreeIter *iter, gpointer unused_data)
+uploads_func_helper(GtkTreeModel *model, GtkTreePath *unused_path,
+	GtkTreeIter *iter, gpointer data)
 {
 	upload_row_data_t *d = NULL;
+	GFunc func = data;
 
 	(void) unused_path;
-	(void) unused_data;
 
 	gtk_tree_model_get(model, iter, c_ul_data, &d, (-1));
 	g_assert(NULL != d);
-	kill_upload(d, NULL);
+	(*func)(d, NULL);
 }
 
 void
@@ -205,10 +236,29 @@ on_button_uploads_kill_clicked(GtkButton *unused_button, gpointer unused_udata)
 	(void) unused_button;
 	(void) unused_udata;
 
-    treeview = GTK_TREE_VIEW(lookup_widget(main_window, "treeview_uploads"));
+    treeview = GTK_TREE_VIEW(gui_main_window_lookup("treeview_uploads"));
     selection = gtk_tree_view_get_selection(treeview);
     gtk_tree_selection_selected_foreach(selection,
-        (GtkTreeSelectionForeachFunc) uploads_kill_helper, NULL);
+		uploads_func_helper, kill_upload);
+}
+
+/**
+ * Initiates a browse host request to the currently selected host.
+ */
+void
+on_popup_uploads_browse_host_activate(GtkMenuItem *unused_menuitem,
+	gpointer unused_udata)
+{
+    GtkTreeView *treeview;
+    GtkTreeSelection *selection;
+
+	(void) unused_menuitem;
+	(void) unused_udata;
+	
+    treeview = GTK_TREE_VIEW(gui_main_window_lookup("treeview_uploads"));
+    selection = gtk_tree_view_get_selection(treeview);
+    gtk_tree_selection_selected_foreach(selection,
+		uploads_func_helper, browse_uploading_host);
 }
 #endif /* USE_GTK2 */
 
