@@ -1,9 +1,7 @@
 /*
- * $Id: visual_progress.c,v 1.26 2004/11/23 19:47:50 graaff Exp $
+ * $Id: visual_progress.c 11773 2006-08-27 06:41:26Z cbiere $
  *
- * Copyright (c) 2003-2004, Hans de Graaff
- *
- * Displaying the visual progress of downloading graphically.
+ * Copyright (c) 2003-2005, Hans de Graaff
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -26,41 +24,27 @@
  */
 
 /**
+ * @ingroup gtk
  * @file
- * 
+ *
+ * Displaying the visual progress of downloading graphically.
+ *
  * Visual progress indicator for files in the download queue.
- * 
- * TODO and other ideas to be implemented.
  *
- * - The current availability info (blue line) is not accurate,
- * because it only aggregates. It should also take into account when
- * we loose a source. As ram mentioned on IRC: 
- *
- *  right, that's why it's better to construct it dynamically when
- *  needed, and have a "one more alive source", "lost one alive
- *  source" events to update the cached merged list when it's needed
- *  only.
- *
- * make colors into properties so that they can be stored in config,
- * should keep hardcoded backups.
- * 
- * Add progress data also to fileinfo table, so that the info is shown for 
- * all current files.
- * 
- * Do not redraw the bar too often, only on event for actual file and
- * perhaps max once a second.
- *
- * Make busy chunks stand out more, e.g. by drawing arrow: *graaff:
- * what about drawing an arrow centered at the start of the downloaded
- * part (beginning of the yellow chunk) of about 5 pixels height?
- * <ram> You'd draw 1, then 3, then 5 then 7 then 9 pixels.  Should be
- * readable that way, no?
+ * @todo Make colors into properties so that they can be stored in config,
+ *       should keep hardcoded backups.
+ * @todo Add progress data also to fileinfo table, so that the info is shown 
+ *       for all current files.
+ * @todo Do not redraw the bar too often, only on event for actual file and
+ *       perhaps max once a second.
  *
+ * @author Hans de Graaff
+ * @date 2003-2005
  */
 
 #include "gui.h"
 
-RCSID("$Id: visual_progress.c,v 1.26 2004/11/23 19:47:50 graaff Exp $");
+RCSID("$Id: visual_progress.c 11773 2006-08-27 06:41:26Z cbiere $")
 
 #include "visual_progress.h"
 
@@ -68,63 +52,64 @@ RCSID("$Id: visual_progress.c,v 1.26 2004/11/23 19:47:50 graaff Exp $");
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/atoms.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
-/* The height of the indicator arrows in visual progress */
-#define VP_ARROW_HEIGHT 5
+/** The height of the indicator arrows in visual progress */
+#define VP_ARROW_HEIGHT 7
 
 /**
- * The context for drawing, including location to draw
+ * The context for drawing, including location to draw.
  */
 typedef struct vp_context {
-	GtkWidget *widget;     /** The widget containing the drawing area */
-    GdkDrawable *drawable; /** The drawable inside the widget */
-    GdkGC *gc;             /** The Graphics Context used in this vp context */
-    gnet_fi_t fih;         /** The most recently used fileinfo handle */
-	gboolean fih_valid;    /** Whether fih is still a valid handle */
+	GtkWidget *widget;      /**< The widget containing the drawing area */
+    GdkDrawable *drawable;  /**< The drawable inside the widget */
+    GdkGC *gc;              /**< The Graphics Context used in this vp context */
+    gnet_fi_t fih;          /**< The most recently used fileinfo handle */
+	gboolean fih_valid;	    /**< Whether fih is still a valid handle */
 } vp_context_t;
 
 /**
- * Locally cached information from fileinfo needed for drawing the graphics 
+ * Locally cached information from fileinfo needed for drawing the graphics.
  */
 typedef struct vp_info {
     gnet_fi_t fi_handle;
-    gchar *file_name;
-    filesize_t file_size;
+    gchar *file_name;		/* atom */
     GSList *chunks_list;
 	GSList *chunks_initial;
 	GSList *ranges_list;
-	filesize_t done_initial;
     vp_context_t *context;
+    filesize_t file_size;
+	filesize_t done_initial;
 } vp_info_t;
 
-static GHashTable *vp_info_hash; /* Hash table with our cached fileinfo info */
+static GHashTable *vp_info_hash; /**< Hash table with our cached fileinfo info */
 
-static GdkColor done;       /* Pre-filled color (green) for DONE chunks */
-static GdkColor done_old;	/* Pre-filled color (dull green) for DONE
-							 * chunks from previous sessions */
-static GdkColor busy;       /* Pre-filled color (yellow) for BUSY chunks */
-static GdkColor arrow;      /* Pre-filled color (blue) for start of BUSY */
-static GdkColor empty;      /* Pre-filled color (red) for EMPTY chunks */
-static GdkColor black;      /* Pre-filled color (black) for general drawing */
-static GdkColor available;  /* Pre-filled color (blue) available on network */
-static GdkColor nosize;     /* Pre-filled color (gray) indicates
+static GdkColor done;       /**< Pre-filled color (green) for DONE chunks */
+static GdkColor done_old;	/**< Pre-filled color (dull green) for DONE
+								 chunks from previous sessions */
+static GdkColor busy;       /**< Pre-filled color (yellow) for BUSY chunks */
+static GdkColor arrow;      /**< Pre-filled color (blue) for start of BUSY */
+static GdkColor empty;      /**< Pre-filled color (red) for EMPTY chunks */
+static GdkColor black;      /**< Pre-filled color (black) for general drawing */
+static GdkColor available;  /**< Pre-filled color (blue) available on network */
+static GdkColor nosize;     /**< Pre-filled color (gray) indicates
 								 chunk information is not available
 								 (e.g. file size == 0 */
-static GdkColor *base;      /* Theme-defined background color */
+static GdkColor *base;      /**< Theme-defined background color */
 
 /**
- * The visual progress context for drawing fileinfo information
+ * The visual progress context for drawing fileinfo information.
  */
 static vp_context_t fi_context;
 
 
 /**
- * Draw a rectangle for visual progress
+ * Draw a rectangle for visual progress.
  */
-void 
-vp_draw_rectangle(vp_info_t *v, filesize_t from, filesize_t to, 
+void
+vp_draw_rectangle(vp_info_t *v, filesize_t from, filesize_t to,
 				  guint top, guint bottom)
 {
     guint s_from;
@@ -134,7 +119,7 @@ vp_draw_rectangle(vp_info_t *v, filesize_t from, filesize_t to,
     g_assert(v->context);
     g_assert(v->context->drawable);
 
-	/* 
+	/*
 	 * Both these variables should be set to a value, otherwise we get
 	 * a division by zero below. We could protect for that, but
 	 * neither should be zero when we end up here, so this can be
@@ -144,22 +129,22 @@ vp_draw_rectangle(vp_info_t *v, filesize_t from, filesize_t to,
 	 * unknown size the file_size == 0, but in this case
 	 * vp_draw_fi_progress catches this case.
 	 */
-	g_assert(v->file_size);
+	g_assert(v->file_size > 0);
 
-	s_from = (gfloat) from * v->context->widget->allocation.width 
+	s_from = ((gdouble) from * v->context->widget->allocation.width)
 		/ v->file_size;
-	s_to   = (gfloat) to   * v->context->widget->allocation.width 
+	s_to   = ((gdouble) to   * v->context->widget->allocation.width)
 		/ v->file_size;
 
-    gdk_draw_rectangle(v->context->drawable, v->context->gc, TRUE, 
+    gdk_draw_rectangle(v->context->drawable, v->context->gc, TRUE,
         s_from, top, s_to - s_from, bottom);
 }
 
 /**
- * Draw a chunk for visual progress
+ * Draw a chunk for visual progress.
  */
-void 
-vp_draw_chunk (gpointer data, gpointer user_data)
+static void
+vp_draw_chunk(gpointer data, gpointer user_data)
 {
     gnet_fi_chunks_t *chunk = data;
     vp_info_t *v = user_data;
@@ -176,18 +161,18 @@ vp_draw_chunk (gpointer data, gpointer user_data)
     }
 
     vp_draw_rectangle(v,
-		chunk->from, chunk->to, 
+		chunk->from, chunk->to,
 		0, v->context->widget->allocation.height);
 }
 
 /**
- * Draw an downward arrow starting at the top
+ * Draw an downward arrow starting at the top.
  */
-void
+static void
 vp_draw_arrow(vp_info_t *v, filesize_t at)
 {
 	guint s_at;
-	guint i;
+	GdkPoint points[3];
 
     g_assert(v);
     g_assert(v->context);
@@ -195,14 +180,23 @@ vp_draw_arrow(vp_info_t *v, filesize_t at)
 
 	g_assert(v->file_size);
 
-	s_at = (gfloat) at * v->context->widget->allocation.width / v->file_size;
+	s_at = ((gdouble) at * v->context->widget->allocation.width) / v->file_size;
 
+	/* Fill the inside of the arrow */
+	points[0].x = s_at - VP_ARROW_HEIGHT;
+	points[0].y = 0;
+	points[1].x = s_at;
+	points[1].y = VP_ARROW_HEIGHT;
+	points[2].x = s_at + VP_ARROW_HEIGHT;
+	points[2].y = 0;
 	gdk_gc_set_foreground(v->context->gc, &arrow);
-	for (i = VP_ARROW_HEIGHT + 1; i-- > 0; /* empty */) {
-		gdk_draw_line(v->context->drawable, v->context->gc,
-		    s_at - i, VP_ARROW_HEIGHT - i,
-			s_at + i, VP_ARROW_HEIGHT - i);
-	}
+	gdk_draw_polygon(v->context->drawable, v->context->gc,
+		TRUE, points, G_N_ELEMENTS(points));
+
+	/* Draw a black border around the arrow */
+	gdk_gc_set_foreground(v->context->gc, &black);
+	gdk_draw_polygon(v->context->drawable, v->context->gc,
+		FALSE, points, G_N_ELEMENTS(points));
 }
 
 /**
@@ -210,8 +204,8 @@ vp_draw_arrow(vp_info_t *v, filesize_t at)
  * This is done in a separate funtion, because the arrows need to be
  * drawn on top of the chunks.
  */
-void
-vp_draw_arrows (gpointer data, gpointer user_data)
+static void
+vp_draw_arrows(gpointer data, gpointer user_data)
 {
     gnet_fi_chunks_t *chunk = data;
     vp_info_t *v = user_data;
@@ -226,7 +220,7 @@ vp_draw_arrows (gpointer data, gpointer user_data)
  * @param data       The HTTP range to draw.
  * @param user_data  A pointer to the vp_info_t structure.
  */
-static void 
+static void
 vp_draw_range (gpointer data, gpointer user_data)
 {
 	http_range_t *range = data;
@@ -234,19 +228,19 @@ vp_draw_range (gpointer data, gpointer user_data)
 
 	gdk_gc_set_foreground(v->context->gc, &available);
 	vp_draw_rectangle(v,
-		range->start, range->end, 
-        v->context->widget->allocation.height - 3, 
+		range->start, range->end,
+        v->context->widget->allocation.height - 3,
         v->context->widget->allocation.height);
 }
 
 
 
 /**
- * Draws a progress bar for the given fi struct in the DrawingArea. 
+ * Draws a progress bar for the given fi struct in the DrawingArea.
  * fih is expected to be a valid fih. Depending on the
  * value of valid the area will be drawn or cleared.
  */
-void 
+void
 vp_draw_fi_progress(gboolean valid, gnet_fi_t fih)
 {
     vp_info_t *v;
@@ -266,33 +260,33 @@ vp_draw_fi_progress(gboolean valid, gnet_fi_t fih)
 				GUINT_TO_POINTER(fih), NULL, &value);
 			g_assert(found);
 			g_assert(value);
-			
+
 			v = value;
 			v->context = &fi_context;
 
 			if (v->file_size > 0) {
-				g_slist_foreach(v->chunks_list, &vp_draw_chunk, v);
-				g_slist_foreach(v->chunks_list, &vp_draw_arrows, v);
-				g_slist_foreach(v->ranges_list, &vp_draw_range, v);
+				g_slist_foreach(v->chunks_list, vp_draw_chunk, v);
+				g_slist_foreach(v->chunks_list, vp_draw_arrows, v);
+				g_slist_foreach(v->ranges_list, vp_draw_range, v);
 			} else {
 				gdk_gc_set_foreground(fi_context.gc, &nosize);
 				gdk_draw_rectangle(fi_context.drawable, fi_context.gc, TRUE,
-								   0, 0, 
-								   fi_context.widget->allocation.width, 
+								   0, 0,
+								   fi_context.widget->allocation.width,
 								   fi_context.widget->allocation.height);
 			}
 		} else {
 			gdk_gc_set_foreground(fi_context.gc, base);
 			gdk_draw_rectangle(fi_context.drawable, fi_context.gc, TRUE,
-							   0, 0, 
-							   fi_context.widget->allocation.width, 
+							   0, 0,
+							   fi_context.widget->allocation.width,
 							   fi_context.widget->allocation.height);
 		}
 	}
 }
 
-/** 
- * Callback for the fileinfo pane GtkDrawingArea 
+/**
+ * Callback for the fileinfo pane GtkDrawingArea.
  */
 void
 on_drawingarea_fi_progress_realize(GtkWidget *widget, gpointer user_data)
@@ -311,7 +305,7 @@ on_drawingarea_fi_progress_realize(GtkWidget *widget, gpointer user_data)
 }
 
 /**
- * Callback for the fileinfo pane GtkDrawingArea
+ * Callback for the fileinfo pane GtkDrawingArea.
  */
 gboolean
 on_drawingarea_fi_progress_expose_event(
@@ -335,18 +329,26 @@ on_drawingarea_fi_progress_expose_event(
 static GSList *
 vp_get_chunks_initial(gnet_fi_t fih) {
 
-	GSList *result;
-	GSList *l;
+	GSList *result, *sl, *prev = NULL;
 
 	result = guc_fi_get_chunks(fih);
 
-	for (l = result; l; ) {
-		gnet_fi_chunks_t *chunk = (gnet_fi_chunks_t *) l->data;
-		l = g_slist_next(l);
+	for (sl = result; sl; /* NOTHING */) {
+		gnet_fi_chunks_t *chunk = sl->data;
+		GSList *next;
+
+		next = g_slist_next(sl);
 		if (DL_CHUNK_DONE != chunk->status) {
-			result = g_slist_remove(result, chunk);
-			wfree(chunk, sizeof(gnet_fi_chunks_t));
+			if (prev) {
+				prev = g_slist_delete_link(prev, sl);
+			} else {
+				result = g_slist_delete_link(result, sl);
+			}
+			wfree(chunk, sizeof *chunk);
+		} else {
+			prev = sl;
 		}
+		sl = next;
 	}
 
 	return result;
@@ -358,24 +360,24 @@ vp_get_chunks_initial(gnet_fi_t fih) {
  *
  * @param fih The fileinfo handle of the entry being added.
  */
-static void 
+static void
 vp_gui_fi_added(gnet_fi_t fih)
 {
-    gnet_fi_info_t *fi = NULL;
-    vp_info_t *new_vp_info = NULL;
+    gnet_fi_info_t *fi;
+    vp_info_t *new_vp_info;
     gnet_fi_status_t s;
 
     fi = guc_fi_get_info(fih);
     guc_fi_get_status(fih, &s);
-    
-    new_vp_info = walloc0(sizeof(*new_vp_info));
+
+    new_vp_info = walloc0(sizeof *new_vp_info);
     new_vp_info->fi_handle = fih;
-    new_vp_info->file_name = g_strdup(fi->file_name);
+    new_vp_info->file_name = atom_str_get(fi->file_name);
     new_vp_info->file_size = s.size;
     new_vp_info->chunks_list = guc_fi_get_chunks(fih);
 	new_vp_info->ranges_list = guc_fi_get_ranges(fih);
 
-	/* 
+	/*
 	 * Keep an optimized copy of the initial chunks list. Also
 	 * remember the amount of data already downloaded. This helps to
 	 * detect cheaply if the file download had restarted due to SHA1
@@ -383,18 +385,35 @@ vp_gui_fi_added(gnet_fi_t fih)
 	 */
 	new_vp_info->chunks_initial = vp_get_chunks_initial(fih);
 	new_vp_info->done_initial = s.done;
-	
+
     g_hash_table_insert(vp_info_hash, GUINT_TO_POINTER(fih), new_vp_info);
-    
+
     guc_fi_free_info(fi);
 }
 
+static void
+vp_info_free(vp_info_t **v_ptr)
+{
+	g_assert(v_ptr);
+
+	if (*v_ptr) {
+		vp_info_t *v = *v_ptr;
+		
+		guc_fi_free_chunks(v->chunks_list);
+		guc_fi_free_chunks(v->chunks_initial);
+		guc_fi_free_ranges(v->ranges_list);
+		atom_str_free_null(&v->file_name);
+		wfree(v, sizeof *v);
+		*v_ptr = NULL;
+	}
+}
+
 /**
- * Handle the event that a fileinfo entry has been removed
+ * Handle the event that a fileinfo entry has been removed.
  *
  * @param fih The fileinfo handle of the entry to be removed
  */
-static void 
+static void
 vp_gui_fi_removed(gnet_fi_t fih)
 {
     gpointer value;
@@ -405,16 +424,10 @@ vp_gui_fi_removed(gnet_fi_t fih)
 		GUINT_TO_POINTER(fih), NULL, &value);
     g_assert(found);
     g_assert(value);
+
 	v = value;
-
     g_hash_table_remove(vp_info_hash, GUINT_TO_POINTER(fih));
-
-    guc_fi_free_chunks(v->chunks_list);
-	guc_fi_free_chunks(v->chunks_initial);
-	guc_fi_free_ranges(v->ranges_list);
-	G_FREE_NULL(v->file_name);
-
-    wfree(v, sizeof(vp_info_t));
+	vp_info_free(&v);
 
     /* Forget the fileinfo handle for which we displayed progress info */
     fi_context.fih_valid = FALSE;
@@ -424,33 +437,34 @@ vp_gui_fi_removed(gnet_fi_t fih)
  * For debugging: print chunk.
  */
 static void
-vp_print_chunk(gnet_fi_chunks_t *c, gboolean show_old)
+vp_print_chunk(FILE *file, const gnet_fi_chunks_t *c, gboolean show_old)
 {
 	if (show_old)
-		printf("%10" PRIu64 " - %10" PRIu64 " %d [%s]\n",
-			(guint64) c->from, (guint64) c->to,
+		fprintf(file, "%10s - %10s %d [%s]\n",
+			uint64_to_string(c->from), uint64_to_string2(c->to),
 			(gint) c->status, c->old ? "O" : "N");
 	else
-		printf("%10" PRIu64 " - %10" PRIu64 " %d\n",
-			(guint64) c->from, (guint64) c->to, (gint) c->status);
+		fprintf(file, "%10s - %10s %d\n",
+			uint64_to_string(c->from), uint64_to_string2(c->to),
+			(gint) c->status);
 }
 
-/*
+/**
  * For debugging: print chunk list.
  */
 static void
-vp_print_chunk_list(GSList *list, gchar *title)
+vp_print_chunk_list(FILE *file, const GSList *list, const gchar *title)
 {
-	GSList *l;
+	const GSList *sl;
 
-	printf("Chunk list \"%s\":\n", title);
+	fprintf(file, "Chunk list \"%s\":\n", title);
 
-	for (l = list; l; l = g_slist_next(l)) {
-		gnet_fi_chunks_t *c = l->data;
-		vp_print_chunk(c, FALSE);
+	for (sl = list; sl; sl = g_slist_next(sl)) {
+		const gnet_fi_chunks_t *c = sl->data;
+		vp_print_chunk(file, c, FALSE);
 	}
 
-	printf("End of list \"%s\".\n", title);
+	fprintf(file, "End of list \"%s\".\n", title);
 }
 
 /**
@@ -466,8 +480,8 @@ vp_create_chunk(filesize_t from, filesize_t to,
 	enum dl_chunk_status status, gboolean old)
 {
 	gnet_fi_chunks_t *chunk;
-	
-	chunk = walloc(sizeof(gnet_fi_chunks_t));
+
+	chunk = walloc(sizeof *chunk);
 	chunk->from = from;
 	chunk->to = to;
 	chunk->status = status;
@@ -475,8 +489,8 @@ vp_create_chunk(filesize_t from, filesize_t to,
 
 #ifdef VP_DEBUG
 	printf("VP adding: ");
-	vp_print_chunk(chunk, TRUE);
-#endif
+	vp_print_chunk(stdout, chunk, TRUE);
+#endif /* VP_DEBUG */
 
 	return chunk;
 }
@@ -485,32 +499,31 @@ vp_create_chunk(filesize_t from, filesize_t to,
  * Assert that a chunks list confirms to the assumptions.
  */
 static gboolean
-vp_assert_chunks_list(GSList *list)
+vp_assert_chunks_list(const GSList *list, const gnet_fi_info_t *fi)
 {
-	GSList *l;
-	gnet_fi_chunks_t *chunk;
+	const GSList *sl;
 	filesize_t last = 0;
 
-	for (l = list; l; l = g_slist_next(l)) {
-		chunk = (gnet_fi_chunks_t *) l->data;
+	for (sl = list; sl; sl = g_slist_next(sl)) {
+		const gnet_fi_chunks_t *chunk = sl->data;
+
 		if (last != chunk->from) {
-			g_warning("**** Chunks list is not consistent! ");
-			vp_print_chunk_list(list, "Chunks list");
+			g_warning("BAD CHUNK LIST for \"%s\"", fi->file_name);
+			vp_print_chunk_list(stderr, list, "Chunks list");
 			return FALSE;
-			break;
 		}
 		last = chunk->to;
 	}
 	return TRUE;
 }
 
-/** 
- * Fileinfo has been changed for a file. Update the information and 
+/**
+ * Fileinfo has been changed for a file. Update the information and
  * draw the information so the changes are visible.
- * 
+ *
  * @param fih Handle for fileinfo data that has been changed.
  */
-static void 
+static void
 vp_gui_fi_status_changed(gnet_fi_t fih)
 {
     vp_info_t *v;
@@ -543,9 +556,8 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 	if (v->done_initial > s.done) {
 		guc_fi_free_chunks(v->chunks_initial);
 		v->chunks_initial = vp_get_chunks_initial(fih);
-		v->done_initial = s.done; 
+		v->done_initial = s.done;
 	}
-	guc_fi_free_info(fi);
 
 	/*
 	 * Use the new chunks list to create a composite with the initial
@@ -556,25 +568,23 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 	new = guc_fi_get_chunks(fih);
 	keep_new = new;	/* So that we can free this list later */
 
-	/* 
+	/*
 	 * Check if this chunks list is valid. If not then skip building a
-	 * new list, and do housekeeping stuff here. We may get an invalid
-	 * list due to an obscure bug in the fileinfo core which has not
-	 * been tracked down yet: #1065382
+	 * new list, and do housekeeping stuff here.
 	 */
-	if (! vp_assert_chunks_list(new)) {
+	if (!vp_assert_chunks_list(new, fi)) {
 		guc_fi_free_chunks(keep_new);
-		return;
+		goto cleanup;
 	}
-		
+
 	guc_fi_free_chunks(v->chunks_list);
 	v->chunks_list = NULL;
 
 	while (old || new) {
 		if (old && new) {
-			oc = (gnet_fi_chunks_t *) old->data;
-			nc = (gnet_fi_chunks_t *) new->data;
-			
+			oc = old->data;
+			nc = new->data;
+
 			/*
 			 * Skip over chunks below the highest mark, they are no longer
 			 * relevant.
@@ -593,23 +603,17 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 			}
 
 			/*
-			 * Check for contiguous list: emit debug info if not
-			 * correct. FIXME: should become an assert once the
-			 * algorithm is stable.
+			 * Check for contiguous list. A failed assertion here
+			 * indicates problems in the algorithm.
 			 */
-			if (nc->from != highest) {
-				g_warning("Visual progress list corruption!");
-				vp_print_chunk_list(v->chunks_initial, "Old");
-				vp_print_chunk_list(keep_new, "New");
-				vp_print_chunk_list(v->chunks_list, "Result");
-			}
+			g_assert(nc->from == highest);
 
 			/*
 			 * The chunks are identical: nothing changed, copy one chunk
 			 */
 			if (oc->from == nc->from && oc->to == nc->to) {
 				highest = nc->to;
-				v->chunks_list = g_slist_append(v->chunks_list, 
+				v->chunks_list = g_slist_append(v->chunks_list,
 				    vp_create_chunk(nc->from, nc->to, nc->status, TRUE));
 				old = g_slist_next(old);
 				new = g_slist_next(new);
@@ -623,7 +627,7 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 			 */
 			if (oc->from >= nc->to) {
 				highest = nc->to;
-				v->chunks_list = g_slist_append(v->chunks_list, 
+				v->chunks_list = g_slist_append(v->chunks_list,
  					vp_create_chunk(nc->from, nc->to, nc->status, FALSE));
 				new = g_slist_next(new);
 				continue;
@@ -646,11 +650,11 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 			 * because we had to back out some data, e.g. on a resume
 			 * mismatch.
 			 */
-			if (oc->from >= nc->from && oc->to <= nc->to) 
+			if (oc->from >= nc->from && oc->to <= nc->to)
 				v->chunks_list = g_slist_append(v->chunks_list,
 				    vp_create_chunk(oc->from, oc->to, oc->status, TRUE));
 			else {
-				/* 
+				/*
 				 * The new chunk we are considering covers a smaller
 				 * area than the old chunk, so some data got lost. To
 				 * cope with this we follow two paths.
@@ -664,7 +668,7 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 					v->chunks_list = g_slist_append(v->chunks_list,
 						vp_create_chunk(nc->from, oc->to, nc->status, TRUE));
 				else {
-					/* 
+					/*
 					 * In this case we copy the chunk and skip
 					 * it. Depending on the situation the next
 					 * iteration will deal with the remainder of this
@@ -683,7 +687,7 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 				v->chunks_list = g_slist_append(v->chunks_list,
 					vp_create_chunk(oc->to, nc->to, nc->status, FALSE));
 			}
-			
+
 			old = g_slist_next(old);
 			new = g_slist_next(new);
 		} else {
@@ -692,43 +696,40 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 			 * the chunks list.
 			 */
 			if (old) {
-				oc = (gnet_fi_chunks_t *) old->data;
+				oc = old->data;
 				v->chunks_list = g_slist_append(v->chunks_list,
 					vp_create_chunk(oc->from, oc->to, oc->status, TRUE));
 				old = g_slist_next(old);
 			}
-				
+
 			if (new) {
-				nc = (gnet_fi_chunks_t *) new->data;
+				nc = new->data;
 				v->chunks_list = g_slist_append(v->chunks_list,
 					vp_create_chunk(nc->from, nc->to, nc->status, FALSE));
 				new = g_slist_next(new);
 			}
 		}
 	}
-	
-	/* 
+
+	/*
 	 * Now that the new list of chunks is merged with the old chunks we
 	 * can safely throw away the new list itself.
 	 */
 	guc_fi_free_chunks(keep_new);
 
-	/* 
-	 * This code is not needed right now because we already dump
-	 * debugging info as soon as we detect a problem.
-	 * vp_assert_chunks_list(v->chunks_list); 
-     */
+cleanup:
+	guc_fi_free_info(fi);
 }
 
 
-/** 
+/**
  * The available ranges information has been changed for a
  * file. Update the information and draw the information so the
  * changes are visible.
- * 
+ *
  * @param fih Handle for fileinfo data that has been changed.
  */
-static void 
+static void
 vp_gui_fi_ranges_changed(gnet_fi_t fih)
 {
     vp_info_t *v;
@@ -742,7 +743,7 @@ vp_gui_fi_ranges_changed(gnet_fi_t fih)
 	v = value;
 
 	/*
-	 * Copy the ranges. These can simply be copied as we do not need to 
+	 * Copy the ranges. These can simply be copied as we do not need to
 	 * apply our own logic to them.
 	 */
 	guc_fi_free_ranges(v->ranges_list);
@@ -751,41 +752,40 @@ vp_gui_fi_ranges_changed(gnet_fi_t fih)
 
 
 /**
- * Free the vp_info_t structs in the vp_info_hash
+ * Free the vp_info_t structs in the vp_info_hash.
  */
-void 
-vp_free_key_value (gpointer key, gpointer value, gpointer user_data)
+void
+vp_free_key_value(gpointer key, gpointer value, gpointer user_data)
 {
+	vp_info_t *v = value;
+
 	(void) key;
 	(void) user_data;
-    guc_fi_free_chunks(((vp_info_t *) value)->chunks_list);
-	guc_fi_free_chunks(((vp_info_t *) value)->chunks_initial);
-	guc_fi_free_ranges(((vp_info_t *) value)->ranges_list);
-	G_FREE_NULL(((vp_info_t *) value)->file_name);
-    wfree(value, sizeof(vp_info_t));
+
+	vp_info_free(&v);
 }
 
 /**
  * Initialize the use of visual progress.
-
+ *
  * Register listeners into the fileinfo structure so that we are
  * notified of fileinfo events, and get a permanent handle to the
  * drawing area for later reuse.
  */
-void 
-vp_gui_init(void) 
+void
+vp_gui_init(void)
 {
     GdkColormap *cmap;
 
     vp_info_hash = g_hash_table_new(NULL, NULL);
 
-    guc_fi_add_listener(vp_gui_fi_added, EV_FI_ADDED, 
+    guc_fi_add_listener(vp_gui_fi_added, EV_FI_ADDED,
 		FREQ_SECS, 0);
-    guc_fi_add_listener(vp_gui_fi_removed, EV_FI_REMOVED, 
+    guc_fi_add_listener(vp_gui_fi_removed, EV_FI_REMOVED,
 		FREQ_SECS, 0);
-    guc_fi_add_listener(vp_gui_fi_status_changed, 
+    guc_fi_add_listener(vp_gui_fi_status_changed,
 		EV_FI_STATUS_CHANGED, FREQ_SECS, 0);
-    guc_fi_add_listener(vp_gui_fi_status_changed, 
+    guc_fi_add_listener(vp_gui_fi_status_changed,
 		EV_FI_STATUS_CHANGED_TRANSIENT, FREQ_SECS, 0);
     guc_fi_add_listener(vp_gui_fi_ranges_changed,
 						EV_FI_RANGES_CHANGED, FREQ_SECS, 0);
@@ -816,29 +816,26 @@ vp_gui_init(void)
 }
 
 /**
- * Undo everything set up in vp_gui_init
+ * Undo everything set up in vp_gui_init.
  */
-void 
+void
 vp_gui_shutdown(void)
 {
     guc_fi_remove_listener(vp_gui_fi_removed, EV_FI_REMOVED);
     guc_fi_remove_listener(vp_gui_fi_added, EV_FI_ADDED);
-    guc_fi_remove_listener(vp_gui_fi_status_changed, 
-		EV_FI_STATUS_CHANGED);
-    guc_fi_remove_listener(vp_gui_fi_status_changed, 
+    guc_fi_remove_listener(vp_gui_fi_status_changed, EV_FI_STATUS_CHANGED);
+    guc_fi_remove_listener(vp_gui_fi_status_changed,
 		EV_FI_STATUS_CHANGED_TRANSIENT);
     guc_fi_remove_listener(vp_gui_fi_ranges_changed, EV_FI_RANGES_CHANGED);
 
     g_hash_table_foreach(vp_info_hash, vp_free_key_value, NULL);
     g_hash_table_destroy(vp_info_hash);
+	vp_info_hash = NULL;
 }
 
-
-
-
-/* 
+/*
  * Local Variables:
  * tab-width:4
  * End:
- * vi: set ts=4:
+ * vi: set ts=4 sw=4 cindent:
  */
